/******************************************************************************
*                _  _               ___             _                         *
*               (_)| |__   _ __    / _ \ _ __ ___  | |__    /\  /\            *
*               | || '_ \ | '__|  / /_\/| '_ ` _ \ | '_ \  / /_/ /            *
*               | || | | || |    / /_\\ | | | | | || |_) |/ __  /             *
*               |_||_| |_||_|    \____/ |_| |_| |_||_.__/ \/ /_/              *
*                                                                             *
*    ihr GmbH                                                                 *
*    Airport Boulevard B210                                                   *
*    77836 Rheinmnster - Germany                                             *
*    http://www.ihr.de                                                        *
*    Phone +49(0) 7229-18475-0                                                *
*    Fax   +49(0) 7229-18475-11                                               *
*                                                                             *
*******************************************************************************
*                                                                             *
* (c) Alle Rechte bleiben bei IHR GmbH, auch fuer den Fall von Schutzrechts-  *
* anmeldungen. Jede Verfuegungsbefugnis, wie Kopier- und Weitergaberecht      *
* bleibt bei IHR GmbH.                                                        *
*                                                                             *
* (c) All rights reserved by IHR GmbH including the right to file             *
* industrial property rights. IHR GmbH retains the sole power of              *
* disposition such as reproduction or distribution.                           *
*                                                                             *
********************     Workfile:      lin_driver_api.c **********************
*                                                                             *
*  PROJECT-DESCRIPTION:  LIN Driver Protocol Layer                            *
*  FILE-DESCRIPTION:     All routines for access from application to          *
*                        LIN driver and callback function from LIN driver     *
*                        to application                                       *
*                                                                             *
*******************************************************************************
*                                                                             *
*       Revision:        $Rev:: 107         $                                 *
*       Responsible:     B.Roegl                                              *
*       Co-Responsible:  P.Koch                                               *
*       Last Modtime:    $Date:: 2016-03-31#$                                 *
*                                                                             *
******************************************************************************/
/**
@file    lin_driver_api.c
@brief   Application Program Interface for the LIN Driver access
*/

/* ===========================================================================
 *  Header files
 * ==========================================================================*/

#include "genLinConfig.h"
#include "lin_slave_task.h"
#include "lin_driver_api.h"



/* ===========================================================================
 *  Global Variables
 * ==========================================================================*/

/* ===========================================================================
 *  Functions
 * ==========================================================================*/

/* ---------------------------------------------------------------------------
 *  l_bool l_sys_init (void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Initialise LIN driver system.
   @pre    MCU started
   @param  void
   @retval void
*/
l_bool l_sys_init (void)
{
   lin_main_init();
   return 0u;
}

#ifdef J2602_SLAVE

/* ---------------------------------------------------------------------------
 *  l_u8* j2602_get_bc_data(l_u8 frame)
 * --------------------------------------------------------------------------*/
/**
   @brief  Read the data of a received broadcastframe
   @pre    LIN driver initialized, broadcast frame received.
   @param  0 Read data from frame zero
   @param  1 Read data from frame one
   @retval l_u8* pointer to data buffer of frame.
*/
/* (MISRA-C:2004 8.10/R, e765) external ... could be made static
 * Reason: Part of API, configuration dependent usage. */
l_u8* j2602_get_bc_data(l_u8 frame)
{
   if (frame == 0u) {
      l_flg_clr_new_bc_data0();
   } else {
      l_flg_clr_new_bc_data1();
   }
    return &j2602_broadcast_bytes[frame][0];
}

#endif /* end #ifdef J2602_SLAVE */

#ifdef LIN_PROTOCOL_VERSION_2_1
/* ---------------------------------------------------------------------------
 *  l_u8 ld_read_configuration (l_u8* const data,l_u8* const length)
 * --------------------------------------------------------------------------*/
/**
   @brief  Fill configuration data to data pointer from application.
   @pre    LIN driver initialized
   @param  l_u8* const data     pointer to application buffer for configuration data
   @param  l_u8* const length   pointer to length information for buffer
   @retval LD_READ_OK           Service was successful
   @retval LD_LENGTH_TOO_SHORT  Configuration size is greater than buffer size, no
                                valid configuration data contained in buffer.
*/
/* (MISRA-C:2004 8.10/R, e765) external ... could be made static
 * Reason: Part of API, configuration dependent usage. */
l_u8 ld_read_configuration (l_u8* const data,l_u8* const length)
{
   l_u8  loc_i = 0u;
   l_u8 ret_val;
   /* (MISRA-C:2004 11.4/A) A cast should not be performed between a pointer to object type and a different pointer to object type
    * Reason: make the pointer (req. by LIN specification) accessible as array. */
   l_u8 (*l_api_tmp_arr)[] = (l_u8 (*)[])data; /*lint !e927*/
   if(*length < (LIN_NUMBER_OF_FRAMES + 1u))
   {
      ret_val = LD_LENGTH_TOO_SHORT;
   }
   else
   {
      *length = LIN_NUMBER_OF_FRAMES + 1u;
      (*l_api_tmp_arr)[0] = g_lin_prod_id.NAD;
      for(loc_i = 0u; loc_i < LIN_NUMBER_OF_FRAMES; loc_i++)
      {
         (*l_api_tmp_arr)[loc_i + 1u] = g_lin_frame_ctrl[loc_i].frame.pid;
      }
      ret_val = LD_READ_OK;
   }
   return ret_val;
}

/* ---------------------------------------------------------------------------
 *  l_u8 ld_set_configuration (l_u8* const data,l_u8* const length)
 * --------------------------------------------------------------------------*/
/**
   @brief  Set configuration with to data from application.
   @pre    LIN driver initialized
   @param  l_u8* const data       pointer to application buffer for configuration data
   @param  l_u8* const length     pointer to length information for buffer
   @retval LD_SET_OK              Service was successful
   @retval LD_LENGTH_NOT_CORRECT  Configuration size is not equal to buffer size, no
                                  valid configuration data contained in buffer.
   @retval LD_DATA_ERROR          Service was not successful
*/
/* (MISRA-C:2004 8.10/R, e765) external ... could be made static
 * Reason: Part of API, configuration dependent usage. */
l_u8 ld_set_configuration (const l_u8* const data,l_u8* const length)
{
   l_u8 loc_i = 0u;
   l_u8 ret_val;
   /* (MISRA-C:2004 11.5/R) A cast shall not be performed that removes any const or volatile qualification from the type addressed by a pointer
    * Reason: make the pointer (req. by LIN specification) accessible as array. */
   const l_u8 (*l_api_tmp_arr)[] = (const l_u8 (*)[])data; /*lint !e927*/
   if(*length != (l_u8)(LIN_NUMBER_OF_FRAMES + 1u))
   {
      ret_val = LD_LENGTH_NOT_CORRECT;
   }
   else
   {
      g_lin_prod_id.NAD = (*l_api_tmp_arr)[0];
      for(loc_i = 0u; loc_i < LIN_NUMBER_OF_FRAMES; loc_i++)
      {
         g_lin_frame_ctrl[loc_i].frame.pid = (*l_api_tmp_arr)[loc_i + 1u];
      }
      ret_val = LD_SET_OK;
   }
   return ret_val;
}
#endif /* end #ifdef LIN_PROTOCOL_VERSION_2_1 */

#if ((defined LIN_MASTER)||(defined J2602_MASTER))
/* ---------------------------------------------------------------------------
 *  l_u8 l_sch_tick (void)
 * --------------------------------------------------------------------------*/
/**
  @brief    Periodic check of LIN scheduletable and to required action

    The function follows a schedule. When a frame becomes due,
    its transmission is initiated. When the end of the current schedule
    is reached, l_sch_tick starts again at the beginning of the schedule.
    The l_sch_tick must be called periodically and individually for each
    interface within the node. The period is the time base set in the LDF.
    The period of the l_sch_tick call effectively sets the time base tick.
    Therefore it is essential that the time base period is uphold with
    minimum jitter. The call to l_sch_tick will not only start the
    transition of the next frame due, it will also update the signal
    values for those signals received since the previous call to l_sch_tick.
  @pre     LIN driver initialised
  @param   void
  @return  void
*/
l_u8 l_sch_tick (void)
{
   l_u8  retval = 0u;

   if(!g_lin_zero_schedule)
   {
      /* First check if schedule table has to be switched to another schedule table
       * After sending an Goto Sleep command, do not switch to a new schedule table */
      if((g_ScheduleIdx != g_nextScheduleIdx) || (g_nextScheduleFrmIdx != 0xFFu))
      {
         g_ScheduleIdx =  g_nextScheduleIdx;
         g_activeScheduleIdx = g_ScheduleIdx;
         g_ScheduleFrmIdx = g_nextScheduleFrmIdx;
         g_nextScheduleFrmIdx = 0xFFu;
         if ((*scheduleList[g_activeScheduleIdx]) == 0)
         {
            g_lin_zero_schedule = L_SET;
         }
      }
      /* check if its time to send next frame header */
      if (!g_lin_zero_schedule)
      {
         if(g_lin_sched_data.lin_current_delay == 0u)
         {
            /* Finish previous frame via timeout procedure to ensure every state is correctly reset upon starting new frame header */
            lin_hal_wait_for_break();

            /*  Start next Frame in Schedule */
            lin_master_task_tx_header((l_u8)(*scheduleList[g_activeScheduleIdx])[g_ScheduleFrmIdx].pid);

            /*  Get the delay until the next Frame */
            g_lin_sched_data.lin_current_delay = (*scheduleList[g_activeScheduleIdx])[g_ScheduleFrmIdx].frame_delay;


            /*  Prepare for next Frame transmission */
            /* get next frame in schedule */
            g_ScheduleFrmIdx ++;
            /* cycle in schedule */
            g_ScheduleFrmIdx %= (schedTblSizeList[g_ScheduleIdx]);

            /* reduce lin_current_delay with main_cyclic_looptime(ms) in 100s */
            g_lin_sched_data.lin_current_delay -=(10u * (LIN_TASK_CYCLE_MS));
         }
         else /* no frame valid this time */
         {
            /* reduce lin_current_delay with main_cyclic_looptime(ms) in 100s
             * and prevent underflow */
            retval = 0u;
            if (g_lin_sched_data.lin_current_delay <= (10u * (LIN_TASK_CYCLE_MS)))
            {
               g_lin_sched_data.lin_current_delay = 0u;
               retval = g_ScheduleFrmIdx + 1u;
            }
            else
            {
               g_lin_sched_data.lin_current_delay -= (10u * (LIN_TASK_CYCLE_MS));
            }
         }
      } /* end if active schedule != 0 */
   }
   return retval;
}
#endif /* #if ((defined LIN_MASTER)||(defined J2602_MASTER)) */


#if ((defined LIN_MASTER)||(defined J2602_MASTER))
/* ---------------------------------------------------------------------------
 *  void l_sch_set (l_u16 schedule, l_u8 entry)
 * --------------------------------------------------------------------------*/
/**
  @brief   The function sets a new schedule in the schedule management
  @pre     LIN driver initialised
  @param   schedule  The new schedule table
  @param   entry  The next frame in new schedule table which had to be executed
  @return  void
*/
void l_sch_set (l_u16 schedule, l_u8 entry)
{
   if(schedule < NumberOfSchedTables)
   {
      g_nextScheduleIdx = schedule;
      g_nextScheduleFrmIdx = entry;
      g_lin_zero_schedule = L_RESET;
   }
   else
   {
      g_lin_zero_schedule = L_SET;
   }
}

void l_sch_change_pid (l_u8 old_pid, l_u8 new_pid)
{
   l_u8 loc_j, loc_k;
   for (loc_j = 0u; loc_j < NumberOfSchedTables; loc_j++)
   {
      for (loc_k = 0u; loc_k < schedTblSizeList[loc_j]; loc_k ++)
      {
         if ((*scheduleList[loc_j])[loc_k].pid == old_pid)
         {
            (*scheduleList[loc_j])[loc_k].pid = new_pid;
         }
      }
   }
}

#endif /* #if ((defined LIN_MASTER)||(defined J2602_MASTER)) */

/* ---------------------------------------------------------------------------
 *  l_u16 l_ifc_read_status(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Get status information from LIN driver.
   @pre    LIN driver initialized
   @param  void
   @retval LIN_status word        MSB contains last PID information
                                  LSB contains status flags
   @see    LIN 2.0 and LIN 2.1 specification
*/
l_u16 l_ifc_read_status(void)
{
   l_u16 loc_temp;

   loc_temp = g_lin_status_word.reg;
   g_lin_status_word.reg = 0u;
   return loc_temp;
}

#ifdef J2602_SLAVE
/* ---------------------------------------------------------------------------
 *  l_bool l_flg_tst_Reset(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Get information if a reset request has been received.
   @pre    LIN driver initialized
   @param  void
   @retval false (zero)        No request has been received.
           true (non-zero)     Reset request has been received.
*/
l_bool l_flg_tst_Reset(void)
{
   return (l_bool)g_j2602_status.flags.reset;
}

/* ---------------------------------------------------------------------------
 *  l_bool l_flg_clr_Reset(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Clear information if a reset request has been received.
   @pre    LIN driver initialized
   @param  void
   @retval void
*/
void l_flg_clr_Reset(void)
{
   g_j2602_status.flags.reset = 0u;
}

/* ---------------------------------------------------------------------------
 *  l_bool l_flg_tst_new_bc_data0(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Get information if a new broadcast frame with ID 0x38 or 0x3A has 
           been received.
   @pre    LIN driver initialized
   @param  void
   @retval false (zero)        No broadcast data in such a frame has been received.
           true (non-zero)     Broadcast data has been received.
*/
l_bool l_flg_tst_new_bc_data0(void)
{
   return (l_bool)g_j2602_status.flags.new_frame0;
}

/* ---------------------------------------------------------------------------
 *  l_bool l_flg_clr_new_bc_data0(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Clear information if a new broadcast frame with ID 0x38 or 0x3A has 
           been received.
   @pre    LIN driver initialized
   @param  void
   @retval void
*/
void l_flg_clr_new_bc_data0(void)
{
   g_j2602_status.flags.new_frame0 = 0u;
}

/* ---------------------------------------------------------------------------
 *  l_bool l_flg_tst_new_bc_data1(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Get information if a new broadcast frame with ID 0x39 or 0x3B has 
           been received.
   @pre    LIN driver initialized
   @param  void
   @retval false (zero)        No broadcast data in such a frame has been received.
           true (non-zero)     Broadcast data has been received.
*/
l_bool l_flg_tst_new_bc_data1(void)
{
   return (l_bool)g_j2602_status.flags.new_frame1;
}

/* ---------------------------------------------------------------------------
 *  l_bool l_flg_clr_new_bc_data1(void)
 * --------------------------------------------------------------------------*/
/**
   @brief  Clear information if a new broadcast frame with ID 0x39 or 0x3B has 
           been received.
   @pre    LIN driver initialized
   @param  void
   @retval void
*/
void l_flg_clr_new_bc_data1(void)
{
   g_j2602_status.flags.new_frame1 = 0u;
}

#endif /* ifdef J2602_SLAVE */
