/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-08-16, BG:   EP-731: Initial version of the revision history  **
** V1.0.1: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Flash: Program a page when the write protection is set                     **
********************************************************************************
** Read the customer id to determine the design step.                         **
** Depending on the design step, the correct method is applied to set the     **
** protection.                                                                **
** A page write is performed in the protected NVM region. This operation      **
** should failed due to the protection.                                       **
** LED_P1_2/LED_P1_4 display pass or fail of this operation.                  **
*******************************************************************************/

#include "tle_device.h"

/* allocate one flash page in RAM */
static uint8 TargetData[FlashPageSize];
static int32_t NVM_Result;

int main(void)
{
  int i;

  /* Initialization of hardware modules based on Config Wizard configuration */ 
  TLE_Init();

  /* Setting the NVM write protection for the data flash           
   * The password (here 0x400000000) is encoded the following way: 
   * bit[31] = 1 => enable read protection (here '0')              
   * bit[30] = 1 => enable write protection (here '1')             
   * bit[29:0] = user password (here all '0')                      
   * The password has to be set either through a BSL command,      
   * or with the user API "user_nvm_password_set"                  
   * If no password was set before, then the password along with   
   * the function "user_nvm_protect_set" is ignored.               
   * The Bits[31:30] are processed in any case. The protection     
   * will then be set temporarily only.                            */
  WDT1_SOW_Service(1);
  
  NVM_Result = (BOOTROM_RETURN_CODE_t)user_nvm_protect_set(0x40000000 , NVM_PASSWORD_SEGMENT_DATA);

  (void)WDT1_Service();
  
  /* Fill up the TargetData array with housenumbers */
  for (i = 0; i < FlashPageSize; i++)
  {
    TargetData[i] = (uint8) i;
  }

  /* Call ProgramPage routine
   * In case of write protection being enabled, the ProgramPage routine returns an error. */
  
  /* Disable interrupts */
  __disable_irq();
  /* Service WDT1 short open window */
  WDT1_SOW_Service(1);
  /* Program NVM page */
  NVM_Result = (BOOTROM_RETURN_CODE_t)user_nvm_write(DataFlashStart, TargetData, FlashPageSize, 0);
  /* Main watchdog service: close short open window */
  (void)WDT1_Service();
  /* Enable interrupts */
  __enable_irq();

  switch (NVM_Result)
  {
    case ERR_LOG_SUCCESS:
    {
      /* Data in Data Flash page 0 correct */
      PORT_P12_Output_High_Set();
    } break;
    case ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED:
    {
      /* NVM is write protected */
      PORT_P14_Output_High_Set();
    } break;
    default:
    {
      /* A not further decoded error has happened */
    } break;
  }
  
  while (1)
  {
    /* Main watchdog service */
    (void)WDT1_Service();

  }
}
