/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     adc2.h
 *
 * \brief    ADC2 low level access library
 *
 * \version  V0.2.1
 * \date     13. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-10-23, AV:   Initial version                                  **
** V0.1.1: 2015-11-18, AV:   Removed unnecessary registers from init routine  **
** V0.1.4: 2017-10-20, KC:   Macros for interrupt register access added       **
** V0.1.5: 2017-11-10, KC:   Calcualtions of ADC2_GetChResult_mVC() corrected **
**                           Return type of ADC2_GetChResult_mVC() changed to **
**                           sint16                                           **
**                           ADC2_GetChResult() converted to non-inline       **
**                           function                                         **
** V0.1.6: 2017-12-21, KC:   Calculations of ADC2_GetChResult_mVC() improved  **
** V0.1.7: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.8: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           ADC2_Init Updated (FILT_LO_CTRL and FILT_UP_CTRL **
**                           initialized)                                     **
** V0.1.9: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to adc2.c                                        **
** V0.2.0: 2021-10-07, BG:   EP-931: Corrected ARMCC compiler v6 errors       **
** V0.2.1: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

#ifndef ADC2_H
#define ADC2_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief ADC2 channel selection macro, channel 0 - VS */
#define ADC2_RESULT_VS          ((uint16)0)        
/**\brief ADC2 channel selection macro, channel 1 - VDDEXT */
#define ADC2_RESULT_VDDEXT      ((uint16)1)        
/**\brief ADC2 channel selection macro, channel 2 - VDDP */
#define ADC2_RESULT_VDDP        ((uint16)2)        
/**\brief ADC2 channel selection macro, channel 3 - VBG */
#define ADC2_RESULT_VBG         ((uint16)3)        
/**\brief ADC2 channel selection macro, channel 4 - VDDC */
#define ADC2_RESULT_VDDC        ((uint16)4)        
/**\brief ADC2 channel selection macro, channel 5 - TEMP1 */
#define ADC2_RESULT_TEMP1       ((uint16)5)        
/**\brief ADC2 channel selection macro, channel 6 - TEMP2 */
#define ADC2_RESULT_TEMP2       ((uint16)6)        

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void ADC2_Init(void);
uint16 ADC2_GetChResult(uint8 channel);
sint16 ADC2_GetChResult_mVC(uint8 channel);
INLINE void ADC2_VS_OV_Int_Clr(void);
INLINE void ADC2_VS_UV_Int_Clr(void);
INLINE void ADC2_VDDEXT_OV_Int_Clr(void);
INLINE void ADC2_VDDEXT_UV_Int_Clr(void);
INLINE void ADC2_VDDP_OV_Int_Clr(void);
INLINE void ADC2_VDDP_UV_Int_Clr(void);
INLINE void ADC2_VBG_OV_Int_Clr(void);
INLINE void ADC2_VBG_UV_Int_Clr(void);
INLINE void ADC2_VDDC_OV_Int_Clr(void);
INLINE void ADC2_VDDC_UV_Int_Clr(void);
INLINE void ADC2_TEMP1_OT_Int_Clr(void);
INLINE void ADC2_TEMP1_WARN_Int_Clr(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/* ADC2 Interrupt Clear */
/** \brief Clear VS Overvoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VS Overvoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VS_OV_IE == 1u)
 *   {
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_OV_IS == 1u)
 *     {
 *       ADC2_VS_UP_CALLBACK();
 *       ADC2_VS_OV_Int_Clr();
 *     }
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VS_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VS_OV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VS_OV_ISC_Msk, 1u);
}

/** \brief Clear VS Undervoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VS Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VS_UV_IE == 1u)
 *   { 
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_UV_IS == 1u)
 *     {
 *       ADC2_VS_LO_CALLBACK();
 *       ADC2_VS_UV_Int_Clr();
 *     }
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VS_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VS_UV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VS_UV_ISC_Msk, 1u);
}

/** \brief Clear VDDEXT Overvoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDEXT Overvoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDDEXT_OV_IE == 1u)
 *   {
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDDEXT_OV_IS == 1u)
 *     {
 *       ADC2_VDDEXT_UP_CALLBACK();
 *       ADC2_VDDEXT_OV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDEXT_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDDEXT_OV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDDEXT_OV_ISC_Msk, 1u);
}

/** \brief Clear VDDEXT Undervoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDEXT Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDDEXT_UV_IE == 1u)
 *   {
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDDEXT_UV_IS == 1u)
 *     {
 *       ADC2_VDDEXT_LO_CALLBACK();
 *       ADC2_VDDEXT_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDEXT_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDDEXT_UV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDDEXT_UV_ISC_Msk, 1u);
}

/** \brief Clear VDDP Overvoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDP Overvoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDD5V_OV_IE == 1u)
 *   {
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDD5V_OV_IS == 1u)
 *     {
 *       ADC2_VDDP_UP_CALLBACK();
 *       ADC2_VDDP_OV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDP_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDD5V_OV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDD5V_OV_ISC_Msk, 1u);
}

/** \brief Clear VDDP Undervoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDP Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDD5V_UV_IE == 1u)
 *   {
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDD5V_UV_IS == 1u)
 *     {
 *       ADC2_VDDP_LO_CALLBACK();
 *       ADC2_VDDP_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDP_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDD5V_UV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDD5V_UV_ISC_Msk, 1u);
}

/** \brief Clear 8 Bit ADC2 Reference Overvoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the 8 Bit ADC2 Reference Overvoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_IRQ_CTRL.bit.REFBG_UPTHWARN_IE == 1u)
 *   { 
 *     if (MF->REF1_STS.bit.REFBG_UPTHWARN_STS == 1u)
 *     {
 *       ADC2_VBG_UP_CALLBACK();
 *       ADC2_VBG_OV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VBG_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint8)SCUPM_SYS_ISCLR_REFBG_UPTHWARN_ISC_Pos, SCUPM_SYS_ISCLR_REFBG_UPTHWARN_ISC_Msk, 1u);
}

/** \brief Clear 8 Bit ADC2 Reference Undervoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the 8 Bit ADC2 Reference Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_IRQ_CTRL.bit.REFBG_LOTHWARN_IE == 1u)
 *   {
 *     if (MF->REF1_STS.bit.REFBG_LOTHWARN_STS == 1u)
 *     {
 *       ADC2_VBG_LO_CALLBACK();
 *       ADC2_VBG_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VBG_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint8)SCUPM_SYS_ISCLR_REFBG_LOTHWARN_ISC_Pos, SCUPM_SYS_ISCLR_REFBG_LOTHWARN_ISC_Msk, 1u);
}

/** \brief Clear VDDC Overvoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDC Overvoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDD1V5_OV_IE == 1u)
 *   { 
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDD1V5_OV_IS == 1u)
 *     {
 *       ADC2_VDDC_UP_CALLBACK();
 *       ADC2_VDDC_OV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDC_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDD1V5_OV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDD1V5_OV_ISC_Msk, 1u);
}

/** \brief Clear VDDC Undervoltage interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDC Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VDD1V5_UV_IE == 1u)
 *   { 
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VDD1V5_UV_IS == 1u)
 *     {
 *       ADC2_VDDC_LO_CALLBACK();
 *       ADC2_VDDC_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_VDDC_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VDD1V5_UV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VDD1V5_UV_ISC_Msk, 1u);
}

/** \brief Clear System Overtemperature Shutdown interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the System Overtemperature Shutdown interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_IRQ_CTRL.bit.SYS_OT_IE == 1u)
 *   { 
 *     if (SCUPM->SYS_IS.bit.SYS_OT_IS == 1u)
 *     {
 *       ADC2_SYS_TEMP_UP_CALLBACK();
 *       ADC2_TEMP1_OT_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_TEMP1_OT_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint8)SCUPM_SYS_ISCLR_SYS_OT_ISC_Pos, SCUPM_SYS_ISCLR_SYS_OT_ISC_Msk, 1u);
}

/** \brief Clear System Overtemperature Prewarning interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the System Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_IRQ_CTRL.bit.SYS_OTWARN_IE == 1u)
 *   {
 *     if (SCUPM->SYS_IS.bit.SYS_OTWARN_IS == 1u)
 *     {
 *       ADC2_SYS_TEMP_LO_CALLBACK();
 *       ADC2_TEMP1_WARN_Int_Clr();
 *     }
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */
INLINE void ADC2_TEMP1_WARN_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint8)SCUPM_SYS_ISCLR_SYS_OTWARN_ISC_Pos, SCUPM_SYS_ISCLR_SYS_OTWARN_ISC_Msk, 1u);
}

#endif

