/**
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     bootrom.h
 *
 * \brief    BootROM low level access library
 *
 * \version  V1.5.6
 * \date     07. Apr 2022
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
** VO           Vanessa Ongaro                                                **
*******************************************************************************/
 
/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2015-01-28, xx:   Initial version matching to Preliminary Firmware **
**                           User Manual Rev0.1 from 2014-11-14               **
** V1.1.0: 2015-08-14, xx:   Update reflecting AB step silicon                **
**                           Removed user_nvm_page_invalid                    **
** V1.2.0: 2015-08-14, xx:   Update reflecting AC step silicon                **
**                           Added user_nvm_mapram_init                       **
**                           Added user_vbg_temperature_get                   **
**                           Added user_ram_init                              **
**                           Changed return type of user_nvm_clk_factor_set   **
** V1.3.0: 2015-09-22, xx:   Added typedef VBG_TEMP_SELECT_t                  **
** V1.4.0: 2016-01-21, xx:   Update reflecting AD and AE step silicon         **
**                           Removed user_ram_init (patch)                    **
** V1.5.0: 2017-01-24, xx:   BootROM error codes added, flash page size,      **
**                           Flash sector size added                          **
** V1.5.1: 2018-11-28, xx:   DoxyGen docu added                               **
**                           Error Codes list cleaned up                      **
** V1.5.2: 2019-03-18, xx:   Unit Testing Macro Condition Added               **
** V1.5.3: 2021-08-16, BG:   EP-731: Updated revision history layout          **
** V1.5.4: 2021-10-07, BG:   EP-754: Added documentation for bootrom types    **
**                           and functions                                    **
** V1.5.5: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V1.5.6: 2022-04-07, VO:   EP-1085: Added corrections to avoid MISRA        **
**                           violations                                       **
*******************************************************************************/

#ifndef _BOOTROM_H
#define _BOOTROM_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/** \brief Size of a flash page, 128 bytes */
#define FlashPageSize               (128U)
/** \brief Size of a flash sector, 32 page á 128 bytes */
#define FlashSectorSize             (4096U)

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \enum BOOTROM_RETURN_CODE_t
 *  \brief This enum defines the Error codes
 */
typedef enum
{
  ERR_LOG_SUCCESS                                    = 0,   /**< Successful execution, no error occured */
  ERR_LOG_ERROR                                      = -1,  /**< Standard Error */
  ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID                = -21, /**< NVM address range is invalid */
  ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED                = -24, /**< NVM semaphore already reserved */
  ERR_LOG_CODE_NVM_VER_ERROR                         = -30, /**< 2 or more bit errors detected in NVM page when verifying NVM data after linear/mappped page programming */
  ERR_LOG_CODE_NVM_PROG_MAPRAM_INIT_FAIL             = -31, /**< NVM mapRAM update failed after mapped page programming or after execution of DH */
  ERR_LOG_CODE_NVM_PROG_VERIFY_MAPRAM_INIT_FAIL      = -32, /**< NVM programming and mapRAM init update failed after mapped page programming */
  ERR_LOG_CODE_NVM_MAPRAM_UNKNOWN_TYPE_USAGE         = -33, /**< MAPRAM physical page number for a given logical sector/page is larger than the number of physical pages in a sector */
  ERR_LOG_CODE_NVM_INIT_MAPRAM_SECTOR                = -35, /**< Mapped page has double mapping or ECC2 error when trying to init mapRAM */
  ERR_LOG_CODE_ACCESS_AB_MODE_ERROR                  = -37, /**< Error when setting the assembly buffer to mode 1 or 2 */
  ERR_LOG_CODE_NVM_PROTECT_REMOVE_PASSWORD_FAILED    = -39, /**< Removing region password(s) failed when trying to remove all region passwords */
  ERR_LOG_CODE_100TP_READ_ADDRESS_INVALID            = -43, /**< Attempt to read NVM 100TP address outside of the valid range */
  ERR_LOG_CODE_100TP_WRITE_COUNT_EXCEEDED            = -44, /**< NVM 100TP page write count was exceeded */
  ERR_LOG_CODE_100TP_WRITE_ADDRESS_INVALID           = -45, /**< Attempt to write NVM 100TP address outside of the valid range */
  ERR_LOG_CODE_CS_PAGE_CHECKSUM                      = -46, /**< NVM config sector checksum calculation failed */
  ERR_LOG_CODE_CS_PAGE_ECC2READ                      = -47, /**< NVM config sector checksum calculation failure based on NVM ECC2 error */
  ERR_LOG_CODE_MBIST_FAILED                          = -62, /**< MBIST test detected an error */
  ERR_LOG_CODE_MBIST_TIMEOUT                         = -63, /**< MBIST test failed due to timeout */
  ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED = -64, /**< Not allowed to change values in write protected config sector */
  ERR_LOG_CODE_USERAPI_CONFIG_SET_PARAMS_INVALID     = -65, /**< Invalid parameters given to user_bsl_config_set */
  ERR_LOG_CODE_NAD_VALUE_INVALID                     = -66, /**< Invalid NAD value given in user_nad_set() */
  ERR_LOG_CODE_100TP_PARAM_INVALID                   = -67, /**< user_nvm_100tp_read/write(): Bad data parameter */
  ERR_LOG_CODE_NVM_CONFIG_NOT_READY                  = -68, /**< Data for user_nvm_config_get() is not available */
  ERR_LOG_CODE_PARAM_INVALID                         = -69, /**< user_nvm_write/branch(): data parameter is invalid */
  ERR_LOG_CODE_USER_CROSS_PAGE_PRG_NOT_SUPPORTED     = -70, /**< user_nvm_write/branch cross-page programming is not supported */
  ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED      = -71, /**< user_nvm_write/branch operation not allowed when NVM is write protected */
  ERR_LOG_CODE_MBIST_RAM_RANGE_INVALID               = -72, /**< user_ram_mbist() RAM range for MBIST is invalid */
  ERR_LOG_CODE_USER_PROTECT_NO_PASSWORD_EXISTS       = -74, /**< user_nvm_password_clear() no password installed when trying to clear password */
  ERR_LOG_CODE_USER_PROTECT_NVM_AND_PWD_ERASED       = -75, /**< user_nvm_password_clear() wrong password given, all NVM sections + NVM passwords are erased */
  ERR_LOG_CODE_USER_PROTECT_NO_CBSL_PWD_CLEAR        = -76, /**< user_nvm_password_clear() clear password for CBSL not supported */
  ERR_LOG_CODE_USER_PROTECT_PWD_INVALID              = -77, /**< user_protect_password_set() provided password not valid */
  ERR_LOG_CODE_USER_PROTECT_PWD_EXISTS               = -78, /**< nvm_protect_password_set() segment password already exists when trying to set a new one in */
  ERR_LOG_CODE_USER_NVM_PROTECT_SEGMENT_INVALID      = -79, /**< Provided segment to change protection is invalid */
  ERR_LOG_CODE_SINGLE_ECC_EVENT_OCCURRED             = -80, /**< user_ecc_events_get()/user_ecc_get() single ECC event has occurred */
  ERR_LOG_CODE_DOUBLE_ECC_EVENT_OCCURRED             = -81, /**< user_ecc_events_get()/user_ecc_get() double ECC event has occurred */
  ERR_LOG_CODE_SINGLE_AND_DOUBLE_ECC_EVENT_OCCURRED  = -82, /**< user_ecc_events_get()/user_ecc_get() single and double ECC events have occurred */
  ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID     = -83, /**< Provided pointer doesn't point to a valid RAM range */
  ERR_LOG_CODE_USER_API_BRANCH_ADDRESS_INVALID       = -84  /**< Provided callback branch address is not within a valid RAM range */
} BOOTROM_RETURN_CODE_t;

/** \enum BSL_INTERFACE_SELECT_t
 *  \brief This enum defines the User API BSL interface Selection.
 */typedef enum
{
  BSL_IF_LIN = 0,                  /**< LIN used as BSL interface. */
  BSL_IF_FAST_LIN = 1              /**< Fast-LIN used as BSL interface */
} BSL_INTERFACE_SELECT_t;

/** \enum NVM_PASSWORD_SEGMENT_t
 *  \brief This enum defines the NVM protection API password segment.
 */
typedef enum _NVM_PASSWORD_SEGMENT_t
{
  NVM_PASSWORD_SEGMENT_BOOT = 0,   /**< NVM password for customer segment, used for customer bootloader */
  NVM_PASSWORD_SEGMENT_CODE = 1,   /**< NVM password for customer code segment, which is not used by the customer bootloader */
  NVM_PASSWORD_SEGMENT_DATA = 2    /**< NVM password for customer data segment */
} NVM_PASSWORD_SEGMENT_t;

typedef void(* user_callback_t) (void);

/** \enum VBG_TEMP_SELECT_t
 *  \brief This enum defines the VBG Temperature Selection.
 */
typedef enum
{
  VBG_TEMP_HOT = 0,                /**< VBG Temperature selection Hot */
  VBG_TEMP_COLD = 1                /**< VBG Temperature selection Cold */
} VBG_TEMP_SELECT_t;


/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

#ifdef UNIT_TESTING_LV2
extern BSL_INTERFACE_SELECT_t user_bsl_config_get(void);
extern int32_t  user_bsl_config_set(BSL_INTERFACE_SELECT_t ser_if);
extern int32_t  user_nvm_protect_clear(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);
extern int32_t  user_nvm_protect_set(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);
extern uint32_t user_nvm_protect_get(NVM_PASSWORD_SEGMENT_t segment);
extern int32_t  user_ecc_events_get(uint32_t *pNVM_Addr);
extern int32_t  user_ecc_check(void);
extern int32_t  user_mbist_set(bool bEnable);
extern uint8_t  user_nac_get(void);
extern int32_t  user_nac_set(uint8_t nac);
extern uint8_t  user_nad_get(void);
extern int32_t  user_nad_set(uint8_t nad);
extern int32_t  user_nvm_100tp_read(uint32_t page_addr, uint32_t offset, void *data, uint32_t count);
extern int32_t  user_nvm_100tp_write(uint32_t page_addr, uint32_t offset, const void *data, uint32_t count);
extern int32_t  user_nvm_config_get(uint8_t * cbsl_nvm_size, uint8_t *code_nvm_size, uint8_t *data_nvm_size);
extern int32_t  user_nvm_page_erase(uint32_t address);
extern int32_t  user_nvm_page_erase_branch(uint32_t address, user_callback_t branch_address);
extern bool     user_nvm_ready_poll(void);
extern int32_t  user_nvm_sector_erase(uint32_t address);
extern int32_t  user_nvm_write(uint32_t address, const void *data, uint8_t count, uint8_t options);
extern int32_t  user_nvm_write_branch(uint32_t address, const void *data, uint8_t count, uint8_t options, user_callback_t branch_address);
extern int32_t  user_ram_mbist(uint32_t start_address, uint32_t end_address);
extern int32_t  user_nvm_mapram_init(void);
extern void     user_nvm_clk_factor_set(uint8_t clk_factor);
extern int32_t  user_nvm_password_set(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);
extern int32_t  user_nvm_password_clear(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);
extern uint32_t user_vbg_temperature_get(VBG_TEMP_SELECT_t temperature);

#else
  
/** \brief This function reads the user BSL interface selection value.
 *
 * \return The currently selected BSL interface. (see \link BSL_INTERFACE_SELECT_t \endlink)  
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a BSL interface value read. 
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_bsl_config_get() == BSL_IF_LIN)
 *     {	 
 *	  	   // LIN used as BSL interface.
 *     }
 *     else 
 *     {	 
 *	       // Fast-LIN used as BSL interface.
 *     }
 *
 *     (void)WDT1_Service();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern BSL_INTERFACE_SELECT_t (*const user_bsl_config_get)(void);

/** \brief This function writes the user BSL interface selection value.
 * \note It returns an error in case the NVM code segment is write protected.
 *
 * \param ser_if BSL interface selection (see \link BSL_INTERFACE_SELECT_t \endlink)
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SET_PARAMS_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR 
 * 
 * \brief <br><b>Example</b><br>
 * \brief This example configures the BSL interface value to Fast-LIN. 
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_bsl_config_set(BSL_IF_FAST_LIN) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // BSL interface value set Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // BSL interface value set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_bsl_config_set)(BSL_INTERFACE_SELECT_t ser_if);

/** \brief This function clears read and/or write protection for any NVM region individually.
 * \note A valid password must be provided in case any valid NVM protection password is installed for this region.
 * \note Set bit 31 of the password parameter to enable read protection. 
 * \note Set bit 30 of the password parameter to enable write protection.
 * \note The bits (0...29) of the password parameter shall match the password installed before.
 * \note In case no valid protection password is currently installed, bits (0...29) are ignored. 
 *
 * \param password Protection password to apply on the given segment
 * \param segment Segment which should be password protected (see \link NVM_PASSWORD_SEGMENT_t \endlink)
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USER_NVM_PROTECT_SEGMENT_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_CS_PAGE_CHECKSUM
 * \retval ERR_LOG_CODE_CS_PAGE_ECC2READ
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_AND_PWD_ERASED 
 * \retval ERR_LOG_CODE_NVM_PROTECT_REMOVE_PASSWORD_FAILED. 
 *
 * \brief <b>Example</b><br>
 * \brief This example clears write protection with a password installed before and equals to 0x40000000 (Write protection enabled with password equals to 0x0) for customer data mapped segment.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_protect_clear(0x40000000, NVM_PASSWORD_SEGMENT_DATA) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // Write Protection Clear Operation has been executed successfully, Password could be successfully applied
 *     }
 *     else 
 *     {	 
 *	       // Write Protection Clear Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_protect_clear)(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);

/** \brief This function sets write protection for any NVM region individually, except CBSL.
 * \note A valid password must be provided in case any valid NVM protection password is installed for this region.
 * \note Set bit 31 of the password parameter to enable read protection. 
 * \note Set bit 30 of the password parameter to enable write protection.
 * \note The bits (0...29) of the password parameter shall match the password installed before.
 * \note In case no valid protection password is currently installed, bits (0...29) are ignored. 
 *
 * \param password Protection password to apply on the given segment
 * \param segment Segment which should be password protected (see \link NVM_PASSWORD_SEGMENT_t \endlink)
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USER_NVM_PROTECT_SEGMENT_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_CS_PAGE_CHECKSUM
 * \retval ERR_LOG_CODE_CS_PAGE_ECC2READ
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_AND_PWD_ERASED 
 * \retval ERR_LOG_CODE_NVM_PROTECT_REMOVE_PASSWORD_FAILED 
 *
 * \brief <b>Example</b><br>
 * \brief This example sets write protection with a password installed before and equals to 0x40000000 (Write protection enabled with password equals to 0x0) for customer data mapped segment.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_protect_set(0x40000000, NVM_PASSWORD_SEGMENT_DATA) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // Write Protection Set Operation has been executed successfully, Password could be successfully applied.
 *     }
 *     else 
 *     {	 
 *	       // Write Protection Set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();  
 * }   
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_protect_set)(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);

/** \brief This function checks for the hardware current applied NVM protection status. 
 *
 * \param segment Which NVM segment to retrieve the current password protection status (see \link NVM_PASSWORD_SEGMENT_t \endlink)
 *  
 * \retval 0x00000000 : Protection disabled
 * \retval 0x80000000 : Read protection enabled
 * \retval 0x40000000 : Write protection enabled
 * \retval 0xC0000000 : Read and write protection enabled 
 * \retval 0xFFFFFFFF : Segment not recognized 
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the NVM protection status for customer data mapped segment.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_protect_get(NVM_PASSWORD_SEGMENT_DATA) == 0x00000000)
 *     {	 
 *	  	   // Protection disabled
 *     }
 *     else if (user_nvm_protect_get(NVM_PASSWORD_SEGMENT_DATA) == 0x80000000) 
 *     {	 
 *	       // Read protection enabled
 *     }
 *     else if (user_nvm_protect_get(NVM_PASSWORD_SEGMENT_DATA) == 0x40000000) 
 *     {	 
 *	       // Write protection enabled
 *     }
 *     else if (user_nvm_protect_get(NVM_PASSWORD_SEGMENT_DATA) == 0xC0000000) 
 *     {	 
 *	       // Read and write protection enabled
 *     }
 *     else 
 *     {	 
 *	       // Segment not recognized
 *     } 
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern uint32_t (*const user_nvm_protect_get)(NVM_PASSWORD_SEGMENT_t segment);

/** \brief This function checks if any single or double ECC events have occurred during runtime. 
 * \brief It reports any single or double ECC event coming from NVM.
 *
 * \param *pNVM_Addr Pointer to NVM Address variable where the double ECC error is stored.
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_SINGLE_ECC_EVENT_OCCURRED
 * \retval ERR_LOG_CODE_DOUBLE_ECC_EVENT_OCCURRED
 * \retval ERR_LOG_CODE_SINGLE_AND_DOUBLE_ECC_EVENT_OCCURRED 
 *
 * \brief <b>Example</b><br>
 * \brief This example checks if any single or double ECC events have occurred during runtime.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint32 fail_addr;
 *	 
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_ecc_events_get(&fail_addr) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // No single or double ECC event have occurred
 *     }
 *     else 
 *     {	 
 *	       // Single/Double ECC Check Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_ecc_events_get)(uint32_t *pNVM_Addr);

/** \brief This function checks for single and double ECC checking over the entire NVM on all NVM linear and NVM non-linear sectors. 
 * \note Any existing ECC errors are cleared before the read starts. 
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_SINGLE_ECC_EVENT_OCCURRED
 * \retval ERR_LOG_CODE_DOUBLE_ECC_EVENT_OCCURRED
 * \retval ERR_LOG_CODE_SINGLE_AND_DOUBLE_ECC_EVENT_OCCURRED 
 *
 * \brief <b>Example</b><br>
 * \brief This example checks for single and double ECC checking over the entire NVM on all NVM linear and NVM non-linear sectors.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_ecc_check() == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // No single or double ECC event have occurred
 *     }
 *     else 
 *     {	 
 *	       // Single/Double ECC Check Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_ecc_check)(void);

/** \brief This function enables a separate MBIST for all possible reset sources, except POR reset and pin reset. 
 * \note It rejects with an error in case the NVM code segment is write protected. 
 *
 * \param bEnable Enable the MBIST test execution if the boolean parameter is set to TRUE, otherwise disable the MBIST test execution.
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a MBSIT test execution.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_mbist_set(true) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // No single or double ECC event have occurred
 *     }
 *     else 
 *     {	 
 *	       // Single/Double ECC Check Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_mbist_set)(bool bEnable);

/** \brief This function reads out the NAC value that is currently configured in the non volatile device configuration memory. 
 * 
 * \return uint8, NAC value that is found in the configuration memory.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads out the NAC value that is currently configured.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     uint8 nac_val;
 *	  
 *     WDT1_SOW_Service(1u);
 *
 *     nac_val = user_nac_get(); 
 *     if (nac_val == 0xFF)
 *     {	 
 *	  	   // no NAC value is currently configured in the configuration memory.
 *     }
 *     else 
 *     {	 
 *	       // Currently configured NAC value is read
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern uint8_t  (*const user_nac_get)(void);

/** \brief This function configures the NAC value in the non volatile device configuration memory. 
 * \note It rejects with an error in case the NVM code segment is write protected. 
 *
 * \param nac NAC value to be stored in the device configuration memory.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs NAC Value Set Operation.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 nac_val;
 *	  
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nac_set(nac_val) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NAC Value Set Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NAC Value Set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nac_set)(uint8_t nac);

/** \brief This function reads out the FASTLIN NAD value that is currently configured in the non volatile device configuration memory. 
 * 
 * \return uint8, The LIN NAD value which is found in the configuration memory.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads out the FASTLIN NAD value that is currently configured.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 nad_val;
 *	  
 *     WDT1_SOW_Service(1u);
 *
 *     nad_val = user_nad_get(); 
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern uint8_t  (*const user_nad_get)(void);

/** \brief This function configures the LIN NAD value in the non volatile device configuration memory. 
 * \note It rejects with an error in case the NVM code segment is write protected. 
 *
 * \param nad LIN NAD value to be stored in the device configuration memory. Valid range is from 0x00-0xFF.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NAD_VALUE_INVALID 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs NAD Value Set Operation.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 nad_val;
 *	  
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nad_set(nad_val) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // LIN NAD Value Set Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // LIN NAD Value Set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nad_set)(uint8_t nad);

/** \brief This function reads data from the customer accessible configuration pages (100TP).
 * \note Invalid parameters (page number out of range, offset plus count larger than page boundary, count is 0) returns an error, and no read operation is performed.
 *
 * \param page_addr Page number where to read from. Valid range: 0 to 7.
 * \param offset Byte offset inside the selected page address, where to start reading from.
 * \param *data Data pointer where to store the data read. Pointer plus valid count must be within valid RAM range or an error code is returned.
 * \param count Amount of data bytes to read. If count is zero, there is no operation performed and an error code is returned.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_100TP_PARAM_INVALID
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_100TP_READ_ADDRESS_INVALID 
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED 
 * \retval ERR_LOG_CODE_CS_PAGE_CHECKSUM 
 * \retval ERR_LOG_CODE_CS_PAGE_ECC2READ 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a complete read of the first 100TP page.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     uint32 data; 
 *     uint8 i;
 * 
 *     WDT1_SOW_Service(1u);
 * 
 *     for ( i = 0; i < FlashPageSize; i++)
 *     {
 *     	   if ( user_nvm_100tp_read(0, i, &data, FlashPageSize) == ERR_LOG_SUCCESS)
 *     	   {
 *	  	       // 100TP Read Operation has been executed successfully
 *     	   }
 *     	   else 
 *     	   {	 
 *	  	   		 // 100TP Read Operation returns a negative error code
 *     	   } 	 
 *     }
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_100tp_read)(uint32_t page_addr, uint32_t offset, void *data, uint32_t count);

/** \brief This function writes data to the configuration NVM, the write address is relative inside the configuration NVM area (8x one page, 1024 bytes).
 * \note Any invalid parameters (page number out of range, offset plus count larger than page boundary, count is 0) returns an error, and no write operation is performed.
 *
 * \param page_addr Page number where to write to. Valid range: 0 to 7.
 * \param offset Byte offset inside the selected page address, where to start writing. Maximum is 126 bytes.
 * \param *data Data pointer where to read the data to write. Pointer plus valid count must be within valid RAM range or an error code is returned.
 * \param count Amount of data bytes to write. If count is zero, there is no write operation performed and an error code is returned. Maximum is 126 bytes.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_100TP_PARAM_INVALID 
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_USERAPI_CONFIG_SECTOR_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_100TP_WRITE_ADDRESS_INVALID
 * \retval ERR_LOG_CODE_100TP_WRITE_COUNT_EXCEEDED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NVM_VER_ERROR 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a complete write of the first 100TP page.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint32 data; 
 *     uint8 i;
 * 
 *     WDT1_SOW_Service(1u);
 * 
 *     for ( i = 0; i < FlashPageSize; i++)
 *     {
 *     	   if ( user_nvm_100tp_write(0, i, &data, FlashPageSize) == ERR_LOG_SUCCESS)
 *     	   {
 *	  	   		 // 100TP Write Operation has been executed successfully
 *     	   }
 *     	   else 
 *     	   {	 
 *	  	   		 // 100TP Write Operation returns a negative error code
 *     	   } 	 
 *     }
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_100tp_write)(uint32_t page_addr, uint32_t offset, const void *data, uint32_t count);

/** \brief This function allows to gather the NVM configuration, this is the number of sectors for user code, user data and user bsl.
 * \note Pointer must be within valid RAM range or an error code is returned.
 *
 * \param *cbsl_nvm_size Pointer where to store the retrieved NVM cbsl size. Valid RAM range is 0x18000000 + device RAM size.
 * \param *code_nvm_size Pointer where to store the retrieved NVM code size. Valid RAM range is 0x18000000 + device RAM size.
 * \param *data_nvm_size Pointer where to store the retrieved NVM data size. Valid RAM range is 0x18000000 + device RAM size.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_NVM_CONFIG_NOT_READY 
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs read of the NVM configuration.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 cbsl_flash_size; 
 *     uint8 code_flash_size; 
 *     uint8 data_flash_size;  
 * 
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_config_get(&cbsl_flash_size, &code_flash_size, &data_flash_size) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NVM Configuration Read Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NVM Configuration Read Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_config_get)(uint8_t * cbsl_nvm_size, uint8_t *code_nvm_size, uint8_t *data_nvm_size);

/** \brief This function erases a given NVM page (address).
 * \note In case of an unsused (new) page in non-linear sector, the function does nothing and returns success.
 * \note In case of erasing a page in linear sector, the function should always perform the erase.
 *
 * \param address Address of the NVM page to erase. Non-aligned address is accepted. Range is 0x11000000 + device NVM size.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a page erase of the first data flash page.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_page_erase(DataFlashStart) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NVM Page Erase Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NVM Page Erase Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_page_erase)(uint32_t address);

/** \brief This function erases a given NVM page (address) and branches to an address (branch_address) for code execution during the NVM operation.
 * \note It rejects with an error in case the accessed NVM page is write protected.
 *
 * \param address Address of the NVM page to erase. Non-aligned address is accepted. Range is 0x11000000 + device NVM size.
 * \param branch_address Function callback address where to jump while waiting for the NVM module to finish the erase operation (see \link user_callback_t \endlink)
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID
 * \retval ERR_LOG_CODE_USER_API_BRANCH_ADDRESS_INVALID
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a page erase of the first data flash page and branches to RAM_Branch_Function address.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_page_erase_branch(DataFlashStart, &RAM_Branch_Function) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NVM Page Erase/Branch Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NVM Page Erase/Branch Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_page_erase_branch)(uint32_t address, user_callback_t branch_address);

/** \brief This function checks for the readiness of the NVM module. 
 * \note It is called within the NVM programming or erase branch callback operation.
 *
 * \retval True : The requested NVM operation is already finished
 * \retval False : The requested NVM operation is not finished 
 *
 * \brief <b>Example</b><br>
 * \brief This example checks for the readiness of the NVM module within the NVM programming.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {  
 *     uint8 page_data[128];
 *     uint8 i;
 *     
 *     for ( i = 0; i < FlashPageSize; i++)
 *     {
 *         page_data[i] = i;
 *     }
 *
 *     WDT1_SOW_Service(1u);
 *     
 *     user_nvm_write_branch(DataFlashStart, &page_data, FlashPageSize, 0, &RAM_Branch_Function);
 * 
 *     if (user_nvm_ready_poll() == true)
 *     {	 
 *	  	   // NVM operation is already finished
 *     }
 *     else 
 *     {	 
 *	       // NVM operation is not finished
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern bool     (*const user_nvm_ready_poll)(void);

/** \brief This function erases the NVM sector-wise.
 * \note It rejects with an error in case the NVM region the address belongs to is write protected.
 *
 * \param address Address of the NVM sector to erase. Non-aligned address is accepted. Range is 0x11000000 + device NVM size.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_NVM_INIT_MAPRAM_SECTOR 
 *
 * \brief <b>Example</b><br>
 * \brief This example erases the data flash sector.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_sector_erase(DataFlashStart) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NVM Sector Erase Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NVM Sector Erase Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_sector_erase)(uint32_t address);

/** \brief This function programs the NVM. It operates on the user NVM, as well as on the user data NVM.
 * \note It shall write a number of bytes (count) from the source (data) to the NVM location (address) with the programming options (options).
 *
 * \param address NVM address where to program the data. Range is 0x11000000 + device NVM size.
 * \param *data Pointer to the data where to read the programming data. Pointer must be within valid RAM range or an error code is returned.
 * \param count Amount of bytes to program.
 * \param options NVM programming options.
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_PARAM_INVALID
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_USER_CROSS_PAGE_PRG_NOT_SUPPORTED
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NVM_MAPRAM_UNKNOWN_TYPE_USAGE
 * \retval ERR_LOG_CODE_NVM_VER_ERROR
 * \retval ERR_LOG_CODE_NVM_PROG_MAPRAM_INIT_FAIL
 * \retval ERR_LOG_CODE_NVM_PROG_VERIFY_MAPRAM_INIT_FAIL 
 * \retval ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID 
 *
 * \brief <b>Example</b><br>
 * \brief This example fills an array of 128 bytes and then it will be programmed to the first page. 
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 page_data[128];
 *     uint8 i;
 *     
 *     for ( i = 0; i < FlashPageSize; i++)
 *     {
 *         page_data[i] = i;
 *	   }
 *
 *	   WDT1_SOW_Service(1);
 * 
 *     if (user_nvm_write(DataFlashStart, &page_data, FlashPageSize, 0) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // NVM Write Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // NVM Write Operation returns a negative error code
 *     }
 * 
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_write)(uint32_t address, const void *data, uint8_t count, uint8_t options);

/** \brief This function programs the NVM. It operates on the user NVM, as well as on the user data NVM.
 * \brief During the NVM operation the program execution branches to a given SRAM location (branch_address) and continues code execution from there.
 * \note It shall write a number of bytes (count) from the source (data) to the NVM location (address) with the programming options (options).
 *
 * \param address NVM address where to program the data. Range is 0x11000000 + device NVM size.
 * \param *data Pointer to the data where to read the programming data. Pointer must be within valid RAM range or an error code is returned.
 * \param count Amount of bytes to program.
 * \param options NVM programming options.
 * \param branch_address Function callback address where to jump while waiting for the NVM module to finish the program operation (see \link user_callback_t \endlink)
 * 
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_PARAM_INVALID
 * \retval ERR_LOG_CODE_USER_API_BRANCH_ADDRESS_INVALID
 * \retval ERR_LOG_CODE_USERAPI_POINTER_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_USER_CROSS_PAGE_PRG_NOT_SUPPORTED
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_WRITE_PROTECTED
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NVM_MAPRAM_UNKNOWN_TYPE_USAGE
 * \retval ERR_LOG_CODE_NVM_VER_ERROR
 * \retval ERR_LOG_CODE_NVM_PROG_MAPRAM_INIT_FAIL
 * \retval ERR_LOG_CODE_NVM_PROG_VERIFY_MAPRAM_INIT_FAIL 
 * \retval ERR_LOG_CODE_NVM_ADDR_RANGE_INVALID 
 *
 * \brief <b>Example</b><br>
 * \brief This example fills an array of 128 bytes and then during the programming to the first page it branches to RAM_Branch_Function address. 
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     uint8 page_data[128];
 *     uint8 i;
 *     
 *     for ( i = 0; i < FlashPageSize; i++)
 *     {
 *         page_data[i] = i;
 *     }
 *
 *     WDT1_SOW_Service(1u);
 *
 *     if ( user_nvm_write_branch(DataFlashStart, &page_data, FlashPageSize, 0, &RAM_Branch_Function) == ERR_LOG_SUCCESS)
 *     {
 *		     // Fast Write Branch Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *		     // Fast Write Branch Operation returns a negative error code
 *     }  
 *
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_write_branch)(uint32_t address, const void *data, uint8_t count, uint8_t options, user_callback_t branch_address);

/** \brief This function performs a MBIST on the integrated SRAM. 
 * \note The function rejects the call in case the parameter exceeds the RAM address range.
 *
 * \param start_address RAM memory address where to start the MBIST test.
 * \param end_address RAM memory address till where to perform the MBIST test.
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_MBIST_RAM_RANGE_INVALID
 * \retval ERR_LOG_CODE_MBIST_FAILED
 * \retval ERR_LOG_CODE_MBIST_TIMEOUT 
 *
 * \brief <b>Example</b><br>
 * \brief This example performs a MBIST over the first half of the RAM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_ram_mbist(0, (uint32)(RAMSize >> 1u)) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // MBIST Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // MBIST Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_ram_mbist)(uint32_t start_address, uint32_t end_address);

/** \brief This function triggers NVM FSM mapRAM update sequence from mapped sector. 
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink)  
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_ERROR 
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_NVM_INIT_MAPRAM_SECTOR 
 *
 * \brief <b>Example</b><br>
 * \brief This example triggers NVM FSM mapRAM update sequence from mapped sector.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_mapram_init() == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // MAPRAM Init Operation has been executed successfully
 *     }
 *     else 
 *     {	 
 *	       // MAPRAM Init Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_mapram_init)(void);

/** \brief This function sets the SCU_SYSCON0.NVMCLKFAC divider. 
 *
 * \param clk_factor Sets the clock factor directly to the value specified. No checks are done on the value.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the clock divider to div 2, fsys/2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     user_nvm_clk_factor_set(1u);
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern void     (*const user_nvm_clk_factor_set)(uint8_t clk_factor);

/** \brief This function sets password for NVM region individually.
 * \note The password parameter consists of a 30-bit password (bit 0...29) and two additional protection bits (bit 30: write protection + bit 31: read protection).
 * \note A valid password must be different from 0x3FFFFFFF and 0x00000000 (bit 0...29).
 *
 * \param password Protection password to apply on the given segment
 * \param segment Segment which should be password protected (see \link NVM_PASSWORD_SEGMENT_t \endlink)
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USER_NVM_SEGMENT_INVALID
 * \retval ERR_LOG_CODE_USER_PROTECT_PWD_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_CS_PAGE_CHECKSUM
 * \retval ERR_LOG_CODE_CS_PAGE_ECC2READ 
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NVM_VER_ERROR
 * \retval ERR_LOG_CODE_USER_PROTECT_PWD_EXISTS 
 *
 * \brief <b>Example</b><br>
 * \brief This example sets password to 0x40000001 (Write protection enabled with password equals to 0x1) for customer data mapped segment.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_password_set(0x40000001, NVM_PASSWORD_SEGMENT_DATA) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // Password Set Operation has been executed successfully, Password could be successfully applied
 *     }
 *     else 
 *     {	 
 *	       // Password Set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_password_set)(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);

/** \brief This function clears the NVM protection password for a given NVM code or data segment.
 * \note The password is only removed in case the correct password is provided.
 *
 * \param password Protection password to apply on the given segment
 * \param segment Segment which should be password protected (see \link NVM_PASSWORD_SEGMENT_t \endlink)
 *
 * \return BOOTROM Return Codes (see \link BOOTROM_RETURN_CODE_t \endlink) 
 * \retval ERR_LOG_SUCCESS
 * \retval ERR_LOG_CODE_USER_PROTECT_NO_CBSL_PWD_CLEAR
 * \retval ERR_LOG_CODE_USER_NVM_PROTECT_SEGMENT_INVALID
 * \retval ERR_LOG_CODE_NVM_SEMAPHORE_RESERVED
 * \retval ERR_LOG_CODE_CS_PAGE_CHECKSUM
 * \retval ERR_LOG_CODE_CS_PAGE_ECC2READ 
 * \retval ERR_LOG_CODE_USER_PROTECT_NO_PASSWORD_EXISTS 
 * \retval ERR_LOG_CODE_ACCESS_AB_MODE_ERROR
 * \retval ERR_LOG_CODE_NVM_PROTECT_REMOVE_PASSWORD_FAILED 
 * \retval ERR_LOG_CODE_USER_PROTECT_NVM_AND_PWD_ERASED 
 * \retval ERR_LOG_CODE_NVM_VER_ERROR 
 *
 * \brief <b>Example</b><br>
 * \brief This example clears password which is set before to 0x40000001 (Write protection enabled with password equals to 0x1) for customer data mapped segment.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {   
 *     WDT1_SOW_Service(1u);
 * 
 *     if (user_nvm_password_set(0x40000001, NVM_PASSWORD_SEGMENT_DATA) == ERR_LOG_SUCCESS)
 *     {	 
 *	  	   // Password Set Operation has been executed successfully, Password could be successfully applied
 *     }
 *     else 
 *     {	 
 *	       // Password Set Operation returns a negative error code
 *     }
 *
 *     (void)WDT1_Service();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern int32_t  (*const user_nvm_password_clear)(uint32_t password, NVM_PASSWORD_SEGMENT_t segment);

/** \brief This function returns the V bandgap temperature hot or cold. 
 * \note The customer can do temperature compensation in software based on these values, as the VBG is the reference for the ADC.
 *
 * \param temperature Selects the temperature to read (see \link VBG_TEMP_SELECT_t \endlink)
 * 
 * \return uint32_t, Vbg value corresponding to the selected temperature.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the VBG Hot Temperature.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint32 temp_val;
 * 
 *     WDT1_SOW_Service(1u);
 * 
 *     temp_val = user_vbg_temperature_get(VBG_TEMP_HOT);
 *
 *     (void)WDT1_Service(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup bootrom_api
 */
extern uint32_t (*const user_vbg_temperature_get)(VBG_TEMP_SELECT_t temperature);

#endif /* UNIT_TESTING_LV2 */
#endif
