/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     gpt12e.h
 *
 * \brief    General Purpose Timer 12 Encoder low level access library
 *
 * \version  V0.1.8
 * \date     13. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version                                  **
** V0.1.4: 2017-10-20, AV:   Macros for interrupt register access added       **
** V0.1.5: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.6: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
** V0.1.7: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to gpt12e.c                                      **
** V0.1.8: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

#ifndef GPT12E_H
#define GPT12E_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void GPT12E_Init(void);
INLINE void GPT12E_T2_Int_Clr(void);
INLINE void GPT12E_T3_Int_Clr(void);
INLINE void GPT12E_T4_Int_Clr(void);
INLINE void GPT12E_T5_Int_Clr(void);
INLINE void GPT12E_T6_Int_Clr(void);
INLINE void GPT12E_CapRel_Int_Clr(void);
INLINE void GPT12E_T2_Int_En(void);
INLINE void GPT12E_T2_Int_Dis(void);
INLINE void GPT12E_T3_Int_En(void);
INLINE void GPT12E_T3_Int_Dis(void);
INLINE void GPT12E_T4_Int_En(void);
INLINE void GPT12E_T4_Int_Dis(void);
INLINE void GPT12E_T5_Int_En(void);
INLINE void GPT12E_T5_Int_Dis(void);
INLINE void GPT12E_T6_Int_En(void);
INLINE void GPT12E_T6_Int_Dis(void);
INLINE void GPT12E_CapRel_Int_En(void);
INLINE void GPT12E_CapRel_Int_Dis(void);
INLINE void GPT12E_T2_Start(void);
INLINE void GPT12E_T2_Stop(void);
INLINE uint16 GPT12E_T2_Read(void);
INLINE void GPT12E_T3_Start(void);
INLINE void GPT12E_T3_Stop(void);
INLINE uint16 GPT12E_T3_Read(void);
INLINE void GPT12E_T4_Start(void);
INLINE void GPT12E_T4_Stop(void);
INLINE uint16 GPT12E_T4_Read(void);
INLINE void GPT12E_T5_Start(void);
INLINE void GPT12E_T5_Stop(void);
INLINE uint16 GPT12E_T5_Read(void);
INLINE void GPT12E_T6_Start(void);
INLINE void GPT12E_T6_Stop(void);
INLINE uint16 GPT12E_T6_Read(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Clear GPT Module 1 Timer 2 interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 2 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T2_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T2 == (uint8)1u)
 *     {
 *         GPT1_T2_CALLBACK();	
 *         GPT12E_T2_Int_Clr(); 
 *     }
 *     GPT12E_T2_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T2_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT1T2C_Pos, (uint32)SCU_GPT12ICLR_GPT1T2C_Msk, 1u);
}

/** \brief Clear GPT Module 1 Timer 3 interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 3 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T3_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T3 == (uint8)1u)
 *     {
 *         GPT1_T3_CALLBACK();	
 *         GPT12E_T3_Int_Clr(); 
 *     }
 *     GPT12E_T3_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T3_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT1T3C_Pos, SCU_GPT12ICLR_GPT1T3C_Msk, 1u);
}

/** \brief Clear GPT Module 1 Timer 4 interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 4 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T4_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T4 == (uint8)1u)
 *     {
 *         GPT1_T4_CALLBACK();	
 *         GPT12E_T4_Int_Clr(); 
 *     }
 *     GPT12E_T4_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T4_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT1T4C_Pos, SCU_GPT12ICLR_GPT1T4C_Msk, 1u);
}

/** \brief Clear GPT Module 2 Timer 5 interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 5 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T5_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T5 == (uint8)1u)
 *     {
 *         GPT2_T5_CALLBACK();	
 *         GPT12E_T5_Int_Clr(); 
 *     }
 *     GPT12E_T5_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T5_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT2T5C_Pos, SCU_GPT12ICLR_GPT2T5C_Msk, 1u);
}

/** \brief Clear GPT Module 2 Timer 6 interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 6 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T6_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T6 == (uint8)1u)
 *     {
 *         GPT2_T6_CALLBACK();	
 *         GPT12E_T6_Int_Clr(); 
 *     }
 *     GPT12E_T6_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T6_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT2T6C_Pos, SCU_GPT12ICLR_GPT2T6C_Msk, 1u);
}

/** \brief Clear GPT Module 1 Capture Reload interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Capture Reload interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_CapRel_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT12CR == (uint8)1u)
 *     {
 *         GPT2_CAPREL_CALLBACK();	
 *         GPT12E_CapRel_Int_Clr(); 
 *     }
 *     GPT12E_CapRel_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_CapRel_Int_Clr(void)
{
  Field_Wrt32(&SCU->GPT12ICLR.reg, (uint8)SCU_GPT12ICLR_GPT12CRC_Pos, SCU_GPT12ICLR_GPT12CRC_Msk, 1u);
}

/* GPT12E Interrupt Enable/Disable */
/** \brief Enable GPT Module 1 Timer 2 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 2 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T2_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T2 == (uint8)1u)
 *     {
 *         GPT1_T2_CALLBACK();	
 *         GPT12E_T2_Int_Clr(); 
 *     }
 *     GPT12E_T2_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T2_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T2IE_Pos, SCU_GPT12IEN_T2IE_Msk, 1u);
}

/** \brief Disable GPT Module 1 Timer 2 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 2 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T2_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T2 == (uint8)1u)
 *     {
 *         GPT1_T2_CALLBACK();	
 *         GPT12E_T2_Int_Clr(); 
 *     }
 *     GPT12E_T2_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T2_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T2IE_Pos, SCU_GPT12IEN_T2IE_Msk, 0u);
}

/** \brief Enable GPT Module 1 Timer 3 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 3 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T3_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T3 == (uint8)1u)
 *     {
 *         GPT1_T3_CALLBACK();	
 *         GPT12E_T3_Int_Clr(); 
 *     }
 *     GPT12E_T3_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T3_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T3IE_Pos, SCU_GPT12IEN_T3IE_Msk, 1u);
}

/** \brief Disable GPT Module 1 Timer 3 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 3 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T3_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T3 == (uint8)1u)
 *     {
 *         GPT1_T3_CALLBACK();	
 *         GPT12E_T3_Int_Clr(); 
 *     }
 *     GPT12E_T3_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T3_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T3IE_Pos, SCU_GPT12IEN_T3IE_Msk, 0u);
}

/** \brief Enable GPT Module 1 Timer 4 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 4 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T4_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T4 == (uint8)1u)
 *     {
 *         GPT1_T4_CALLBACK();	
 *         GPT12E_T4_Int_Clr(); 
 *     }
 *     GPT12E_T4_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T4_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T4IE_Pos, SCU_GPT12IEN_T4IE_Msk, 1u);
}

/** \brief Disable GPT Module 1 Timer 4 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Timer 4 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T4_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT1T4 == (uint8)1u)
 *     {
 *         GPT1_T4_CALLBACK();	
 *         GPT12E_T4_Int_Clr(); 
 *     }
 *     GPT12E_T4_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T4_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T4IE_Pos, SCU_GPT12IEN_T4IE_Msk, 0u);
}

/** \brief Enable GPT Module 2 Timer 5 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 5 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T5_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T5 == (uint8)1u)
 *     {
 *         GPT2_T5_CALLBACK();	
 *         GPT12E_T5_Int_Clr(); 
 *     }
 *     GPT12E_T5_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T5_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T5IE_Pos, SCU_GPT12IEN_T5IE_Msk, 1u);
}

/** \brief Disable GPT Module 2 Timer 5 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 5 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T5_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T5 == (uint8)1u)
 *     {
 *         GPT2_T5_CALLBACK();	
 *         GPT12E_T5_Int_Clr(); 
 *     }
 *     GPT12E_T5_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T5_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T5IE_Pos, SCU_GPT12IEN_T5IE_Msk, 0u);
}

/** \brief Enable GPT Module 2 Timer 6 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 6 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T6_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T6 == (uint8)1u)
 *     {
 *         GPT2_T6_CALLBACK();	
 *         GPT12E_T6_Int_Clr(); 
 *     }
 *     GPT12E_T6_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T6_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T6IE_Pos, SCU_GPT12IEN_T6IE_Msk, 1u);
}

/** \brief Disable GPT Module 2 Timer 6 interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 2 Timer 6 interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_T6_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT2T6 == (uint8)1u)
 *     {
 *         GPT2_T6_CALLBACK();	
 *         GPT12E_T6_Int_Clr(); 
 *     }
 *     GPT12E_T6_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T6_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_T6IE_Pos, SCU_GPT12IEN_T6IE_Msk, 0u);
}

/** \brief Enable GPT Module 1 Capture Reload interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Capture Reload interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_CapRel_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT12CR == (uint8)1u)
 *     {
 *         GPT2_CAPREL_CALLBACK();	
 *         GPT12E_CapRel_Int_Clr(); 
 *     }
 *     GPT12E_CapRel_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_CapRel_Int_En(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_CRIE_Pos, SCU_GPT12IEN_CRIE_Msk, 1u);
}

/** \brief Disable GPT Module 1 Capture Reload interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the GPT Module 1 Capture Reload interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     GPT12E_CapRel_Int_En();	 
 *     if ((uint8)SCU->GPT12IRC.bit.GPT12CR == (uint8)1u)
 *     {
 *         GPT2_CAPREL_CALLBACK();	
 *         GPT12E_CapRel_Int_Clr(); 
 *     }
 *     GPT12E_CapRel_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_CapRel_Int_Dis(void)
{
  Field_Mod32(&SCU->GPT12IEN.reg, (uint8)SCU_GPT12IEN_CRIE_Pos, SCU_GPT12IEN_CRIE_Msk, 0u);
}

/* GPT12E Start/Stop/Read */
/** \brief Start Timer T2.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Timer T2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {	 
 *    GPT12E_T2_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T2_Start(void)
{
  GPT12E->T2CON.bit.T2R = 1u;
}

/** \brief Stop Timer T2.
 *
 * \brief <b>Example</b><br>
 * \brief This example stops Timer T2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T2_Stop();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T2_Stop(void)
{
  GPT12E->T2CON.bit.T2R = 0u;
}

/** \brief Read Timer T2 Value.
 *
 * \return T2 Value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads Timer T2 Value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint16 timer;
 * 
 *    timer = GPT12E_T2_Read();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE uint16 GPT12E_T2_Read(void)
{
  return (uint16)GPT12E->T2.reg;
}

/** \brief Start Timer T3.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Timer T3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T3_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T3_Start(void)
{
  GPT12E->T3CON.bit.T3R = 1u;
}

/** \brief Stop Timer T3.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example stops Timer T3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {	 
 *    GPT12E_T3_Stop();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T3_Stop(void)
{
  GPT12E->T3CON.bit.T3R = 0u;
}

/** \brief Read Timer T3 Value.
 *
 * \return T3 Value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads Timer T3 Value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint16 timer;
 * 
 *    timer = GPT12E_T3_Read();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE uint16 GPT12E_T3_Read(void)
{
  return (uint16)GPT12E->T3.reg;
}

/** \brief Start Timer T4.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Timer T4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T4_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T4_Start(void)
{
  GPT12E->T4CON.bit.T4R = 1u;
}

/** \brief Stop Timer T4.
 *
 * \brief <b>Example</b><br>
 * \brief This example stops Timer T4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T4_Stop();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T4_Stop(void)
{
  GPT12E->T4CON.bit.T4R = 0u;
}

/** \brief Read Timer T4 Value.
 *
 * \return T4 Value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads Timer T4 Value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint16 timer;
 * 
 *    timer = GPT12E_T4_Read();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE uint16 GPT12E_T4_Read(void)
{
  return (uint16)GPT12E->T4.reg;
}

/** \brief Start Timer T5.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Timer T5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T5_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T5_Start(void)
{
  GPT12E->T5CON.bit.T5R = 1u;
}

/** \brief Stop Timer T5.
 *
 * \brief <b>Example</b><br>
 * \brief This example stops Timer T5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {	 
 *    GPT12E_T5_Stop();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T5_Stop(void)
{
  GPT12E->T5CON.bit.T5R = 0u;
}

/** \brief Read Timer T5 Value.
 *
 * \return T5 Value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads Timer T5 Value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint16 timer;
 * 
 *    timer = GPT12E_T5_Read();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE uint16 GPT12E_T5_Read(void)
{
  return (uint16)GPT12E->T5.reg;
}

/** \brief Start Timer T6.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Timer T6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T6_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T6_Start(void)
{
  GPT12E->T6CON.bit.T6R = 1u;
}

/** \brief Stop Timer T6.
 *
 * \brief <b>Example</b><br>
 * \brief This example stops Timer T6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *    GPT12E_T6_Stop();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE void GPT12E_T6_Stop(void)
{
  GPT12E->T6CON.bit.T6R = 0u;
}

/** \brief Read Timer T6 Value.
 *
 * \return T6 Value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads Timer T6 Value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint16 timer;
 * 
 *    timer = GPT12E_T6_Read();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup gpt12e_api
 */
INLINE uint16 GPT12E_T6_Read(void)
{
  return (uint16)GPT12E->T6.reg;
}

#endif
