/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     hs.h
 *
 * \brief    High Side Switch low level access library
 *
 * \version  V0.2.1
 * \date     13. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AV           Axel Viebig                                                   **
** DM           Daniel Mysliwitz                                              **
** MK           Martin Kilian                                                 **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-10-23, AV:   Initial version                                  **
** V0.1.1: 2015-11-11, AV:   Added methods for HS module                      **
** V0.1.2: 2015-11-23, AV:   Changed recognition of 7 variants to enable HS2  **
**                           section                                          **
** V0.1.3: 2015-11-28, AV:   Init routine extended with conditional           **
**                           HIGHSIDE_CTRL access                             **
** V0.1.4: 2016-06-28, DM:   HSx_Set_Switch corrected                         **
** V0.1.5: 2017-04-05, MK:   Added define macros for HS Registers in HS.h     **
** V0.1.6: 2017-10-20, KC:   Macros with _ISts_ renamed to _Int_              **
**                           Macros with "prewarn" removed                    **
**                           Macros corrected to right IRQCLR register        **
** V0.1.7: 2017-12-11, KC:   Macros HSx_OverCur_Th_Sel() added                **
** V0.1.8: 2018-11-05, CE:   hs.h: Macros for HS_IRQCLR corrected             **
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.9: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           HS1_Init and HS2_Init Updated (Macro Condition   **
**                           used)                                            **
** V0.2.0: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to hs.c                                          **
** V0.2.1: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

#ifndef HS_H
#define HS_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"
#include "tle_variants.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief HS OverCurrent Thresholds Macro, 25mA */
#define HS_Oc_25mA  0u
/**\brief HS OverCurrent Thresholds Macro, 50mA */
#define HS_Oc_50mA  1u
/**\brief HS OverCurrent Thresholds Macro, 100mA */
#define HS_Oc_100mA 2u
/**\brief HS OverCurrent Thresholds Macro, 150mA */
#define HS_Oc_150mA 3u

/**\brief HS1 PWM Source Selection Macro, CC60 */
#define HS_PWM_CC60   0u
/**\brief HS1 PWM Source Selection Macro, CC61 */
#define HS_PWM_CC61   1u
/**\brief HS1 PWM Source Selection Macro, CC62 */
#define HS_PWM_CC62   2u
/**\brief HS1 PWM Source Selection Macro, COUT60 */
#define HS_PWM_COUT60 3u
/**\brief HS1 PWM Source Selection Macro, COUT61 */
#define HS_PWM_COUT61 4u
/**\brief HS1 PWM Source Selection Macro, COUT62 */
#define HS_PWM_COUT62 5u
/**\brief HS1 PWM Source Selection Macro, T3OUT */
#define HS_PWM_T3OUT  6u

/**\brief Blanking Time Filter Select for HS1 Overcurrent/Overtemperature and Open Load Detection Macro, 4 μs filter time */
#define HS_Filt_4us  0u
/**\brief Blanking Time Filter Select for HS1 Overcurrent/Overtemperature and Open Load Detection Macro, 8 μs filter time */
#define HS_Filt_8us  1u
/**\brief Blanking Time Filter Select for HS1 Overcurrent/Overtemperature and Open Load Detection Macro, 16 μs filter time */
#define HS_Filt_16us 2u
/**\brief Blanking Time Filter Select for HS1 Overcurrent/Overtemperature and Open Load Detection Macro, 32 μs filter time */
#define HS_Filt_32us 3u

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void HS1_Init(void);
#if (UC_FEAT_HS > 1u)
void HS2_Init(void);
#endif
INLINE void HS1_En(void);
INLINE void HS1_Dis(void);
INLINE void HS1_PWM_En(void);
INLINE void HS1_PWM_Dis(void);
INLINE void HS1_On(void);
INLINE void HS1_Off(void);
INLINE void HS1_OL_En(void);
INLINE void HS1_OL_Dis(void);
INLINE void HS1_SR_Fast_Set(void);
INLINE void HS1_SR_Slow_Set(void);
INLINE void HS1_OverCur_Th_Sel(uint32 th);
INLINE uint8 HS1_OpenLoad_Sts(void);
INLINE uint8 HS1_OverTemp_Sts(void);
INLINE uint8 HS1_OverCur_Int_Sts(void);
INLINE uint8 HS1_OpenLoad_Int_Sts(void);
INLINE uint8 HS1_OverTemp_Int_Sts(void);
INLINE void HS1_OpenLoad_Sts_Clr(void);
INLINE void HS1_OverTemp_Sts_Clr(void);
INLINE void HS1_OverCur_Int_Clr(void);
INLINE void HS1_OpenLoad_Int_Clr(void);
INLINE void HS1_OverTemp_Int_Clr(void);
INLINE void HS1_OverCur_Int_En(void);
INLINE void HS1_OpenLoad_Int_En(void);
INLINE void HS1_OverTemp_Int_En(void);
INLINE void HS1_PWM_Src_Sel(uint32 hs_pwm);
INLINE void HS1_OverCur_Filt_time(uint32 hs_filt);
INLINE void HS1_OpenLoad_Filt_time(uint32 hs_filt);
#if(UC_FEAT_HS == 2u)
INLINE void HS2_En(void);
INLINE void HS2_Dis(void);
INLINE void HS2_PWM_En(void);
INLINE void HS2_PWM_Dis(void);
INLINE void HS2_On(void);
INLINE void HS2_Off(void);
INLINE void HS2_OL_En(void);
INLINE void HS2_OL_Dis(void);
INLINE void HS2_SR_Fast_Set(void);
INLINE void HS2_SR_Slow_Set(void);
INLINE void HS2_OverCur_Th_Sel(uint32 th);
INLINE uint8 HS2_OpenLoad_Sts(void);
INLINE uint8 HS2_OverTemp_Sts(void);
INLINE uint8 HS2_OverCur_Int_Sts(void);
INLINE uint8 HS2_OpenLoad_Int_Sts(void);
INLINE uint8 HS2_OverTemp_Int_Sts(void);
INLINE void HS2_OpenLoad_Sts_Clr(void);
INLINE void HS2_OverTemp_Sts_Clr(void);
INLINE void HS2_OverCur_Int_Clr(void);
INLINE void HS2_OpenLoad_Int_Clr(void);
INLINE void HS2_OverTemp_Int_Clr(void);
INLINE void HS2_OverCur_Int_En(void);
INLINE void HS2_OpenLoad_Int_En(void);
INLINE void HS2_OverTemp_Int_En(void);
INLINE void HS2_PWM_Src_Sel(uint32 hs_pwm);
INLINE void HS2_OverCur_Filt_time(uint32 hs_filt);
INLINE void HS2_OpenLoad_Filt_time(uint32 hs_filt);
#endif

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Enable High Side 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_EN_Pos, HS_CTRL_HS1_EN_Msk, 1u);
}

/** \brief Disable High Side 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_EN_Pos, HS_CTRL_HS1_EN_Msk, 0u);
}

/** \brief Enable High Side 1 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 1 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_PWM_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_PWM_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_PWM_Pos, HS_CTRL_HS1_PWM_Msk, 1u);
}

/** \brief Disable High Side 1 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 1 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_PWM_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_PWM_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_PWM_Pos, HS_CTRL_HS1_PWM_Msk, 0u);
}

/** \brief Set High Side 1 to ON.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 1 to ON.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_On(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_On(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_ON_Pos, HS_CTRL_HS1_ON_Msk, 1u);
}

/** \brief Set High Side 1 to OFF.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 1 to OFF.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_Off(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_Off(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_ON_Pos, HS_CTRL_HS1_ON_Msk, 0u);
}

/** \brief Enable High Side 1 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 1 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_OL_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_OL_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_OL_EN_Pos, HS_CTRL_HS1_OL_EN_Msk, 1u);
}

/** \brief Disable High Side 1 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 1 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_OL_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_OL_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_OL_EN_Pos, HS_CTRL_HS1_OL_EN_Msk, 0u);
}

/** \brief Enable High Side 1 Slew Rate 30V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 1 Slew Rate 30V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_SR_Fast_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_SR_Fast_Set(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_SRCTL_SEL_Pos, HS_CTRL_HS1_SRCTL_SEL_Msk, 1u);
}

/** \brief Enable High Side 1 Slew Rate 10V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 1 Slew Rate 10V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_SR_Slow_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_SR_Slow_Set(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_SRCTL_SEL_Pos, HS_CTRL_HS1_SRCTL_SEL_Msk, 0u);
}

/** \brief Select High Side 1 Overcurrent Threshold.
 *
 * \param th High Side 1 Overcurrent Threshold Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 1 Overcurrent Threshold to 100 mA min.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_OverCur_Th_Sel(HS_Oc_100mA); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_OverCur_Th_Sel(uint32 th)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS1_OC_SEL_Pos, HS_CTRL_HS1_OC_SEL_Msk, th);
}

#if(UC_FEAT_HS == 2u)    
/** \brief Enable High Side 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_EN_Pos, HS_CTRL_HS2_EN_Msk, 1u);
}

/** \brief Disable High Side 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_EN_Pos, HS_CTRL_HS2_EN_Msk, 0u);
}

/** \brief Enable High Side 2 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 2 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_PWM_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_PWM_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_PWM_Pos, HS_CTRL_HS2_PWM_Msk, 1u);
}

/** \brief Disable High Side 2 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 2 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_PWM_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_PWM_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_PWM_Pos, HS_CTRL_HS2_PWM_Msk, 0u);
}

/** \brief Set High Side 2 to ON.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 2 to ON.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_On(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_On(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_ON_Pos, HS_CTRL_HS2_ON_Msk, 1u);
}

/** \brief Set High Side 2 to OFF.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 2 to OFF.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_Off(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_Off(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_ON_Pos, HS_CTRL_HS2_ON_Msk, 0u);
}

/** \brief Enable High Side 2 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 2 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_OL_EN(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_OL_En(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_OL_EN_Pos, HS_CTRL_HS2_OL_EN_Msk, 1u);
}

/** \brief Disable High Side 2 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables High Side 2 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_OL_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_OL_Dis(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_OL_EN_Pos, HS_CTRL_HS2_OL_EN_Msk, 0u);
}

/** \brief Enable High Side 2 Slew Rate 30V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 2 Slew Rate 30V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_SR_Fast_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_SR_Fast_Set(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_SRCTL_SEL_Pos, HS_CTRL_HS2_SRCTL_SEL_Msk, 1u);
}

/** \brief Enable High Side 2 Slew Rate 10V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables High Side 2 Slew Rate 10V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_SR_Slow_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_SR_Slow_Set(void)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_SRCTL_SEL_Pos, HS_CTRL_HS2_SRCTL_SEL_Msk, 0u);
}

/** \brief Select High Side 2 Overcurrent Threshold.
 *
 * \param th, High Side 2 Overcurrent Threshold Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 2 Overcurrent Threshold to 100 mA min.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_OverCur_Th_Sel(HS_Oc_100mA); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_OverCur_Th_Sel(uint32 th)
{
  Field_Mod32(&HS->CTRL.reg, (uint8)HS_CTRL_HS2_OC_SEL_Pos, HS_CTRL_HS2_OC_SEL_Msk, th);
}

#endif

/* Register HS_IRQS: High-Side Driver Interrupt Status */
#if(UC_FEAT_HS == 2u)
/** \brief Read HS2 Open Load status.
 *
 * \retval 1 if HS2 Open Load is occurred
 * \retval 0 if HS2 Open Load is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS2 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS2_OpenLoad_Sts() == 1u)
 *   {
 *     HS2_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS2_OpenLoad_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS2_OL_STS_Pos, HS_IRQS_HS2_OL_STS_Msk);
}

/** \brief Read HS2 Overtemperature Status.
 *
 * \retval 1 if HS2 Overtemperature is occurred
 * \retval 0 if HS2 Overtemperature is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS2 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS2_OverTemp_Sts() == 1u)
 *   {
 *     HS2_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS2_OverTemp_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS2_OT_STS_Pos, HS_IRQS_HS2_OT_STS_Msk);
}

/** \brief Read HS2 Overcurrent interrupt Status.
 *
 * \retval 1 if HS2 Overcurrent interrupt is occurred
 * \retval 0 if HS2 Overcurrent interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverCur_Int_En(); 
 *   if (HS2_OverCur_Int_Sts() == 1u)
 *   {
 *     HS2_OC_CALLBACK();
 *     HS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS2_OverCur_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS2_OC_IS_Pos, HS_IRQS_HS2_OC_IS_Msk);
}

/** \brief Read HS2 Open Load interrupt status.
 *
 * \retval 1 if HS2 Open Load interrupt is occurred
 * \retval 0 if HS2 Open Load interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OpenLoad_Int_En(); 
 *   if (HS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS2_OL_CALLBACK();
 *     HS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS2_OpenLoad_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS2_OL_IS_Pos, HS_IRQS_HS2_OL_IS_Msk);
}

/** \brief Read HS2 Overtemperature interrupt status.
 *
 * \retval 1 if HS2 Overtemperature interrupt is occurred
 * \retval 0 if HS2 Overtemperature interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverTemp_Int_En(); 
 *   if (HS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS2_OT_CALLBACK();
 *     HS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS2_OverTemp_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS2_OT_IS_Pos, HS_IRQS_HS2_OT_IS_Msk);
}

#endif
/** \brief Read HS1 Open Load status.
 *
 * \retval 1 if HS1 Open Load is occurred
 * \retval 0 if HS1 Open Load is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS1 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS1_OpenLoad_Sts() == 1u)
 *   {
 *     HS1_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS1_OpenLoad_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS1_OL_STS_Pos, HS_IRQS_HS1_OL_STS_Msk);
}

/** \brief Read HS1 Overtemperature Status.
 *
 * \retval 1 if HS1 Overtemperature is occurred
 * \retval 0 if HS1 Overtemperature is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS1 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS1_OverTemp_Sts() == 1u)
 *   {
 *     HS1_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS1_OverTemp_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS1_OT_STS_Pos, HS_IRQS_HS1_OT_STS_Msk);
}

/** \brief Read HS1 Overcurrent interrupt Status.
 *
 * \retval 1 if HS1 Overcurrent interrupt is occurred
 * \retval 0 if HS1 Overcurrent interrupt is not occurred 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverCur_Int_En(); 
 *   if (HS1_OverCur_Int_Sts() == 1u)
 *   {
 *     HS1_OC_CALLBACK();
 *     HS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS1_OverCur_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS1_OC_IS_Pos, HS_IRQS_HS1_OC_IS_Msk);
}

/** \brief Read HS1 Open Load interrupt Status.
 *
 * \retval 1 if HS1 Open Load interrupt is occurred
 * \retval 0 if HS1 Open Load interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OpenLoad_Int_En(); 
 *   if (HS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS1_OL_CALLBACK();
 *     HS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS1_OpenLoad_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS1_OL_IS_Pos, HS_IRQS_HS1_OL_IS_Msk);
}

/** \brief Read HS1 Overtemperature interrupt Status.
 *
 * \retval 1 if HS1 Overtemperature interrupt is occurred
 * \retval 0 if HS1 Overtemperature interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverTemp_Int_En(); 
 *   if (HS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS1_OT_CALLBACK();
 *     HS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE uint8 HS1_OverTemp_Int_Sts(void)
{
  return u1_Field_Rd32(&HS->IRQS.reg, (uint8)HS_IRQS_HS1_OT_IS_Pos, HS_IRQS_HS1_OT_IS_Msk);
}


/* Register HS_IRQCLR: High-Side Interrupt Status Clear */
#if(UC_FEAT_HS == 2u)
/** \brief Clear HS2 Open Load flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS2 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS2_OpenLoad_Sts() == 1u)
 *   {
 *     HS2_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OpenLoad_Sts_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS2_OL_SC_Pos, HS_IRQCLR_HS2_OL_SC_Msk, 1u);
}

/** \brief Clear HS2 Overtemperature flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS2 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS2_OverTemp_Sts() == 1u)
 *   {
 *     HS2_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OverTemp_Sts_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS2_OT_SC_Pos, HS_IRQCLR_HS2_OT_SC_Msk, 1u);
}

/** \brief Clear HS2 Overcurrent interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverCur_Int_En(); 
 *   if (HS2_OverCur_Int_Sts() == 1u)
 *   {
 *     HS2_OC_CALLBACK();
 *     HS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OverCur_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS2_OC_ISC_Pos, HS_IRQCLR_HS2_OC_ISC_Msk, 1u);
} 

/** \brief Clear HS2 Open Load interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OpenLoad_Int_En(); 
 *   if (HS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS2_OL_CALLBACK();
 *     HS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OpenLoad_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS2_OL_ISC_Pos, HS_IRQCLR_HS2_OL_ISC_Msk, 1u);
}  

/** \brief Clear HS2 Overtemperature interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverTemp_Int_En(); 
 *   if (HS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS2_OT_CALLBACK();
 *     HS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OverTemp_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS2_OT_ISC_Pos, HS_IRQCLR_HS2_OT_ISC_Msk, 1u);
}   

#endif

/** \brief Clear HS1 Open Load flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS1 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS1_OpenLoad_Sts() == 1u)
 *   {
 *     HS1_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OpenLoad_Sts_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS1_OL_SC_Pos, HS_IRQCLR_HS1_OL_SC_Msk, 1u);
}

/** \brief Clear HS1 Overtemperature flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks HS1 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (HS1_OverTemp_Sts() == 1u)
 *   {
 *     HS1_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OverTemp_Sts_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS1_OT_SC_Pos, HS_IRQCLR_HS1_OT_SC_Msk, 1u);
}

/** \brief Clear HS1 Overcurrent interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverCur_Int_En(); 
 *   if (HS1_OverCur_Int_Sts() == 1u)
 *   {
 *     HS1_OC_CALLBACK();
 *     HS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OverCur_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS1_OC_ISC_Pos, HS_IRQCLR_HS1_OC_ISC_Msk, 1u);
} 

/** \brief Clear HS1 Open Load interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OpenLoad_Int_En(); 
 *   if (HS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS1_OL_CALLBACK();
 *     HS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OpenLoad_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS1_OL_ISC_Pos, HS_IRQCLR_HS1_OL_ISC_Msk, 1u);
} 

/** \brief Clear HS1 Overtemperature interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverTemp_Int_En(); 
 *   if (HS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS1_OT_CALLBACK();
 *     HS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OverTemp_Int_Clr(void)
{
  Field_Wrt32(&HS->IRQCLR.reg, (uint8)HS_IRQCLR_HS1_OT_ISC_Pos, HS_IRQCLR_HS1_OT_ISC_Msk, 1u);
} 


/* Register LS_IRQEN: High-Side Interrupt Enable */
#if(UC_FEAT_HS == 2u)
/** \brief Enable HS2 Overcurrent interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverCur_Int_En(); 
 *   if (HS2_OverCur_Int_Sts() == 1u)
 *   {
 *     HS2_OC_CALLBACK();
 *     HS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OverCur_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS2_OC_IEN_Pos, HS_IRQEN_HS2_OC_IEN_Msk, 1u);
}

/** \brief Enable HS2 Open Load interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OpenLoad_Int_En(); 
 *   if (HS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS2_OL_CALLBACK();
 *     HS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OpenLoad_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS2_OL_IEN_Pos, HS_IRQEN_HS2_OL_IEN_Msk, 1u);
}

/** \brief Enable HS2 Overtemperature interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS2_OverTemp_Int_En(); 
 *   if (HS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS2_OT_CALLBACK();
 *     HS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS2_OverTemp_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS2_OT_IEN_Pos, HS_IRQEN_HS2_OT_IEN_Msk, 1u);
}

#endif

/** \brief Enable HS1 Overcurrent interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverCur_Int_En(); 
 *   if (HS1_OverCur_Int_Sts() == 1u)
 *   {
 *     HS1_OC_CALLBACK();
 *     HS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OverCur_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS1_OC_IEN_Pos, HS_IRQEN_HS1_OC_IEN_Msk, 1u);
}

/** \brief Enable HS1 Open Load interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OpenLoad_Int_En(); 
 *   if (HS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     HS1_OL_CALLBACK();
 *     HS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OpenLoad_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS1_OL_IEN_Pos, HS_IRQEN_HS1_OL_IEN_Msk, 1u);
}

/** \brief Enable HS1 Overtemperature interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the HS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   HS1_OverTemp_Int_En(); 
 *   if (HS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     HS1_OT_CALLBACK();
 *     HS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup hs_api
 */
INLINE void HS1_OverTemp_Int_En(void)
{
  Field_Mod32(&HS->IRQEN.reg, (uint8)HS_IRQEN_HS1_OT_IEN_Pos, HS_IRQEN_HS1_OT_IEN_Msk, 1u);
}


/* Register LS_PWMSRCSEL: High-Side PWM Selection */
/** \brief Select High Side 1 PWM Source.
 *
 * \param hs_pwm High Side 1 PWM Source Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 1 PWM Source to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_PWM_Src_Sel(HS_PWM_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_PWM_Src_Sel(uint32 hs_pwm)
{
  Field_Mod32(&HS->PWMSRCSEL.reg, (uint8)HS_PWMSRCSEL_HS1_SRC_SEL_Pos, HS_PWMSRCSEL_HS1_SRC_SEL_Msk, hs_pwm);
}

#if(UC_FEAT_HS == 2u)
/** \brief Select High Side 2 PWM Source.
 *
 * \param hs_pwm High Side 2 PWM Source Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets High Side 2 PWM Source to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_PWM_Src_Sel(HS_PWM_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_PWM_Src_Sel(uint32 hs_pwm)
{
  Field_Mod32(&HS->PWMSRCSEL.reg, (uint8)HS_PWMSRCSEL_HS2_SRC_SEL_Pos, HS_PWMSRCSEL_HS2_SRC_SEL_Msk, hs_pwm);
}

#endif

/* Register LS_LS1_TRIM Low Side Trimming Register */
/** \brief Select HS1 Blanking Time Filter for Overcurrent/Overtemperature Detection.
 *
 * \param hs_filt HS1 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets HS1 Blanking Time Filter for Overcurrent/Overtemperature Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_OverCur_Filt_time(HS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_OverCur_Filt_time(uint32 hs_filt)
{
  Field_Mod32(&HS->HS1_TRIM.reg, (uint8)HS_HS1_TRIM_HS1_OC_OT_BTFILT_SEL_Pos, HS_HS1_TRIM_HS1_OC_OT_BTFILT_SEL_Msk, hs_filt);
}

/** \brief Select HS1 Blanking Time Filter for Open Load Detection.
 *
 * \param hs_filt HS1 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets HS1 Blanking Time Filter for Open Load Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS1_En();
 *     HS1_OpenLoad_Filt_time(HS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS1_OpenLoad_Filt_time(uint32 hs_filt)
{
  Field_Mod32(&HS->HS1_TRIM.reg, (uint8)HS_HS1_TRIM_HS1_OL_BTFILT_SEL_Pos, HS_HS1_TRIM_HS1_OL_BTFILT_SEL_Msk, hs_filt);
}

#if(UC_FEAT_HS == 2u)
/** \brief Select HS2 Blanking Time Filter for Overcurrent/Overtemperature Detection.
 *
 * \param hs_filt HS2 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets HS2 Blanking Time Filter for Overcurrent/Overtemperature Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_OverCur_Filt_time(HS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_OverCur_Filt_time(uint32 hs_filt)
{
  Field_Mod32(&HS->HS2_TRIM.reg, (uint8)HS_HS2_TRIM_HS2_OC_OT_BTFILT_SEL_Pos, HS_HS2_TRIM_HS2_OC_OT_BTFILT_SEL_Msk, hs_filt);
}

/** \brief Select HS2 Blanking Time Filter for Open Load Detection.
 *
 * \param hs_filt HS2 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets HS2 Blanking Time Filter for Open Load Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     HS2_En();
 *     HS2_OpenLoad_Filt_time(HS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup hs_api
 */
INLINE void HS2_OpenLoad_Filt_time(uint32 hs_filt)
{
  Field_Mod32(&HS->HS2_TRIM.reg, (uint8)HS_HS2_TRIM_HS2_OL_BTFILT_SEL_Pos, HS_HS2_TRIM_HS2_OL_BTFILT_SEL_Msk, hs_filt);
}

#endif

#endif
