/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     ls.h
 *
 * \brief    Low Side Switch low level access library
 *
 * \version  V0.1.8
 * \date     13. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AV           Axel Viebig                                                   **
** DM           Daniel Mysliwitz                                              **
** MK           Martin Kilian                                                 **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-10-23, AV:   Initial version                                  **
** V0.1.1: 2015-11-11, AV:   Added methods for LS module                      **
** V0.1.2: 2016-06-28, DM:   LSx_Set_Switch corrected                         **
** V0.1.3: 2017-04-05, MK:   Added Define Macros for LS Registers in LS.h     **
** V0.1.4: 2017-10-20, KC:   Macros with _ISts_ renamed to _Int_              **
**                           Macros corrected to right IRQCLR register        **
** V0.1.5: 2018-11-05, CE:   Macros for LS_IRQCLR corrected                   **
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.6: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
** V0.1.7: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to ls.c                                          **
** V0.1.8: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

#ifndef LS_H
#define LS_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief LS Interrupt Mask */
#define LS_IRQ_BITS 0x00F000F0

/* Register LS_PWMSRCSEL: Low-Side PWM Selection Macros */
/**\brief Low-Side PWM Selection Macro, LS_PWM_CC60 */
#define LS_PWM_CC60   0u
/**\brief Low-Side PWM Selection Macro, LS_PWM_CC61 */
#define LS_PWM_CC61   1u
/**\brief Low-Side PWM Selection Macro, LS_PWM_CC62 */
#define LS_PWM_CC62   2u
/**\brief Low-Side PWM Selection Macro, LS_PWM_COUT60 */
#define LS_PWM_COUT60 3u
/**\brief Low-Side PWM Selection Macro, LS_PWM_COUT61 */
#define LS_PWM_COUT61 4u
/**\brief Low-Side PWM Selection Macro, LS_PWM_COUT62 */
#define LS_PWM_COUT62 5u
/**\brief Low-Side PWM Selection Macro, LS_PWM_T3OUT */
#define LS_PWM_T3OUT  6u

/* Register LS_LS1_TRIM Low Side Trimming Register Macros */
/**\brief Low-Side Overcurrent BlankTime Selection Macro, LS_Filt_4us */
#define LS_Filt_4us  0u
/**\brief Low-Side Overcurrent BlankTime Selection Macro, LS_Filt_8us */
#define LS_Filt_8us  1u
/**\brief Low-Side Overcurrent BlankTime Selection Macro, LS_Filt_16us */
#define LS_Filt_16us 2u
/**\brief Low-Side Overcurrent BlankTime Selection Macro, LS_Filt_32us */
#define LS_Filt_32us 3u

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \enum LS_Sw_Int
 *  \brief This enum lists the Low Side channel configurations. 
 */
typedef enum _LS_Sw_Int
{
  Int_Ls1_Ints    = 17U,  /**< \brief LS1 interrupts global enable            */
  Int_Ls1_Oc      = 7U,   /**< \brief LS1 Over-Current interrupt enable       */
  Int_Ls1_Ol      = 6U,   /**< \brief LS1 Over-Load interrupt enable          */
  Int_Ls1_Ot      = 5U,   /**< \brief LS1 Over-Temperature interrupt en.      */
  Int_Ls1_Ot_Prew = 4U,   /**< \brief LS1 Over-Temperature Prewarn int.en.    */

  Int_Ls2_Ints    = 18U,  /**< \brief LS2 interrupts global enable            */
  Int_Ls2_Oc      = 23U,  /**< \brief LS2 Over-Current interrupt enable       */
  Int_Ls2_Ol      = 22U,  /**< \brief LS2 Over-Load interrupt enable          */ 
  Int_Ls2_Ot      = 21U,  /**< \brief LS2 Over-Temperature interrupt en.      */
  Int_Ls2_Ot_Prew = 20U  /**< \brief LS2 Over-Temperature Prewarn int.en.    */  
} LS_Sw_Int;

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void LS1_Init(void);
void LS2_Init(void);
INLINE void LS1_En(void);
INLINE void LS1_Dis(void);
INLINE void LS1_PWM_En(void);
INLINE void LS1_PWM_Dis(void);
INLINE void LS1_On(void);
INLINE void LS1_Off(void);
INLINE void LS1_OL_En(void);
INLINE void LS1_OL_Dis(void);
INLINE void LS1_SR_Fast_Set(void);
INLINE void LS1_SR_Slow_Set(void);
INLINE void LS2_En(void);
INLINE void LS2_Dis(void);
INLINE void LS2_PWM_En(void);
INLINE void LS2_PWM_Dis(void);
INLINE void LS2_On(void);
INLINE void LS2_Off(void);
INLINE void LS2_OL_En(void);
INLINE void LS2_OL_Dis(void);
INLINE void LS2_SR_Fast_Set(void);
INLINE void LS2_SR_Slow_Set(void);
INLINE uint8 LS2_OpenLoad_Sts(void);
INLINE uint8 LS2_OverTemp_Sts(void);
INLINE uint8 LS2_OverTemp_Prewarn_Sts(void);
INLINE uint8 LS2_OverCur_Int_Sts(void);
INLINE uint8 LS2_OpenLoad_Int_Sts(void);
INLINE uint8 LS2_OverTemp_Int_Sts(void);
INLINE uint8 LS2_OverTemp_Prewarn_Int_Sts(void);
INLINE uint8 LS1_OpenLoad_Sts(void);
INLINE uint8 LS1_OverTemp_Sts(void);
INLINE uint8 LS1_OverTemp_Prewarn_Sts(void);
INLINE uint8 LS1_OverCur_Int_Sts(void);
INLINE uint8 LS1_OpenLoad_Int_Sts(void);
INLINE uint8 LS1_OverTemp_Int_Sts(void);
INLINE uint8 LS1_OverTemp_Prewarn_Int_Sts(void);
INLINE void LS2_OpenLoad_Sts_Clr(void);
INLINE void LS2_OverTemp_Sts_Clr(void);
INLINE void LS2_OverTemp_Prewarn_Sts_Clr(void);
INLINE void LS2_OverCur_Int_Clr(void);
INLINE void LS2_OpenLoad_Int_Clr(void);
INLINE void LS2_OverTemp_Int_Clr(void);
INLINE void LS2_OverTemp_Prewarn_Int_Clr(void);
INLINE void LS1_OpenLoad_Sts_Clr(void);
INLINE void LS1_OverTemp_Sts_Clr(void);
INLINE void LS1_OverTemp_Prewarn_Sts_Clr(void);
INLINE void LS1_OverCur_Int_Clr(void);
INLINE void LS1_OpenLoad_Int_Clr(void);
INLINE void LS1_OverTemp_Int_Clr(void);
INLINE void LS1_OverTemp_Prewarn_Int_Clr(void);
INLINE void LS2_OverCur_Int_En(void);
INLINE void LS2_OpenLoad_Int_En(void);
INLINE void LS2_OverTemp_Int_En(void);
INLINE void LS2_OverTemp_Prewarn_Int_En(void);
INLINE void LS1_OverCur_Int_En(void);
INLINE void LS1_OpenLoad_Int_En(void);
INLINE void LS1_OverTemp_Int_En(void);
INLINE void LS1_OverTemp_Prewarn_Int_En(void);
INLINE void LS1_PWM_Src_Sel(uint32 ls_pwm);
INLINE void LS2_PWM_Src_Sel(uint32 ls_pwm);
INLINE void LS1_OverCur_Filt_time(uint32 ls_filt);
INLINE void LS1_OpenLoad_Filt_time(uint32 ls_filt);
INLINE void LS2_OverCur_Filt_time(uint32 ls_filt);
INLINE void LS2_OpenLoad_Filt_time(uint32 ls_filt);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Enable Low Side 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_EN_Pos, LS_CTRL_LS1_EN_Msk, 1u);
}

/** \brief Disable Low Side 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_EN_Pos, LS_CTRL_LS1_EN_Msk, 0u);
}

/** \brief Enable Low Side 1 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 1 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_PWM_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_PWM_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_PWM_Pos, LS_CTRL_LS1_PWM_Msk, 1u);
}

/** \brief Disable Low Side 1 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 1 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_PWM_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_PWM_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_PWM_Pos, LS_CTRL_LS1_PWM_Msk, 0u);
}

/** \brief Set Low Side 1 to ON.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 1 to ON.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_On(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_On(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_ON_Pos, LS_CTRL_LS1_ON_Msk, 1u);
}

/** \brief Set Low Side 1 to OFF.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 1 to OFF.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_Off(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_Off(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_ON_Pos, LS_CTRL_LS1_ON_Msk, 0u);
}

/** \brief Enable Low Side 1 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 1 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_OL_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_OL_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_OL_EN_Pos, LS_CTRL_LS1_OL_EN_Msk, 1u);
}

/** \brief Disable Low Side 1 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 1 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_OL_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_OL_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_OL_EN_Pos, LS_CTRL_LS1_OL_EN_Msk, 0u);
}

/** \brief Enable Low Side 1 Slew Rate 30V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 1 Slew Rate 30V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_SR_Fast_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_SR_Fast_Set(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_SRCTL_SEL_Pos, LS_CTRL_LS1_SRCTL_SEL_Msk, 1u);
}

/** \brief Enable Low Side 1 Slew Rate 10V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 1 Slew Rate 10V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_SR_Slow_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_SR_Slow_Set(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS1_SRCTL_SEL_Pos, LS_CTRL_LS1_SRCTL_SEL_Msk, 0u);
}

/** \brief Enable Low Side 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_EN_Pos, LS_CTRL_LS2_EN_Msk, 1u);
}

/** \brief Disable Low Side 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_EN_Pos, LS_CTRL_LS2_EN_Msk, 0u);
}

/** \brief Enable Low Side 2 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 2 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_PWM_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_PWM_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_PWM_Pos, LS_CTRL_LS2_PWM_Msk, 1u);
}

/** \brief Disable Low Side 2 PWM.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 2 PWM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_PWM_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_PWM_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_PWM_Pos, LS_CTRL_LS2_PWM_Msk, 0u);
}

/** \brief Set Low Side 2 to ON.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 2 to ON.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_On(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_On(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_ON_Pos, LS_CTRL_LS2_ON_Msk, 1u);
}

/** \brief Set Low Side 2 to OFF.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 2 to OFF.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_Off(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_Off(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_ON_Pos, LS_CTRL_LS2_ON_Msk, 0u);
}

/** \brief Enable Low Side 2 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 2 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_OL_En(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_OL_En(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_OL_EN_Pos, LS_CTRL_LS2_OL_EN_Msk, 1u);
}

/** \brief Disable Low Side 2 Open Load Detection.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Low Side 2 Open Load Detection.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_OL_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_OL_Dis(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_OL_EN_Pos, LS_CTRL_LS2_OL_EN_Msk, 0u);
}

/** \brief Enable Low Side 2 Slew Rate 30V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 2 Slew Rate 30V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_SR_Fast_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_SR_Fast_Set(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_SRCTL_SEL_Pos, LS_CTRL_LS2_SRCTL_SEL_Msk, 1u);
}

/** \brief Enable Low Side 2 Slew Rate 10V/μs.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Low Side 2 Slew Rate 10V/μs.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_SR_Slow_Set(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_SR_Slow_Set(void)
{
  Field_Mod32(&LS->CTRL.reg, (uint8)LS_CTRL_LS2_SRCTL_SEL_Pos, LS_CTRL_LS2_SRCTL_SEL_Msk, 0u);
}


/* Register LS_IRQS: Low-Side Driver Interrupt Status */
/** \brief Read LS2 Open Load status.
 *
 * \retval 1 if LS2 Open Load is occurred
 * \retval 0 if LS2 Open Load is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OpenLoad_Sts() == 1u)
 *   {
 *     LS2_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OpenLoad_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OL_STS_Pos, LS_IRQS_LS2_OL_STS_Msk);
}

/** \brief Read LS2 Overtemperature Status.
 *
 * \retval 1 if LS2 Overtemperature is occurred
 * \retval 0 if LS2 Overtemperature is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OverTemp_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OverTemp_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OT_STS_Pos, LS_IRQS_LS2_OT_STS_Msk);
}

/** \brief Read LS2 Overtemperature Prewarning Status.
 *
 * \retval 1 if LS2 Overtemperature Prewarning is occurred
 * \retval 0 if LS2 Overtemperature Prewarning is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Overtemperature Prewarning Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OverTemp_Prewarn_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Prewarn_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OverTemp_Prewarn_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OT_PREWARN_STS_Pos, LS_IRQS_LS2_OT_PREWARN_STS_Msk);
}

/** \brief Read LS2 Overcurrent interrupt Status.
 *
 * \retval 1 if LS2 Overcurrent interrupt is occurred
 * \retval 0 if LS2 Overcurrent interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverCur_Int_En(); 
 *   if (LS2_OverCur_Int_Sts() == 1u)
 *   {
 *     LS2_OC_CALLBACK();
 *     LS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OverCur_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OC_IS_Pos, LS_IRQS_LS2_OC_IS_Msk);
}

/** \brief Read LS2 Open Load interrupt status.
 *
 * \retval 1 if LS2 Open Load interrupt is occurred
 * \retval 0 if LS2 Open Load interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OpenLoad_Int_En(); 
 *   if (LS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS2_OL_CALLBACK();
 *     LS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OpenLoad_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OL_IS_Pos, LS_IRQS_LS2_OL_IS_Msk);
}

/** \brief Read LS2 Overtemperature interrupt status.
 *
 * \retval 1 if LS2 Overtemperature interrupt is occurred
 * \retval 0 if LS2 Overtemperature interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Int_En(); 
 *   if (LS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS2_OT_CALLBACK();
 *     LS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OverTemp_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OT_IS_Pos, LS_IRQS_LS2_OT_IS_Msk);
}

/** \brief Read LS2 Overtemperature Prewarning interrupt status.
 *
 * \retval 1 if LS2 Overtemperature Prewarning interrupt is occurred
 * \retval 0 if LS2 Overtemperature Prewarning interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Prewarn_Int_En(); 
 *   if (LS2_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS2_OverTemp_Prewarn_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS2_OT_PREWARN_IS_Pos, LS_IRQS_LS2_OT_PREWARN_IS_Msk);
}

/** \brief Read LS1 Open Load status.
 *
 * \retval 1 if LS1 Open Load is occurred
 * \retval 0 if LS1 Open Load is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OpenLoad_Sts() == 1u)
 *   {
 *     LS1_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OpenLoad_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OL_STS_Pos, LS_IRQS_LS1_OL_STS_Msk);
}

/** \brief Read LS1 Overtemperature Status.
 *
 * \retval 1 if LS1 Overtemperature is occurred
 * \retval 0 if LS1 Overtemperature is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OverTemp_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OverTemp_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OT_STS_Pos, LS_IRQS_LS1_OT_STS_Msk);
}

/** \brief Read LS1 Overtemperature Prewarning Status.
 *
 * \retval 1 if LS1 Overtemperature Prewarning is occurred
 * \retval 0 if LS1 Overtemperature Prewarning is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Overtemperature Prewarning Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OverTemp_Prewarn_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Prewarn_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OverTemp_Prewarn_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OT_PREWARN_STS_Pos, LS_IRQS_LS1_OT_PREWARN_STS_Msk);
}

/** \brief Read LS1 Overcurrent interrupt Status.
 *
 * \retval 1 if LS1 Overcurrent interrupt is occurred
 * \retval 0 if LS1 Overcurrent interrupt is not occurred 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverCur_Int_En(); 
 *   if (LS1_OverCur_Int_Sts() == 1u)
 *   {
 *     LS1_OC_CALLBACK();
 *     LS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OverCur_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OC_IS_Pos, LS_IRQS_LS1_OC_IS_Msk);
}

/** \brief Read LS1 Open Load interrupt Status.
 *
 * \retval 1 if LS1 Open Load interrupt is occurred
 * \retval 0 if LS1 Open Load interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OpenLoad_Int_En(); 
 *   if (LS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS1_OL_CALLBACK();
 *     LS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OpenLoad_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OL_IS_Pos, LS_IRQS_LS1_OL_IS_Msk);
}

/** \brief Read LS1 Overtemperature interrupt Status.
 *
 * \retval 1 if LS1 Overtemperature interrupt is occurred
 * \retval 0 if LS1 Overtemperature interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Int_En(); 
 *   if (LS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS1_OT_CALLBACK();
 *     LS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OverTemp_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OT_IS_Pos, LS_IRQS_LS1_OT_IS_Msk);
}

/** \brief Read LS1 Overtemperature Prewarning interrupt Status.
 *
 * \retval 1 if LS1 Overtemperature Prewarning interrupt is occurred
 * \retval 0 if LS1 Overtemperature Prewarning interrupt is not occurred
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Prewarn_Int_En(); 
 *   if (LS1_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE uint8 LS1_OverTemp_Prewarn_Int_Sts(void)
{
  return u1_Field_Rd32(&LS->IRQS.reg, (uint8)LS_IRQS_LS1_OT_PREWARN_IS_Pos, LS_IRQS_LS1_OT_PREWARN_IS_Msk);
}


/* Register LS_IRQCLR: Low-Side Interrupt Status Clear */
/** \brief Clear LS2 Open Load flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OpenLoad_Sts() == 1u)
 *   {
 *     LS2_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OpenLoad_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OL_SC_Pos, LS_IRQCLR_LS2_OL_SC_Msk, 1u);
}

/** \brief Clear LS2 Overtemperature flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OverTemp_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OT_SC_Pos, LS_IRQCLR_LS2_OT_SC_Msk, 1u);
}

/** \brief Clear LS2 Overtemperature Prewarning flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS2 Overtemperature Prewarning Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS2_OverTemp_Prewarn_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Prewarn_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Prewarn_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OT_PREWARN_SC_Pos, LS_IRQCLR_LS2_OT_PREWARN_SC_Msk, 1u);
}

/** \brief Clear LS2 Overcurrent interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverCur_Int_En(); 
 *   if (LS2_OverCur_Int_Sts() == 1u)
 *   {
 *     LS2_OC_CALLBACK();
 *     LS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverCur_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OC_ISC_Pos, LS_IRQCLR_LS2_OC_ISC_Msk, 1u);
} 

/** \brief Clear LS2 Open Load interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OpenLoad_Int_En(); 
 *   if (LS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS2_OL_CALLBACK();
 *     LS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OpenLoad_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OL_ISC_Pos, LS_IRQCLR_LS2_OL_ISC_Msk, 1u);
}  

/** \brief Clear LS2 Overtemperature interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Int_En(); 
 *   if (LS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS2_OT_CALLBACK();
 *     LS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OT_ISC_Pos, LS_IRQCLR_LS2_OT_ISC_Msk, 1u);
}   

/** \brief Clear LS2 Overtemperature Prewarning interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Prewarn_Int_En(); 
 *   if (LS2_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Prewarn_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS2_OT_PREWARN_ISC_Pos, LS_IRQCLR_LS2_OT_PREWARN_ISC_Msk, 1u);
}   

/** \brief Clear LS1 Open Load flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Open Load Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OpenLoad_Sts() == 1u)
 *   {
 *     LS1_OpenLoad_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OpenLoad_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OL_SC_Pos, LS_IRQCLR_LS1_OL_SC_Msk, 1u);
}

/** \brief Clear LS1 Overtemperature flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Overtemperature Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OverTemp_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OT_SC_Pos, LS_IRQCLR_LS1_OT_SC_Msk, 1u);
}

/** \brief Clear LS1 Overtemperature Prewarning flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks LS1 Overtemperature Prewarning Status and clears its flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (LS1_OverTemp_Prewarn_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Prewarn_Sts_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Prewarn_Sts_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OT_PREWARN_SC_Pos, LS_IRQCLR_LS1_OT_PREWARN_SC_Msk, 1u);
}

/** \brief Clear LS1 Overcurrent interrupt flag.
 *
 * \return None 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverCur_Int_En(); 
 *   if (LS1_OverCur_Int_Sts() == 1u)
 *   {
 *     LS1_OC_CALLBACK();
 *     LS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverCur_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OC_ISC_Pos, LS_IRQCLR_LS1_OC_ISC_Msk, 1u);
} 

/** \brief Clear LS1 Open Load interrupt flag.
 *
 * \return None 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OpenLoad_Int_En(); 
 *   if (LS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS1_OL_CALLBACK();
 *     LS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OpenLoad_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OL_ISC_Pos, LS_IRQCLR_LS1_OL_ISC_Msk, 1u);
} 

/** \brief Clear LS1 Overtemperature interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Int_En(); 
 *   if (LS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS1_OT_CALLBACK();
 *     LS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OT_ISC_Pos, LS_IRQCLR_LS1_OT_ISC_Msk, 1u);
}  

/** \brief Clear LS1 Overtemperature Prewarning interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Prewarn_Int_En(); 
 *   if (LS1_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Prewarn_Int_Clr(void)
{
  Field_Wrt32(&LS->IRQCLR.reg, (uint8)LS_IRQCLR_LS1_OT_PREWARN_ISC_Pos, LS_IRQCLR_LS1_OT_PREWARN_ISC_Msk, 1u);
}  

/* Register LS_IRQEN: Low-Side Interrupt Enable */
/** \brief Enable LS2 Overcurrent interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverCur_Int_En(); 
 *   if (LS2_OverCur_Int_Sts() == 1u)
 *   {
 *     LS2_OC_CALLBACK();
 *     LS2_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverCur_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS2_OC_IEN_Pos, LS_IRQEN_LS2_OC_IEN_Msk, 1u);
}

/** \brief Enable LS2 Open Load interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OpenLoad_Int_En(); 
 *   if (LS2_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS2_OL_CALLBACK();
 *     LS2_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OpenLoad_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS2_OL_IEN_Pos, LS_IRQEN_LS2_OL_IEN_Msk, 1u);
}

/** \brief Enable LS2 Overtemperature interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Int_En(); 
 *   if (LS2_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS2_OT_CALLBACK();
 *     LS2_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS2_OT_IEN_Pos, LS_IRQEN_LS2_OT_IEN_Msk, 1u);
}

/** \brief Enable LS2 Overtemperature Prewarning interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS2 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS2_OverTemp_Prewarn_Int_En(); 
 *   if (LS2_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS2_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS2_OverTemp_Prewarn_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS2_OT_PREWARN_IEN_Pos, LS_IRQEN_LS2_OT_PREWARN_IEN_Msk, 1u);
} 

/** \brief Enable LS1 Overcurrent interrupt.
 *
 * \return None 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverCur_Int_En(); 
 *   if (LS1_OverCur_Int_Sts() == 1u)
 *   {
 *     LS1_OC_CALLBACK();
 *     LS1_OverCur_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverCur_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS1_OC_IEN_Pos, LS_IRQEN_LS1_OC_IEN_Msk, 1u);
} 

/** \brief Enable LS1 Open Load interrupt.
 *
 * \return None 
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Open Load interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OpenLoad_Int_En(); 
 *   if (LS1_OpenLoad_Int_Sts() == 1u)
 *   {
 *     LS1_OL_CALLBACK();
 *     LS1_OpenLoad_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OpenLoad_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS1_OL_IEN_Pos, LS_IRQEN_LS1_OL_IEN_Msk, 1u);
} 

/** \brief Enable LS1 Overtemperature interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Int_En(); 
 *   if (LS1_OverTemp_Int_Sts() == 1u)
 *   {
 *     LS1_OT_CALLBACK();
 *     LS1_OverTemp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS1_OT_IEN_Pos, LS_IRQEN_LS1_OT_IEN_Msk, 1u);
}  

/** \brief Enable LS1 Overtemperature Prewarning interrupt.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LS1 Overtemperature Prewarning interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   LS1_OverTemp_Prewarn_Int_En(); 
 *   if (LS1_OverTemp_Prewarn_Int_Sts() == 1u)
 *   {
 *     LS1_OverTemp_Prewarn_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ls_api
 */
INLINE void LS1_OverTemp_Prewarn_Int_En(void)
{
  Field_Mod32(&LS->IRQEN.reg, (uint8)LS_IRQEN_LS1_OT_PREWARN_IEN_Pos, LS_IRQEN_LS1_OT_PREWARN_IEN_Msk, 1u);
} 

/* Register LS_PWMSRCSEL: Low-Side PWM Selection */
/** \brief Select Low Side 1 PWM Source.
 *
 * \param ls_pwm Low Side 1 PWM Source Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 1 PWM Source to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_PWM_Src_Sel(LS_PWM_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_PWM_Src_Sel(uint32 ls_pwm)
{
  Field_Mod32(&LS->PWMSRCSEL.reg, (uint8)LS_PWMSRCSEL_LS1_SRC_SEL_Pos, LS_PWMSRCSEL_LS1_SRC_SEL_Msk, ls_pwm);
}

/** \brief Select Low Side 2 PWM Source.
 *
 * \param ls_pwm Low Side 2 PWM Source Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Low Side 2 PWM Source to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_PWM_Src_Sel(LS_PWM_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_PWM_Src_Sel(uint32 ls_pwm)
{
  Field_Mod32(&LS->PWMSRCSEL.reg, (uint8)LS_PWMSRCSEL_LS2_SRC_SEL_Pos, LS_PWMSRCSEL_LS2_SRC_SEL_Msk, ls_pwm);
}

/* Register LS_LS1_TRIM Low Side Trimming Register */
/** \brief Select LS1 Blanking Time Filter for Overcurrent/Overtemperature Detection.
 *
 * \param ls_filt LS1 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets LS1 Blanking Time Filter for Overcurrent/Overtemperature Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_OverCur_Filt_time(LS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_OverCur_Filt_time(uint32 ls_filt)
{
  Field_Mod32(&LS->LS1_TRIM.reg, (uint8)LS_LS1_TRIM_LS1_OC_BTFILT_SEL_Pos, LS_LS1_TRIM_LS1_OC_BTFILT_SEL_Msk, ls_filt);
}

/** \brief Select LS1 Blanking Time Filter for Open Load Detection.
 *
 * \param ls_filt LS1 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets LS1 Blanking Time Filter for Open Load Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS1_En();
 *     LS1_OpenLoad_Filt_time(LS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS1_OpenLoad_Filt_time(uint32 ls_filt)
{
  Field_Mod32(&LS->LS1_TRIM.reg, (uint8)LS_LS1_TRIM_LS1_OL_BTFILT_SEL_Pos, LS_LS1_TRIM_LS1_OL_BTFILT_SEL_Msk, ls_filt);
}

/** \brief Select LS2 Blanking Time Filter for Overcurrent/Overtemperature Detection.
 *
 * \param ls_filt LS2 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets LS2 Blanking Time Filter for Overcurrent/Overtemperature Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_OverCur_Filt_time(LS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_OverCur_Filt_time(uint32 ls_filt)
{
  Field_Mod32(&LS->LS2_TRIM.reg, (uint8)LS_LS2_TRIM_LS2_OC_BTFILT_SEL_Pos, LS_LS2_TRIM_LS2_OC_BTFILT_SEL_Msk, ls_filt);
}

/** \brief Select LS2 Blanking Time Filter for Open Load Detection.
 *
 * \param ls_filt LS2 Blanking Time Filter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets LS2 Blanking Time Filter for Open Load Detection to 8 μs filter time.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * { 
 *     LS2_En();
 *     LS2_OpenLoad_Filt_time(LS_Filt_8us); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ls_api
 */
INLINE void LS2_OpenLoad_Filt_time(uint32 ls_filt)
{
  Field_Mod32(&LS->LS2_TRIM.reg, (uint8)LS_LS2_TRIM_LS2_OL_BTFILT_SEL_Pos, LS_LS2_TRIM_LS2_OL_BTFILT_SEL_Msk, ls_filt);
}

#endif
