/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     pmu.h
 *
 * \brief    PMU low level access library
 *
 * \version  V0.1.8
 * \date     21. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-10-23, AV:   Initial version                                  **
** V0.1.2: 2017-10-20, KC:   Macros for interrupt register access added       **
** V0.1.3: 2017-11-29, KC:   Paramater of PMU_VDDEXT_Error_Clear              **
**                           and PMU_VDDEXT_Set changed from uint8 to uint16  **
**                           Interrupt clear macros corrected                 **
** V0.1.4: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.5: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
** V0.1.6: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to pmu.c                                         **
** V0.1.7: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.1.8: 2021-12-21, BG:   EP-997: Added missing void as parameter for      **
**                           PMU_VDDEXT_UV_Int_Clr()                          **
*******************************************************************************/

#ifndef PMU_H
#define PMU_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/* Typedef Reset Status */
/**\brief PMU Reset Status Flag Mask, Lockup-Reset Flag*/
#define PMU_RESET_STS_LOCKUP   (0x0400u)
/**\brief PMU Reset Status Flag Mask, Soft-Reset Flag*/
#define PMU_RESET_STS_PMU_SOFT (0x0200u)
/**\brief PMU Reset Status Flag Mask, Internal Watchdog Reset Flag*/
#define PMU_RESET_STS_IntWDT   (0x0100u)
/**\brief PMU Reset Status Flag Mask, Power-On Reset Flag*/
#define PMU_RESET_STS_POR      (0x0080u)
/**\brief PMU Reset Status Flag Mask, PIN-Reset Flag*/
#define PMU_RESET_STS_PIN      (0x0040u)
/**\brief PMU Reset Status Flag Mask, External Watchdog (WDT1) Reset Flag*/
#define PMU_RESET_STS_ExtWDT1  (0x0020u)
/**\brief PMU Reset Status Flag Mask, Clock Watchdog (CLKWDT) Reset Flag*/
#define PMU_RESET_STS_ClkWDT   (0x0010u)
/**\brief PMU Reset Status Flag Mask, Low Priority Resets*/
#define PMU_RESET_STS_LPR      (0x0008u)
/**\brief PMU Reset Status Flag Mask, Flag which indicates a reset caused by Sleep-Exit*/
#define PMU_RESET_STS_SleepEx  (0x0004u)
/**\brief PMU Reset Status Flag Mask, Flag which indicates a reset caused by Stop-Exit*/
#define PMU_RESET_STS_WAKE     (0x0002u)
/**\brief PMU Reset Status Flag Mask, Flag which indicates a reset caused by a System Fail reported in the corresponding Fail Register*/
#define PMU_RESET_STS_SYS_FAIL (0x0001u)

/* VDDEXT Flags */
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Overtemperature Status clear*/
#define PMU_VDDEXT_OT_SC        (0x2000u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Undervoltage Interrupt Status clear*/
#define PMU_VDDEXT_UV_ISC       (0x1000u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Overtemperature Interrupt Status clear*/
#define PMU_VDDEXT_OT_ISC       (0x0800u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Stable*/
#define PMU_VDDEXT_STABLE       (0x0080u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Overtemperature*/
#define PMU_VDDEXT_OT           (0x0040u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Overtemperature Status*/
#define PMU_VDDEXT_OT_STS       (0x0020u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Undervoltage Interrupt Status*/
#define PMU_VDDEXT_UV_IS        (0x0010u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply OverTemperature Interrupt Status*/
#define PMU_VDDEXT_OT_IS        (0x0008u)
/**\brief PMU VDDEXT Status Flag Mask, Enabling of VDDEXT Supply status information as interrupt source*/
#define PMU_VDDEXT_FAIL_EN      (0x0004u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply for Cyclic Sense Enable*/
#define PMU_VDDEXT_CYC_EN       (0x0002u)
/**\brief PMU VDDEXT Status Flag Mask, VDDEXT Supply Enable*/
#define PMU_VDDEXT_ENABLE       (0x0001u)

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void PMU_Init(void);
INLINE void PMU_VDDEXT_OT_Int_Clr(void);   
INLINE void PMU_VDDEXT_UV_Int_Clr(void);   
INLINE uint32 PMU_Get_Reset_Status(void);
INLINE void PMU_Clear_Reset_Status(void);
INLINE uint8 PMU_VDDEXT_On(void);
INLINE uint8 PMU_VDDEXT_Off(void);
INLINE void PMU_VDDEXT_Error_Clear(uint16 ErrorFlagMask);
INLINE void PMU_VDDEXT_Set(uint16 FlagMask);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Clear VDDEXT Supply OverTemperature interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDEXT Supply OverTemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (PMU->VDDEXT_CTRL.bit.VDDEXT_FAIL_EN == 1u) 
 *   {	 
 *     if (PMU->VDDEXT_CTRL.bit.VDDEXT_OT_IS == 1u)
 *     {
 *       PMU_VDDEXT_OT_CALLBACK();
 *       PMU_VDDEXT_OT_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup pmu_api
 */
INLINE void PMU_VDDEXT_OT_Int_Clr(void)   
{
	Field_Mod32(&PMU->VDDEXT_CTRL.reg, (uint8)PMU_VDDEXT_CTRL_VDDEXT_OT_ISC_Pos, PMU_VDDEXT_CTRL_VDDEXT_OT_ISC_Msk, 1u);	
}

/** \brief Clear VDDEXT Supply Undervoltage interrupt flag.
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VDDEXT Supply Undervoltage interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (PMU->VDDEXT_CTRL.bit.VDDEXT_FAIL_EN == 1u) 
 *   { 
 *     if (PMU->VDDEXT_CTRL.bit.VDDEXT_UV_IS == 1u)
 *     {
 *       PMU_VDDEXT_UV_CALLBACK();
 *       PMU_VDDEXT_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup pmu_api
 */
INLINE void PMU_VDDEXT_UV_Int_Clr(void)   
{
	Field_Mod32(&PMU->VDDEXT_CTRL.reg, (uint8)PMU_VDDEXT_CTRL_VDDEXT_UV_ISC_Pos, PMU_VDDEXT_CTRL_VDDEXT_UV_ISC_Msk, 1u);	
}

/** \brief Read out the Reset Status register
 *
 * \return combination of reset status bits
 *
 * \brief <b>Example</b><br>
 * \brief This example checks and clears Reset Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (PMU_Get_Reset_Status() != (uint16)0)
 *   {
 *     PMU_Clear_Reset_Status();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup pmu_api
 */
INLINE uint32 PMU_Get_Reset_Status(void)
{
  return ((uint32) PMU->RESET_STS.reg);
}

/** \brief Clear the Reset Status register
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example checks and clears Reset Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (PMU_Get_Reset_Status() != (uint16)0)
 *   {
 *     PMU_Clear_Reset_Status();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup pmu_api
 */
INLINE void PMU_Clear_Reset_Status(void)
{
  PMU->RESET_STS.reg = (uint32) 0;
}

/** \brief Turn VDDEXT on.
 *
 * \return status of VDDEXT
 *
 * \brief <b>Example</b><br> 
 * \brief This example switches VDDEXT on and waits until VDDEXT is in stable condition.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   while (PMU_VDDEXT_On() == (uint8)0)
 *   {
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup pmu_api
 */
INLINE uint8 PMU_VDDEXT_On(void)
{
  PMU->VDDEXT_CTRL.bit.VDDEXT_ENABLE = 1u;
  return ((uint8) PMU->VDDEXT_CTRL.bit.VDDEXT_STABLE);
}

/** \brief Switch off (disables) VDDEXT and return the stable state of VDDEXT (VDDEXT Supply works inside its specified range)
 *
 * \return status of VDDEXT
 *
 * \brief <b>Example</b><br> 
 * \brief This example switches VDDEXT off and waits until VDDEXT is in stable condition.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   while (PMU_VDDEXT_Off() == (uint8)0)
 *   {
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup pmu_api
 */
INLINE uint8 PMU_VDDEXT_Off(void)
{
  PMU->VDDEXT_CTRL.bit.VDDEXT_ENABLE = 0u;
  return ((uint8) PMU->VDDEXT_CTRL.bit.VDDEXT_STABLE);
}

/** \brief Clear VDDEXT Error Flags
 *
 * \param ErrorFlagMask VDDEXT Error Flags Mask
 *
 * \brief <b>Example</b><br>
 * \brief This example clears VDDEXT Supply Undervoltage Interrupt flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   PMU_VDDEXT_Error_Clear(0x1000);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup pmu_api
 */
INLINE void PMU_VDDEXT_Error_Clear(uint16 ErrorFlagMask)
{
  PMU->VDDEXT_CTRL.reg &= (uint16) ~ ErrorFlagMask;
}

/** \brief Set VDDEXT Bit
 *
 * \param FlagMask VDDEXT Flags Mask
 *
 * \brief <b>Example</b><br>
 * \brief This example enables VDDEXT Supply for Cyclic Sense.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   PMU_VDDEXT_Set(2u);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup pmu_api
 */
INLINE void PMU_VDDEXT_Set(uint16 FlagMask)
{
  PMU->VDDEXT_CTRL.reg = (uint16) FlagMask;
}

#endif
