/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     scu.h
 *
 * \brief    SCU low level access library
 *
 * \version  V0.4.6
 * \date     11. Jul 2022
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** TS           T&S                                                           **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
** VO           Vanessa Ongaro                                                **
** PS           Patrik Schwarz                                                **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version, ported from V0.5.1 (Proteus SDK)**
** V0.1.1: 2015-11-27, AV:   Include ssc_defines.h (def. SCU_MODPISEL)        **
** V0.1.2: 2017-06-19, KC:   global variable SystemFrequency added            **
**                           global interrupt disabled during SCU_ClkInit()   **
** V0.3.0: 2017-10-20, KC:   Removed conditional compilation for              **
**                           SCU_EnterSleepMode() and SCU_EnterStopMode()     **
**                           Version jump to be consistent with pdsc-file     **
** V0.3.1: 2017-12-06, KC:   Dummy read (GPIO_WAKE_STATUS) fixes problems     **
**                           when entering sleep mode                         **
**                           For XTAL enabling MODPISEL1 is set in SCU_ClkInit**
** V0.3.2: 2017-12-08, KC:   SCU_EnterSleepMode(): LIN functions removed and  **
**                           dummy read of CNF_WAKE_FILTER added              **
**                           Include of lin.h removed                         **
**                           Variable type of "dummy" changed to 32bit        **
** V0.3.3: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.3.4: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           SCU_EnterStopMode and SCU_EnterSleepMode         **
**                           functions updated                                **
**                           CMSIS Intrinsic functions updated                **
** V0.3.5: 2019-04-23, JO:   Changed return value of function                 **
**                           SCU_ChangeNVMProtection from int32_t to sint32_t **
**                           to match the return value of BootROM functions   **
**                           Reworked SCU_EnterSlowMode and added parameter   **
**                           for prescaler                                    **
** V0.3.6: 2020-12-04, JO:   EP-636: Corrected switching to LP_CLK in         **
**                           functions SCU_ClkInit and SCU_EnterStopMode      **
** V0.3.7: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to scu.c                                         **
**                           Changed from anonymous to named typedefs to      **
**                           prevent MISRA warning                            **
** V0.3.8: 2021-08-24, VO:   EP-826: Added clearing of GPIO_WAKE_STS when     **
**                           entering stop mode                               **
** V0.3.9: 2021-08-25, VO:   EP-823: Updated sequence to enter stop mode      **
** V0.4.0: 2021-10-07, BG:   EP-931: Corrected ARMCC compiler v5/v6 warnings  **
**                           for SCU_EnterSleepMode and SCU_EnterStopMode     **
** V0.4.1: 2021-10-14, BG:   EP-915: Updated LIN -> TRX                       **
** V0.4.2: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.4.3: 2021-12-21, BG:   EP-997: Corrected MISRA 2012 violations          **
** V0.4.4: 2022-03-09, PS:   EP-399: Fixed Wrong OT_STS after wake from Stop  **
**                           mode without reset, added proper event clearing  **
** V0.4.5: 2022-04-06, VO:   EP-1085: Added corrections to avoid MISRA        **
**                           violations                                       **
** V0.4.6: 2022-07-11, VO:   EP-1210: Added '__attribute__((noreturn))' to    **
**                           definition of SCU_EnterSleepMode(void)           **
*******************************************************************************/


#ifndef SCU_H
#define SCU_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"
#include "bootrom.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief  PASSWD Phrases, PASSWD Opened */
#define PASSWD_Open       (0x98U)
/**\brief  PASSWD Phrases, PASSWD Closed */
#define PASSWD_Close      (0xA8U)

/* NVM Protection actions */
/**\brief  NVM Protection actions, CLEAR */
#define PROTECTION_CLEAR  (1U)
/**\brief  NVM Protection actions, SET */
#define PROTECTION_SET    (0U)

/* Slow Down mode prescaler */
#define SLOWDOWN_PRESCALER_1     (0U)
#define SLOWDOWN_PRESCALER_2     (1U)
#define SLOWDOWN_PRESCALER_3     (2U)
#define SLOWDOWN_PRESCALER_4     (3U)
#define SLOWDOWN_PRESCALER_8     (4U)
#define SLOWDOWN_PRESCALER_16    (5U)
#define SLOWDOWN_PRESCALER_24    (6U)
#define SLOWDOWN_PRESCALER_32    (7U)
#define SLOWDOWN_PRESCALER_48    (8U)
#define SLOWDOWN_PRESCALER_64    (9U)
#define SLOWDOWN_PRESCALER_96   (10U)
#define SLOWDOWN_PRESCALER_128  (11U)
#define SLOWDOWN_PRESCALER_192  (12U)
#define SLOWDOWN_PRESCALER_256  (13U)
#define SLOWDOWN_PRESCALER_384  (14U)
#define SLOWDOWN_PRESCALER_512  (15U)

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \enum TScu_Mod
 *  \brief This enum lists the SCU Modules Configuration. 
 */
typedef enum _TScu_Mod
{
  Mod_ADC1 = 0x00,    /**< SCU Modules Select: ADC1 */
  Mod_SSC1 = 0x01,    /**< SCU Modules Select: SSC1 */
  Mod_CCU6 = 0x02,    /**< SCU Modules Select: CCU6 */
  Mod_Timer2 = 0x03,  /**< SCU Modules Select: Timer2 */
  Mod_GPT12 = 0x04,   /**< SCU Modules Select: GPT12 */
  Mod_SSC2 = 0x08,    /**< SCU Modules Select: SSC2 */
  Mod_Timer21 = 0x0A  /**< SCU Modules Select: Timer21 */
} TScu_Mod;

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void SCU_ClkInit(void);
void SCU_Init(void);
void SCU_EnterSleepMode(void);
void SCU_EnterStopMode(void);
void SCU_EnterSlowMode(uint8 divider_scaled);
void SCU_ExitSlowMode(void);
sint32 SCU_ChangeNVMProtection(NVM_PASSWORD_SEGMENT_t Protection, uint8 Action);
INLINE void SCU_Disable_Module(TScu_Mod Module);
INLINE void SCU_Enable_Module(TScu_Mod Module);
INLINE void SCU_OpenPASSWD(void);
INLINE void SCU_ClosePASSWD(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Disable a given peripheral module
 *
 * \param Module SCU Modules \link TScu_Mod \endlink
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC1 Module.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_Disable_Module(Mod_ADC1);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup scu_api
 */ 
INLINE void SCU_Disable_Module(TScu_Mod Module)
{
  Field_Mod32(&SCU->PMCON.reg, (uint8)Module, (uint32)1 << (uint8)Module, 1u);	
}

/** \brief Enable a given peripheral module
 *
 * \param Module SCU Modules \link TScu_Mod \endlink
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Module.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_Enable_Module(Mod_ADC1);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup scu_api
 */ 
INLINE void SCU_Enable_Module(TScu_Mod Module)
{
  Field_Mod32(&SCU->PMCON.reg, (uint8)Module, (uint32)1 << (uint8)Module, 0u);
}

/** \brief Open the PASSWD Register protection scheme for 32 tCLK.
 *
 * \return None
 *
 * \brief <b>Example</b><br> 
 * \brief This example opens the PASSWD Register protection scheme.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_OpenPASSWD();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup scu_api
 */ 
INLINE void SCU_OpenPASSWD(void)
{
  SCU->PASSWD.reg = (uint8) PASSWD_Open;
}

/** \brief Close the PASSWD Register protection scheme.
 *
 * \return None
 *
 * \brief <b>Example</b><br> 
 * \brief This example closes the PASSWD Register protection scheme.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_ClosePASSWD();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup scu_api
 */ 
INLINE void SCU_ClosePASSWD(void)
{
  SCU->PASSWD.reg = (uint8) PASSWD_Close;
}

#endif
