/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     ssc.h
 *
 * \brief    High-Speed Synchronous Serial Interface low level access library
 *
 * \version  V0.1.7
 * \date     13. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version, ported from V0.1.1 (Proteus SDK)**
** V0.1.1: 2015-11-21, AV:   Bugfix in Init routines                          **
** V0.1.2: 2017-06-19, KC:   SendWord functions return received data word     **
** V0.1.3: 2017-10-20, KC:   Macros for interupt handling added               **
** V0.1.4: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.5: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
** V0.1.6: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to ssc.c                                         **
** V0.1.7: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
*******************************************************************************/

#ifndef SSC_H
#define SSC_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief SSC1 Manual Baudrate*/
#define SSC1_tBit_us (1.0 / (SSC1_MAN_BAUDRATE / 1000.0))
/**\brief SSC2 Manual Baudrate*/
#define SSC2_tBit_us (1.0 / (SSC2_MAN_BAUDRATE / 1000.0))

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void SSC1_Init(void);
void SSC2_Init(void);
INLINE void SSC1_TX_Int_Clr(void);
INLINE void SSC1_RX_Int_Clr(void);
INLINE void SSC1_ERR_Int_Clr(void);
INLINE void SSC2_TX_Int_Clr(void);
INLINE void SSC2_RX_Int_Clr(void);
INLINE void SSC2_ERR_Int_Clr(void);
INLINE void SSC1_TX_Int_En(void);
INLINE void SSC1_TX_Int_Dis(void);
INLINE void SSC1_RX_Int_En(void);
INLINE void SSC1_RX_Int_Dis(void);
INLINE void SSC1_ERR_Int_En(void);
INLINE void SSC1_ERR_Int_Dis(void);
INLINE void SSC2_TX_Int_En(void);
INLINE void SSC2_TX_Int_Dis(void);
INLINE void SSC2_RX_Int_En(void);
INLINE void SSC2_RX_Int_Dis(void);
INLINE void SSC2_ERR_Int_En(void);
INLINE void SSC2_ERR_Int_Dis(void);
INLINE uint16 SSC1_SendWord(uint16 DataWord);
INLINE uint16 SSC1_ReadWord(void);
INLINE uint16 SSC2_SendWord(uint16 DataWord);
INLINE uint16 SSC2_ReadWord(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Clear transmit interrupt flag for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the transmit interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_TX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.TIR1 == (uint8)1)
 *   {
 *     SSC1_TX_CALLBACK();
 *     SSC1_TX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_TX_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON2CLR.reg, (uint8)SCU_IRCON2CLR_TIR1C_Pos, SCU_IRCON2CLR_TIR1C_Msk, 1u);
}

/** \brief Clear receive interrupt flag for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_RX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.RIR1 == (uint8)1)
 *   {
 *     SSC1_RX_CALLBACK();
 *     SSC1_RX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_RX_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON2CLR.reg, (uint8)SCU_IRCON2CLR_RIR1C_Pos, SCU_IRCON2CLR_RIR1C_Msk, 1u);
}

/** \brief Clear error interrupt flag for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_ERR_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.EIR1 == (uint8)1)
 *   {
 *     SSC1_ERR_CALLBACK();
 *     SSC1_ERR_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_ERR_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON2CLR.reg, (uint8)SCU_IRCON2CLR_EIR1C_Pos, SCU_IRCON2CLR_EIR1C_Msk, 1u);
}

/** \brief Clear transmit interrupt flag for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats thd transmit interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_TX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.TIR2 == (uint8)1)
 *   {
 *     SSC2_TX_CALLBACK();
 *     SSC2_TX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_TX_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON3CLR.reg, (uint8)SCU_IRCON3CLR_TIR2C_Pos, SCU_IRCON3CLR_TIR2C_Msk, 1u);
}

/** \brief Clear receive interrupt flag for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_RX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.RIR2 == (uint8)1)
 *   {
 *     SSC2_RX_CALLBACK();
 *     SSC2_RX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_RX_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON3CLR.reg, (uint8)SCU_IRCON3CLR_RIR2C_Pos, SCU_IRCON3CLR_RIR2C_Msk, 1u);
}

/** \brief Clear error interrupt flag for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_ERR_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.EIR2 == (uint8)1)
 *   {
 *     SSC2_ERR_CALLBACK();
 *     SSC2_ERR_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_ERR_Int_Clr(void)
{
  Field_Wrt32(&SCU->IRCON3CLR.reg, (uint8)SCU_IRCON3CLR_EIR2C_Pos, SCU_IRCON3CLR_EIR2C_Msk, 1u);
}

/** \brief Enable transmit interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats thd transmit interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_TX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.TIR1 == (uint8)1)
 *   {
 *     SSC1_TX_CALLBACK();
 *     SSC1_TX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_TX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_TIREN1_Pos, SCU_MODIEN1_TIREN1_Msk, 1u);
}

/** \brief Disable transmit interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats thd transmit interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_TX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.TIR1 == (uint8)1)
 *   {
 *     SSC1_TX_CALLBACK();
 *     SSC1_TX_Int_Clr();
 *   }
 *   SSC1_TX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_TX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_TIREN1_Pos, SCU_MODIEN1_TIREN1_Msk, 0u);
}

/** \brief Enable receive interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_RX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.RIR1 == (uint8)1)
 *   {
 *     SSC1_RX_CALLBACK();
 *     SSC1_RX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ssc_api
 */
INLINE void SSC1_RX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_RIREN1_Pos, SCU_MODIEN1_RIREN1_Msk, 1u);
}

/** \brief Disable receive interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_RX_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.RIR1 == (uint8)1)
 *   {
 *     SSC1_RX_CALLBACK();
 *     SSC1_RX_Int_Clr();
 *   }
 *   SSC1_RX_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ssc_api
 */
INLINE void SSC1_RX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_RIREN1_Pos, SCU_MODIEN1_RIREN1_Msk, 0u);
}

/** \brief Enable error interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_ERR_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.EIR1 == (uint8)1)
 *   {
 *     SSC1_ERR_CALLBACK();
 *     SSC1_ERR_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_ERR_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_EIREN1_Pos, SCU_MODIEN1_EIREN1_Msk, 1u);
}

/** \brief Disable error interrupt for SSC1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC1_ERR_Int_En();
 *   if ((uint8)SCU->IRCON2.bit.EIR1 == (uint8)1)
 *   {
 *     SSC1_ERR_CALLBACK();
 *     SSC1_ERR_Int_Clr();
 *   }
 *   SSC1_ERR_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC1_ERR_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_EIREN1_Pos, SCU_MODIEN1_EIREN1_Msk, 0u);
}

/** \brief Enable transmit interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats thd transmit interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_TX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.TIR2 == (uint8)1)
 *   {
 *     SSC2_TX_CALLBACK();
 *     SSC2_TX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_TX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_TIREN2_Pos, SCU_MODIEN1_TIREN2_Msk, 1u);
}

/** \brief Disable transmit interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats thd transmit interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_TX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.TIR2 == (uint8)1)
 *   {
 *     SSC2_TX_CALLBACK();
 *     SSC2_TX_Int_Clr();
 *   }
 *   SSC2_TX_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_TX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_TIREN2_Pos, SCU_MODIEN1_TIREN2_Msk, 0u);
}

/** \brief Enable receive interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_RX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.RIR2 == (uint8)1)
 *   {
 *     SSC2_RX_CALLBACK();
 *     SSC2_RX_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_RX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_RIREN2_Pos, SCU_MODIEN1_RIREN2_Msk, 1u);
}

/** \brief Disable receive interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_RX_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.RIR2 == (uint8)1)
 *   {
 *     SSC2_RX_CALLBACK();
 *     SSC2_RX_Int_Clr();
 *   }
 *   SSC2_RX_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_RX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_RIREN2_Pos, SCU_MODIEN1_RIREN2_Msk, 0u);
}

/** \brief Enable error interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_ERR_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.EIR2 == (uint8)1)
 *   {
 *     SSC2_ERR_CALLBACK();
 *     SSC2_ERR_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_ERR_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_EIREN2_Pos, SCU_MODIEN1_EIREN2_Msk, 1u);
}

/** \brief Disable error interrupt for SSC2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the error interrupt for SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SSC2_ERR_Int_En();
 *   if ((uint8)SCU->IRCON3.bit.EIR2 == (uint8)1)
 *   {
 *     SSC2_ERR_CALLBACK();
 *     SSC2_ERR_Int_Clr();
 *   }
 *   SSC2_ERR_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE void SSC2_ERR_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN1.reg, (uint8)SCU_MODIEN1_EIREN2_Pos, SCU_MODIEN1_EIREN2_Msk, 0u);
}

/** \brief Send data word via SSC1.
 *
 * \param DataWord Data to send
 * \return Received data
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" with SSC1 and reads the received data.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 data;
 * 
 *     data = SSC1_SendWord(0x41); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE uint16 SSC1_SendWord(uint16 DataWord)
{
  SSC1->TB.reg = DataWord;
  return( (uint16)SSC1->RB.reg);
}

/** \brief Read data word from SSC1 receive buffer.
 *
 * \return Received data
 *
 * \brief <b>Example</b><br>
 * \brief This example receives data with SSC1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 data;
 * 
 *     data = SSC1_ReadWord(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ssc_api
 */
INLINE uint16 SSC1_ReadWord(void)
{
  return  (uint16)SSC1->RB.reg;
}

/** \brief Send data word via SSC2.
 *
 * \param DataWord Data to send
 * \return Received data
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" with SSC2 and reads the received data.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 data;
 * 
 *     data = SSC2_SendWord(0x41); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ssc_api
 */
INLINE uint16 SSC2_SendWord(uint16 DataWord)
{
  SSC2->TB.reg = DataWord;
  return( (uint16)SSC2->RB.reg);
}

/** \brief Read data word from SSC2 receive buffer.
 *
 * \return Received data
 *
 * \brief <b>Example</b><br>
 * \brief This example receives data with SSC2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 data;
 * 
 *     data = SSC2_ReadWord(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ssc_api
 */
INLINE uint16 SSC2_ReadWord(void)
{
  return  (uint16)SSC2->RB.reg;
}

#endif
