/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     timer2x.h
 *
 * \brief    Timer2/Timer21 low level access library
 *
 * \version  V0.1.9
 * \date     21. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version, ported from V0.1.2 (Proteus SDK)**
** V0.1.1: 2015-11-09, AV:   corrected T2->T20,                               **
**                           renamed T20->TIMER2 and T21->TIMER21             **
**                           renamed TIMER2_T2[A-Z]->TIMER2_[A-Z]             **
**                           renamed TIMER21_T2[A-Z]->TIMER21_[A-Z]           **
**                           renamed Timer2->TIMER2                           **
** V0.1.3: 2017-10-19, KC:   macros for TIMER2x bit accesses added            **
**                           Set_Reload function added                        **
**                           API extended                                     **
** V0.1.4: 2018-11-05, CE:   TIMER2_Value_Get - opening bracket added in macro**
**                           lines 76-91: position of inverting sign corrected**
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.5: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           TIMER2_Clk_Prescaler_Dis function updated        **
** V0.1.6: 2019-03-29, JO:   corerrected TIMER2_Interval_Timer_Setup          **
**                           changed order of functions in timer2x.h          **
** V0.1.7: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Moved DoxyGen documentation for global functions **
**                           to timer2x.c                                     **
**                           Changed from anonymous to named typedefs to      **
**                           prevent MISRA warning                            **
** V0.1.8: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.1.9: 2021-12-21, BG:   EP-997: Corrected MISRA 2012 violations          **
*******************************************************************************/

#ifndef TIMER2X_H
#define TIMER2X_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \enum TIMER2x_Clock_Prescaler
 *  \brief This enum lists TIMER2x Clock Prescaler Configuration. 
 */
typedef enum _TIMER2x_Clock_Prescaler
{
  TIMER2x_Clk_Div_1   = 0u, /**< Timer2x Input Clock Select: fsys */
  TIMER2x_Clk_Div_2   = 1u, /**< Timer2x Input Clock Select: fsys/2 */
  TIMER2x_Clk_Div_4   = 2u, /**< Timer2x Input Clock Select: fsys/4 */
  TIMER2x_Clk_Div_8   = 3u, /**< Timer2x Input Clock Select: fsys/8 */  
  TIMER2x_Clk_Div_16  = 4u, /**< Timer2x Input Clock Select: fsys/16 */
  TIMER2x_Clk_Div_32  = 5u, /**< Timer2x Input Clock Select: fsys/32 */
  TIMER2x_Clk_Div_64  = 6u, /**< Timer2x Input Clock Select: fsys/64 */
  TIMER2x_Clk_Div_128 = 7u  /**< Timer2x Input Clock Select: fsys/128 */
} TIMER2x_Clock_Prescaler;

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

/* TIMER2 */
void TIMER2_Init(void);
bool TIMER2_Interval_Timer_Setup(uint32 timer_interval_us);
INLINE void TIMER2_Clk_Prescaler_En(void);
INLINE void TIMER2_Clk_Prescaler_Dis(void);
INLINE void TIMER2_Clk_Prescaler_Sel(uint32 t2pre);
INLINE uint8 TIMER2_Clk_Prescaler_Get(void);
INLINE uint8 TIMER2_Overflow_Sts(void);
INLINE uint8 TIMER2_External_Sts(void);
INLINE void TIMER2_External_Ctrl_En(void);
INLINE void TIMER2_External_Ctrl_Dis(void);
INLINE void TIMER2_Reload_Capture_Rising_Sel(void);
INLINE void TIMER2_Reload_Capture_Falling_Sel(void);
INLINE void TIMER2_ExtStart_Rising_Sel(void);
INLINE void TIMER2_ExtStart_Falling_Sel(void);
INLINE void TIMER2_UpDownCount_En(void);
INLINE void TIMER2_UpDownCount_Dis(void);
INLINE void TIMER2_Mode_Capture_Set(void);
INLINE void TIMER2_Mode_Reload_Set(void);
INLINE void TIMER2_Mode_Counter_Set(void);
INLINE void TIMER2_Mode_Timer_Set(void);
INLINE void TIMER2_Overflow_Int_Clr(void);
INLINE void TIMER2_External_Int_Clr(void);
INLINE void TIMER2_Overflow_Int_En(void);
INLINE void TIMER2_Overflow_Int_Dis(void);
INLINE void TIMER2_External_Int_En(void);
INLINE void TIMER2_External_Int_Dis(void);
INLINE uint16 TIMER2_Value_Get(void);
INLINE void TIMER2_Value_Set(uint16 t2);
INLINE void TIMER2_Reload_Value_Set(uint16 t2);
INLINE void TIMER2_Start(void);
INLINE void TIMER2_Stop(void);
INLINE void TIMER2_ExtStart(void);
INLINE void TIMER2_ExtStop(void);
INLINE uint16 TIMER2_Get_Count(void);
INLINE uint16 TIMER2_Get_Capture(void);
INLINE uint16 TIMER2_Capture_Value_Get(void);
INLINE void TIMER2_Clear_Count(void);

/* TIMER21 */
void TIMER21_Init(void);
bool TIMER21_Interval_Timer_Setup(uint32 timer_interval_us);
INLINE void TIMER21_Clk_Prescaler_En(void);
INLINE void TIMER21_Clk_Prescaler_Dis(void);
INLINE void TIMER21_Clk_Prescaler_Sel(uint32 t2pre);
INLINE uint8 TIMER21_Clk_Prescaler_Get(void);
INLINE uint8 TIMER21_Overflow_Sts(void);
INLINE uint8 TIMER21_External_Sts(void);
INLINE void TIMER21_ExternalCtrl_En(void);
INLINE void TIMER21_ExternalCtrl_Dis(void);
INLINE void TIMER21_ReloadCapture_RisingEdge_Set(void);
INLINE void TIMER21_ReloadCapture_FallingEdge_Set(void);
INLINE void TIMER21_ExtStart_RisingEdge_Set(void);
INLINE void TIMER21_ExtStart_FallingEdge_Set(void);
INLINE void TIMER21_UpDownCount_En(void);
INLINE void TIMER21_UpDownCount_Dis(void);
INLINE void TIMER21_Mode_Capture_Set(void);
INLINE void TIMER21_Mode_Reload_Set(void);
INLINE void TIMER21_Mode_Counter_Set(void);
INLINE void TIMER21_Mode_Timer_Set(void);
INLINE void TIMER21_Overflow_Int_Clr(void);
INLINE void TIMER21_External_Int_Clr(void);
INLINE void TIMER21_Overflow_Int_En(void);
INLINE void TIMER21_Overflow_Int_Dis(void);
INLINE void TIMER21_External_Int_En(void);
INLINE void TIMER21_External_Int_Dis(void);
INLINE uint16 TIMER21_Value_Get(void);
INLINE void TIMER21_Value_Set(uint16 t21);
INLINE void TIMER21_Reload_Value_Set(uint16 t21);
INLINE void TIMER21_Start(void);
INLINE void TIMER21_Stop(void);
INLINE void TIMER21_ExtStart(void);
INLINE void TIMER21_ExtStop(void);
INLINE uint16 TIMER21_Get_Count(void);
INLINE uint16 TIMER21_Get_Capture(void);
INLINE uint16 TIMER21_Capture_Value_Get(void);
INLINE void TIMER21_Clear_Count(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/* TIMER2 */
/** \brief Enable TIMER2 clock prescaler.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables TIMER2 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Clk_Prescaler_En();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Clk_Prescaler_En(void )
{
  Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_PREN_Pos, TIMER2_MOD_PREN_Msk, 1u);
}

/** \brief Disable TIMER2 clock prescaler.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables TIMER2 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Clk_Prescaler_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Clk_Prescaler_Dis(void)
{
  Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_PREN_Pos, TIMER2_MOD_PREN_Msk, 0u);
}

/** \brief Set TIMER2 clock prescaler.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER2 clock prescaler to DIV2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Clk_Prescaler_En();
 *     TIMER2_Clk_Prescaler_Sel(1u);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Clk_Prescaler_Sel(uint32 t2pre)
{
  Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_T2PRE_Pos, TIMER2_MOD_T2PRE_Msk, t2pre);
}

/** \brief Read TIMER2 clock prescaler.
 *
 * \return TIMER2 clock prescaler
 *
 * \brief <b>Example</b><br>
 * \brief This example reads TIMER2 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 pre;
 * 
 *     TIMER2_Clk_Prescaler_En();
 *     pre = TIMER2_Clk_Prescaler_Get();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER2_Clk_Prescaler_Get(void)               
{
	return(u8_Field_Rd32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_T2PRE_Pos  , TIMER2_MOD_T2PRE_Msk));
} 

/** \brief Read TIMER2 Overflow Status.
 *
 * \return TIMER2 Overflow Status
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER2 Overflow Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Overflow_Int_En();
 *	   if (TIMER2_Overflow_Sts() == (uint8)1)
 *	   {
 *       TIMER2_TF2_CALLBACK();	 
 *     }
 *     TIMER2_Overflow_Int_Clr();
 *     TIMER2_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER2_Overflow_Sts(void)                    
{
	return(u1_Field_Rd32(&TIMER2->CON.reg, (uint8)TIMER2_CON_TF2_Pos    , TIMER2_CON_TF2_Msk));
} 

/** \brief Read TIMER2 External Status.
 *
 * \return TIMER2 External Status
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER2 External Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Ctrl_En();
 *     if (TIMER2_External_Sts() == (uint8)1)
 *	   {
 *       TIMER2_EXF2_CALLBACK();	 
 *     }
 *     TIMER2_External_Ctrl_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER2_External_Sts(void)                    
{
	return(u1_Field_Rd32(&TIMER2->CON.reg, (uint8)TIMER2_CON_EXF2_Pos   , TIMER2_CON_EXF2_Msk));
} 

/** \brief Enable TIMER2 External Control.
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER2 External Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Int_En();	 
 *     TIMER2_External_Ctrl_En();
 *     if (TIMER2_External_Sts() == (uint8)1)
 *	   {
 *       TIMER2_EXF2_CALLBACK();	 
 *     }
 *     TIMER2_External_Int_Clr();
 *     TIMER2_External_Ctrl_Dis(); 
 *     TIMER2_External_Int_Dis(); 
 * } 
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_External_Ctrl_En(void )
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_EXEN2_Pos, TIMER2_CON_EXEN2_Msk, 1u);
}

/** \brief Disable TIMER2 External Control.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads TIMER2 External Status while External control is disabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 * 
 *     TIMER2_External_Ctrl_En();
 *     sts = TIMER2_External_Sts();
 *     TIMER2_External_Ctrl_Dis();  
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_External_Ctrl_Dis(void )
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_EXEN2_Pos, TIMER2_CON_EXEN2_Msk, 0u);
}

/** \brief Set Rising Edge to Capture/Reload Mode for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to Capture/Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Reload_Capture_Rising_Sel();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Reload_Capture_Rising_Sel(void)    
{
	Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_EDGESEL_Pos, TIMER2_MOD_EDGESEL_Msk, 1u);
} 

/** \brief Set Falling Edge to Capture/Reload Mode for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Falling Edge to Capture/Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Reload_Capture_Falling_Sel();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */			
INLINE void TIMER2_Reload_Capture_Falling_Sel(void)
{
	Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_EDGESEL_Pos, TIMER2_MOD_EDGESEL_Msk, 0u);
} 

/** \brief Set Rising Edge to External Start for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_ExtStart();	 
 *     TIMER2_ExtStart_Rising_Sel();
 *     TIMER2_ExtStop();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */		
INLINE void TIMER2_ExtStart_Rising_Sel(void)          
{
	Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_T2REGS_Pos , TIMER2_MOD_T2REGS_Msk , 1u);
} 

/** \brief Set Falling Edge to External Start for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Falling Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_ExtStart();	 
 *     TIMER2_ExtStart_Falling_Sel();
 *     TIMER2_ExtStop(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */	
INLINE void TIMER2_ExtStart_Falling_Sel(void)         
{
	Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_T2REGS_Pos , TIMER2_MOD_T2REGS_Msk , 0u);
} 

/** \brief Enable Up/Down Counter for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Up/Down Counter for TIMER2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_UpDownCount_En();	 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */	
INLINE void TIMER2_UpDownCount_En(void)
{
  Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_DCEN_Pos, TIMER2_MOD_DCEN_Msk, 1u);
}

/** \brief Disable Up/Down Counter for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Up Counter once TR2 is set and External Control was disabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Ctrl_Dis();
 *     TIMER2_UpDownCount_Dis();
 *     TIMER2_Start();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_UpDownCount_Dis(void)
{
  Field_Mod32(&TIMER2->MOD.reg, (uint8)TIMER2_MOD_DCEN_Pos, TIMER2_MOD_DCEN_Msk, 0u);
}

/** \brief Select Capture Mode for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enters 16-bit Capture Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Ctrl_En();
 *     TIMER2_Mode_Capture_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Mode_Capture_Set(void)
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_CP_RL2_Pos, TIMER2_CON_CP_RL2_Msk, 1u);
}

/** \brief Select Reload Mode for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enters Auto-Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Mode_Reload_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Mode_Reload_Set(void)
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_CP_RL2_Pos, TIMER2_CON_CP_RL2_Msk, 0u);
}

/** \brief Select Mode Counter for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Mode Counter.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Mode_Counter_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Mode_Counter_Set(void)
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_C_T2_Pos, TIMER2_CON_C_T2_Msk, 1u);
}

/** \brief Select Mode Timer for TIMER2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Mode Timer.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Mode_Timer_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Mode_Timer_Set(void)
{
  Field_Mod32(&TIMER2->CON.reg, (uint8)TIMER2_CON_C_T2_Pos, TIMER2_CON_C_T2_Msk, 0u);
}

/** \brief Clear TIMER2 Overflow Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER2 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Overflow_Int_En();
 *	   if (TIMER2_Overflow_Sts() == 1)
 *	   {
 *       TIMER2_TF2_CALLBACK();
 *       TIMER2_Overflow_Int_Clr();
 *     }
 *     TIMER2_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Overflow_Int_Clr(void)
{
  Field_Wrt32(&TIMER2->ICLR.reg, (uint8)TIMER2_ICLR_TF2CLR_Pos, TIMER2_ICLR_TF2CLR_Msk, 1u);
}

/** \brief Clear TIMER2 External Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER2 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Int_En();	 
 *     if (TIMER2_External_Sts() == 1)
 *	   {
 *       TIMER2_EXF2_CALLBACK();
 *       TIMER2_External_Int_Clr(); 
 *     }
 *     TIMER2_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_External_Int_Clr(void)
{
  Field_Wrt32(&TIMER2->ICLR.reg, (uint8)TIMER2_ICLR_EXF2CLR_Pos, TIMER2_ICLR_EXF2CLR_Msk, 1u);
}

/** \brief Enable TIMER2 Overflow Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER2 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Overflow_Int_En();
 *	   if (TIMER2_Overflow_Sts() == (uint8)1)
 *	   {
 *       TIMER2_TF2_CALLBACK();
 *       TIMER2_Overflow_Int_Clr();
 *     }
 *     TIMER2_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Overflow_Int_En(void)
{
  Field_Mod32(&TIMER2->CON1.reg, (uint8)TIMER2_CON1_TF2EN_Pos, TIMER2_CON1_TF2EN_Msk, 1u);
}

/** \brief Disable TIMER2 Overflow Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER2 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Overflow_Int_En();
 *	   if (TIMER2_Overflow_Sts() == (uint8)1)
 *	   {
 *       TIMER2_TF2_CALLBACK();
 *       TIMER2_Overflow_Int_Clr();
 *     }
 *     TIMER2_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Overflow_Int_Dis(void)
{
  Field_Mod32(&TIMER2->CON1.reg, (uint8)TIMER2_CON1_TF2EN_Pos, TIMER2_CON1_TF2EN_Msk, 0u);
}

/** \brief Enable TIMER2 External Interrupt.
 *
 * \brief <b>Example</b><br> 
 * \brief This example treats the TIMER2 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Int_En();	 
 *     if (TIMER2_External_Sts() == 1)
 *	   {
 *       TIMER2_EXF2_CALLBACK();
 *       TIMER2_External_Int_Clr(); 
 *     }
 *     TIMER2_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_External_Int_En(void)
{
  Field_Mod32(&TIMER2->CON1.reg, (uint8)TIMER2_CON1_EXF2EN_Pos, TIMER2_CON1_EXF2EN_Msk, 1u);
}

/** \brief Disable TIMER2 External Interrupt.
 *
 * \brief <b>Example</b><br> 
 * \brief This example treats the TIMER2 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_External_Int_En();	 
 *     if (TIMER2_External_Sts() == 1)
 *	   {
 *       TIMER2_EXF2_CALLBACK();
 *       TIMER2_External_Int_Clr(); 
 *     }
 *     TIMER2_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_External_Int_Dis(void)
{
  Field_Mod32(&TIMER2->CON1.reg, (uint8)TIMER2_CON1_EXF2EN_Pos, TIMER2_CON1_EXF2EN_Msk, 0u);
}

/** \brief Read the TIMER2 count value
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER2 count value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *
 *     timer_ticks = TIMER2_Value_Get();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup timer2x_api
 */
INLINE uint16 TIMER2_Value_Get(void)
{
	return(u16_Field_Rd32(&TIMER2->CNT.reg, (uint8)0x00, 0xFFFF));	
} 

/** \brief Set TIMER2 Value.
 *
 * \param t2 TIMER2 Value
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER2 Value to 5000 ticks.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Value_Set(0x1388);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup timer2x_api
 */ 
INLINE void TIMER2_Value_Set(uint16 t2)
{
	Field_Wrt32(&TIMER2->CNT.reg, (uint8)TIMER2_CNT_T2H_Pos, TIMER2_CNT_T2H_Msk, ((uint32)t2 >> 8u));																		        
	Field_Mod32(&TIMER2->CNT.reg, (uint8)TIMER2_CNT_T2L_Pos, TIMER2_CNT_T2L_Msk, ((uint32)t2 & (uint32)0xFF));
} 

/** \brief Set TIMER2 Reload Value.
 *
 * \param t2 TIMER2 Reload Value
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER2 Reload Value to 5000.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Mode_Reload_Set();
 *     TIMER2_Reload_Value_Set(0x1388);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup timer2x_api
 */
INLINE void TIMER2_Reload_Value_Set(uint16 t2)
{
	Field_Wrt32(&TIMER2->RC.reg, (uint8)TIMER2_RC_RCH2_Pos, TIMER2_RC_RCH2_Msk, ((uint32)t2 >> 8u));
	Field_Mod32(&TIMER2->RC.reg, (uint8)TIMER2_RC_RCL2_Pos, TIMER2_RC_RCL2_Msk, ((uint32)t2 & (uint32)0xFF));
} 

/** \brief Start the TIMER2 by software
 *
 * \brief <b>Example</b><br>
 * \brief This example starts the TIMER2 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Start(void)
{
  TIMER2->CON.bit.TR2 = 1;
}

/** \brief Stop the TIMER2 by software
 *
 * \brief <b>Example</b><br>
 * \brief This example stops and reset TIMER2 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_Stop();
 *     TIMER2_Clear_Count(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Stop(void)
{
  TIMER2->CON.bit.TR2 = 0;
}

/** \brief Prepare TIMER2 to be started externally
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_ExtStart();	 
 *     TIMER2_ExtStart_Rising_Sel();
 *     TIMER2_ExtStop();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_ExtStart(void)
{
  TIMER2->MOD.bit.T2RHEN = 1;
}

/** \brief Disable TIMER2 to be started externally
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER2_ExtStart();	 
 *     TIMER2_ExtStart_Rising_Sel();
 *     TIMER2_ExtStop();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_ExtStop(void)
{
  TIMER2->MOD.bit.T2RHEN = 0;
}

/** \brief Read the TIMER2 count value
 *
 * \return TIMER2 count value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER2 count value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *     timer_ticks = TIMER2_Get_Count();	 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER2_Get_Count(void)
{
  return (uint16) TIMER2->CNT.reg;
}

/** \brief Read the TIMER2 capture value
 *
 * \return TIMER2 capture value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER2 capture value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *	 
 *     timer_ticks = TIMER2_Get_Capture();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER2_Get_Capture(void)
{
  return (uint16) TIMER2->RC.reg;
}

/** \brief Read the TIMER2 capture value
 *
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER2 capture value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *	 
 *     timer_ticks = TIMER2_Capture_Value_Get();	 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup timer2x_api
 */
INLINE uint16 TIMER2_Capture_Value_Get(void)
{
	return(u16_Field_Rd32(&TIMER2->RC.reg, (uint8)0x00, 0xFFFF));	
} 

/** \brief Clear the TIMER2 count value
 *
 * \brief <b>Example</b><br>
 * \brief This example stops and reset TIMER2 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {	 
 *     TIMER2_Stop();
 *     TIMER2_Clear_Count();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE void TIMER2_Clear_Count(void)
{
  TIMER2->CNT.reg = (uint32) 0;
}

/* TIMER21 */
/** \brief Enable TIMER21 clock prescaler.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables TIMER21 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Clk_Prescaler_En();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Clk_Prescaler_En(void)
{
  Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_PREN_Pos, TIMER21_MOD_PREN_Msk, 1u);
}

/** \brief Disable TIMER21 clock prescaler.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables TIMER21 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Clk_Prescaler_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Clk_Prescaler_Dis(void)
{
  Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_PREN_Pos, TIMER21_MOD_PREN_Msk, 0u);
}

/** \brief Set TIMER21 clock prescaler.
 *
 * \param t2pre TIMER21 clock prescaler
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER21 clock prescaler to DIV2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Clk_Prescaler_En();
 *     TIMER21_Clk_Prescaler_Sel(1u);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Clk_Prescaler_Sel(uint32 t2pre)
{
  Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_T2PRE_Pos, TIMER21_MOD_T2PRE_Msk, t2pre);
}

/** \brief Read TIMER21 clock prescaler.
 *
 * \return TIMER21 clock prescaler
 *
 * \brief <b>Example</b><br>
 * \brief This example reads TIMER21 clock prescaler.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 pre;
 *
 *     TIMER21_Clk_Prescaler_En();
 *     pre = TIMER21_Clk_Prescaler_Get();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER21_Clk_Prescaler_Get(void)              
{
	return(u8_Field_Rd32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_T2PRE_Pos  , TIMER21_MOD_T2PRE_Msk));
} 

/** \brief Read TIMER21 Overflow Status.
 *
 * \return TIMER21 Overflow Status
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER21 Overflow Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Overflow_Int_En();
 *	   if (TIMER21_Overflow_Sts() == (uint8)1)
 *	   {
 *       TIMER21_TF2_CALLBACK();	 
 *     }
 *     TIMER21_Overflow_Int_Clr();
 *     TIMER21_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER21_Overflow_Sts(void)
{
  return u1_Field_Rd32(&TIMER21->CON.reg, (uint8)TIMER21_CON_TF2_Pos, TIMER21_CON_TF2_Msk);
}

/** \brief Read TIMER21 External Status.
 *
 * \return TIMER21 External Status
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER21 External Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExternalCtrl_En();
 *     if (TIMER21_External_Sts() == (uint8)1)
 *	   {
 *       TIMER21_EXF2_CALLBACK();	 
 *     }
 *     TIMER21_ExternalCtrl_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint8 TIMER21_External_Sts(void)
{
  return u1_Field_Rd32(&TIMER21->CON.reg, (uint8)TIMER21_CON_EXF2_Pos, TIMER21_CON_EXF2_Msk);
}

/** \brief Enable TIMER21 External Control.
 *
 * \brief <b>Example</b><br>
 * \brief This example checks the TIMER21 External Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_External_Int_En();	 
 *     TIMER21_ExternalCtrl_En();
 *     if (TIMER21_External_Sts() == (uint8)1)
 *	   {
 *       TIMER21_EXF2_CALLBACK();	 
 *     }
 *     TIMER21_External_Int_Clr();
 *     TIMER21_ExternalCtrl_Dis(); 
 *     TIMER21_External_Int_Dis(); 
 * } 
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_ExternalCtrl_En(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_EXEN2_Pos, TIMER21_CON_EXEN2_Msk, 1u);
}

/** \brief Disable TIMER21 External Control.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads TIMER21 External Status while External control is disabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 * 	 
 *     TIMER21_ExternalCtrl_En();
 *     sts = TIMER21_External_Sts();
 *     TIMER21_ExternalCtrl_Dis();  
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_ExternalCtrl_Dis(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_EXEN2_Pos, TIMER21_CON_EXEN2_Msk, 0u);
}

/** \brief Set Rising Edge to Capture/Reload Mode for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to Capture/Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ReloadCapture_RisingEdge_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_ReloadCapture_RisingEdge_Set(void)																	        
{
	Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_EDGESEL_Pos, TIMER21_MOD_EDGESEL_Msk, 1u);
}

/** \brief Set Falling Edge to Capture/Reload Mode for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Falling Edge to Capture/Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ReloadCapture_FallingEdge_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */			
INLINE void TIMER21_ReloadCapture_FallingEdge_Set(void)																			        
{
	Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_EDGESEL_Pos, TIMER21_MOD_EDGESEL_Msk, 0u);
} 

/** \brief Set Rising Edge to External Start for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExtStart_RisingEdge_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */			
INLINE void TIMER21_ExtStart_RisingEdge_Set(void)          
{
	Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_T2REGS_Pos , TIMER21_MOD_T2REGS_Msk , 1u);
} 

/** \brief Set Falling Edge to External Start for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Falling Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExtStart_FallingEdge_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */	
INLINE void TIMER21_ExtStart_FallingEdge_Set(void)         
{
	Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_T2REGS_Pos , TIMER21_MOD_T2REGS_Msk , 0u);
} 

/** \brief Enable Up/Down Counter for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Up/Down Counter for TIMER21.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_UpDownCount_En(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_UpDownCount_En(void)
{
  Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_DCEN_Pos, TIMER21_MOD_DCEN_Msk, 1u);
}

/** \brief Disable Up/Down Counter for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts Up Counter once TR2 is set and External Control was disabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExternalCtrl_Dis();
 *     TIMER21_UpDownCount_Dis();
 *     TIMER21_Start();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_UpDownCount_Dis(void)
{
  Field_Mod32(&TIMER21->MOD.reg, (uint8)TIMER21_MOD_DCEN_Pos, TIMER21_MOD_DCEN_Msk, 0u);
}

/** \brief Select Capture Mode for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example enters 16-bit Capture Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExternalCtrl_En();
 *     TIMER21_Mode_Capture_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Mode_Capture_Set(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_CP_RL2_Pos, TIMER21_CON_CP_RL2_Msk, 1u);
}

/** \brief Select Reload Mode for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example enters Auto-Reload Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Mode_Reload_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Mode_Reload_Set(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_CP_RL2_Pos, TIMER21_CON_CP_RL2_Msk, 0u);
}

/** \brief Select Mode Counter for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Mode Counter.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Mode_Counter_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Mode_Counter_Set(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_C_T2_Pos, TIMER21_CON_C_T2_Msk, 1u);
}

/** \brief Select Mode Timer for TIMER21.
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Mode Timer.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Mode_Timer_Set();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Mode_Timer_Set(void)
{
  Field_Mod32(&TIMER21->CON.reg, (uint8)TIMER21_CON_C_T2_Pos, TIMER21_CON_C_T2_Msk, 0u);
}

/** \brief Clear TIMER21 Overflow Interrupt status.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Overflow_Int_En();
 *	   if (TIMER21_Overflow_Sts() == 1)
 *	   {
 *       TIMER21_TF2_CALLBACK();
 *       TIMER21_Overflow_Int_Clr();
 *     }
 *     TIMER21_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Overflow_Int_Clr(void)
{
  Field_Wrt32(&TIMER21->ICLR.reg, (uint8)TIMER21_ICLR_TF2CLR_Pos, TIMER21_ICLR_TF2CLR_Msk, 1u);
}

/** \brief Clear TIMER21 External Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_External_Int_En();	 
 *     if (TIMER21_External_Sts() == 1)
 *	   {
 *       TIMER21_EXF2_CALLBACK();
 *       TIMER21_External_Int_Clr(); 
 *     }
 *     TIMER21_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_External_Int_Clr(void)
{
  Field_Wrt32(&TIMER21->ICLR.reg, (uint8)TIMER21_ICLR_EXF2CLR_Pos, TIMER21_ICLR_EXF2CLR_Msk, 1u);
}

/** \brief Enable TIMER21 Overflow Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Overflow_Int_En();
 *	   if (TIMER21_Overflow_Sts() == 1)
 *	   {
 *       TIMER21_TF2_CALLBACK();
 *       TIMER21_Overflow_Int_Clr();
 *     }
 *     TIMER21_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Overflow_Int_En(void)
{
  Field_Mod32(&TIMER21->CON1.reg, (uint8)TIMER21_CON1_TF2EN_Pos, TIMER21_CON1_TF2EN_Msk, 1u);
}

/** \brief Disable TIMER21 Overflow Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 Overflow Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Overflow_Int_En();
 *	   if (TIMER21_Overflow_Sts() == 1)
 *	   {
 *       TIMER21_TF2_CALLBACK();
 *       TIMER21_Overflow_Int_Clr();
 *     }
 *     TIMER21_Overflow_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Overflow_Int_Dis(void)
{
  Field_Mod32(&TIMER21->CON1.reg, (uint8)TIMER21_CON1_TF2EN_Pos, TIMER21_CON1_TF2EN_Msk, 0u);
}

/** \brief Enable TIMER21 External Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_External_Int_En();	 
 *     if (TIMER21_External_Sts() == 1)
 *	   {
 *       TIMER21_EXF2_CALLBACK();
 *       TIMER21_External_Int_Clr(); 
 *     }
 *     TIMER21_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_External_Int_En(void)
{
  Field_Mod32(&TIMER21->CON1.reg, (uint8)TIMER21_CON1_EXF2EN_Pos, TIMER21_CON1_EXF2EN_Msk, 1u);
}


/** \brief Disable TIMER21 External Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TIMER21 External Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_External_Int_En();	 
 *     if (TIMER21_External_Sts() == 1)
 *	   {
 *       TIMER21_EXF2_CALLBACK();
 *       TIMER21_External_Int_Clr(); 
 *     }
 *     TIMER21_External_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_External_Int_Dis(void)
{
  Field_Mod32(&TIMER21->CON1.reg, (uint8)TIMER21_CON1_EXF2EN_Pos, TIMER21_CON1_EXF2EN_Msk, 0u);
}


/** \brief Read the TIMER21 count value
 *
 * \return TIMER21 count value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER21 count value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *
 *     timer_ticks = TIMER21_Value_Get();	 
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER21_Value_Get(void)
{
	return(u16_Field_Rd32(&TIMER21->CNT.reg, (uint8)0x00, 0xFFFF));	
} 

/** \brief Set TIMER21 Value.
 *
 * \param t21 TIMER21 Value
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER21 Value to 5000.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Value_Set(ox1388);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Value_Set(uint16 t21)
{
	Field_Wrt32(&TIMER21->CNT.reg  , (uint8)TIMER21_CNT_T2H_Pos      , TIMER21_CNT_T2H_Msk, ((uint32)t21 >> 8u));																		        
	Field_Mod32(&TIMER21->CNT.reg  , (uint8)TIMER21_CNT_T2L_Pos      , TIMER21_CNT_T2L_Msk, ((uint32)t21 & (uint32)0xFF));
}

/** \brief Set TIMER21 Reload Value.
 *
 * \param t21 TIMER21 Reload Value
 * \return None
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TIMER21 Reload Value to 5000.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Mode_Reload_Set();
 *     TIMER21_Reload_Value_Set(0x1388);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */                                         
INLINE void TIMER21_Reload_Value_Set(uint16 t21)
{
	Field_Wrt32(&TIMER21->RC.reg , (uint8)TIMER21_RC_RCH2_Pos     , TIMER21_RC_RCH2_Msk, ((uint32)t21 >> 8u));
	Field_Mod32(&TIMER21->RC.reg , (uint8)TIMER21_RC_RCL2_Pos     , TIMER21_RC_RCL2_Msk, ((uint32)t21 & (uint32)0xFF));
}

/** \brief Start the TIMER21 by software
 *
 * \brief <b>Example</b><br>
 * \brief This example starts the TIMER21 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Start();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Start(void)
{
  TIMER21->CON.bit.TR2 = 1;
}

/** \brief Stop the TIMER21 by software
 *
 * \brief <b>Example</b><br>
 * \brief This example stops and reset TIMER21 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_Stop();
 *     TIMER21_Clear_Count(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Stop(void)
{
  TIMER21->CON.bit.TR2 = 0;
}

/** \brief Prepare TIMER21 to be started externally
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExtStart();	 
 *     TIMER21_ExtStart_RisingEdge_Set();
 *     TIMER21_ExtStop();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_ExtStart(void)
{
  TIMER21->MOD.bit.T2RHEN = 1;
}

/** \brief Disable TIMER21 to be started externally
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Rising Edge to External Start.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TIMER21_ExtStart();	 
 *     TIMER21_ExtStart_RisingEdge_Set();
 *     TIMER21_ExtStop();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_ExtStop(void)
{
  TIMER21->MOD.bit.T2RHEN = 0;
}

/** \brief Read the TIMER21 count value
 *
 * \return TIMER21 count value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER21 count value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *
 *     timer_ticks = TIMER21_Get_Count();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER21_Get_Count(void)
{
  return ((uint16) TIMER21->CNT.reg );
}

/** \brief Read the TIMER21 capture value
 *
 * \return TIMER21 capture value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER21 capture value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *	 
 *     timer_ticks = TIMER21_Get_Capture();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER21_Get_Capture(void)
{
  return (uint16) TIMER21->RC.reg;
}

/** \brief Read the TIMER21 capture value
 *
 * \return TIMER21 capture value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the TIMER21 capture value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 timer_ticks;
 *	 
 *     timer_ticks = TIMER21_Capture_Value_Get();	 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup TIMER2x_api
 */
INLINE uint16 TIMER21_Capture_Value_Get(void)
{
	return(u16_Field_Rd32(&TIMER21->RC.reg, (uint8)0x00, 0xFFFF));	
}

/** \brief Clear the TIMER21 count value
 *
 * \brief <b>Example</b><br>
 * \brief This example stops and reset TIMER21 by software.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {	 
 *     TIMER21_Stop();
 *     TIMER21_Clear_Count();  
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup TIMER2x_api
 */
INLINE void TIMER21_Clear_Count(void)
{
  TIMER21->CNT.reg = (uint32) 0;
}

#endif
