/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     trx.h
 *
 * \brief    TRX low level access library
 *
 * \version  V0.2.0
 * \date     21. Dec 2021
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** TS           T&S                                                           **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version                                  **
** V0.1.1: 2015-11-30, AV:   Extended init routine                            **
** V0.1.2: 2017-01-13, DM:   Mode initialization corrected                    **
** V0.1.3: 2017-06-19, KC:   Register added in initialization                 **
** V0.1.4: 2017-10-20, KC:   Macros for interrupt register access added       **
** V0.1.5: 2018-11-28, DM:   Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.6: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
** V0.1.7: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to lin.c                                         **
** V0.1.8: 2021-10-14, BG:   EP-915: Updated LIN -> TRX                       **
** V0.1.9: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.2.0: 2021-12-21, BG:   EP-997: Corrected MISRA 2012 violations          **
*******************************************************************************/

#ifndef TRX_H
#define TRX_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief TRX MODE, SLEEP MODE */
#define TRX_MODE_SLEEP    (0u)
/**\brief TRX MODE, RCV ONLY MODE */
#define TRX_MODE_RCV_ONLY (1u)
/**\brief TRX MODE, NORMAL MODE */
#define TRX_MODE_NORMAL   (3u)

/**\brief TRX MODE READ, SLEEP MODE */
#define TRX_GET_MODE_SLEEP    (1u)
/**\brief TRX MODE READ, RCV ONLY MODE */
#define TRX_GET_MODE_RCV_ONLY (5u)
/**\brief TRX MODE READ, NORMAL MODE */
#define TRX_GET_MODE_NORMAL   (7u)

/**\brief TRX SLOPE MODE, NORMAL SLOPE */
#define TRX_SLOPE_NORMAL  (0u)
/**\brief TRX SLOPE MODE, FAST SLOPE */
#define TRX_SLOPE_FAST    (1u)
/**\brief TRX SLOPE MODE, LOW SLOPE */
#define TRX_SLOPE_LOW     (2u)
/**\brief TRX SLOPE MODE, FLASH SLOPE */
#define TRX_SLOPE_FLASH   (3u)

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void TRX_Init(void);
INLINE void TRX_Time_Out_Int_Clr(void);
INLINE void TRX_Over_Curr_Int_Clr(void);
INLINE void TRX_Over_Temp_Int_Clr(void);
INLINE void TRX_Slope_Error_Int_Clr(void);
INLINE void LIN_End_Of_Sync_Int_Clr(void);
INLINE void LIN_Err_In_Sync_Int_Clr(void);
INLINE void TRX_Time_Out_Int_En(void);
INLINE void TRX_Time_Out_Int_Dis(void);
INLINE void TRX_Over_Curr_Int_En(void);
INLINE void TRX_Over_Curr_Int_Dis(void);
INLINE void TRX_Over_Temp_Int_En(void);
INLINE void TRX_Over_Temp_Int_Dis(void);
INLINE void TRX_Slope_Error_Int_En(void);
INLINE void TRX_Slope_Error_Int_Dis(void);
INLINE void LIN_Sync_Int_En(void);
INLINE void LIN_Sync_Int_Dis(void);
INLINE void TRX_Set_Mode(uint8 Mode);
INLINE uint32 TRX_Get_Mode(void);
INLINE void TRX_Set_Slope(uint8 SlopeMode);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Clear TRX TXD time-out interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX TXD time-out interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Time_Out_Int_En();
 *   if ((uint8)TRX->IRQS.bit.TXD_TMOUT_IS == (uint8)1)
 *   {
 *     TRX_TMOUT_CALLBACK();
 *     TRX_Time_Out_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Time_Out_Int_Clr(void)
{
  Field_Wrt32(&TRX->IRQCLR.reg, (uint8)TRX_IRQCLR_TXD_TMOUT_ISC_Pos, TRX_IRQCLR_TXD_TMOUT_ISC_Msk, 1u);
}

/** \brief Clear TRX Receiver Overcurrent interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Curr_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OC_IS == (uint8)1)
 *   {
 *     TRX_OC_CALLBACK();
 *     TRX_Over_Curr_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Curr_Int_Clr(void)
{
  Field_Wrt32(&TRX->IRQCLR.reg, (uint8)TRX_IRQCLR_OC_ISC_Pos, TRX_IRQCLR_OC_ISC_Msk, 1u);
}

/** \brief Clear TRX Receiver Overtemperature interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Temp_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OT_IS == (uint8)1)
 *   {
 *     TRX_OT_CALLBACK();
 *     TRX_Over_Temp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Temp_Int_Clr(void)
{
  Field_Wrt32(&TRX->IRQCLR.reg, (uint8)TRX_IRQCLR_OT_ISC_Pos, TRX_IRQCLR_OT_ISC_Msk, 1u);
}

/** \brief Clear TRX Transceiver Mode Error - Slope Mode Error interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Transceiver Mode Error - Slope Mode Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Slope_Error_Int_En();
 *   if ((uint8)TRX->IRQS.bit.M_SM_ERR_IS == (uint8)1)
 *   {
 *     TRX_SMERR_CALLBACK();
 *     TRX_Slope_Error_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Slope_Error_Int_Clr(void)
{
  Field_Wrt32(&TRX->IRQCLR.reg, (uint8)TRX_IRQCLR_M_SM_ERR_ISC_Pos, TRX_IRQCLR_M_SM_ERR_ISC_Msk, 1u);
}

/** \brief Enable TRX Transceiver Overcurrent interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Curr_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OC_IS == (uint8)1)
 *   {
 *     TRX_OC_CALLBACK();
 *     TRX_Over_Curr_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Curr_Int_En(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_OC_IEN_Pos, TRX_IRQEN_OC_IEN_Msk, 1u);
}

/** \brief Disable TRX Transceiver Overcurrent interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overcurrent interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Curr_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OC_IS == (uint8)1)
 *   {
 *     TRX_OC_CALLBACK();
 *     TRX_Over_Curr_Int_Clr();
 *   }
 *   TRX_Over_Curr_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Curr_Int_Dis(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_OC_IEN_Pos, TRX_IRQEN_OC_IEN_Msk, 0u);
}

/** \brief Enable TRX Transceiver Overtemperature interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Temp_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OT_IS == (uint8)1)
 *   {
 *     TRX_OT_CALLBACK();
 *     TRX_Over_Temp_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Temp_Int_En(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_OT_IEN_Pos, TRX_IRQEN_OT_IEN_Msk, 1u);
}

/** \brief Disable TRX Transceiver Overtemperature interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Receiver Overtemperature interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Over_Temp_Int_En();
 *   if ((uint8)TRX->IRQS.bit.OT_IS == (uint8)1)
 *   {
 *     TRX_OT_CALLBACK();
 *     TRX_Over_Temp_Int_Clr();
 *   }
 *   TRX_Over_Temp_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Over_Temp_Int_Dis(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_OT_IEN_Pos, TRX_IRQEN_OT_IEN_Msk, 0u);
}

/** \brief Enable TRX Transceiver TxD-Timeout interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX TXD time-out interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Time_Out_Int_En();
 *   if ((uint8)TRX->IRQS.bit.TXD_TMOUT_IS == (uint8)1)
 *   {
 *     TRX_TMOUT_CALLBACK();
 *     TRX_Time_Out_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Time_Out_Int_En(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_TXD_TMOUT_IEN_Pos, TRX_IRQEN_TXD_TMOUT_IEN_Msk, 1u);
}

/** \brief Disable TRX TXD time-out interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX TXD time-out interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Time_Out_Int_En();
 *   if ((uint8)TRX->IRQS.bit.TXD_TMOUT_IS == (uint8)1)
 *   {
 *     TRX_TMOUT_CALLBACK();
 *     TRX_Time_Out_Int_Clr();
 *   }
 *   TRX_Time_Out_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Time_Out_Int_Dis(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_TXD_TMOUT_IEN_Pos, TRX_IRQEN_TXD_TMOUT_IEN_Msk, 0u);
}

/** \brief Enable TRX Transceiver Mode Error - Slope Mode Error interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Transceiver Mode Error - Slope Mode Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Slope_Error_Int_En();
 *   if ((uint8)TRX->IRQS.bit.M_SM_ERR_IS == (uint8)1)
 *   {
 *     TRX_SMERR_CALLBACK();
 *     TRX_Slope_Error_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Slope_Error_Int_En(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_M_SM_ERR_IEN_Pos, TRX_IRQEN_M_SM_ERR_IEN_Msk, 1u);
}

/** \brief Disable TRX Transceiver Mode Error - Slope Mode Error interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the TRX Transceiver Mode Error - Slope Mode Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   TRX_Slope_Error_Int_En();
 *   if ((uint8)TRX->IRQS.bit.M_SM_ERR_IS == (uint8)1)
 *   {
 *     TRX_SMERR_CALLBACK();
 *     TRX_Slope_Error_Int_Clr();
 *   }
 *   TRX_Slope_Error_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Slope_Error_Int_Dis(void)
{
  Field_Mod32(&TRX->IRQEN.reg, (uint8)TRX_IRQEN_M_SM_ERR_IEN_Pos, TRX_IRQEN_M_SM_ERR_IEN_Msk, 0u);
}

/** \brief Set TRX Mode
 *
 * \param TRX Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TRX Transmitter Sleep Mode as Transceiver power mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TRX_Set_Mode(0x00); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void TRX_Set_Mode(uint8 Mode)
{
  TRX->CTRL.bit.MODE = Mode;
}

/** \brief Get TRX Mode
 *
 * \return Trx Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example reads TRX Transmitter Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *    uint32 mode;
 * 
 *    mode = TRX_Get_Mode(); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup trx_api
 */
INLINE uint32 TRX_Get_Mode(void)
{
  return(TRX->CTRL.bit.MODE_FB);
}

/** \brief Set TRX Slope Mode
 *
 * \param SlopeMode TRX Trx Slope Mode Selection
 *
 * \brief <b>Example</b><br>
 * \brief This example sets TRX Normal Mode as Transmitter Slope mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     TRX_Set_Slope(0x00); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup trx_api
 */
INLINE void TRX_Set_Slope(uint8 SlopeMode)
{
  TRX->CTRL.bit.MODE = TRX_MODE_SLEEP;
  TRX->CTRL.bit.SM = SlopeMode;
  TRX->CTRL.bit.MODE = TRX_MODE_NORMAL;
}

/** \brief Clear End of SYN Byte interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the LIN End of SYN Byte interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 * 	 LIN_Sync_Int_En();
 *   if ((uint8)SCU->LINST.bit.EOFSYN == (uint8)1u)
 *   {
 *     LIN_EOF_CALLBACK();
 *     LIN_End_Of_Sync_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void LIN_End_Of_Sync_Int_Clr(void)
{
  Field_Wrt32(&SCU->LINSCLR.reg, (uint8)SCU_LINSCLR_EOFSYNC_Pos, SCU_LINSCLR_EOFSYNC_Msk, 1u);
}

/** \brief Clear SYN Byte Error interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example threats the SYN Byte Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 * 	 LIN_Sync_Int_En();
 *   if ((uint8)SCU->LINST.bit.ERRSYN == (uint8)1u)
 *   {
 *     LIN_ERR_CALLBACK();
 *     LIN_Err_In_Sync_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void LIN_Err_In_Sync_Int_Clr(void)
{
  Field_Wrt32(&SCU->LINSCLR.reg, (uint8)SCU_LINSCLR_ERRSYNC_Pos, SCU_LINSCLR_ERRSYNC_Msk, 1u);
}

/** \brief Enable SYN Byte Error interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example threats the SYN Byte Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 * 	 LIN_Sync_Int_En();
 *   if ((uint8)SCU->LINST.bit.ERRSYN == (uint8)1u)
 *   {
 *     LIN_ERR_CALLBACK();
 *     LIN_Err_In_Sync_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void LIN_Sync_Int_En(void)
{
  Field_Mod32(&SCU->LINST.reg, (uint8)SCU_LINST_SYNEN_Pos, SCU_LINST_SYNEN_Msk, 1u);
}

/** \brief Disable SYN Byte Error interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example threats the SYN Byte Error interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 * 	 LIN_Sync_Int_En();
 *   if ((uint8)SCU->LINST.bit.ERRSYN == (uint8)1u)
 *   {
 *     LIN_ERR_CALLBACK();
 *     LIN_Err_In_Sync_Int_Clr();
 *   }
 * 	 LIN_Sync_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup trx_api
 */
INLINE void LIN_Sync_Int_Dis(void)
{
  Field_Mod32(&SCU->LINST.reg, (uint8)SCU_LINST_SYNEN_Pos, SCU_LINST_SYNEN_Msk, 0u);
}

#endif
