/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     uart.h
 *
 * \brief    UART low level access library
 *
 * \version  V0.2.3
 * \date     06. Apr 2022
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
** VO           Vanessa Ongaro                                                **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version, ported from V0.5.1 (Proteus SDK)**
** V0.1.1: 2015-11-06, AV:   Improved UART init/send routines (now blocking   **
**                           until send is complete). Intended for sending    **
**                           from low(est) priority task.                     **
** V0.1.2: 2015-11-12, AV:   Added UARTx_Read_Byte, UARTx_Send_String and     **
**                           UART2_Send_HEX functions as basic communication  **
**                           routines                                         **
** V0.1.3: 2015-07-15, DM:   STDIN/STDOUT function added                      **
** V0.1.4: 2017-06-19, KC:   GetByte, isByteReceived and isByteTransmitted    **
**                           inline functions added                           **
**                           Baudrate_Set function added                      **
**                           Blocking characteristic of send routines removed **
** V0.1.5: 2018-04-27, KC:   uart_defines.h moved from uart.c to uart.h       **
** V0.1.6: 2018-11-05, CE:   UART1_BaudRate_Set() - calculation optimized     **
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
**                           UART1_BaudRate_Value_Set,UART2_BaudRate_Value_Set**
**                           functions added                                  **
** V0.1.7: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           UART2_BaudRate_Set function updated              **
**                           UARTx Clock Structure added (for Unit Testing    **
**                           full coverage reason)                            **
** V0.1.8: 2019-03-29, JO:   Used variable uart*.clock                        **
**                           Removed UART1_Send_HEX, UART1_Send_DEC,          **
**                           UART1_Send_String, UART2_Send_HEX, UART2_Send_DEC**
**                           and UART2_Send_String                            **
**                           Added range for functions UART1/2_BaudRate_Set   **
** V0.1.9: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to uart.c                                        **
**                           Changed from anonymous to named typedefs to      **
**                           prevent MISRA warning                            **
** V0.2.0: 2021-10-07, BG:   EP-931: Corrected ARMCC compiler v6 errors       **
** V0.2.1: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.2.2: 2021-12-21, BG:   EP-997: Corrected MISRA 2012 violations          **
** V0.2.3: 2022-04-06, VO:   EP-1085: Added corrections to avoid MISRA        **
**                           violations                                       **
*******************************************************************************/

#ifndef UART_H
#define UART_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "uart_defines.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/**\brief UART Mask Interrupt Macro, Receive Interrupt*/
#define UART_MASK_INT_RI (0x01u)
/**\brief UART Mask Interrupt Macro, Transmit Interrupt*/
#define UART_MASK_INT_TI (0x02u)

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \struct TUart
 *  \brief This struct lists parameter (clock) of UART1/UART2.
 */
typedef struct _TUart
{
  uint32 clock;
} TUart;

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void UART1_Init(void);
void UART1_BaudRate_Set(uint32 baudrate);
void UART2_Init(void);
void UART2_BaudRate_Set(uint32 baudrate);
#if ((UART1_STD_EN == 1u) || (UART2_STD_EN == 1u))
sint32 stdout_putchar(sint32 Char);
sint32 stdin_getchar(void);
void ttywrch(sint32 ch);
#endif
INLINE void UART1_BaudRate_Value_Set(uint16 br_val);
INLINE void UART1_BaudRateGen_Dis(void);
INLINE void UART1_BaudRateGen_En(void);
INLINE void UART2_BaudRate_Value_Set(uint16 br_val);
INLINE void UART2_BaudRateGen_Dis(void);
INLINE void UART2_BaudRateGen_En(void);
INLINE uint8 UART1_RX_Sts(void);          
INLINE uint8 UART1_TX_Sts(void);          
INLINE uint8 UART2_RX_Sts(void);          
INLINE uint8 UART2_TX_Sts(void);          
INLINE void UART1_Receiver_En(void);
INLINE void UART1_Receiver_Dis(void);
INLINE void UART2_Receiver_En(void);
INLINE void UART2_Receiver_Dis(void);
INLINE void UART1_RX_Int_Clr(void);
INLINE void UART1_TX_Int_Clr(void);
INLINE void UART2_RX_Int_Clr(void);
INLINE void UART2_TX_Int_Clr(void);
INLINE void UART1_RX_Int_En(void);
INLINE void UART1_RX_Int_Dis(void);
INLINE void UART1_TX_Int_En(void);
INLINE void UART1_TX_Int_Dis(void);
INLINE void UART2_RX_Int_En(void);
INLINE void UART2_RX_Int_Dis(void);
INLINE void UART2_TX_Int_En(void);
INLINE void UART2_TX_Int_Dis(void);
INLINE void UART1_Send_Byte(uint8 c);
INLINE uint8 UART1_Get_Byte(void);
INLINE bool UART1_isByteReceived(void);
INLINE bool UART1_isByteTransmitted(void);
INLINE bool UART1_Read_Byte(uint8* pc);
INLINE void UART2_Send_Byte(uint8 c);
INLINE uint8 UART2_Get_Byte(void);
INLINE bool UART2_isByteReceived(void);
INLINE bool UART2_isByteTransmitted(void);
INLINE bool UART2_Read_Byte(uint8* pc);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Write the baudrate timer register (UART1).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1 (baud-rate is equal to 19200 when frequency is equal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRateGen_Dis();
 *     UART1_BaudRate_Value_Set(0x1046);
 *     UART1_BaudRateGen_En();
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 *     UART1_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup uart_api
 */
INLINE void UART1_BaudRate_Value_Set(uint16 br_val)
{
  Field_Wrt32(&SCU->BG1.reg, (uint8)SCU_BG1_BG1_BR_VALUE_Pos, SCU_BG1_BG1_BR_VALUE_Msk, ((uint32)br_val >> 5u));
  Field_Wrt32(&SCU->BGL1.reg, (uint8)SCU_BGL1_BG1_FD_SEL_Pos, SCU_BGL1_BG1_FD_SEL_Msk, ((uint32)br_val & (uint32)0x1F));
}

/** \brief Disable Baud-rate generator (UART1).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1 (baudrate is equal to 19200 when frequency is equal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRateGen_Dis();
 *     UART1_BaudRate_Value_Set(0x1046);
 *     UART1_BaudRateGen_En();
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 *     UART1_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART1_BaudRateGen_Dis(void)
{
  Field_Mod32(&SCU->BCON1.reg, (uint8)SCU_BCON1_BR1_R_Pos, SCU_BCON1_BR1_R_Msk, 0u);
}

/** \brief Enable Baud-rate generator (UART1).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1 (baudrate is equal to 19200 when frequency is equal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRateGen_Dis();
 *     UART1_BaudRate_Value_Set(0x1046);
 *     UART1_BaudRateGen_En();
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 *     UART1_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART1_BaudRateGen_En(void)
{
  Field_Mod32(&SCU->BCON1.reg, (uint8)SCU_BCON1_BR1_R_Pos, SCU_BCON1_BR1_R_Msk, 1u);
}

/** \brief Write the baudrate timer register (UART2).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2 (baud-rate is equal to 19200 when frequency is equal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRateGen_Dis();
 *     UART2_BaudRate_Value_Set(0x1046);
 *     UART2_BaudRateGen_En();
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 *     UART2_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup uart_api
 */
INLINE void UART2_BaudRate_Value_Set(uint16 br_val)
{
  Field_Wrt32(&SCU->BG2.reg, (uint8)SCU_BG2_BG2_BR_VALUE_Pos, SCU_BG2_BG2_BR_VALUE_Msk, ((uint32)br_val >> 5u));
  Field_Wrt32(&SCU->BGL2.reg, (uint8)SCU_BGL2_BG2_FD_SEL_Pos, SCU_BGL2_BG2_FD_SEL_Msk, ((uint32)br_val & (uint32)0x1F));
}

/** \brief Disable Baud-rate generator (UART2).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2 (baudrate is equal to 19200 when frequency is equeal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRateGen_Dis();
 *     UART2_BaudRate_Value_Set(0x1046);
 *     UART2_BaudRateGen_En();
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 *     UART2_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART2_BaudRateGen_Dis(void)
{
  Field_Mod32(&SCU->BCON2.reg, (uint8)SCU_BCON2_BR2_R_Pos, SCU_BCON2_BR2_R_Msk, 0u);
}

/** \brief Enable Baud-rate generator (UART2).
 * \note Setting new Baud-rate value must be when BaudRateGen is disabled.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2 (baudrate is equal to 19200 when frequency is equeal to 40MHz).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRateGen_Dis();
 *     UART2_BaudRate_Value_Set(0x1046);
 *     UART2_BaudRateGen_En();
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 *     UART2_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART2_BaudRateGen_En(void)
{
  Field_Mod32(&SCU->BCON2.reg, (uint8)SCU_BCON2_BR2_R_Pos, SCU_BCON2_BR2_R_Msk, 1u);
}

/** \brief Read the receive interrupt flag of UART1.
 *
 * \return Receive Interrupt Flag Status
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 * 
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_RX_Sts() == (uint8)1u)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART1_RX_Sts(void)          
{
	return(u1_Field_Rd32(&UART1->SCON.reg, (uint8)UART1_SCON_RI_Pos, UART1_SCON_RI_Msk));
}

/** \brief Read the transmit interrupt flag of UART1.
 *
 * \return Transmit Interrupt Flag Status
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_TX_Sts() == (uint8)1u)
 *	   {
 *       UART1_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART1_TX_Sts(void)          
{
	return(u1_Field_Rd32(&UART1->SCON.reg, (uint8)UART1_SCON_TI_Pos, UART1_SCON_TI_Msk));
}

/** \brief Read the receive interrupt flag of UART2.
 *
 * \return Receive Interrupt Flag Status
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_RX_Sts() == (uint8)1u)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART2_RX_Sts(void)          
{
	return(u1_Field_Rd32(&UART2->SCON.reg, (uint8)UART2_SCON_RI_Pos, UART2_SCON_RI_Msk));
}

/** \brief Read the transmit interrupt flag of UART2.
 *
 * \return Transmit Interrupt Flag Status
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_TX_Sts() == (uint8)1u)
 *	   {
 *       UART2_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART2_TX_Sts(void)          
{
	return(u1_Field_Rd32(&UART2->SCON.reg, (uint8)UART2_SCON_TI_Pos, UART2_SCON_TI_Msk));
}

/** \brief Enable Receiver of Serial Port (UART1).
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART1_Receiver_En(void)
{
  Field_Mod32(&UART1->SCON.reg, (uint8)UART1_SCON_REN_Pos, UART1_SCON_REN_Msk, 1u);
}

/** \brief Disable Receiver of Serial Port (UART1).
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 *     UART1_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART1_Receiver_Dis(void)
{
  Field_Mod32(&UART1->SCON.reg, (uint8)UART1_SCON_REN_Pos, UART1_SCON_REN_Msk, 0u);
}

/** \brief Enable Receiver of Serial Port (UART2).
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART2_Receiver_En(void)
{
  Field_Mod32(&UART2->SCON.reg, (uint8)UART2_SCON_REN_Pos, UART2_SCON_REN_Msk, 1u);
}

/** \brief Disable Receiver of Serial Port (UART2).
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 *     UART2_Receiver_Dis();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART2_Receiver_Dis(void)
{
  Field_Mod32(&UART2->SCON.reg, (uint8)UART2_SCON_REN_Pos, UART2_SCON_REN_Msk, 0u);
}

/** \brief Clear receive interrupt flag for UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *       UART1_RX_Int_Clr(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART1_RX_Int_Clr(void)
{
  Field_Wrt32(&UART1->SCONCLR.reg, (uint8)UART1_SCONCLR_RICLR_Pos, UART1_SCONCLR_RICLR_Msk, 1u);
}

/** \brief Clear transmit interrupt flag of UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteTransmitted() == TRUE)
 *	   {
 *       UART1_Send_Byte(0x41); 
 *       UART1_TX_Int_Clr();
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_TX_Int_Clr(void)
{
  Field_Wrt32(&UART1->SCONCLR.reg, (uint8)UART1_SCONCLR_TICLR_Pos, UART1_SCONCLR_TICLR_Msk, 1u);
}

/** \brief Clear receive interrupt flag of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *       UART2_RX_Int_Clr(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE void UART2_RX_Int_Clr(void)
{
  Field_Wrt32(&UART2->SCONCLR.reg, (uint8)UART2_SCONCLR_RICLR_Pos, UART2_SCONCLR_RICLR_Msk, 1u);
}

/** \brief Clear transmit interrupt flag of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteTransmitted() == TRUE)
 *	   {
 *       UART2_Send_Byte(0x41); 
 *       UART2_TX_Int_Clr();
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_TX_Int_Clr(void)
{
  Field_Wrt32(&UART2->SCONCLR.reg, (uint8)UART2_SCONCLR_TICLR_Pos, UART2_SCONCLR_TICLR_Msk, 1u);
}

/** \brief Enable receive interrupt of UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_RX_Int_En();
 *     if (UART1_RX_Sts() == (uint8)1u)
 *		 {
 *       UART1_RX_CALLBACK();
 *       UART1_RX_Int_Clr();
 *     }	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_RX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_RIEN1_Pos, SCU_MODIEN2_RIEN1_Msk, 1u);
}

/** \brief Disable receive interrupt of UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_RX_Int_En();
 *     if (UART1_RX_Sts() == (uint8)1u)
 *		 {
 *       UART1_RX_CALLBACK();
 *       UART1_RX_Int_Clr();
 *     }	 
 *     UART1_RX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_RX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_RIEN1_Pos, SCU_MODIEN2_RIEN1_Msk, 0u);
}

/** \brief Enable transmit interrupt of UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the transmit interrupt for UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_TX_Int_En();
 *     if (UART1_TX_Sts() == (uint8)1u)
 *		 {
 *       UART1_TX_CALLBACK();
 *       UART1_TX_Int_Clr();
 *     }	 
 *     UART1_TX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_TX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_TIEN1_Pos, SCU_MODIEN2_TIEN1_Msk, 1u);
}

/** \brief Disable transmit interrupt of UART1.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the transmit interrupt for UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_TX_Int_En();
 *     if (UART1_TX_Sts() == (uint8)1u)
 *		 {
 *       UART1_TX_CALLBACK();
 *       UART1_TX_Int_Clr();
 *     }	 
 *     UART1_TX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_TX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_TIEN1_Pos, SCU_MODIEN2_TIEN1_Msk, 0u);
}

/** \brief Enable receive interrupt of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_RX_Int_En();
 *     if (UART2_RX_Sts() == (uint8)1u)
 *		 {
 *       UART2_RX_CALLBACK();
 *       UART2_RX_Int_Clr();
 *     }	 
 *     UART2_RX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_RX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_RIEN2_Pos, SCU_MODIEN2_RIEN2_Msk, 1u);
}

/** \brief Disable receive interrupt of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the receive interrupt for UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_RX_Int_En();
 *     if (UART2_RX_Sts() == (uint8)1u)
 *		 {
 *       UART2_RX_CALLBACK();
 *       UART2_RX_Int_Clr();
 *     }	 
 *     UART2_RX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_RX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_RIEN2_Pos, SCU_MODIEN2_RIEN2_Msk, 0u);
}

/** \brief Enable transmit interrupt of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the transmit interrupt for UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_TX_Int_En();
 *     if (UART2_TX_Sts() == (uint8)1u)
 *		 {
 *       UART2_TX_CALLBACK();
 *       UART2_TX_Int_Clr();
 *     }	 
 *     UART2_TX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_TX_Int_En(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_TIEN2_Pos, SCU_MODIEN2_TIEN2_Msk, 1u);
}

/** \brief Disable transmit interrupt of UART2.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the transmit interrupt for UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_TX_Int_En();
 *     if (UART2_TX_Sts() == (uint8)1u)
 *		 {
 *       UART2_TX_CALLBACK();
 *       UART2_TX_Int_Clr();
 *     }	 
 *     UART2_TX_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_TX_Int_Dis(void)
{
  Field_Mod32(&SCU->MODIEN2.reg, (uint8)SCU_MODIEN2_TIEN2_Pos, SCU_MODIEN2_TIEN2_Msk, 0u);
}

/** \brief Clear the UART1 Transmit interrupt and sends a Byte via UART1.
 *
 * \param c Byte to send
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteTransmitted() == TRUE)
 *	   {
 *       UART1_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART1_Send_Byte(uint8 c)
{
  UART1_TX_Int_Clr();
  UART1->SBUF.reg = c;
}

/** \brief Clear the UART1 receive interrupt and returns the UART1 buffer.
 *
 * \return Received Byte
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART1_Get_Byte(void)
{
  UART1_RX_Int_Clr();
  return((uint8)UART1->SBUF.reg);
}

/** \brief Check if UART1 has received Byte or not.
 *
 * \retval TRUE a byte was received
 * \retval FALSE no byte was received
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART1_Receiver_En();
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteReceived() == TRUE)
 *	   {
 *       ch = UART1_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE bool UART1_isByteReceived(void)
{
  bool bRes = false;
  if (UART1_RX_Sts() == (uint8)1)
  {
    bRes = true;
  }
  return (bRes);
}

/** \brief Check if UART1 has transmitted a Byte or not.
 *
 * \retval TRUE a byte was transmitted
 * \retval FALSE no byte was transmitted
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_BaudRate_Set(19200);
 *     if (UART1_isByteTransmitted() == TRUE)
 *	   {
 *       UART1_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE bool UART1_isByteTransmitted(void)
{
  bool bRes = false;
  if (UART1_TX_Sts() == (uint8)1)
  {
    bRes = true;
  }
  return (bRes);
}

/** \brief Read a Byte from UART1
 *
 * \param pc Byte read from UART receive buffer
 * \return true if a byte was received
 *
 * \ingroup uart_api
 */
INLINE bool UART1_Read_Byte(uint8* pc)
{ 
  bool result = false;
  
  if (UART1->SCON.bit.RI == 1u)
  {
    *pc = (uint8)UART1->SBUF.reg;
    UART1->SCONCLR.bit.RICLR = 1u;
    result = true;
  }
  return result;
}


/** \brief Clear the UART2 Transmit interrupt and sends a Byte via UART2.
 *
 * \param c Byte to send
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteTransmitted() == TRUE)
 *	   {
 *       UART2_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE void UART2_Send_Byte(uint8 c)
{
  UART2_TX_Int_Clr();
  UART2->SBUF.reg = c;
}

/** \brief Clear the UART2 receive interrupt and returns the UART2 buffer.
 *
 * \return Received Byte
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE uint8 UART2_Get_Byte(void)
{
  UART2_RX_Int_Clr();
  return((uint8)UART2->SBUF.reg);
}

/** \brief Check if UART2 has received Byte or not.
 *
 * \retval TRUE a byte was received
 * \retval FALSE no byte was receive
 *
 * \brief <b>Example</b><br>
 * \brief This example receives a Byte via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 ch;
 *
 *     UART2_Receiver_En();
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteReceived() == TRUE)
 *	   {
 *       ch = UART2_Get_Byte(); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
INLINE bool UART2_isByteReceived(void)
{
  bool bRes = false;
  if (UART2_RX_Sts() == (uint8)1)
  {
    bRes = true;
  }
  return (bRes);
}

/** \brief Check if UART2 has transmitted a Byte or not.
 *
 * \retval TRUE a byte was transmitted
 * \retval FALSE no byte was transmitted
 *
 * \brief <b>Example</b><br>
 * \brief This example sends "A" via UART2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_BaudRate_Set(19200);
 *     if (UART2_isByteTransmitted() == TRUE)
 *	   {
 *       UART2_Send_Byte(0x41); 
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
INLINE bool UART2_isByteTransmitted(void)
{
  bool bRes = false;
  if (UART2_TX_Sts() == (uint8)1)
  {
    bRes = true;
  }
  return (bRes);
}

/** \brief Read a Byte from UART2
 *
 * \param pc Byte read from UART receive buffer
 * \return true if a byte was received
 *
 * \ingroup uart_api
 */
INLINE bool UART2_Read_Byte(uint8* pc)
{ 
  bool result = false;
  
  if (UART2->SCON.bit.RI == 1u)
  {
    *pc = (uint8)UART2->SBUF.reg;
    UART2->SCONCLR.bit.RICLR = 1u;
    result = true;
  }
  return result;
}

#endif
