/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     wdt1.h
 *
 * \brief    Window Watchdog 1 low level access library
 *
 * \version  V0.3.2
 * \date     03. Mar 2022
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** AV           Axel Viebig                                                   **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
** VO           Vanessa Ongaro                                                **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-09-30, AV:   Initial version, ported from Proteus SDK         **
** V0.1.1: 2015-11-18, AV:   Added function Delay_ms                          **
** V0.1.2: 2016-05-03, DM:   Function Delay_ms removed                        **
** V0.1.3: 2017-06-19, KC:   Macros One_us and SysTickRL are moved to wdt.h   **
**                           Delay_us() function updated                      **
** V0.2.5: 2017-10-20, KC:   Version number changed to be consistent with pdsc**
** V0.2.6: 2018-11-05, CE:   Typecasts in macro for SysTickRL changed from    **
**                           sint32 to uint32                                 **
**                           Macro for SysTickRL used for writing of          **
**                           "CPU->SYSTICK_RVR.reg"                           **
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.2.7: 2019-03-18, TS:   Delay_us function updated                        **
**                           Systick_Value_Get, Systick_ReloadValue_Get and   **
**                           SysTick_ReloadValue_Set functions added          **
** V0.2.8: 2019-03-18, JO:   Split WDT1_Init to WDT1_Init and SysTick_Init    **
**                           Changed return value of function WDT1_Service    **
**                           from int to uint8                                **
** V0.2.9: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to wdt1.c                                        **
** V0.3.0: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.3.1: 2021-12-17, BG:   EP-769: Corrected function Delay_us in case the  **
**                           current systick value is 0 or the reload value   **
** V0.3.2: 2022-03-03, VO:   EP-1032: Corrected define of One_us              **
*******************************************************************************/

#ifndef _WDT1_H
#define _WDT1_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "sfr_access.h"

/*******************************************************************************
**                      External Variable Declarations                        **
*******************************************************************************/
extern uint32 WD_Counter;

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

#define One_us ((uint32)SCU_FSYS / 1000000u)
#define SysTickRL ((uint32)SCU_FSYS / (uint32)SysTickFreq)

/**\brief SysTick 1kHz*/
#define SysTickFreq     1000u

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void WDT1_Init(void);
void SysTick_Init(void);
void WDT1_Stop(void);
uint8 WDT1_Service(void);
void WDT1_SOW_Service(uint32 NoOfSOW);
void Delay_us(uint32 delay_time_us);
INLINE void WDT1_Window_Count(void);
INLINE uint32 Systick_Value_Get(void);
INLINE uint32 Systick_ReloadValue_Get(void);
INLINE void SysTick_ReloadValue_Set(uint32 val);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/** \brief Increment the WDT1 Window Counter
 *
 * \brief <b>Example</b><br>
 * \brief This example increments the WDT1 Window Counter.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     WDT1_Window_Count();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup wdt1_api
 */
INLINE void WDT1_Window_Count(void)
{
  WD_Counter++;
}

/** \brief Return the current SysTick timer count value
 *
 * \return current SysTick timer count value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the current SysTick timer count value
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint32 val;
 *
 *     val = Systick_Value_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup wdt1_api
 */
INLINE uint32 Systick_Value_Get(void)
{
  return (u32_Field_Rd32(&CPU->SYSTICK_CVR.reg, (uint8)CPU_SYSTICK_CVR_CURRENT_Pos, CPU_SYSTICK_CVR_CURRENT_Msk));
}

/** \brief Return the current SysTick reload value
 *
 * \return current SysTick reload value
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the current SysTick reload value
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint32 val;
 *
 *     val = Systick_ReloadValue_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup wdt1_api
 */
INLINE uint32 Systick_ReloadValue_Get(void)
{
  return (u32_Field_Rd32(&CPU->SYSTICK_RVR.reg, (uint8)CPU_SYSTICK_RVR_RELOAD_Pos, CPU_SYSTICK_RVR_RELOAD_Msk));
}

/** \brief Set the SysTick Reload value
 *
 * \param val reload value for SysTick timer
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the current SysTick reload to Max value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SysTick_ReloadValue_Set(0xFFFFFF);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup wdt1_api
 */
INLINE void SysTick_ReloadValue_Set(uint32 val)
{
  Field_Wrt32(&CPU->SYSTICK_RVR.reg, (uint8)CPU_SYSTICK_RVR_RELOAD_Pos, CPU_SYSTICK_RVR_RELOAD_Msk, val);
}

#endif
