/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "adc1.h"
#include "adc1_defines.h"
#include "wdt1.h"
#include "int.h"
#include "tle_variants.h"

/*******************************************************************************
**                       Private Function Declarations                        **
*******************************************************************************/

static uint32 ADC1_GetChCorrFactMul1024(uint8 adc1_ch);

/*******************************************************************************
**                        Private Function Definitions                        **
*******************************************************************************/

static uint32 ADC1_GetChCorrFactMul1024(uint8 adc1_ch)
{
  uint32 result = 0;
  
  if (adc1_ch < 2u)
  {
    /* fs = Vref_in_mV / correction_factor / full_scale_bits * 1024 */
    /* fs = 1211 / 0,047 / 2^10 * 1024 */
    result = (uint32)25766;      /* ch0-1 */
  }
#if (UC_FEAT_MON == 5u)
  else if (adc1_ch < 7u)
#else
  else if (adc1_ch < 6u)
#endif
  {
    /* fs = 1211 / 0,039 */
    result = (uint32) 31051;    /* ch2-6 for devices with MON5, else ch2-5 */
  }
  else 
  {
    /* fs = 1211 / 0,219 */
    result = (uint32) 5530;     /* ch7-12 for devices with MON5, else ch6-12 */
  }
  return result;
}

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Initialize the ADC1
 *
 * \ingroup adc1_api
 */
void ADC1_Init(void)
{
  uint16_t i;
  /* Clock Configuration */
  ADC1->CTRL3.reg = (uint32) ADC1_CTRL3;    //1. clock+mode
                                            //2. sequencer config
                                            //3. configure channels, thresholds, filters, calibration, interrupts
                                            //4. configure esm
                                            //5. configure eim
                                            //6. configure interrupts
                                            //7. enable adc1
                                            

  ADC1->CTRL2.reg = (uint32) ADC1_CTRL2;
  ADC1->CTRL5.reg = (uint32) ADC1_CTRL5;  


  /* Sequencer0-1 configuration */
  ADC1->SQ0_1.reg = (uint32) ADC1_SQ0_1; 
  /* Sequencer2-3 configuration */
  ADC1->SQ2_3.reg = (uint32) ADC1_SQ2_3;  
  /* Sequencer4-5 configuration */
  ADC1->SQ4_5.reg = (uint32) ADC1_SQ4_5;  
  /* Sequencer6-7 configuration */
  ADC1->SQ6_7.reg = (uint32) ADC1_SQ6_7;
  /* Sequencer8-9 configuration */
  ADC1->SQ8_9.reg = (uint32) ADC1_SQ8_9;
  /* Sequencer10-11 configuration */
  ADC1->SQ10_11.reg = (uint32) ADC1_SQ10_11;
  
#if (ADC1_DUIN_SEL > 0u)
  SCU->MODPISEL4.reg = (uint32) SCU_MODPISEL4;
  ADC1->DCHTH1_4_LOWER.reg = ADC1_DCHTH1_4_LOWER;
  ADC1->DCHTH1_4_UPPER.reg = ADC1_DCHTH1_4_UPPER;
#endif  
  
  ADC1->TH0_3_LOWER.reg = (uint32_t)ADC1_TH0_3_LOWER; 
  ADC1->TH0_3_UPPER.reg = ADC1_TH0_3_UPPER; 
  ADC1->TH4_7_LOWER.reg = ADC1_TH4_7_LOWER; 
  ADC1->TH4_7_UPPER.reg = ADC1_TH4_7_UPPER; 
  ADC1->TH8_11_LOWER.reg = ADC1_TH8_11_LOWER;
  ADC1->TH8_11_UPPER.reg = ADC1_TH8_11_UPPER;
#if (CONFIGWIZARD == 2)  
  ADC1->CNT0_3_LOWER.reg = ADC1_CNT0_3_LOWER;
  ADC1->CNT0_3_UPPER.reg = ADC1_CNT0_3_UPPER;
  ADC1->CNT4_7_LOWER.reg = ADC1_CNT4_7_LOWER;
  ADC1->CNT4_7_UPPER.reg = ADC1_CNT4_7_UPPER;
  ADC1->CNT8_11_LOWER.reg = ADC1_CNT8_11_LOWER;
  ADC1->CNT8_11_UPPER.reg = ADC1_CNT8_11_UPPER;
  ADC1->DCHCNT1_4_LOWER.reg = ADC1_DCHCNT1_4_LOWER;
  ADC1->DCHCNT1_4_UPPER.reg = ADC1_DCHCNT1_4_UPPER;
  ADC1->MMODE0_11.reg = ADC1_MMODE0_11;
#endif
  
#if (CONFIGWIZARD == 1)  
  ADC1->CHx_EIM.reg = (uint32) ADC1_CHX_EIM;
  ADC1->CHx_ESM.reg = (uint32) ADC1_CHX_ESM;
#else /* (CONFIGWIZARD == 2) */
  ADC1->CHx_EIM.reg = (uint32) ADC1_CHx_EIM;
  ADC1->CHx_ESM.reg = (uint32) ADC1_CHx_ESM;
#endif
  
  ADC1->DIFFCH_OUT1.reg = (uint32) ADC1_DIFFCH_OUT1;  
  ADC1->DIFFCH_OUT2.reg = (uint32) ADC1_DIFFCH_OUT2;  
  ADC1->DIFFCH_OUT3.reg = (uint32) ADC1_DIFFCH_OUT3;  
  ADC1->DIFFCH_OUT4.reg = (uint32) ADC1_DIFFCH_OUT4;  
  
  ADC1->DUIN_SEL.reg = (uint32) ADC1_DUIN_SEL;  
  
  ADC1->FILTCOEFF0_11.reg = (uint32) ADC1_FILTCOEFF0_11;  
  
  ADC1->FILT_OUT0.reg = ADC1_FILT_OUT0;
  ADC1->FILT_OUT1.reg = ADC1_FILT_OUT1;
  ADC1->FILT_OUT2.reg = ADC1_FILT_OUT2;
  ADC1->FILT_OUT3.reg = ADC1_FILT_OUT3;
  ADC1->FILT_OUT4.reg = ADC1_FILT_OUT4;
  ADC1->FILT_OUT5.reg = ADC1_FILT_OUT5;
  ADC1->FILT_OUT6.reg = ADC1_FILT_OUT6;
  ADC1->FILT_OUT7.reg = ADC1_FILT_OUT7;
  ADC1->FILT_OUT8.reg = ADC1_FILT_OUT8;
  ADC1->FILT_OUT9.reg = ADC1_FILT_OUT9;
  ADC1->FILT_OUT10.reg = ADC1_FILT_OUT10;
  ADC1->FILT_OUT11.reg = ADC1_FILT_OUT11;
  
  ADC1->FILT_LO_CTRL.reg = ADC1_FILT_LO_CTRL;
  ADC1->FILT_UP_CTRL.reg = ADC1_FILT_UP_CTRL;
  ADC1->IRQEN_1.reg = ADC1_IRQEN_1;
  ADC1->IRQEN_2.reg = ADC1_IRQEN_2;

  /* Enable ADC1 */ 

  INT_Disable_Global_Int();

  /* Enable ADC1 in Peripheral Management Register */
  SCU->PMCON.bit.ADC1_DIS = 0;
  /* Enable Measurement Core Module*/
  ADC1->CTRL3.bit.MCM_PD_N = 1;
  /* Switch ADC1 on */
  ADC1->CTRL_STS.bit.PD_N = 1;

  /* apply 10ms delay to give ADC1 time to turn on */
  for (i = 0u; i < 20u; i++)
  {
    Delay_us(500);
    (void)WDT1_Service();
  }
  
  /* Clear all collected interrupts during ADC1 startup period */
  ADC1->IRQCLR_1.reg=0xFFFFFFFFu;
  /* Clear all collected interrupts during ADC1 startup period */
  ADC1->IRQCLR_2.reg=0xFFFFFFFFu;     

  INT_Enable_Global_Int();  
}

/** \brief Get ADC1 Channel result value
 *
 * \param[in] channel Channel number
 * \param[out] *pVar ADC value (10bit right aligned in 16 bit return value)
 * \retval true *pVar valid
 * \retval false *pVar invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC1 Channel 5 result in Milli Volt (mV).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var; 
 * 
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     if (ADC1_GetChResult(&var, ADC1_CH5) == true)
 *     {
 *	       value_valid = ADC1_GetChResult_mV(&var_mV, ADC1_CH5);
 *     }
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */ 
bool ADC1_GetChResult(uint16 * pVar, uint8 channel)
{
  const volatile uint32 *pBaseAddr;
  uint32 addr;
  uint8 vf;
  uint16 idx;
  bool res;

  res = false;

  /* Calculate the base address */
  if(channel < 12u)
  {
    /* channels 0 to 11 linear in memory */
    addr = (uint32)&ADC1->FILT_OUT0.reg;
    idx = (uint16)((uint16)channel << 2u);
    addr += idx;
    pBaseAddr = (volatile uint32 *) addr;

  } 
  else 
  {
    /* channel 12 is not directly after channel 11 --> extra handling needed */
    pBaseAddr = &ADC1->FILT_OUT12.reg;
  }

  /* Pos-/Msk-Fields are taken from Channel0, these values are the same for each result register */
  vf = u1_Field_Rd32(pBaseAddr, (uint8)ADC1_FILT_OUT0_VF0_Pos, ADC1_FILT_OUT0_VF0_Msk);

  /* update the value only if there is valid data in result register */
  if (vf == (uint8)1)
  {
    /* Pos-/Msk-Fields are taken from Channel0, these values are the same for each result register */
    *pVar = u16_Field_Rd32(pBaseAddr, (uint8)ADC1_FILT_OUT0_FILT_OUT_CH0_Pos, ADC1_FILT_OUT0_FILT_OUT_CH0_Msk);
    *pVar >>= 2u;
    res = true;
  }
  return (res);
}

/** \brief Get ADC1 Channel result in millivolt (mV)
 *
 * \param[in] channel Channel number
 * \param[out] *pVar_mV ADC value in mV
 * \retval true *pVar_mV valid
 * \retval false *pVar_mV invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC1 Channel 5 result in Milli Volt (mV).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var; 
 * 
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     if (ADC1_GetChResult(&var, ADC1_CH5) == true)
 *     {
 *	       value_valid = ADC1_GetChResult_mV(&var_mV, ADC1_CH5);
 *     }
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */ 
bool ADC1_GetChResult_mV(uint16 * pVar_mV, uint8 channel)
{
  uint16 var;
  uint32 fs; 
  bool res;
  
  res = ADC1_GetChResult(&var, channel);
  if (res == true)
  {
    if(var > (uint16)0u)
    {
      fs = ADC1_GetChCorrFactMul1024(channel);
      *pVar_mV = (uint16) (((var * fs)-(fs >> 1u)) >> 10u);
    }
    else
	{
	  *pVar_mV = 0;
	}
  }
  return (res);
}

/** \brief Get ADC1 EIM result value
 *
 * \param[out] *pVar ADC value (10bit right aligned in 16 bit return value)
 * \retval true *pVar_mV valid
 * \retval false *pVar_mV invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, programs channel 8 (P2.2) and reads the result value in mV.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var;  
 *	 	 
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_SetEIMChannel( ADC1_P22 ); 
 *     if (ADC1_GetEIMResult(&var) == true)
 *     {
 *	       value_valid = ADC1_GetEIMResult_mV(&var_mV);
 *     } 
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */ 
bool ADC1_GetEIMResult(uint16 * pVar)
{
  uint8 vf;
  bool res;
  res = false;

  vf = u1_Field_Rd32(&ADC1->FILT_OUTEIM.reg, (uint8)ADC1_FILT_OUTEIM_VF_EIM_Pos, ADC1_FILT_OUTEIM_VF_EIM_Msk);

  if (vf == (uint8)1)
  {
	*pVar = u16_Field_Rd32(&ADC1->FILT_OUTEIM.reg, (uint8)ADC1_FILT_OUTEIM_FILT_OUT_EIM_Pos, ADC1_FILT_OUTEIM_FILT_OUT_EIM_Msk);
	*pVar >>= 2u;
    res = true;
  }
  return (res);
}

/** \brief Get ADC1 EIM result value in millivolt
 *
 * \param[out] *pVar_mV ADC value in mV
 * \retval true *pVar_mV valid
 * \retval false *pVar_mV invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, programs channel 8 (P2.2) and reads the result value in mV.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var;  
 *	 	 
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_SetEIMChannel( ADC1_P22 ); 
 *     if (ADC1_GetEIMResult(&var) == true)
 *     {
 *	       value_valid = ADC1_GetEIMResult_mV(&var_mV);
 *     } 
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */ 
bool ADC1_GetEIMResult_mV(uint16 * pVar_mV)
{
  uint16 var;
  uint32 fs;
  bool res;
  
  res = ADC1_GetEIMResult(&var);
  if (res == true)
  {
    if(var > (uint16)0)
    {
      fs = ADC1_GetChCorrFactMul1024(ADC1->CHx_EIM.bit.EIM_CHx);
      *pVar_mV = (uint16) (((var * fs)-(fs >> 1u)) >> 10u);
    }
    else
	{
	  *pVar_mV = 0;
	}
  }
  return (res);
}

/** \brief Get ADC1 software mode result
 *
 * \param[out] *pVar result for the recent software mode conversion
 * \retval true *pVar valid
 * \retval false *pVar invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC1 latest software mode result in Milli Volt (mV).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var;
 * 
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_SW_Ch_Sel( ADC1_P27 ); 
 *     if (ADC1_GetSwModeResult(&var) == true)
 *     {
 *	       value_valid = ADC1_GetSwModeResult_mV(&var_mV);
 *     }  
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~    
 * \ingroup ADC1_api
 */ 
bool ADC1_GetSwModeResult(uint16 * pVar)
{
  uint8 channel;
  bool res;

  channel = ADC1->CTRL_STS.bit.SW_CH_SEL;

  res = ADC1_GetChResult(pVar, channel);
  
  return (res);
}

/** \brief Get ADC1 software mode result in millivolt
 *
 * \param[out] *pVar_mV ADC value in mV of the recent conversion
 * \retval true pVar_mV  valid
 * \retval false pVar_mV  invalid
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC1 latest software mode result in Milli Volt (mV).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     bool value_valid;
 *     uint16 var_mV;
 *     uint16 var;
 * 
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_SW_Ch_Sel( ADC1_P27 ); 
 *     if (ADC1_GetSwModeResult(&var) == true)
 *     {
 *	       value_valid = ADC1_GetSwModeResult_mV(&var_mV);
 *     }  
 *     
 *     return(value_valid); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */ 
bool ADC1_GetSwModeResult_mV(uint16 * pVar_mV)
{
  uint16 var;
  uint32 fs;
  bool result_valid;
  
  result_valid = ADC1_GetSwModeResult(&var);
  if (result_valid == true)
  {
    if(var > (uint16)0)
    {    
      fs = ADC1_GetChCorrFactMul1024(ADC1->CTRL_STS.bit.SW_CH_SEL);
      *pVar_mV = (uint16) (((var * fs)-(fs >> 1u)) >> 10u);
    }
    else
	{
	  *pVar_mV = 0;
	}
  }
  return (result_valid);
}

