/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "adc2.h"
#include "adc2_defines.h"

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Initialize the ADC2 module.
 *
 * \ingroup adc2_api
 */
void ADC2_Init(void)
{
#if ((ADC2_CTRL2 & 1u) != 0u)
  ADC2->CTRL2.reg = (uint32) ADC2_CTRL2;
  ADC2->CTRL4.reg = (uint32) ADC2_CTRL4;
  ADC2->TH0_3_LOWER.reg = (uint32) ADC2_TH0_3_LOWER;
  ADC2->TH0_3_UPPER.reg = (uint32) ADC2_TH0_3_UPPER;
  ADC2->TH4_7_LOWER.reg = (uint32) ADC2_TH4_7_LOWER;
  ADC2->FILT_LO_CTRL.reg = (uint32) ADC2_FILT_LO_CTRL;
  ADC2->FILT_UP_CTRL.reg = (uint32) ADC2_FILT_UP_CTRL;	
#endif
}

/** \brief Get ADC2 Channel result value.
 *
 * \param channel Channel number(defined in adc2.h)
 * \return native ADC2 result value
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC2 Channel 4 result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 data; 
 *
 *	   data = ADC2_GetChResult(ADC2_RESULT_VDDC);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup adc2_api
 */ 
uint16 ADC2_GetChResult(uint8 channel)
{
  const volatile uint32 *pBaseAddr;
  volatile uint16 var;
  uint32 addr;
  uint16 idx;

  addr = (uint32)&ADC2->FILT_OUT0.reg;
  idx = (uint16)((uint16)channel << 2u);
  addr += idx;
  pBaseAddr = (volatile uint32 *) addr;

  var = u16_Field_Rd32(pBaseAddr, (uint8)ADC2_FILT_OUT0_OUT_CH0_Pos, ADC2_FILT_OUT0_OUT_CH0_Msk);

  return (var);
}

/** \brief Get ADC2 Channels 0-4 result in mV and channels 5-6 in °C
 *
 * \param[in] channel Channel number
 * \return sint16 ADC2 result values in mV or °C
 *
 * \brief <b>Example</b><br>
 * \brief This example returns the ADC2 Channel 5 result in °C.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     sint16 data; 
 *
 *	   data = ADC2_GetChResult_mVC(ADC2_RESULT_TEMP1);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup adc2_api
 */ 
sint16 ADC2_GetChResult_mVC(uint8 channel)
{
  uint16 var;
  uint16 fs;
  sint16 result;
  uint32 res_ui32;
  sint32 res_si32;
 
  var = ADC2_GetChResult(channel);
  if (channel == 0u)
  {
    /* Channel 0 in mV */
	/* 1.211 * 1000 / 0.039 / 1024 * 1024 */
    fs = 31051;
	if (var > 0u)
	{
	  /* calculate voltage in mV */
	  res_ui32 = (((uint32)var * (uint32)fs) - ((uint32)fs >> 1u)) >> 10u; 
	  result = (sint16)res_ui32; 
	}
	else
	{
	  /* 0 mV */
	  result = 0;
	}
  }
  else if (channel < 3u) 
  {
    /* Channel 1 and 2 in mV */
	/* 1.211 * 1000 / 0.203 */
    fs = 5966;
	if (var > 0u)
	{
	  /* calculate voltage in mV */
	  res_ui32 = (((uint32)var * (uint32)fs) - ((uint32)fs >> 1u)) >> 10u; 
	  result = (sint16)res_ui32; 
	}
	else
	{
	  /* 0 mV */
	  result = 0;
	}
  }
  else if (channel < 5u) 
  {
    /* Channel 3 and 4 in mV */
	/* 1.211 * 1000 / 0.75 */
    fs = 1615;
	if (var > 0u)
	{
	  /* calculate voltage in mV */
	  res_ui32 = (((uint32)var * (uint32)fs) - ((uint32)fs >> 1u)) >> 10u; 
	  result = (sint16)res_ui32; 
	}
	else
	{
	  /* 0 mV */
	  result = 0;
	}
  }
  else if (channel < 7u)
  {
    /* Channel 5 and 6 in °C */
	/* Calculate temperature in °C = (var*131-69662) / 256 */
    res_si32 = ((((sint32)(131) * (sint32)var) - (sint32)69662)) / (sint32)256;
    result = (sint16)res_si32;
  }
  else
  {
    /* invalid channel */
	result = 0;
  }
  
  return result;
}
