/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "mon.h"
#include "wdt1.h"
#include "mon_defines.h"
#include "int_defines.h"
#include "isr_defines.h"

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Initialize the MON module.
 * 
 * \ingroup mon_api
 */
void MONx_Init(void)
{
#if (CONFIGWIZARD == 1)  
  PMU->MON_CNF1.reg = (uint32) MON_CNF14;
#else /* CONFIGWIZARD == 2 */
  PMU->MON_CNF1.reg = (uint32) PMU_MON_CNF1;
#endif
  SCU->MONIEN.reg = (uint32) SCU_MONIEN;
  SCU->EXICON1.reg = (uint32) SCU_EXICON1;  
#if (UC_FEAT_MON > 4u)
  #if (CONFIGWIZARD == 1)  
    PMU->MON_CNF2.reg = (uint32) MON_CNF5;
  #else /* CONFIGWIZARD == 2 */
    PMU->MON_CNF2.reg = (uint32) PMU_MON_CNF2;
  #endif
#endif
}

/** \brief Read out the MONx Status
 *
 * \param Monx 1..5  MONx status flag
 * \return None
 *
 * \brief <b>Example</b><br> 
 * \brief This example reads MON4 Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 status;
 * 
 *     status = MONx_Get_Status(4u); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup mon_api
 */
uint8 MONx_Get_Status(uint8 Monx)
{
  uint8 result;
  
  if ((Monx < 5u) && (Monx > 0u))
  {
    result = ((uint8)((PMU->MON_CNF1.reg >> ((uint8)((uint8)Monx << 3u) - 1u)) & 1u));
  }
  else
  { 
    result = ((uint8)PMU->MON_CNF2.bit.MON5_STS);
  }
  return result;
}

/** \brief Debounce the MONx input and read the logical input value
 *
 * \param Monx 1..5  MONx status flag 
 * \param MonActiveState 0..1  Expected active state of the MONx input
 * \return None
 *
 * \brief <b>Example</b><br> 
 * \brief This example Debounces the MON2 input to Low Level and reads the logical input value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     MONx_Debounce(2u,0u); 
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup mon_api
 */
uint8 MONx_Debounce(uint8 Monx, uint8 MonActiveState)
{
  uint32 debounce = 1;
  uint8 stat;
  
  if(MonActiveState == (uint8)0u)
  {
	stat = 1;
  }
  else
  {
	stat = 0;
  }
  
  while (debounce > 0u)
  {
  /* wait until MONx button is released and remember that it was pressed */
    while (MONx_Get_Status(Monx) == MonActiveState)
    {
      (void)WDT1_Service();
      stat = MonActiveState;
      debounce = 20000;
    }
    debounce--;
  }
  return (stat);
}
