/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "scu.h"
#include "wdt1.h"
#include "trx.h"
#include "scu_defines.h"
#include "cmsis_misra.h"

/*******************************************************************************
**                         Global Variable Definitions                        **
*******************************************************************************/

uint32 SystemFrequency = (uint32)SCU_FSYS;

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Initialize the system clocks.
 *
 * \ingroup scu_api
 */
void SCU_ClkInit(void)
{
  sint32 int_was_mask;
  /* disable all interrupts                */
  int_was_mask = CMSIS_Irq_Dis();
  /***************************************************************************
   ** NVM Protection Control                                                **
   **************************************************************************/
#if (SCU_NVM_BOOT_PROT == 1u)
  (void)user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, NVM_PASSWORD_SEGMENT_BOOT);
#endif
#if (SCU_NVM_CODE_PROT == 1u)
  (void)user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, NVM_PASSWORD_SEGMENT_CODE);
#endif
#if (SCU_NVM_DATA_PROT == 1u)
  (void)user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, NVM_PASSWORD_SEGMENT_DATA);
#endif
 /***************************************************************************
  ** PLL/SYSCLK Control                                                    **
  **************************************************************************/
  SCU->NMICON.bit.NMIPLL = 0u;

  /* enable XTAL1/2 pins */
  SCU_OpenPASSWD();
  SCU->MODPISEL1.reg = (uint32)(SCU_MODPISEL1 & SCU_MODPISEL1_XTAL12EN_Msk);
  SCU->PASSWD.reg = PASSWD_Close;

  /* select LP_CLK */
  SCU_OpenPASSWD();
  SCU->SYSCON0.bit.SYSCLKSEL = 2u;
  SCU_ClosePASSWD();

  /* Oscillator Select */
  SCU_OpenPASSWD();
  SCU->OSC_CON.reg = (uint32)SCU_OSC_CON;
  SCU_ClosePASSWD();

  SCU_OpenPASSWD();
  SCU->PLL_CON.reg = (uint32) SCU_PLL_CON;
  SCU_ClosePASSWD();

  SCU_OpenPASSWD();
  SCU->CMCON1.reg = (uint32) SCU_CMCON1;
  SCU_ClosePASSWD();

  SCU->PLL_CON.bit.RESLD = 1u;
  /* set PLL_CON.bit.VCOBYP=0 */
  SCU->PLL_CON.reg=(SCU->PLL_CON.reg&~SCU_PLL_CON_VCOBYP_Msk)|SCU_PLL_CON_UNPROT_VCOBYP_Msk;
  /* set PLL_CON.bit.OSCDISC=0 */
  SCU->PLL_CON.reg=(SCU->PLL_CON.reg&~SCU_PLL_CON_OSCDISC_Msk)|SCU_PLL_CON_UNPROT_OSCDISC_Msk;

  while (u1_Field_Rd32(&SCU->PLL_CON.reg, (uint8)SCU_PLL_CON_LOCK_Pos, SCU_PLL_CON_LOCK_Msk) == (uint8)0)
  {
  }

  SCU_OpenPASSWD();
  /* 0u << 6u */
  SCU->SYSCON0.reg = 0u;
  SCU_ClosePASSWD();

  SCU->NMISRCLR.bit.FNMIPLLC = 1u;
  
  /***************************************************************************
   ** Analog Clock Control                                                  **
   ***************************************************************************/

  /* set factor for MI_CLK and Filt_CLK*/
  SCU->APCLK.reg = (uint32) SCU_APCLK;

  /* apply setting by toggling APCLK_SET */
  SCU_OpenPASSWD();
  SCU->APCLK_CTRL.bit.APCLK_SET = 1u;
  SCU_ClosePASSWD();
  CMSIS_NOP();
  SCU_OpenPASSWD();
  SCU->APCLK_CTRL.bit.APCLK_SET = 0u;
  SCU_ClosePASSWD();

  /* enable interrupts                     */
  if (int_was_mask == 0)
  {
    CMSIS_Irq_En();
  }
}

/** \brief Initialize the SCU module.
 *
 * \ingroup scu_api
 */
void SCU_Init(void)
{
  /***************************************************************************
   ** System Clock Output Control                                           **
   ***************************************************************************/
  SCU->COCON.reg = (uint32) (SCU_COCON);

 /***************************************************************************
  ** Module Pin Select                                                     **
  **************************************************************************/
#ifdef SCU_MODPISEL
  SCU->MODPISEL.reg = (uint32) SCU_MODPISEL;
#endif
#ifdef SCU_MODPISEL1
  SCU->MODPISEL1.reg = (uint8) SCU_MODPISEL1;
#endif
#ifdef SCU_MODPISEL2
  SCU->MODPISEL2.reg = (uint8) SCU_MODPISEL2;
#endif
#ifdef SCU_MODPISEL3
  SCU->MODPISEL3.reg = (uint8) SCU_MODPISEL3;
#endif
  SCU->GPT12PISEL.reg = (uint8) SCU_GPT12PISEL;
}

/** \brief Set the device into Sleep Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example Sets the device into Sleep Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_EnterSleepMode();
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup scu_api
 */
#ifndef UNIT_TESTING_LV2
  void __attribute__((noreturn)) SCU_EnterSleepMode(void)
#else
  void SCU_EnterSleepMode(void)
#endif
{
  /* This function assumes the desired wake up sources are enabled already   */
  #if defined (__CC_ARM) /* ARMCC compiler v5 */
    __attribute__((unused)) uint32 dummy;
  #else
    uint32 dummy;
  #endif
  /* set Trx into Sleep Mode           */
  TRX_Set_Mode(TRX_MODE_SLEEP);
  /* wait until Trx switched into Sleep Mode */
  while (TRX_Get_Mode() != TRX_GET_MODE_SLEEP)
  {
  }
  /* wait approx. 100us in order to pass   *
   * the filter time + margin inside the   *
   * Trx. to avoid a false wake up     */
  Delay_us(100u); 
  /* dummy read to clr wake up status to prevent the device from wakeing up  *
   * immediately because of still set wake up flags                          */
  dummy = PMU->WAKE_STATUS.reg;
  dummy = PMU->GPIO_WAKE_STATUS.reg;
  dummy = PMU->CNF_WAKE_FILTER.reg;  
  /* stop WDT1 (SysTick) to prevent any SysTick interrupt to disturb the     *
   * Sleep Mode Entry sequence                                               */
  WDT1_Stop();
  /* Trigger a ShortOpenWindow on WDT1 to prevent an unserviced WDT1 by      *
   * accident                                                                */
  WDT1_SOW_Service(1);
  /* disable all interrupts                   */
  (void)CMSIS_Irq_Dis(); 
  /* open passwd to access PMCON0 register    */
  SCU_OpenPASSWD();
  
  SCU->PMCON0.reg = 0x02; /* set SleepMode flag in PMCON0 */
  CMSIS_WFE();
  CMSIS_WFE();
  /* dont do anything anymore            *
   * device is in SleepMode              */
#ifndef UNIT_TESTING_LV2
  for (;;)
  {
  }
#endif    
    /* device is in Sleep Mode now wake up performes RESET */
  }

/** \brief Set the device into Stop Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example Sets the device into Stop Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_EnterStopMode();
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup scu_api
 */
void SCU_EnterStopMode(void)
{
  uint32 u32_loc_pmcon0;
  sint32 int_was_mask;
  #if defined (__CC_ARM) /* ARMCC compiler v5 */
    __attribute__((unused)) uint32 dummy;
  #else
    uint32 dummy;
  #endif
	uint32 backup_HS_CTRL;
	
	/* backup current HS settings */
	backup_HS_CTRL = HS->CTRL.reg;
	
	/* disable the HSx switch */
	HS->CTRL.bit.HS1_EN = 0;
	HS->CTRL.bit.HS2_EN = 0;
	HS->CTRL.bit.HS1_ON = 0;
	HS->CTRL.bit.HS2_ON = 0;

  /* dummy read to clr wake up status      *
   * to prevent the device from wakeing up *
   * immediately because of still set      *
   * wake up flags                         */
  dummy = PMU->WAKE_STATUS.reg;
  /* dummy read to clr GPIO wake up status */
  dummy = PMU->GPIO_WAKE_STATUS.reg;
  /* stop WDT1 (SysTick) to prevent any    *
   * SysTick interrupt to disturb the      *
   * Sleep Mode Entry sequence             */
  WDT1_Stop();
  /* Trigger a ShortOpenWindow on WDT1     *
   * to prevent an unserviced WDT1 by      *
   * accident                              */
  WDT1_SOW_Service(1u);
  /* disable all interrupts                */
  int_was_mask = CMSIS_Irq_Dis();

  /* select LP_CLK as sys clock            */
  SCU_OpenPASSWD();
  SCU->SYSCON0.bit.SYSCLKSEL = 2U;
  SCU_ClosePASSWD();

  /* select OSC_PLL async.                 */
  SCU_OpenPASSWD();
  SCU->OSC_CON.bit.OSCSS = 3U;
  SCU_ClosePASSWD();

  /* read current PMCON0 content           */
  u32_loc_pmcon0 = SCU->PMCON0.reg;
  /* set PowerDown flag in PMCON0          */
  Field_Mod32(&u32_loc_pmcon0, (uint32)SCU_PMCON0_PD_Pos, (uint32)SCU_PMCON0_PD_Msk, 1U);
  /* open passwd to access PMCON0 register */
  SCU_OpenPASSWD();
  SCU->PMCON0.reg = u32_loc_pmcon0;
  CMSIS_SEV();
  CMSIS_WFE();
  CMSIS_WFE();
  CMSIS_NOP();
  CMSIS_NOP();
  CMSIS_NOP();
  CMSIS_NOP();

  /* dont do anything anymore            *
   * device is in PowerDown Mode         */
  /***************************************/
  /********* DEVICE IN STOP MODE *********/
  /***************************************/
  /* Device has been woken up            */
  /* reset PLL locking                   */
  SCU->PLL_CON.bit.RESLD = 1;
  /* set PLL_CON.bit.VCOBYP=0 */
  SCU->PLL_CON.reg=(SCU->PLL_CON.reg&~SCU_PLL_CON_VCOBYP_Msk) | SCU_PLL_CON_UNPROT_VCOBYP_Msk;
  /* set PLL_CON.bit.OSCDISC=0 */
  SCU->PLL_CON.reg=(SCU->PLL_CON.reg&~SCU_PLL_CON_OSCDISC_Msk) | SCU_PLL_CON_UNPROT_OSCDISC_Msk;
    
  /* wait for PLL being locked           */
  while (u1_Field_Rd32(&SCU->PLL_CON.reg, (uint8)SCU_PLL_CON_LOCK_Pos, SCU_PLL_CON_LOCK_Msk) == (uint8)0)
  {
  }
  
  /* restore previous HS settings */
  PMU->HIGHSIDE_CTRL.reg = (uint32)0;
  if (HS->IRQS.bit.HS1_OT_STS == 1u) 
  {
  	HS->CTRL.bit.HS1_EN = 1u;
  	HS->IRQCLR.reg = (uint32)0x00002020;
  }
  if (HS->IRQS.bit.HS2_OT_STS == 1u)
  {
  	HS->CTRL.bit.HS2_EN = 1u;
  	HS->IRQCLR.reg = (uint32)0x20200000;
  }
  HS->CTRL.reg = backup_HS_CTRL;

  /* switch fSYS back to                 *
   * user configuration                  */
  SCU_OpenPASSWD();
  SCU->SYSCON0.reg = (uint8)0;
  SCU_ClosePASSWD();

  /* enable interrupts                     */
  if (int_was_mask == 0)
  {
    CMSIS_Irq_En();
  }
  
  /* Init SysTick                          */
  SysTick_Init();
  
  /* Init and service WDT1                 */
  WDT1_Init();
}

/** \brief Set the device into Slow Down Mode
 * \warning At prescalers of 384 or higher 
 * the regular Systick execution at an 1ms interval is not possible anymore.
 * The functions WDT1_Init_SlowMode needs to set the Systick interrupt
 * to a lower calling frequency, 
 * WDT1_Window_Count also needs to be adapted in this case.
 *
 * \param divider_scaled Divider from 1 up to 512 scaled to a value from 0 to 15;
 * for scaling see User manual, register CMCON1->CLKREL.
 * The macros SLOWDOWN_PRESCALER_x are recommended as parameter.
 *
 * \brief <b>Example</b><br>
 * \brief This example Sets the device into Slow Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_EnterSlowMode(SLOWDOWN_PRESCALER_16);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup scu_api
 */
void SCU_EnterSlowMode(uint8 divider_scaled)
{
  sint32 int_was_mask;
  uint16 divider[16] = {1, 2, 3, 4, 8, 16, 24, 32, 48, 64, 96, 128, 192, 256, 384, 512};
  uint32 reload_value;
  uint8 divider_scaled_local = divider_scaled;
  
  if(divider_scaled_local > (uint8)0x0F)
  {
    divider_scaled_local = 0x0F;
  }
  
  /* set the Clock Output Divider */
  Field_Mod32(&SCU->CMCON1.reg, (uint8)SCU_CMCON1_CLKREL_Pos, SCU_CMCON1_CLKREL_Msk, divider_scaled_local);
  /* stop WDT1 (SysTick) to prevent any    *
   * SysTick interrupt to disturb the      *
   * Sleep Mode Entry sequence             */
  WDT1_Stop();

  /* Trigger a ShortOpenWindow on WDT1     *
   * to prevent an unserviced WDT1 by      *
   * accident                              */
  WDT1_SOW_Service(3u);

  /* disable all interrupts                */
  int_was_mask = CMSIS_Irq_Dis();

  /* open passwd to access PMCON0 register */
  SCU_OpenPASSWD();
  /* set SlowMode flag in PMCON0           */
  Field_Wrt32(&SCU->PMCON0.reg, (uint8)SCU_PMCON0_SD_Pos, SCU_PMCON0_SD_Msk, 0x1);
  SCU_ClosePASSWD();
  
  /***************************************/
  /********* DEVICE IN SLOW MODE *********/
  /***************************************/
  /* enable interrupts                     */
  if (int_was_mask == 0)
  {
    CMSIS_Irq_En();
  }
  
  //lint -e9029
  /* set reload value of systick           */
  reload_value = ((uint32)SCU_FSYS /  divider[divider_scaled_local]) / (uint32)SysTickFreq;
  //lint +e9029
  /* program SysTick Timer */
  SysTick_ReloadValue_Set(reload_value);
  
  CPU->SYSTICK_CVR.bit.CURRENT = 0x0;
  
  /* Enable SysTick                        */
  CPU->SYSTICK_CSR.bit.ENABLE = 1;
  
  /* Init and service WDT1                 */
  WDT1_Init();
}

/** \brief Get the device out of Slow Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example Gets the device out of Slow Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_ExitSlowMode();
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup scu_api
 */
void SCU_ExitSlowMode(void)
{
  sint32 int_was_mask;
  /* stop WDT1 (SysTick) to prevent any    *
   * SysTick interrupt to disturb the      *
   * Slow Mode Exit sequence               */
  WDT1_Stop();

  /* Trigger a ShortOpenWindow on WDT1     *
   * to prevent an unserviced WDT1 by      *
   * accident                              */
  WDT1_SOW_Service(3);

  /* disable all interrupts                */
  int_was_mask = CMSIS_Irq_Dis();

  /* open passwd to access PMCON0 register */
  SCU_OpenPASSWD();
  /* reset SlowDown flag in PMCON0         */
  Field_Wrt32(&SCU->PMCON0.reg, (uint8)SCU_PMCON0_SD_Pos, (uint8)SCU_PMCON0_SD_Msk, 0);
  SCU_ClosePASSWD();
  
  /***************************************/
  /***** DEVICE back in NORMAL MODE ******/
  /***************************************/
  /* enable interrupts                     */
  if (int_was_mask == 0)
  {
    CMSIS_Irq_En();
  }
    
  /* Init SysTick                          */
  SysTick_Init();
  
  /* Init and service WDT1                 */
  WDT1_Init();
}

/** \brief Set the Write/Read Protection for the Code/Data Flash
 *
 * \param[in] Protection: BOOT/CODE/DATA
 * \param[in] Action: CLEAR/SET
 * \return ERR_LOG_SUCCESS or error code
 *
 * \brief <b>Example</b><br>
 * \brief This example writes Protection for the Code Flash.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SCU_ChangeNVMProtection(NVM_PASSWORD_SEGMENT_CODE, PROTECTION_SET);
 * }
 * ~~~~~~~~~~~~~~~   
 * \ingroup scu_api
 */
sint32 SCU_ChangeNVMProtection(NVM_PASSWORD_SEGMENT_t Protection, uint8 Action)
{
  sint32 result;

  if (Action == PROTECTION_CLEAR)
  {
    if (Protection == NVM_PASSWORD_SEGMENT_BOOT)
    {
      result = user_nvm_protect_clear((uint32) SCU_NVM_PROT_PW, Protection);
    }
    else if (Protection == NVM_PASSWORD_SEGMENT_CODE)
    {
      result = user_nvm_protect_clear((uint32) SCU_NVM_PROT_PW, Protection);
    }
    else if (Protection == NVM_PASSWORD_SEGMENT_DATA)
    {
      result = user_nvm_protect_clear((uint32) SCU_NVM_PROT_PW, Protection);
    }
    else
    {
      result = (sint32)ERR_LOG_ERROR;
    }
  }
  else
  {
    if (Protection == NVM_PASSWORD_SEGMENT_BOOT)
    {
      result = user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, Protection);
    }
    else if (Protection == NVM_PASSWORD_SEGMENT_CODE)
    {
      result = user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, Protection);
    }  
    else if (Protection == NVM_PASSWORD_SEGMENT_DATA)
    {
      result = user_nvm_protect_set((uint32) SCU_NVM_PROT_PW, Protection);
    }
    else
    {
      result = (sint32)ERR_LOG_ERROR;
    }
  }
  return result;
}
