/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "uart.h"
#include "wdt1.h"
#include "sfr_access.h"
#include "scu_defines.h"

/*******************************************************************************
**                       External Variable Declarations                       **
*******************************************************************************/

static TUart uart1 = {0};
static TUart uart2 = {0};

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Initialize the UART1 module.
 *
 * \ingroup uart_api
 */
void UART1_Init(void)
{
#if (UART1_Configuration_En == 1u)
  UART1->SCON.reg = UART1_SCON;
  SCU->BCON1.reg = 0;
  SCU->BG1.reg =  UART1_BRVAL;
  SCU->BGL1.reg = UART1_FD;
  SCU->BCON1.reg = SCU_BCON1;
  UART1->SCONCLR.bit.RICLR = 1u;
  UART1->SCONCLR.bit.TICLR = 1u;
  uart1.clock = UART1_CLK;
#endif
}

/** \brief Set the baudrate of UART1.
 *
 * \param baudrate, e.g. 19200, or 115200
 *
 * \brief <b>Example</b><br>
 * \brief This example sets UART1 BAUDRATE to 19200.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART1_BaudRate_Set(19200);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup uart_api
 */
void UART1_BaudRate_Set(uint32 baudrate)
{
  uint16 brval_u11q5;
  
  if ((baudrate >= (uint16)1221) && (baudrate <= (uint32)1250000))
  {
    brval_u11q5 = (uint16)(((((uint32)uart1.clock * (uint32)1E6) << 1u) / baudrate));
  
    UART1_BaudRateGen_Dis();
  
    UART1_BaudRate_Value_Set(brval_u11q5);
  
    UART1_BaudRateGen_En();
  } 
}

/** \brief Initialize the UART2 module.
 *
 * \ingroup uart_api
 */
void UART2_Init(void)
{
#if (UART2_Configuration_En == 1u)
  UART2->SCON.reg = (uint32) UART2_SCON;
  SCU->BCON2.reg = (uint32) 0;
  SCU->BG2.reg = (uint32) UART2_BRVAL;
  SCU->BGL2.reg = (uint32) UART2_FD;
  SCU->BCON2.reg = (uint32) SCU_BCON2;
  UART2->SCONCLR.bit.RICLR = 1u;
  UART2->SCONCLR.bit.TICLR = 1u;
  uart2.clock = UART2_CLK;
#endif
}

/** \brief Set the baudrate of UART2.
 *
 * \param baudrate, e.g. 19200, or 115200
 *
 * \brief <b>Example</b><br>
 * \brief This example sets UART2 BAUDRATE to 19200.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     UART2_BaudRate_Set(19200);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup uart_api
 */
void UART2_BaudRate_Set(uint32 baudrate)
{
  uint16 brval_u11q5;
  
  if ((baudrate >= (uint16)1221) && (baudrate <= (uint32)1250000))
  {
    brval_u11q5 = (uint16)(((((uint32)uart2.clock * (uint32)1E6) << 1u) / baudrate));
  
    UART2_BaudRateGen_Dis();

    UART2_BaudRate_Value_Set(brval_u11q5);
  
    UART2_BaudRateGen_En();
  }
}

#if ((UART1_STD_EN == 1u) && (UART2_STD_EN == 0u))
/** \brief Send a character via UART1.
 *
 * \param[in] Char Character to send
 * \return Sent character
 *
 * \ingroup uart_api
 */
sint32 stdout_putchar(sint32 Char)
{
  UART1_Send_Byte((uint8)Char);
  
  if (Char == (sint32)'\n')
  {
    /* line feed: */
    while (UART1_TX_Sts() == 0u)
    {
      /* Execute wait function until character is transmitted */
      (void)WDT1_Service();
    }
    /* Clear TI bit */
    UART1->SCONCLR.reg = UART_MASK_INT_TI;

    /* Send additional carriage return */
    UART1->SBUF.reg = (uint8)'\r';
  }
  while (UART1_TX_Sts() == 0u)
  {
    /* Execute wait function until character is transmitted */
    (void)WDT1_Service();
  }
  return Char;
} /* End of stdout_putchar */ 


/** \brief Receive a character via UART1.
 *
 * \return Received character
 *
 * \ingroup uart_api
 */
sint32 stdin_getchar(void)
{
  while (UART1_RX_Sts() == 0u)
  {
    /* Execute wait function until character is received */
    (void)WDT1_Service();
  }

  /* Clear RI bit */
  UART1->SCONCLR.reg = UART_MASK_INT_RI;

  /* Get character */
  return (sint32)UART1->SBUF.reg;
} /* End of stdin_getchar */ 

/** \brief TTY write character via UART1.
 *
 * \ingroup uart_api
 */
void ttywrch(sint32 ch)
{
  (void)stdout_putchar(ch);
}
#endif /* ((UART1_STD_EN == 1u) && (UART2_STD_EN == 0u)) */

#if ((UART1_STD_EN == 0u) && (UART2_STD_EN == 1u))
/** \brief Send a character via UART2.
 *
 * \param Char Character to send
 * \return Sent character
 *
 * \ingroup uart_api
 */
sint32 stdout_putchar(sint32 Char)
{
  UART2_Send_Byte((uint8)Char);
  
  if (Char == (sint32)'\n')
  {
    /* line feed: */
    while (UART2_TX_Sts() == 0u)
    {
      /* Execute wait function until character is transmitted */
      (void)WDT1_Service();
    }
    /* Clear TI bit */
    UART2->SCONCLR.reg = UART_MASK_INT_TI;

    /* Send additional carriage return */
    UART2->SBUF.reg = (uint8)'\r';
  }
  while (UART2_TX_Sts() == 0u)
  {
    /* Execute wait function until character is transmitted */
    (void)WDT1_Service();
  }
  return Char;
} /* End of stdout_putchar */ 


/** \brief Receive a character via UART2.
 *
 * \return Received character
 *
 * \ingroup uart_api
 */
sint32 stdin_getchar(void)
{
  while (UART2_RX_Sts() == 0u)
  {
    /* Execute wait function until character is received */
    (void)WDT1_Service();
  }

  /* Clear RI bit */
  UART2->SCONCLR.reg = UART_MASK_INT_RI;

  /* Get character */
  return (sint32)UART2->SBUF.reg;
} /* End of stdin_getchar */ 

/** \brief TTY write character via UART2.
 *
 * \ingroup uart_api
 */
void ttywrch(int ch)
{
  (void)stdout_putchar(ch);
}
#endif /* ((UART1_STD_EN == 0u) && (UART2_STD_EN == 1u)) */

#if ((UART1_STD_EN == 1u) && (UART2_STD_EN == 1u))
 #error only one UART can be selected as STDIN/STDOUT device at a time
#endif
