/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "wdt1.h"
#include "scu_defines.h"

/*******************************************************************************
**                         Private Macro Definitions                          **
*******************************************************************************/

#define SYSTICK_DELAY_THRESHOLD 100U

/*******************************************************************************
**                        Private Variable Declarations                       **
*******************************************************************************/

/**\brief Window counter for WDT1. It is incremented by the SysTick interrupt
 *\brief on every Millisecond. It is used by the function WDT1_Service()
 *\brief to determine the right service point for the WDT1.
 */
uint32 WD_Counter;
/**\brief is set in case a WDT1 Short Open Window is active.*/
static bool bSOWactive;

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Perform the initial service of the WDT1 (closes the long open window). 
 *\brief Reset the \link WD_Counter \endlink to '0'.
 *
 * \ingroup wdt1_api
 */
void WDT1_Init(void)
{
  /* trigger inital WDT1 service */
  SCUPM->WDT1_TRIG.reg = (uint8) SCUPM_WDT1_TRIG;
  /* reset window counter */
  WD_Counter = 0u;
  /* reset SOW active signal */
  bSOWactive = false;
}

/** \brief Initialize the SysTick timer to 1ms interval and enable the SysTick interrupt.
 *
 * \ingroup wdt1_api
 */
void SysTick_Init(void)
{
  /* program SysTick Timer; SysTickRL: SysTick reload value based on SystemFrequency calculated in wdt1.h */
  CPU->SYSTICK_RVR.reg = (uint32)SysTickRL;
  /* reset SysTick timer */
  CPU->SYSTICK_CVR.reg = 0u;
  /* CLKSRC=CPU clk */ 
  CPU->SYSTICK_CSR.bit.CLKSOURCE = 1u;
  /* TICK Interrupt = enabled */
  CPU->SYSTICK_CSR.bit.TICKINT = 1u;
  /* ENABLE SysTick Timer */
  CPU->SYSTICK_CSR.bit.ENABLE = 1u;
}

/** \brief Stop the service of WDT1 by stopping the SysTick timer.
 * 
 *\warning Handle this function with care, as a WDT1 itself will not be stopped,
 *\warning just the service of it is stopped. This might lead to device resets.
 *
 * \brief <b>Example</b><br>
 * \brief This example Stops the WDT1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     WDT1_Stop();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup wdt1_api
 */
void WDT1_Stop(void)
{
  /* disable SysTick Timer */
  CPU->SYSTICK_CSR.bit.ENABLE = 0u;
}

/** \brief Service the WDT1 in the open window.
 *
 * \retval 0 WDT1 not serviced (not in open window)
 * \retval 1 WDT1 serviced
 *
 * \brief <b>Example</b><br>
 * \brief This example services the WDT1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     (void)WDT1_Service();
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup wdt1_api
 */
uint8 WDT1_Service(void)
{
  uint8 result;
  result = 0;
  /* check if Window Counter is beyond 70% of WDT1 period  *
   * or if a SOW service has been done before              */

/*lint -e9050 Supressing MISRA 2012 Rule 12.1 */
  if ((WD_Counter > (uint32)SCUPM_WDT1_TRIGGER) || (bSOWactive == true))
/*lint +e9050 */
  {
    /* service WDT1 */
    SCUPM->WDT1_TRIG.reg = (uint8) SCUPM_WDT1_TRIG;
    /* reset window counter */
    WD_Counter = 0u;
    /* reset "short open window" active flag */
    bSOWactive = false;

    result = 1;
  }
  return (result);
}

/** \brief Trigger a short-window (~30ms) of the WDT1.
 *
 * \param NoOfSOW number of consecutive Short open windows allowed
 *
 * \brief <b>Example</b><br>
 * \brief This example sets one successive Short Open Window to be allowed and triggers a SOW.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     WDT1_SOW_Service(1u);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup wdt1_api
 */
void WDT1_SOW_Service(uint32 NoOfSOW)
{
  SCUPM->WDT1_TRIG.reg = (NoOfSOW & 3u) << 6u;
  bSOWactive = true;
}

/** \brief Delay the regular program execution by a given number of microseconds.
 * \brief The function returns if the given time has elapsed.
 * \brief Small microsecond delay times are getting falsen by the execution time of the function itself.
 * 
 * \warning Handle this function with care, as WDT1/WDT will not be serviced during the delay time. 
 * \warning The user has to take care of WDT1/WDT service by himself.
 * 
 * \param delay_time_us Delay time in Microseconds
 *
 * \brief <b>Example</b><br>
 * \brief This example sets a delay of 100 us.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     Delay_us(100);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup wdt1_api
 */
void Delay_us(uint32 delay_time_us)
{
  uint32 systick_target_val;
  uint32 systick_val;
  uint32 delay_count;
  uint32 systick_cur;
  uint32 systick_rl;

  /* Get current systick value */
  systick_rl = Systick_ReloadValue_Get();
  /* Adapt systick value into range SYSTICK_DELAY_THRESHOLD...reload value-SYSTICK_DELAY_THRESHOLD to avoid getting stuck in a while, wait in case needed */
  do
  {
    systick_val = Systick_Value_Get();
  } while ((systick_val < SYSTICK_DELAY_THRESHOLD) || (systick_val > (systick_rl - SYSTICK_DELAY_THRESHOLD)));

  while (delay_time_us >= (uint32)1000)
  {
    /* Wait for underflow */
    while (Systick_Value_Get() < systick_val)
    {}

    /* Wait target underflowed */
    while (Systick_Value_Get() > systick_val)
    {}

    delay_time_us -= (uint32)1000;
  }

  delay_count = delay_time_us * (uint32)One_us;

  /* Adapt systick value into range SYSTICK_DELAY_THRESHOLD...reload value-SYSTICK_DELAY_THRESHOLD to avoid getting stuck in a while, wait in case needed */
  systick_val = Systick_Value_Get();
	
  if (systick_val >= delay_count)
  {
    systick_target_val = systick_val - delay_count;

    /* Wait underflow with 1 countdown detection */
    do
    {
      systick_cur = Systick_Value_Get();
    }
    while ((systick_cur > systick_target_val) && (systick_cur < systick_val));
  }
  else
  {
    systick_target_val = systick_rl - ( delay_count - systick_val );

    /* Wait for underflow */
    while (Systick_Value_Get() < systick_val)
    {}

    /* Wait target underflowed */
    while (Systick_Value_Get() > systick_target_val)
    {}
  }
  
  return;
}







