/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/**
 * \file     adc1.h
 *
 * \brief    ADC1 low level access library
 *
 * \version  V0.2.5
 * \date     23. Jun 2022
 *
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AV           Axel Viebig                                                   **
** MK           Martin Kilian                                                 **
** KC           Kay Claussen                                                  **
** CE           Christian Engel                                               **
** TS           T&S                                                           **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
** VO           Vanessa Ongaro                                                **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2015-10-23, AV:   Initial version                                  **
** V0.1.1: 2015-11-03, AV:   Several changes for DU mode                      **
** V0.1.2: 2015-11-11, AV:   Range corrections in INLINE functions (adc1.h)   **
**                           DU mode changes due to reorganized CW XML        **
**                           Introduced waiting for acknowledge on mode change**
** V0.1.3: 2015-11-18, AV:   Improved init routine to allow for clean startup **
**                            without having a bunch of irrelevant interrupts **
** V0.1.4: 2017-04-20, MK:   Added define macros for registers access         **
** V0.1.5: 2017-11-10, KC:   Six inline fuctions converted to non-inline      **
**                           Calculation in mV functions corrected            **
** V0.1.6: 2017-11-13, KC:   Calculation result for Ch6 pending on device     **
**                           Added 2 macros for clearing ADC interrupts       **
** V0.1.7: 2017-12-20, KC:   mV calcualtion corrected                         **
** V0.1.8: 2018-11-05, CE:   ADC1_GetChResult: Non-linearity of channel 12    **
**                           offset address corrected                         **
**                           ADC1_DPP1_En() and ADC1_DPP1_Dis() corrected     **
**                           ConfigWizard2 integration:                       **
**                               ADC1_CHX_ESM renamed to ADC1_CHx_ESM,        **
**                               ADC1_CHX_EIM renamed to ADC1_CHx_EIM         **
**                           Macros formed into inline functions              **
**                           SFR bitfield accesses replaced by functions      **
**                           (Field_Mod(), ...)                               **
**                           DoxyGen docu added                               **
** V0.1.9: 2019-03-18, TS:   All positions for Field_X functions are cast     **
**                           with uint8                                       **
**                           ADC1_GetSwModeResult function updated            **
** V0.2.0: 2019-05-07, JO:   Removed function ADC1_SOOC_Set                   **
** V0.2.1: 2021-08-16, BG:   EP-731: Updated revision history layout          **
**                           Added inline functions declarations              **
**                           Moved DoxyGen documentation for global functions **
**                           to adc1.c                                        **
** V0.2.2: 2021-10-07, BG:   EP-931: Corrected ARMCC compiler v6 errors       **
** V0.2.3: 2021-12-13, JO:   EP-933: Updated copyright and branding           **
** V0.2.4: 2021-12-21, BG:   EP-997: Corrected MISRA 2012 violations          **
** V0.2.5: 2022-06-23, VO:   EP-1083: Added missing registers to init (only   **
**                           for CW2): ADC1_CNTx_x_LOWER, ADC1_CNTx_x_UPPER,  **
**                           ADC1_DCHCNT1_4_UPPER/LOWER, ADC1_MMODE0_11       **
*******************************************************************************/

#ifndef ADC1_H
#define ADC1_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/

#include "tle984x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/

/* Mode Selection */
/**\brief ADC1 Mode Selection, Software Mode */
#define SOFT_MODE (1u)
/**\brief Mode Selection, Sequencer Mode */
#define SEQ_MODE  (0u)

/* Sequencer Configuration */
/**\brief ADC1 Sequencer Configuration, skip channel measurement */
#define SKIP      (0u)
/**\brief Sequencer Configuration, perform channel measurement */
#define MEAS    (1u)

/* Data Width Selection */
/**\brief ADC1 conversion data width selection, 10 bit conversion */
#define BIT10     (0u)
/**\brief ADC1 conversion data width selection, 8 bit conversion */
#define BIT8      (1u)

/* Interrupt Enable/Disable */
/**\brief ADC1 Interrupt, Disable*/
#define INTDIS    (0u)
/**\brief ADC1 Interrupt, Enable*/
#define INTEN     (1u)

/* Result Register WFR mode */
/**\brief ADC1 Result Register, Overwrite Mode*/
#define OVERWRITE (0u)
/**\brief ADC1 Result Register, Wait For Read Mode*/
#define WFR       (1u)

/* Result Register Channel */
/**\brief ADC1 channel selection macro, channel 0 - VBAT_SENSE */
#define ADC1_CH0  (0u)
/**\brief ADC1 channel selection macro, channel 1 - VS */
#define ADC1_CH1  (1u)
/**\brief ADC1 channel selection macro, channel 2 - MON1 */
#define ADC1_CH2  (2u)
/**\brief ADC1 channel selection macro, channel 3 - MON2 */
#define ADC1_CH3  (3u)
/**\brief ADC1 channel selection macro, channel 4 - MON3 */
#define ADC1_CH4  (4u)
/**\brief ADC1 channel selection macro, channel 5 - MON4 */
#define ADC1_CH5  (5u)
/**\brief ADC1 channel selection macro, channel 6 - MON5 */
#define ADC1_CH6  (6u)
/**\brief ADC1 channel selection macro, channel 7 - P2.1 */
#define ADC1_CH7  (7u)
/**\brief ADC1 channel selection macro, channel 8 - P2.2 */
#define ADC1_CH8  (8u)
/**\brief ADC1 channel selection macro, channel 9 - P2.3 */
#define ADC1_CH9  (9u)
/**\brief ADC1 channel selection macro, channel 10 - P2.6 */
#define ADC1_CH10 (10u)
/**\brief ADC1 channel selection macro, channel 11 - P2.7 */
#define ADC1_CH11 (11u)
/**\brief ADC1 channel selection macro, channel 12 - P2.0 */
#define ADC1_CH12 (12u)

/* ADC1 Channel Connections */
/**\brief ADC1 channel selection macro, channel 0 - VBAT_SENSE */
#define ADC1_VBAT_SENSE (ADC1_CH0)
/**\brief ADC1 channel selection macro, channel 1 - VS */
#define ADC1_VS         (ADC1_CH1)
/**\brief ADC1 channel selection macro, channel 2 - MON1 */
#define ADC1_MON1       (ADC1_CH2)
/**\brief ADC1 channel selection macro, channel 3 - MON2 */
#define ADC1_MON2       (ADC1_CH3)
/**\brief ADC1 channel selection macro, channel 4 - MON3 */
#define ADC1_MON3       (ADC1_CH4)
/**\brief ADC1 channel selection macro, channel 5 - MON4 */
#define ADC1_MON4       (ADC1_CH5)
/**\brief ADC1 channel selection macro, channel 6 - MON5 */
#define ADC1_MON5       (ADC1_CH6)
/**\brief ADC1 channel selection macro, channel 7 - P2.1 */
#define ADC1_P21        (ADC1_CH7)
/**\brief ADC1 channel selection macro, channel 8 - P2.2 */
#define ADC1_P22        (ADC1_CH8)
/**\brief ADC1 channel selection macro, channel 9 - P2.3 */
#define ADC1_P23        (ADC1_CH9)
/**\brief ADC1 channel selection macro, channel 10 - P2.6 */
#define ADC1_P26        (ADC1_CH10)
/**\brief ADC1 channel selection macro, channel 11 - P2.7 */
#define ADC1_P27        (ADC1_CH11)
/**\brief ADC1 channel selection macro, channel 12 - P2.0 */
#define ADC1_P20        (ADC1_CH12)

/* Sequencer/ESM Channel Enable Bitposition */
/* ADC1_MASK_CHx Bit */
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 0 - VBAT_SENSE */
#define ADC1_MASK_CH0  (1u << 0u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 1 - VS */
#define ADC1_MASK_CH1  (1u << 1u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 2 - MON1 */
#define ADC1_MASK_CH2  (1u << 2u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 3 - MON2 */
#define ADC1_MASK_CH3  (1u << 3u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 4 - MON3 */
#define ADC1_MASK_CH4  (1u << 4u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 5 - MON4 */
#define ADC1_MASK_CH5  (1u << 5u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 6 - MON5 */
#define ADC1_MASK_CH6  (1u << 6u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 7 - P2.1 */
#define ADC1_MASK_CH7  (1u << 7u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 8 - P2.2 */
#define ADC1_MASK_CH8  (1u << 8u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 9 - P2.3 */
#define ADC1_MASK_CH9  (1u << 9u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 10 - P2.6 */
#define ADC1_MASK_CH10 (1u << 10u)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 11 - P2.7 */
#define ADC1_MASK_CH11 (1u << 11u)

/**\brief ADC1 Sequencer/ESM channel selection macro, channel 0 - VBAT_SENSE */
#define ADC1_MASK_VBAT_SENSE (ADC1_MASK_CH0)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 1 - VS */
#define ADC1_MASK_VS         (ADC1_MASK_CH1)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 2 - MON1 */
#define ADC1_MASK_MON1       (ADC1_MASK_CH2)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 3 - MON2 */
#define ADC1_MASK_MON2       (ADC1_MASK_CH3)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 4 - MON3 */
#define ADC1_MASK_MON3       (ADC1_MASK_CH4)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 5 - MON4 */
#define ADC1_MASK_MON4       (ADC1_MASK_CH5)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 6 - MON5 */
#define ADC1_MASK_MON5       (ADC1_MASK_CH6)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 7 - P2.1 */
#define ADC1_MASK_P21        (ADC1_MASK_CH7)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 8 - P2.2 */
#define ADC1_MASK_P22        (ADC1_MASK_CH8)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 9 - P2.3 */
#define ADC1_MASK_P23        (ADC1_MASK_CH9)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 10 - P2.6 */
#define ADC1_MASK_P26        (ADC1_MASK_CH10)
/**\brief ADC1 Sequencer/ESM channel selection macro, channel 11 - P2.7 */
#define ADC1_MASK_P27        (ADC1_MASK_CH11)

/* ADC1_CTRL3 Sample Time Definitions*/
/**\brief ADC1 Sample Time Selection Macro, MICLK4 */
#define ADC1_MICLK4  (0u)
/**\brief ADC1 Sample Time Selection Macro, MICLK6 */
#define ADC1_MICLK6  (1u)
/**\brief ADC1 Sample Time Selection Macro, MICLK8 */
#define ADC1_MICLK8  (2u)
/**\brief ADC1 Sample Time Selection Macro, MICLK10 */
#define ADC1_MICLK10 (3u)
/**\brief ADC1 Sample Time Selection Macro, MICLK12 */
#define ADC1_MICLK12 (4u)
/**\brief ADC1 Sample Time Selection Macro, MICLK14 */
#define ADC1_MICLK14 (5u)
/**\brief ADC1 Sample Time Selection Macro, MICLK16 */
#define ADC1_MICLK16 (6u)
/**\brief ADC1 Sample Time Selection Macro, MICLK18 */
#define ADC1_MICLK18 (7u)
/**\brief ADC1 Sample Time Selection Macro, MICLK20 */
#define ADC1_MICLK20 (8u)
/**\brief ADC1 Sample Time Selection Macro, MICLK22 */
#define ADC1_MICLK22 (9u)

/* ADC1 Overvoltage Measurement Mode */
/**\brief ADC1 Overvoltage Measurement Mode Macro, upper and lower voltage */
#define ADC1_MMODE_0  0u
/**\brief ADC1 Overvoltage Measurement Mode Macro, undervoltage */
#define ADC1_MMODE_UV 1u
/**\brief ADC1 Overvoltage Measurement Mode Macro, overvoltage */
#define ADC1_MMODE_OV 2u

/* ADC1 Lower Hysteresis Trigger Level  */
/**\brief ADC1 Lower Hysteresis Trigger Level Macro, HYSTOFF */
#define ADC1_HYST_OFF (0u)
/**\brief ADC1 Lower Hysteresis Trigger Level Macro, HYST4 */
#define ADC1_HYST_4   (1u)
/**\brief ADC1 Lower Hysteresis Trigger Level Macro, HYST8 */
#define ADC1_HYST_8   (2u)
/**\brief ADC1 Lower Hysteresis Trigger Level Macro, HYST16 */
#define ADC1_HYST_16  (3u)

/* ADC1 Lower Counter Trigger Level  */
/**\brief ADC1 Lower Counter Trigger Level Macro, 1 measurement */
#define ADC1_CNT_1  (0u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 2 measurements */
#define ADC1_CNT_2  (1u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 4 measurements */
#define ADC1_CNT_4  (2u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 8 measurements */
#define ADC1_CNT_8  (3u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 16 measurements */
#define ADC1_CNT_16 (4u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 32 measurements */
#define ADC1_CNT_32 (5u)
/**\brief ADC1 Lower Counter Trigger Level Macro, 63 measurements */
#define ADC1_CNT_63 (6u)

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/

/** \enum du_triggen_sel
 *  \brief This enum lists the Differential Unit Trigger options
 */
typedef enum _du_triggen_sel
{
  DU_TRIGGEN_CC60   = (uint32)0u, /**< CC60 is selected */
  DU_TRIGGEN_CC61   = (uint32)1u, /**< CC61 is selected */
  DU_TRIGGEN_CC62   = (uint32)2u, /**< CC62 is selected */
  DU_TRIGGEN_COUT60 = (uint32)3u, /**< COUT60 is selected */
  DU_TRIGGEN_COUT61 = (uint32)4u, /**< COUT61 is selected */
  DU_TRIGGEN_COUT62 = (uint32)5u, /**< COUT62 is selected */
  DU_TRIGGEN_T3OUT  = (uint32)6u, /**< T3OUT is selected */
  DU_TRIGGEN_COUT63 = (uint32)7u, /**< COUT63 is selected */
  DU_TRIGGEN_Ch0t   = (uint32)0u,
  DU_TRIGGEN_Ch1t   = (uint32)1u,
  DU_TRIGGEN_Ch2t   = (uint32)2u,
  DU_TRIGGEN_Ch0c   = (uint32)3u,
  DU_TRIGGEN_Ch1c   = (uint32)4u,
  DU_TRIGGEN_Ch2c   = (uint32)5u,
  DU_TRIGGEN_Out_T3 = (uint32)6u,
  DU_TRIGGEN_Ch3c   = (uint32)7u
} du_triggen_sel;

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/

void ADC1_Init(void);
bool ADC1_GetChResult(uint16 * pVar, uint8 channel);
bool ADC1_GetChResult_mV(uint16 * pVar_mV, uint8 channel);
bool ADC1_GetEIMResult(uint16 * pVar);
bool ADC1_GetEIMResult_mV(uint16 * pVar_mV);
bool ADC1_GetSwModeResult(uint16 * pVar);
bool ADC1_GetSwModeResult_mV(uint16 * pVar_mV);
INLINE void ADC1_Power_On(void);
INLINE void ADC1_Power_Off(void);
INLINE void ADC1_SOS_Set(void);
INLINE void ADC1_DPP1_En(void);
INLINE void ADC1_DPP1_Dis(void);
INLINE void ADC1_SW_Ch_Sel(uint32 adc1_ch);
INLINE uint8 ADC1_EOC_Sts(void);
INLINE uint8 ADC1_HV_Ready_Sts(void);
INLINE uint8 ADC1_Current_Ch_Sts(void);                            
INLINE uint8 ADC1_Current_SQ_Sts(void);                           
INLINE uint8 ADC1_ESM_Active_Sts(void);
INLINE uint8 ADC1_EIM_Active_Sts(void);
INLINE uint8 ADC1_SQ_Stop_Sts(void);
INLINE uint8 ADC1_SQ_Feedback_Sts(void);
INLINE void ADC1_Calibration_En(uint32 mask_ch);
INLINE void ADC1_Ch_Sample_Time_LV_Set(uint32 miclk);
INLINE void ADC1_Ch_Sample_Time_HV_Set(uint32 miclk);
INLINE uint8 ADC1_MCM_Ready_Sts(void);
INLINE uint8 ADC1_EOC_Fail_Sts(void);
INLINE void ADC1_EOC_Fail_Clr(void);
INLINE void ADC1_Software_Mode_Sel(void);
INLINE void ADC1_Sequencer_Mode_Sel(void);
INLINE void ADC1_MCM_Power_On(void);
INLINE void ADC1_MCM_Power_Off(void);
INLINE void ADC1_Calibration_Time_Set(uint32 time);
INLINE void ADC1_Ch_Filter_Set(uint32 mask_ch);
INLINE void ADC1_EIM_Chx_En(uint32 adc1_ch);
INLINE void ADC1_EIM_Rep_Set(uint32 meas);
INLINE void ADC1_EIM_En(void);
INLINE void ADC1_EIM_Dis(void);
INLINE void ADC1_EIM_Trig_Set(uint32 eim_trig);
INLINE void ADC1_ESM_Sequence_Set(uint16 mask_ch);                                
INLINE void ADC1_ESM_Trig_Set(uint32 esm_trig);
INLINE void ADC1_ESM_En(void);
INLINE void ADC1_ESM_Dis(void);
INLINE uint8 ADC1_ESM_Sts(void);
INLINE void ADC1_SW_Max_Time_Set(uint32 time);
INLINE void ADC1_Ch0_WaitForReady_Set(void);
INLINE void ADC1_Ch0_Overwrite_Set(void);
INLINE void ADC1_Ch1_WaitForReady_Set(void);
INLINE void ADC1_Ch1_Overwrite_Set(void);
INLINE void ADC1_Ch2_WaitForReady_Set(void);
INLINE void ADC1_Ch2_Overwrite_Set(void);
INLINE void ADC1_Ch3_WaitForReady_Set(void);
INLINE void ADC1_Ch3_Overwrite_Set(void);
INLINE void ADC1_Ch4_WaitForReady_Set(void);
INLINE void ADC1_Ch4_Overwrite_Set(void);
INLINE void ADC1_Ch5_WaitForReady_Set(void);
INLINE void ADC1_Ch5_Overwrite_Set(void);
INLINE void ADC1_Ch6_WaitForReady_Set(void);
INLINE void ADC1_Ch6_Overwrite_Set(void);
INLINE void ADC1_Ch7_WaitForReady_Set(void);
INLINE void ADC1_Ch7_Overwrite_Set(void);
INLINE void ADC1_Ch8_WaitForReady_Set(void);
INLINE void ADC1_Ch8_Overwrite_Set(void);
INLINE void ADC1_Ch9_WaitForReady_Set(void);
INLINE void ADC1_Ch9_Overwrite_Set(void);
INLINE void ADC1_Ch10_WaitForReady_Set(void);
INLINE void ADC1_Ch10_Overwrite_Set(void);
INLINE void ADC1_Ch11_WaitForReady_Set(void);
INLINE void ADC1_Ch11_Overwrite_Set(void);
INLINE void ADC1_Ch12_WaitForReady_Set(void);
INLINE void ADC1_Ch12_Overwrite_Set(void);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_WaitForReady_Set(void);
INLINE void ADC1_DU1_Overwrite_Set(void);
INLINE void ADC1_DU2_WaitForReady_Set(void);
INLINE void ADC1_DU2_Overwrite_Set(void);
INLINE void ADC1_DU3_WaitForReady_Set(void);
INLINE void ADC1_DU3_Overwrite_Set(void);
INLINE void ADC1_DU4_WaitForReady_Set(void);
INLINE void ADC1_DU4_Overwrite_Set(void);
#endif
INLINE uint16 ADC1_Ch0_Result_Get(void);
INLINE uint16 ADC1_Ch1_Result_Get(void);
INLINE uint16 ADC1_Ch2_Result_Get(void);
INLINE uint16 ADC1_Ch3_Result_Get(void);
INLINE uint16 ADC1_Ch4_Result_Get(void);
INLINE uint16 ADC1_Ch5_Result_Get(void);
INLINE uint16 ADC1_Ch6_Result_Get(void);
INLINE uint16 ADC1_Ch7_Result_Get(void);
INLINE uint16 ADC1_Ch8_Result_Get(void);
INLINE uint16 ADC1_Ch9_Result_Get(void);
INLINE uint16 ADC1_Ch10_Result_Get(void);
INLINE uint16 ADC1_Ch11_Result_Get(void);
INLINE uint16 ADC1_Ch12_Result_Get(void);
#if defined(TLE9845QX)
INLINE uint16 ADC1_DU1_Result_Get(void);
INLINE uint16 ADC1_DU2_Result_Get(void);
INLINE uint16 ADC1_DU3_Result_Get(void);
INLINE uint16 ADC1_DU4_Result_Get(void);
#endif
INLINE uint16 ADC1_VBAT_SENSE_Result_Get(void);
INLINE uint16 ADC1_VS_Result_Get(void);
INLINE uint16 ADC1_MON1_Result_Get(void);
INLINE uint16 ADC1_MON2_Result_Get(void);
INLINE uint16 ADC1_MON3_Result_Get(void);
INLINE uint16 ADC1_MON4_Result_Get(void);
INLINE uint16 ADC1_MON5_Result_Get(void);
INLINE uint16 ADC1_P21_Result_Get(void);
INLINE uint16 ADC1_P22_Result_Get(void);
INLINE uint16 ADC1_P23_Result_Get(void);
INLINE uint16 ADC1_P26_Result_Get(void);
INLINE uint16 ADC1_P27_Result_Get(void);
INLINE uint16 ADC1_P20_Result_Get(void);
INLINE void ADC1_Sequence0_Set(uint32 mask_ch);
INLINE void ADC1_Sequence1_Set(uint32 mask_ch);
INLINE void ADC1_Sequence2_Set(uint32 mask_ch);
INLINE void ADC1_Sequence3_Set(uint32 mask_ch);
INLINE void ADC1_Sequence4_Set(uint32 mask_ch);
INLINE void ADC1_Sequence5_Set(uint32 mask_ch);
INLINE void ADC1_Sequence6_Set(uint32 mask_ch);
INLINE void ADC1_Sequence7_Set(uint32 mask_ch);
INLINE void ADC1_Sequence8_Set(uint32 mask_ch);
INLINE void ADC1_Sequence9_Set(uint32 mask_ch);
INLINE void ADC1_Sequence10_Set(uint32 mask_ch);
INLINE void ADC1_Sequence11_Set(uint32 mask_ch);
INLINE void ADC1_Ch0_Filt_Up_En(void);
INLINE void ADC1_Ch1_Filt_Up_En(void);
INLINE void ADC1_Ch2_Filt_Up_En(void);
INLINE void ADC1_Ch3_Filt_Up_En(void);
INLINE void ADC1_Ch4_Filt_Up_En(void);
INLINE void ADC1_Ch5_Filt_Up_En(void);
INLINE void ADC1_Ch6_Filt_Up_En(void);
INLINE void ADC1_Ch7_Filt_Up_En(void);
INLINE void ADC1_Ch8_Filt_Up_En(void);
INLINE void ADC1_Ch9_Filt_Up_En(void);
INLINE void ADC1_Ch10_Filt_Up_En(void);
INLINE void ADC1_Ch11_Filt_Up_En(void);
INLINE void ADC1_Ch0_Filt_Up_Dis(void);
INLINE void ADC1_Ch1_Filt_Up_Dis(void);
INLINE void ADC1_Ch2_Filt_Up_Dis(void);
INLINE void ADC1_Ch3_Filt_Up_Dis(void);
INLINE void ADC1_Ch4_Filt_Up_Dis(void);
INLINE void ADC1_Ch5_Filt_Up_Dis(void);
INLINE void ADC1_Ch6_Filt_Up_Dis(void);
INLINE void ADC1_Ch7_Filt_Up_Dis(void);
INLINE void ADC1_Ch8_Filt_Up_Dis(void);
INLINE void ADC1_Ch9_Filt_Up_Dis(void);
INLINE void ADC1_Ch10_Filt_Up_Dis(void);
INLINE void ADC1_Ch11_Filt_Up_Dis(void);
INLINE void ADC1_Ch0_Filt_Lo_En(void);
INLINE void ADC1_Ch1_Filt_Lo_En(void);
INLINE void ADC1_Ch2_Filt_Lo_En(void);
INLINE void ADC1_Ch3_Filt_Lo_En(void);
INLINE void ADC1_Ch4_Filt_Lo_En(void);
INLINE void ADC1_Ch5_Filt_Lo_En(void);
INLINE void ADC1_Ch6_Filt_Lo_En(void);
INLINE void ADC1_Ch7_Filt_Lo_En(void);
INLINE void ADC1_Ch8_Filt_Lo_En(void);
INLINE void ADC1_Ch9_Filt_Lo_En(void);
INLINE void ADC1_Ch10_Filt_Lo_En(void);
INLINE void ADC1_Ch11_Filt_Lo_En(void);
INLINE void ADC1_Ch0_Filt_Lo_Dis(void);
INLINE void ADC1_Ch1_Filt_Lo_Dis(void);
INLINE void ADC1_Ch2_Filt_Lo_Dis(void);
INLINE void ADC1_Ch3_Filt_Lo_Dis(void);
INLINE void ADC1_Ch4_Filt_Lo_Dis(void);
INLINE void ADC1_Ch5_Filt_Lo_Dis(void);
INLINE void ADC1_Ch6_Filt_Lo_Dis(void);
INLINE void ADC1_Ch7_Filt_Lo_Dis(void);
INLINE void ADC1_Ch8_Filt_Lo_Dis(void);
INLINE void ADC1_Ch9_Filt_Lo_Dis(void);
INLINE void ADC1_Ch10_Filt_Lo_Dis(void);
INLINE void ADC1_Ch11_Filt_Lo_Dis(void);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Sts_Clr(void);
INLINE void ADC1_DU1_Up_Sts_Clr(void);
INLINE void ADC1_DU2_Lo_Sts_Clr(void);
INLINE void ADC1_DU2_Up_Sts_Clr(void);
INLINE void ADC1_DU3_Lo_Sts_Clr(void);
INLINE void ADC1_DU3_Up_Sts_Clr(void);
INLINE void ADC1_DU4_Lo_Sts_Clr(void);
INLINE void ADC1_DU4_Up_Sts_Clr(void);
#endif
#if defined(TLE9845QX)
INLINE void ADC1_DU1_En(void);
INLINE void ADC1_DU2_En(void);
INLINE void ADC1_DU3_En(void);
INLINE void ADC1_DU4_En(void);
INLINE void ADC1_DU1_Dis(void);
INLINE void ADC1_DU2_Dis(void);
INLINE void ADC1_DU3_Dis(void);
INLINE void ADC1_DU4_Dis(void);
INLINE uint8 ADC1_DU1_Result_Neg_Sts(void);
INLINE uint8 ADC1_DU2_Result_Neg_Sts(void);
INLINE uint8 ADC1_DU3_Result_Neg_Sts(void);
INLINE uint8 ADC1_DU4_Result_Neg_Sts(void);
#endif
INLINE void ADC1_Ch0_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch1_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch2_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch3_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch4_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch5_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch6_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch7_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch8_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch9_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch10_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_Ch11_Mode_Meas_Sel(uint32 mmode);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_DU2_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_DU3_Mode_Meas_Sel(uint32 mmode);
INLINE void ADC1_DU4_Mode_Meas_Sel(uint32 mmode);
#endif
INLINE void ADC1_Ch0_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch1_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch2_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch3_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch4_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch5_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch6_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch7_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch8_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch9_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch10_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch11_Lo_Th_Set(uint32 trig_val);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_DU2_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_DU3_Lo_Th_Set(uint32 trig_val);
INLINE void ADC1_DU4_Lo_Th_Set(uint32 trig_val);
#endif
INLINE void ADC1_Ch0_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch1_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch2_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch3_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch4_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch5_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch6_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch7_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch8_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch9_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch10_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_Ch11_Up_Th_Set(uint32 trig_val);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_DU2_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_DU3_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_DU4_Up_Th_Set(uint32 trig_val);
INLINE void ADC1_DU1_Trigger_Sel(du_triggen_sel trig_Ch);
INLINE void ADC1_DU2_Trigger_Sel(du_triggen_sel trig_Ch);
INLINE void ADC1_DU3_Trigger_Sel(du_triggen_sel trig_Ch);
INLINE void ADC1_DU4_Trigger_Sel(du_triggen_sel trig_Ch);
#endif
INLINE void ADC1_Ch0_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch1_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch2_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch3_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch4_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch5_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch6_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch7_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch8_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch9_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch10_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_Ch11_Lo_hys_Set(uint32 hyst);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_DU2_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_DU3_Lo_hys_Set(uint32 hyst);
INLINE void ADC1_DU4_Lo_hys_Set(uint32 hyst);
#endif
INLINE void ADC1_Ch0_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch1_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch2_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch3_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch4_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch5_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch6_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch7_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch8_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch9_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch10_Up_hys_Set(uint32 hyst);
INLINE void ADC1_Ch11_Up_hys_Set(uint32 hyst);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Up_hys_Set(uint32 hyst);
INLINE void ADC1_DU2_Up_hys_Set(uint32 hyst);
INLINE void ADC1_DU3_Up_hys_Set(uint32 hyst);
INLINE void ADC1_DU4_Up_hys_Set(uint32 hyst);
#endif
INLINE void ADC1_Ch0_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch1_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch2_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch3_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch4_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch5_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch6_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch7_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch8_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch9_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch10_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch11_Lo_Cnt_Set(uint32 cnt);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU2_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU3_Lo_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU4_Lo_Cnt_Set(uint32 cnt);
#endif
INLINE void ADC1_Ch0_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch1_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch2_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch3_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch4_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch5_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch6_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch7_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch8_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch9_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch10_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_Ch11_Up_Cnt_Set(uint32 cnt);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU2_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU3_Up_Cnt_Set(uint32 cnt);
INLINE void ADC1_DU4_Up_Cnt_Set(uint32 cnt);
#endif
INLINE void ADC1_VBAT_OV_Int_Clr(void);
INLINE void ADC1_VBAT_UV_Int_Clr(void);
INLINE void ADC1_VbatSense_Int_Clr(void);
INLINE void ADC1_VS_Int_Clr(void);
INLINE void ADC1_MON1_Int_Clr(void);
INLINE void ADC1_MON2_Int_Clr(void);
INLINE void ADC1_MON3_Int_Clr(void);
INLINE void ADC1_MON4_Int_Clr(void);
INLINE void ADC1_MON5_Int_Clr(void);
INLINE void ADC1_P21_Int_Clr(void);
INLINE void ADC1_P22_Int_Clr(void);
INLINE void ADC1_P23_Int_Clr(void);
INLINE void ADC1_P26_Int_Clr(void);
INLINE void ADC1_P27_Int_Clr(void);
INLINE void ADC1_P20_Int_Clr(void);
INLINE void ADC1_EIM_Int_Clr(void);
INLINE void ADC1_ESM_Int_Clr(void);
INLINE void ADC1_VS_Lo_Int_Clr(void);
INLINE void ADC1_MON1_Lo_Int_Clr(void);
INLINE void ADC1_MON2_Lo_Int_Clr(void);
INLINE void ADC1_MON3_Lo_Int_Clr(void);
INLINE void ADC1_MON4_Lo_Int_Clr(void);
INLINE void ADC1_MON5_Lo_Int_Clr(void);
INLINE void ADC1_P21_Lo_Int_Clr(void);
INLINE void ADC1_P22_Lo_Int_Clr(void);
INLINE void ADC1_P23_Lo_Int_Clr(void);
INLINE void ADC1_P26_Lo_Int_Clr(void);
INLINE void ADC1_P27_Lo_Int_Clr(void);
INLINE void ADC1_VS_Up_Int_Clr(void);
INLINE void ADC1_MON1_Up_Int_Clr(void);
INLINE void ADC1_MON2_Up_Int_Clr(void);
INLINE void ADC1_MON3_Up_Int_Clr(void);
INLINE void ADC1_MON4_Up_Int_Clr(void);
INLINE void ADC1_MON5_Up_Int_Clr(void);
INLINE void ADC1_P21_Up_Int_Clr(void);
INLINE void ADC1_P22_Up_Int_Clr(void);
INLINE void ADC1_P23_Up_Int_Clr(void);
INLINE void ADC1_P26_Up_Int_Clr(void);
INLINE void ADC1_P27_Up_Int_Clr(void);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Int_Clr(void);
INLINE void ADC1_DU1_Up_Int_Clr(void);
INLINE void ADC1_DU2_Lo_Int_Clr(void);
INLINE void ADC1_DU2_Up_Int_Clr(void);
INLINE void ADC1_DU3_Lo_Int_Clr(void);
INLINE void ADC1_DU3_Up_Int_Clr(void);
INLINE void ADC1_DU4_Lo_Int_Clr(void);
INLINE void ADC1_DU4_Up_Int_Clr(void);
#endif
INLINE void ADC1_VBatSense_Int_En(void);
INLINE void ADC1_VS_Int_En(void);
INLINE void ADC1_MON1_Int_En(void);
INLINE void ADC1_MON2_Int_En(void);
INLINE void ADC1_MON3_Int_En(void);
INLINE void ADC1_MON4_Int_En(void);
INLINE void ADC1_MON5_Int_En(void);
INLINE void ADC1_P21_Int_En(void);
INLINE void ADC1_P22_Int_En(void);
INLINE void ADC1_P23_Int_En(void);
INLINE void ADC1_P26_Int_En(void);
INLINE void ADC1_P27_Int_En(void);
INLINE void ADC1_P20_Int_En(void);
INLINE void ADC1_EIM_Int_En(void);
INLINE void ADC1_ESM_Int_En(void);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Int_En(void);
INLINE void ADC1_DU1_Up_Int_En(void);
INLINE void ADC1_DU2_Lo_Int_En(void);
INLINE void ADC1_DU2_Up_Int_En(void);
INLINE void ADC1_DU3_Lo_Int_En(void);
INLINE void ADC1_DU3_Up_Int_En(void);
INLINE void ADC1_DU4_Lo_Int_En(void);
INLINE void ADC1_DU4_Up_Int_En(void);
#endif
INLINE void ADC1_VbatSense_Int_Dis(void);
INLINE void ADC1_VS_Int_Dis(void);
INLINE void ADC1_MON1_Int_Dis(void);
INLINE void ADC1_MON2_Int_Dis(void);
INLINE void ADC1_MON3_Int_Dis(void);
INLINE void ADC1_MON4_Int_Dis(void);
INLINE void ADC1_MON5_Int_Dis(void);
INLINE void ADC1_P21_Int_Dis(void);
INLINE void ADC1_P22_Int_Dis(void);
INLINE void ADC1_P23_Int_Dis(void);
INLINE void ADC1_P26_Int_Dis(void);
INLINE void ADC1_P27_Int_Dis(void);
INLINE void ADC1_P20_Int_Dis(void);
INLINE void ADC1_EIM_Int_Dis(void);
INLINE void ADC1_ESM_Int_Dis(void);
#if defined(TLE9845QX)
INLINE void ADC1_DU1_Lo_Int_Dis(void);
INLINE void ADC1_DU1_Up_Int_Dis(void);
INLINE void ADC1_DU2_Lo_Int_Dis(void);
INLINE void ADC1_DU2_Up_Int_Dis(void);
INLINE void ADC1_DU3_Lo_Int_Dis(void);
INLINE void ADC1_DU3_Up_Int_Dis(void);
INLINE void ADC1_DU4_Lo_Int_Dis(void);
INLINE void ADC1_DU4_Up_Int_Dis(void);
#endif
INLINE void ADC1_VS_Lo_Int_En(void);
INLINE void ADC1_MON1_Lo_Int_En(void);
INLINE void ADC1_MON2_Lo_Int_En(void);
INLINE void ADC1_MON3_Lo_Int_En(void);
INLINE void ADC1_MON4_Lo_Int_En(void);
INLINE void ADC1_MON5_Lo_Int_En(void);
INLINE void ADC1_P21_Lo_Int_En(void);
INLINE void ADC1_P22_Lo_Int_En(void);
INLINE void ADC1_P23_Lo_Int_En(void);
INLINE void ADC1_P26_Lo_Int_En(void);
INLINE void ADC1_P27_Lo_Int_En(void);
INLINE void ADC1_VS_Up_Int_En(void);
INLINE void ADC1_MON1_Up_Int_En(void);
INLINE void ADC1_MON2_Up_Int_En(void);
INLINE void ADC1_MON3_Up_Int_En(void);
INLINE void ADC1_MON4_Up_Int_En(void);
INLINE void ADC1_MON5_Up_Int_En(void);
INLINE void ADC1_P21_Up_Int_En(void);
INLINE void ADC1_P22_Up_Int_En(void);
INLINE void ADC1_P23_Up_Int_En(void);
INLINE void ADC1_P26_Up_Int_En(void);
INLINE void ADC1_P27_Up_Int_En(void);
INLINE void ADC1_VS_Lo_Int_Dis(void);
INLINE void ADC1_MON1_Lo_Int_Dis(void);
INLINE void ADC1_MON2_Lo_Int_Dis(void);
INLINE void ADC1_MON3_Lo_Int_Dis(void);
INLINE void ADC1_MON4_Lo_Int_Dis(void);
INLINE void ADC1_MON5_Lo_Int_Dis(void);
INLINE void ADC1_P21_Lo_Int_Dis(void);
INLINE void ADC1_P22_Lo_Int_Dis(void);
INLINE void ADC1_P23_Lo_Int_Dis(void);
INLINE void ADC1_P26_Lo_Int_Dis(void);
INLINE void ADC1_P27_Lo_Int_Dis(void);
INLINE void ADC1_VS_Up_Int_Dis(void);
INLINE void ADC1_MON1_Up_Int_Dis(void);
INLINE void ADC1_MON2_Up_Int_Dis(void);
INLINE void ADC1_MON3_Up_Int_Dis(void);
INLINE void ADC1_MON4_Up_Int_Dis(void);
INLINE void ADC1_MON5_Up_Int_Dis(void);
INLINE void ADC1_P21_Up_Int_Dis(void);
INLINE void ADC1_P22_Up_Int_Dis(void);
INLINE void ADC1_P23_Up_Int_Dis(void);
INLINE void ADC1_P26_Up_Int_Dis(void);
INLINE void ADC1_P27_Up_Int_Dis(void);
INLINE void ADC1_SetEIMChannel(uint8 channel);
INLINE void ADC1_SetSwMode_Channel(uint8 channel);
INLINE void ADC1_SetSeqMode(void);
INLINE void ADC1_SetSwMode(void);
INLINE void ADC1_SetSocSwMode(uint8 channel);
INLINE bool ADC1_GetEocSwMode(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

/* ADC1 global configuration flags */
/** \brief Enable the ADC1 module.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables the ADC1 module.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Power_On(void)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_PD_N_Pos, ADC1_CTRL_STS_PD_N_Msk, 1u);
}

/** \brief Disable the ADC1 module.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables the ADC1 module.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_Off();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Power_Off(void)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_PD_N_Pos, ADC1_CTRL_STS_PD_N_Msk, 0u);
}

/** \brief Start an ADC1 Conversion, for Software mode only.
 *
 * \brief <b>Example</b><br>
 * \brief This example starts an ADC1 conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel(); 
 *     ADC1_SOS_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_SOS_Set(void)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_SOS_Pos, ADC1_CTRL_STS_SOS_Msk, 1u);
}

/** \brief Enable DPP1 Startup.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables DPP1 Startup.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_DPP1_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DPP1_En(void)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_STRTUP_DIS_Pos, ADC1_CTRL_STS_STRTUP_DIS_Msk, 0u);
}

/** \brief Disable DPP1 Startup.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables DPP1 Startup.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_DPP1_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DPP1_Dis(void)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_STRTUP_DIS_Pos, ADC1_CTRL_STS_STRTUP_DIS_Msk, 1u);
}

/** \brief Select a channel for the software conversion.
 *
 * \param adc1_ch channel which should be measured
 *
 * \brief <b>Example</b><br>
 * \brief This example starts an ADC1 conversion on channel 12 - P2.0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Software_Mode_Sel(); 
 *     ADC1_SW_Ch_Sel(ADC1_P20);
 *     ADC1_SOS_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api1
 */
INLINE void ADC1_SW_Ch_Sel(uint32 adc1_ch)
{
  Field_Mod32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_SW_CH_SEL_Pos, ADC1_CTRL_STS_SW_CH_SEL_Msk, adc1_ch);
}

/* ADC1 Status Flags */
/** \brief Read the End-of-Conversion status.
 *
 * \retval 1 if conversion has ended
 * \retval 0 if conversion still ongoing
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Software Mode, starts a measurement and waits for it to be finished.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_SW_Ch_Sel( ADC1_MON1 );
 *     ADC1_SOS_Set();
 *
 *     while ( ADC1_EOC_Sts() == 0 ) {}
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_EOC_Sts(void)
{
  return u1_Field_Rd32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_EOC_Pos, ADC1_CTRL_STS_EOC_Msk);
}

/** \brief Read the HVADC Ready bit.
 *
 * \retval 1 if HVADC is Ready: Set automatically 5 ADC clock cycles after module is enabled
 * \retval 0 if HVADC is Not ready: Module in power down or in init phase
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and waits HVADC to be ready.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *
 *     while ( ADC1_HV_Ready_Sts() == 0 ) {}
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_HV_Ready_Sts(void)
{
  return u1_Field_Rd32(&ADC1->CTRL_STS.reg, (uint8)ADC1_CTRL_STS_READY_Pos, ADC1_CTRL_STS_READY_Msk);
}

/* ADC1 Sequencer Feedback Current ADC1 Channel */
/** \brief Read the currently active channel status.
 *
 * \return uint8, number of the current channel, values between 0..11
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Software Mode and waits until Sequence 6 is enabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_SW_Ch_Sel( ADC1_MON5 );
 *     ADC1_SOS_Set();
 *     while (ADC1_Current_Ch_Sts() & 0x6 != 0x6)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_Current_Ch_Sts(void)                            
{
	return u8_Field_Rd32(&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_CHx_Pos, ADC1_SQ_FB_CHx_Msk);
}

/** \brief Read the currently active channel status in Sequencer Mode.
 *
 * \return uint8, number of the current sequence, values between 0..11
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode and waits until Sequence 6 is enabled.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 );
 *     while (ADC1_Current_SQ_Sts() & 0x6 != 0x6)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_Current_SQ_Sts(void)                           
{
	return u8_Field_Rd32  (&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_SQx_Pos, ADC1_SQ_FB_SQx_Msk);
}

/** \brief Read the active status of the Exceptional Sequencer Measurement (ESM).
 *
 * \retval 1 ESM ongoing
 * \retval 0 no ESM active
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 module and ESM, switches to Sequencer Mode and reads the active status of the Exceptional Sequencer Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_ESM_En();
 *     ADC1_Sequencer_Mode_Sel(); 
 *     ADC1_Sequencer_Mode_Sel(); 
 *     ADC1_ESM_Sequence_Set( ADC1_CH2 );
 *
 *     while (ADC1_ESM_Active_Sts() == 0u)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_ESM_Active_Sts(void)
{
  return u1_Field_Rd32(&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_ESM_ACTIVE_Pos, ADC1_SQ_FB_ESM_ACTIVE_Msk);
}

/** \brief Read the active status of the Exceptional Interrupt Measurement (EIM).
 *
 * \retval 1 EIM ongoing
 * \retval 0 no EIM active
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 module and EIM, switches to Sequencer Mode and reads the active status of the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_EIM_En();  
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_EIM_Chx_En( ADC1_MON1 );
 *
 *     while (ADC1_EIM_Active_Sts() == 0u)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_EIM_Active_Sts(void)
{
  return u1_Field_Rd32(&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_EIM_ACTIVE_Pos, ADC1_SQ_FB_EIM_ACTIVE_Msk);
}

/** \brief Read the ADC1 Sequencer Stop Signal for DPP Status.
 *
 * \retval 0 if DPP is Running.
 * \retval 1 if DPP is Stopped.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode and reads the DPP Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel(); 
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 ); 
 *     while (ADC1_SQ_Stop_Sts() == 0)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_SQ_Stop_Sts(void)
{
  return u1_Field_Rd32(&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_SQ_STOP_Pos, ADC1_SQ_FB_SQ_STOP_Msk);
}

/** \brief Read the Current Sequence that caused software mode.
 *
 * \return uint8, Current Sequence.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Current Sequence that caused software mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *      
 *     sts = ADC1_SQ_Feedback_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_SQ_Feedback_Sts(void)
{
  return u8_Field_Rd32(&ADC1->SQ_FB.reg, (uint8)ADC1_SQ_FB_SQ_FB_Pos, ADC1_SQ_FB_SQ_FB_Msk);
}

/* ADC1_CTRL2 */
/** \brief Enable ADC1 Channels Calibration.
 *
 * \param mask_ch Mask for which the calibration is enabled
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Channel 11 (P2.7) Calibration.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Calibration_En(ADC1_MASK_P27); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Calibration_En(uint32 mask_ch)
{
  Field_Mod32(&ADC1->CTRL2.reg, (uint8)ADC1_CTRL2_CAL_EN_Pos, ADC1_CTRL2_CAL_EN_Msk, mask_ch);
}

/* ADC1_CTRL3 */
/** \brief Select Sample time of ADC1 (Low Voltage Channel).
 *
 * \param miclk MI_CLK periods
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Sample time Low Voltage Channel of ADC1 to MICLK6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Ch_Sample_Time_LV_Set(ADC1_MICLK6); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch_Sample_Time_LV_Set(uint32 miclk)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_SAMPLE_TIME_LVCH_Pos, ADC1_CTRL3_SAMPLE_TIME_LVCH_Msk, miclk);
}			 

/** \brief Select Sample time of ADC1 (High Voltage Channel).
 *
 * \param miclk MI_CLK periods
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Sample time High Voltage Channel of ADC1 to MICLK6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Ch_Sample_Time_HV_Set(1u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch_Sample_Time_HV_Set(uint32 miclk)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_SAMPLE_TIME_HVCH_Pos, ADC1_CTRL3_SAMPLE_TIME_HVCH_Msk, miclk);
}			 

/** \brief Read the Ready Signal Status for MCM after Power On or Reset.
 *
 * \retval 0 if MCM is Not Ready.
 * \retval 1 if MCM is Ready.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Ready Signal Status for MCM after Power On or Reset.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 * 
 *     ADC1_Power_On(); 
 *     ADC1_MCM_Power_On();  
 *     sts = ADC1_MCM_Ready_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_MCM_Ready_Sts(void)
{
  return u1_Field_Rd32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_MCM_RDY_Pos, ADC1_CTRL3_MCM_RDY_Msk);
}

/** \brief Read the Fail of ADC End of Conversion Signal Status.
 *
 * \retval 0 if ADC EoC is available.
 * \retval 1 if ADC EoC is not available.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Fail of ADC End of Conversion Signal Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 * 
 *     ADC1_Power_On();  
 *     sts = ADC1_EOC_Fail_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_EOC_Fail_Sts(void)
{
  return u1_Field_Rd32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_EoC_FAIL_Pos, ADC1_CTRL3_EoC_FAIL_Msk);
}

/** \brief Clear Fail of ADC End of Conversion Signal Flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears Fail of ADC End of Conversion Signal Flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_EOC_Fail_Clr();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_EOC_Fail_Clr(void)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_EoC_FAIL_CLR_Pos, ADC1_CTRL3_EoC_FAIL_CLR_Msk, 1u);
}

/** \brief ADC1 selects the Software Mode, measurements are performed on user request.
 * \note EIM/ESM measurements are only possible in \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Software Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_SW_Ch_Sel( ADC1_P20 );
 *     ADC1_SOS_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Software_Mode_Sel(void)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_SW_MODE_Pos, ADC1_CTRL3_SW_MODE_Msk, 1u);
}

/** \brief ADC1 selects the Sequencer Mode.
 * \note EIM/ESM measurements are only possible in Sequencer mode.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequencer_Mode_Sel(void)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_SW_MODE_Pos, ADC1_CTRL3_SW_MODE_Msk, 0u);
}

/** \brief Enable Power Down Signal for MCM.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Power Down Signal for MCM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_MCM_Power_On(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_MCM_Power_On(void)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_MCM_PD_N_Pos, ADC1_CTRL3_MCM_PD_N_Msk, 1u);
}

/** \brief Disable Power Down Signal for MCM.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Power Down Signal for MCM.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_MCM_Power_Off(); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_MCM_Power_Off(void)
{
  Field_Mod32(&ADC1->CTRL3.reg, (uint8)ADC1_CTRL3_MCM_PD_N_Pos, ADC1_CTRL3_MCM_PD_N_Msk, 0u);
}


/* ADC1_CTRL4 */
/** \brief Select Maximum ADC1 Calibration Time.
 *
 * \param time in MI_CLK periods
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Maximum ADC1 Calibration Time to 2 Sequences.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Calibration_Time_Set(1u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Calibration_Time_Set(uint32 time)
{
  Field_Mod32(&ADC1->CTRL4.reg, (uint8)ADC1_CTRL4_MAX_CALTIME_Pos, ADC1_CTRL4_MAX_CALTIME_Msk, time);
}

/* ADC1_CTRL5*/
/** \brief Select Output Filter for ADC1 Channels.
 *
 * \param mask_ch Mask for which the Output Filter is enabled
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Channel 2 IIR Data for FILT_OUT2 Register.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Ch_Filter_Set(ADC1_MASK_CH2); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch_Filter_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->CTRL5.reg, (uint8)ADC1_CTRL5_FILT_OUT_SEL_11_0_Pos, ADC1_CTRL5_FILT_OUT_SEL_11_0_Msk, mask_ch);
}

/* ADC1 EIM Channel Setting */
/** \brief Set EIM channel for measurement.
 *
 * \param adc1_ch channel to be measured
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_EIM_Chx_En( ADC1_CH2 );
 *
 *     while (ADC1->FILT_OUTEIM.bit.VF_EIM == 0u)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Chx_En(uint32 adc1_ch)
{
  Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_EIM_CHx_Pos, ADC1_CHx_EIM_EIM_CHx_Msk, adc1_ch);
}

/** \brief Set ADC1 EIM Repeat Counter
 *
 * \param meas repeat counter for EIM measurement
 * \return none
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and sets EIM Repeat Counter to 4 Measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_EIM_Rep_Set(2u);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Rep_Set(uint32 meas)
{
  Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_EIM_REP_Pos, ADC1_CHx_EIM_EIM_REP_Msk, meas);
}

/** \brief Enable Exceptional interrupt measurement (EIM) Trigger Event.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_EIM_En();
 *   ADC1_EIM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.EIM_IS == 1u)
 *   {
 *     ADC1_EIM_CALLBACK();
 *     ADC1_EIM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_En(void)
{
  Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_EIM_EN_Pos, ADC1_CHx_EIM_EIM_EN_Msk, 1u);
}

/** \brief Disable Exceptional interrupt measurement (EIM) Trigger Event.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_EIM_En();
 *   ADC1_EIM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.EIM_IS == 1u)
 *   {
 *     ADC1_EIM_CALLBACK();
 *     ADC1_EIM_Int_Clr();
 *   }
 *   ADC1_EIM_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Dis(void)
{
  Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_EIM_EN_Pos, ADC1_CHx_EIM_EIM_EN_Msk, 0u);
}

/** \brief Get ADC1 EIM Trigger Selection
 *
 * \param eim_trig Trigger select
 * \return none
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and sets EIM Trigger Selection to Timer2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_EIM_Trig_Set(4u);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Trig_Set(uint32 eim_trig)
{
	Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_ADC1_EIM_TRIG_SEL_Pos, ADC1_CHx_EIM_ADC1_EIM_TRIG_SEL_Msk, eim_trig);	
}

/* ADC1 ESM Channel Settings */
/** \brief Set channels in ESM sequence.
 *
 * \param mask_ch bit mask of channels to be measured
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1, switches to Sequencer Mode, programs channel 6 (MON5), and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_ESM_Sequence_Set( ADC1_MASK_MON5 );
 *
 *     while (ADC1->FILT_OUT6.bit.VF6 == 0u)
 *     {
 *     }
 *
 *     adc_result = ADC1_P20_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Sequence_Set(uint16 mask_ch)                                
{
	Field_Mod32(&ADC1->CHx_ESM.reg, (uint8)ADC1_CHx_ESM_ESM_0_Pos, ADC1_CHx_ESM_ESM_0_Msk, (mask_ch));
}
							  
/** \brief Set ADC1 ESM Trigger Selection
 *
 * \param esm_trig Trigger select
 * \return none
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and sets ESM Trigger Selection to Timer2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_ESM_Trig_Set(4u);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Trig_Set(uint32 esm_trig)
{
	Field_Mod32(&ADC1->CHx_ESM.reg, (uint8)ADC1_CHx_ESM_ADC1_ESM_TRIG_SEL_Pos, ADC1_CHx_ESM_ADC1_ESM_TRIG_SEL_Msk, esm_trig);		
} 

/** \brief Enable Exceptional Sequence measurement (ESM) Trigger Event.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Sequence Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_ESM_En();
 *   ADC1_ESM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.ESM_IS == 1u)
 *   {
 *     ADC1_ESM_CALLBACK();
 *     ADC1_ESM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */ 
INLINE void ADC1_ESM_En(void)
{
  Field_Mod32(&ADC1->CHx_ESM.reg, (uint8)ADC1_CHx_ESM_ESM_EN_Pos, ADC1_CHx_ESM_ESM_EN_Msk, 1u);
}

/** \brief Disable Exceptional Sequence measurement (ESM) Trigger Event.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Sequence Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_ESM_En();
 *   ADC1_ESM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.ESM_IS == 1u)
 *   {
 *     ADC1_ESM_CALLBACK();
 *     ADC1_ESM_Int_Clr();
 *   }
 *   ADC1_ESM_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Dis(void)
{
  Field_Mod32(&ADC1->CHx_ESM.reg, (uint8)ADC1_CHx_ESM_ESM_EN_Pos, ADC1_CHx_ESM_ESM_EN_Msk, 0u);
}

/** \brief Read the Exceptional Sequence Measurement Status.
 *
 * \retval 0 if Exceptional Sequence Measurement is not done.
 * \retval 1 if Exceptional Sequence Measurement is done.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Fail of ADC End of Conversion Signal Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *
 *     sts = ADC1_ESM_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_ESM_Sts(void)
{
  return u1_Field_Rd32(&ADC1->CHx_ESM.reg, (uint8)ADC1_CHx_ESM_ESM_STS_Pos, ADC1_CHx_ESM_ESM_STS_Msk);
}

/* ADC1 Maximum Time in Software Mode */
/** \brief Set Maximum Time in Software Mode
 *
 * \param time Maximum time in Software Mode with the unit of 50 ns.
 * \return none
 *
 * \brief <b>Example</b><br>
 * \brief This example sets Maximum Time in Software Mode to 1us.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_SW_Max_Time_Set(20u);
 * }
 * ~~~~~~~~~~~~~~~  
 * \ingroup ADC1_api
 */
INLINE void ADC1_SW_Max_Time_Set(uint32 time)
{
	Field_Wrt32(&ADC1->MAX_TIME.reg, (uint8)ADC1_MAX_TIME_MAX_TIME_Pos, ADC1_MAX_TIME_MAX_TIME_Msk, time);		
} 

/* Result Mode */
/** \brief Set the result register channel 0 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 0 (VBAT_SENSE) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VBAT_SENSE );
 *     ADC1_Ch0_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT0.bit.VF0 == 1u)
 *     {
 *         adc_result = ADC1_VBAT_SENSE_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT0.reg, (uint8)ADC1_FILT_OUT0_WFR0_Pos, ADC1_FILT_OUT0_WFR0_Msk, 1u);
}

/** \brief Set the result register channel 0 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 0 (VBAT_SENSE) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();


 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VBAT_SENSE );
 *     ADC1_Ch0_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT0.bit.VF0 == 1u)
 *     {
 *         adc_result = ADC1_VBAT_SENSE_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT0.reg, (uint8)ADC1_FILT_OUT0_WFR0_Pos, ADC1_FILT_OUT0_WFR0_Msk, 0u);
}

/** \brief Set the result register channel 1 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 1 (VS) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();


 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VS );
 *     ADC1_Ch1_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT1.bit.VF1 == 1u)
 *     {
 *         adc_result = ADC1_VS_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT1.reg, (uint8)ADC1_FILT_OUT1_WFR1_Pos, ADC1_FILT_OUT1_WFR1_Msk, 1u);
}

/** \brief Set the result register channel 1 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 1 (VS) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VS );
 *     ADC1_Ch1_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT1.bit.VF1 == 1u)
 *     {
 *         adc_result = ADC1_VS_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT1.reg, (uint8)ADC1_FILT_OUT1_WFR1_Pos, ADC1_FILT_OUT1_WFR1_Msk, 0u);
}

/** \brief Set the result register channel 2 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON1 );
 *     ADC1_Ch2_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT2.bit.VF2 == 1u)
 *     {
 *         adc_result = ADC1_MON1_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT2.reg, (uint8)ADC1_FILT_OUT2_WFR2_Pos, ADC1_FILT_OUT2_WFR2_Msk, 1u);
}

/** \brief Set the result register channel 2 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON1 );
 *     ADC1_Ch2_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT2.bit.VF2 == 1u)
 *     {
 *         adc_result = ADC1_MON1_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT2.reg, (uint8)ADC1_FILT_OUT2_WFR2_Pos, ADC1_FILT_OUT2_WFR2_Msk, 0u);
}

/** \brief Set the result register channel 3 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON2 );
 *     ADC1_Ch3_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT3.bit.VF3 == 1u)
 *     {
 *         adc_result = ADC1_MON2_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT3.reg, (uint8)ADC1_FILT_OUT3_WFR3_Pos, ADC1_FILT_OUT3_WFR3_Msk, 1u);
}

/** \brief Set the result register channel 3 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON2 );
 *     ADC1_Ch3_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT3.bit.VF3 == 1u)
 *     {
 *         adc_result = ADC1_MON2_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT3.reg, (uint8)ADC1_FILT_OUT3_WFR3_Pos, ADC1_FILT_OUT3_WFR3_Msk, 0u);
}

/** \brief Set the result register channel 4 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 4 (MON3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON3 );
 *     ADC1_Ch4_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT4.bit.VF4 == 1u)
 *     {
 *         adc_result = ADC1_MON3_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT4.reg, (uint8)ADC1_FILT_OUT4_WFR4_Pos, ADC1_FILT_OUT4_WFR4_Msk, 1u);
}

/** \brief Set the result register channel 4 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 4 (MON3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON3 );
 *     ADC1_Ch4_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT4.bit.VF4 == 1u)
 *     {
 *         adc_result = ADC1_MON3_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT4.reg, (uint8)ADC1_FILT_OUT4_WFR4_Pos, ADC1_FILT_OUT4_WFR4_Msk, 0u);
}

/** \brief Set the result register channel 5 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 5 (MON4) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON4 );
 *     ADC1_Ch5_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT5.bit.VF5 == 1u)
 *     {
 *         adc_result = ADC1_MON4_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT5.reg, (uint8)ADC1_FILT_OUT5_WFR5_Pos, ADC1_FILT_OUT5_WFR5_Msk, 1u);
}

/** \brief Set the result register channel 5 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 5 (MON4) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON4 );
 *     ADC1_Ch5_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT5.bit.VF5 == 1u)
 *     {
 *         adc_result = ADC1_MON4_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT5.reg, (uint8)ADC1_FILT_OUT5_WFR5_Pos, ADC1_FILT_OUT5_WFR5_Msk, 0u);
}

/** \brief Set the result register channel 6 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 6 (MON5) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 );
 *     ADC1_Ch6_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT6.bit.VF6 == 1u)
 *     {
 *         adc_result = ADC1_MON5_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT6.reg, (uint8)ADC1_FILT_OUT6_WFR6_Pos, ADC1_FILT_OUT6_WFR6_Msk, 1u);
}

/** \brief Set the result register channel 6 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 6 (MON5) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 );
 *     ADC1_Ch6_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT6.bit.VF6 == 1u)
 *     {
 *         adc_result = ADC1_MON5_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT6.reg, (uint8)ADC1_FILT_OUT6_WFR6_Pos, ADC1_FILT_OUT6_WFR6_Msk, 0u);
}

/** \brief Set the result register channel 7 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 7 (P2.1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P21 );
 *     ADC1_Ch7_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT7.bit.VF7 == 1u)
 *     {
 *         adc_result = ADC1_P21_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT7.reg, (uint8)ADC1_FILT_OUT7_WFR7_Pos, ADC1_FILT_OUT7_WFR7_Msk, 1u);
}

/** \brief Set the result register channel 7 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 7 (P2.1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P21 );
 *     ADC1_Ch7_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT7.bit.VF7 == 1u)
 *     {
 *         adc_result = ADC1_P21_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT7.reg, (uint8)ADC1_FILT_OUT7_WFR7_Pos, ADC1_FILT_OUT7_WFR7_Msk, 0u);
}

/** \brief Set the result register channel 8 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 8 (P2.2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P22 );
 *     ADC1_Ch8_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT8.bit.VF8 == 1u)
 *     {
 *         adc_result = ADC1_P22_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT8.reg, (uint8)ADC1_FILT_OUT8_WFR8_Pos, ADC1_FILT_OUT8_WFR8_Msk, 1u);
}

/** \brief Set the result register channel 8 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 8 (P2.2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P22 );
 *     ADC1_Ch8_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT8.bit.VF8 == 1u)
 *     {
 *         adc_result = ADC1_P22_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT8.reg, (uint8)ADC1_FILT_OUT8_WFR8_Pos, ADC1_FILT_OUT8_WFR8_Msk, 0u);
}

/** \brief Set the result register channel 9 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 9 (P2.3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P23 );
 *     ADC1_Ch9_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT9.bit.VF9 == 1u)
 *     {
 *         adc_result = ADC1_P23_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT9.reg, (uint8)ADC1_FILT_OUT9_WFR9_Pos, ADC1_FILT_OUT9_WFR9_Msk, 1u);
}

/** \brief Set the result register channel 9 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 9 (P2.3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P23 );
 *     ADC1_Ch9_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT9.bit.VF9 == 1u)
 *     {
 *         adc_result = ADC1_P23_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT9.reg, (uint8)ADC1_FILT_OUT9_WFR9_Pos, ADC1_FILT_OUT9_WFR9_Msk, 0u);
}

/** \brief Set the result register channel 10 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 10 (P2.6) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P26 );
 *     ADC1_Ch10_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT10.bit.VF10 == 1u)
 *     {
 *         adc_result = ADC1_P26_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT10.reg, (uint8)ADC1_FILT_OUT10_WFR10_Pos, ADC1_FILT_OUT10_WFR10_Msk, 1u);
}

/** \brief Set the result register channel 10 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 10 (P2.6) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P26 );
 *     ADC1_Ch10_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT10.bit.VF10 == 1u)
 *     {
 *         adc_result = ADC1_P26_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT10.reg, (uint8)ADC1_FILT_OUT10_WFR10_Pos, ADC1_FILT_OUT10_WFR10_Msk, 0u);
}

/** \brief Set the result register channel 11 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 11 (P2.7) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P27 );
 *     ADC1_Ch11_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT11.bit.VF11 == 1u)
 *     {
 *         adc_result = ADC1_P27_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT11.reg, (uint8)ADC1_FILT_OUT11_WFR11_Pos, ADC1_FILT_OUT11_WFR11_Msk, 1u);
}

/** \brief Set the result register channel 11 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 11 (P2.7) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P27 );
 *     ADC1_Ch11_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT11.bit.VF11 == 1u)
 *     {
 *         adc_result = ADC1_P27_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT11.reg, (uint8)ADC1_FILT_OUT11_WFR11_Pos, ADC1_FILT_OUT11_WFR11_Msk, 0u);
}

/** \brief Set the result register channel 12 to "wait for read".
 * \note A valid result will not be overwritten by a subsequent conversion until the result is read.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Ch12_WaitForReady_Set();
 *
 *     if (ADC1->FILT_OUT12.bit.VF12 == 1u)
 *     {
 *         adc_result = ADC1_P20_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch12_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT12.reg, (uint8)ADC1_FILT_OUT12_WFR12_Pos, ADC1_FILT_OUT12_WFR12_Msk, 1u);
}

/** \brief Set the result register channel 12 to "overwrite".
 * \note A valid result will be overwritten by a subsequent conversion, a previous unread conversion is lost.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Ch12_Overwrite_Set();
 *
 *     if (ADC1->FILT_OUT12.bit.VF12 == 1u)
 *     {
 *         adc_result = ADC1_P20_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch12_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->FILT_OUT12.reg, (uint8)ADC1_FILT_OUT12_WFR12_Pos, ADC1_FILT_OUT12_WFR12_Msk, 0u);
}

#if defined(TLE9845QX)
/** \brief Set the result register Differential Channel Output 1 to "wait for read".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 1 to "wait for read".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_WaitForReady_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT1.reg, (uint8)ADC1_DIFFCH_OUT1_DWFR1_Pos, ADC1_DIFFCH_OUT1_DWFR1_Msk, 1u);
}

/** \brief Set the result register Differential Channel Output 1 to "overwrite".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 1 to "overwrite".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Overwrite_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT1.reg, (uint8)ADC1_DIFFCH_OUT1_DWFR1_Pos, ADC1_DIFFCH_OUT1_DWFR1_Msk, 0u);
}

/** \brief Set the result register Differential Channel Output 2 to "wait for read".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 2 to "wait for read".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_WaitForReady_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT2.reg, (uint8)ADC1_DIFFCH_OUT2_DWFR2_Pos, ADC1_DIFFCH_OUT2_DWFR2_Msk, 1u);
}

/** \brief Set the result register Differential Channel Output 2 to "overwrite".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 2 to "overwrite".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Overwrite_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT2.reg, (uint8)ADC1_DIFFCH_OUT2_DWFR2_Pos, ADC1_DIFFCH_OUT2_DWFR2_Msk, 0u);
}

/** \brief Set the result register Differential Channel Output 3 to "wait for read".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 3 to "wait for read".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_WaitForReady_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT3.reg, (uint8)ADC1_DIFFCH_OUT3_DWFR3_Pos, ADC1_DIFFCH_OUT3_DWFR3_Msk, 1u);
}

/** \brief Set the result register Differential Channel Output 3 to "overwrite".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 3 to "overwrite".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Overwrite_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT3.reg, (uint8)ADC1_DIFFCH_OUT3_DWFR3_Pos, ADC1_DIFFCH_OUT3_DWFR3_Msk, 0u);
}

/** \brief Set the result register Differential Channel Output 4 to "wait for read".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 4 to "wait for read".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_WaitForReady_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_WaitForReady_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT4.reg, (uint8)ADC1_DIFFCH_OUT4_DWFR4_Pos, ADC1_DIFFCH_OUT4_DWFR4_Msk, 1u);
}

/** \brief Set the result register Differential Channel Output 4 to "overwrite".
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the result register Differential Channel Output 4 to "overwrite".
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Overwrite_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Overwrite_Set(void)
{
  Field_Mod32(&ADC1->DIFFCH_OUT4.reg, (uint8)ADC1_DIFFCH_OUT4_DWFR4_Pos, ADC1_DIFFCH_OUT4_DWFR4_Msk, 0u);
}

#endif

/* Get ADC1 Result Value */
/** \brief Read the converted value from the channel 0 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 0 (VBAT_SENSE) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_VBAT_SENSE );
 *
 *     if (ADC1->FILT_OUT0.bit.VF0 == 1u)
 *     {
 *         adc_result = ADC1_Ch0_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch0_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT0.reg, (uint8)ADC1_FILT_OUT0_FILT_OUT_CH0_Pos, ADC1_FILT_OUT0_FILT_OUT_CH0_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 1 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 1 (VS) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VS );
 *
 *     if (ADC1->FILT_OUT1.bit.VF1 == 1u)
 *     {
 *         adc_result = ADC1_Ch1_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch1_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT1.reg, (uint8)ADC1_FILT_OUT1_FILT_OUT_CH1_Pos, ADC1_FILT_OUT1_FILT_OUT_CH1_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 2 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON1 );
 *
 *     if (ADC1->FILT_OUT2.bit.VF2 == 1u)
 *     {
 *         adc_result = ADC1_Ch2_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch2_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT2.reg, (uint8)ADC1_FILT_OUT2_FILT_OUT_CH2_Pos, ADC1_FILT_OUT2_FILT_OUT_CH2_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 3 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON2 );
 *
 *     if (ADC1->FILT_OUT3.bit.VF3 == 1u)
 *     {
 *         adc_result = ADC1_Ch3_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch3_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT3.reg, (uint8)ADC1_FILT_OUT3_FILT_OUT_CH3_Pos, ADC1_FILT_OUT3_FILT_OUT_CH3_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 4 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 4 (MON3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON3 );
 *
 *     if (ADC1->FILT_OUT4.bit.VF4 == 1u)
 *     {
 *         adc_result = ADC1_Ch4_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch4_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT4.reg, (uint8)ADC1_FILT_OUT4_FILT_OUT_CH4_Pos, ADC1_FILT_OUT4_FILT_OUT_CH4_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 5 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 5 (MON4) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON4 );
 *
 *     if (ADC1->FILT_OUT5.bit.VF5 == 1u)
 *     {
 *         adc_result = ADC1_Ch5_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch5_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT5.reg, (uint8)ADC1_FILT_OUT5_FILT_OUT_CH5_Pos, ADC1_FILT_OUT5_FILT_OUT_CH5_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 6 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 6 (MON5) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 );
 *
 *     if (ADC1->FILT_OUT6.bit.VF6 == 1u)
 *     {
 *         adc_result = ADC1_Ch6_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch6_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT6.reg, (uint8)ADC1_FILT_OUT6_FILT_OUT_CH6_Pos, ADC1_FILT_OUT6_FILT_OUT_CH6_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 7 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 7 (P2.1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P21 );
 *
 *     if (ADC1->FILT_OUT7.bit.VF7 == 1u)
 *     {
 *         adc_result = ADC1_Ch7_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch7_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT7.reg, (uint8)ADC1_FILT_OUT7_FILT_OUT_CH7_Pos, ADC1_FILT_OUT7_FILT_OUT_CH7_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 8 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 8 (P2.2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P22 );
 *
 *     if (ADC1->FILT_OUT8.bit.VF8 == 1u)
 *     {
 *         adc_result = ADC1_Ch8_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch8_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT8.reg, (uint8)ADC1_FILT_OUT8_FILT_OUT_CH8_Pos, ADC1_FILT_OUT8_FILT_OUT_CH8_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 9 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 9 (P2.3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P23 );
 *
 *     if (ADC1->FILT_OUT9.bit.VF9 == 1u)
 *     {
 *         adc_result = ADC1_Ch9_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch9_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT9.reg, (uint8)ADC1_FILT_OUT9_FILT_OUT_CH9_Pos, ADC1_FILT_OUT9_FILT_OUT_CH9_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 10 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 10 (P2.6) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P26 );
 *
 *     if (ADC1->FILT_OUT10.bit.VF10 == 1u)
 *     {
 *         adc_result = ADC1_Ch10_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch10_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT10.reg, (uint8)ADC1_FILT_OUT10_FILT_OUT_CH10_Pos, ADC1_FILT_OUT10_FILT_OUT_CH10_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 11 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 11 (P2.7) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P27 );
 *
 *     if (ADC1->FILT_OUT11.bit.VF11 == 1u)
 *     {
 *         adc_result = ADC1_Ch11_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch11_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT11.reg, (uint8)ADC1_FILT_OUT11_FILT_OUT_CH11_Pos, ADC1_FILT_OUT11_FILT_OUT_CH11_Msk) >> 2u);
}

/** \brief Read the converted value from the channel 12 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *
 *     if (ADC1->FILT_OUT12.bit.VF12 == 1u)
 *     {
 *         adc_result = ADC1_Ch12_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_Ch12_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->FILT_OUT12.reg, (uint8)ADC1_FILT_OUT12_FILT_OUT_CH12_Pos, ADC1_FILT_OUT12_FILT_OUT_CH12_Msk) >> 2u);
}

#if defined(TLE9845QX)
/** \brief Read the converted value from the differential channel 1 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the converted value from the differential channel 1 result register.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     adc_result = ADC1_DU1_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_DU1_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->DIFFCH_OUT1.reg, (uint8)ADC1_DIFFCH_OUT1_DCH1_Pos, ADC1_DIFFCH_OUT1_DCH1_Msk) >> 2u);
}

/** \brief Read the converted value from the differential channel 2 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the converted value from the differential channel 2 result register.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     adc_result = ADC1_DU2_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_DU2_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->DIFFCH_OUT2.reg, (uint8)ADC1_DIFFCH_OUT2_DCH2_Pos, ADC1_DIFFCH_OUT2_DCH2_Msk) >> 2u);
}

/** \brief Read the converted value from the differential channel 3 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the converted value from the differential channel 3 result register.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     adc_result = ADC1_DU3_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_DU3_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->DIFFCH_OUT3.reg, (uint8)ADC1_DIFFCH_OUT3_DCH3_Pos, ADC1_DIFFCH_OUT3_DCH3_Msk) >> 2u);
}

/** \brief Read the converted value from the differential channel 4 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the converted value from the differential channel 4 result register.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     adc_result = ADC1_DU4_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_DU4_Result_Get(void)
{
  return (u16_Field_Rd32(&ADC1->DIFFCH_OUT4.reg, (uint8)ADC1_DIFFCH_OUT4_DCH4_Pos, ADC1_DIFFCH_OUT4_DCH4_Msk) >> 2u);
}

#endif

/** \brief Read the converted value from the channel 0 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 0 (VBAT_SENSE) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_VBAT_SENSE );
 *
 *     if (ADC1->FILT_OUT0.bit.VF0 == 1u)
 *     {
 *         adc_result = ADC1_VBAT_SENSE_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_VBAT_SENSE_Result_Get(void)
{
  return ADC1_Ch0_Result_Get();
}

/** \brief Read the converted value from the channel 1 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 1 (VS) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_VS );
 *
 *     if (ADC1->FILT_OUT1.bit.VF1 == 1u)
 *     {
 *         adc_result = ADC1_Ch1_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_VS_Result_Get(void)
{
  return ADC1_Ch1_Result_Get();
}

/** \brief Read the converted value from the channel 2 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON1 );
 *
 *     if (ADC1->FILT_OUT2.bit.VF2 == 1u)
 *     {
 *         adc_result = ADC1_Ch2_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_MON1_Result_Get(void)
{
  return ADC1_Ch2_Result_Get();
}

/** \brief Read the converted value from the channel 3 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON2 );
 *
 *     if (ADC1->FILT_OUT3.bit.VF3 == 1u)
 *     {
 *         adc_result = ADC1_Ch3_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_MON2_Result_Get(void)
{
  return ADC1_Ch3_Result_Get();
}

/** \brief Read the converted value from the channel 4 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 4 (MON3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON3 );
 *
 *     if (ADC1->FILT_OUT4.bit.VF4 == 1u)
 *     {
 *         adc_result = ADC1_Ch4_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_MON3_Result_Get(void)
{
  return ADC1_Ch4_Result_Get();
}

/** \brief Read the converted value from the channel 5 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 5 (MON4) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON4 );
 *
 *     if (ADC1->FILT_OUT5.bit.VF5 == 1u)
 *     {
 *         adc_result = ADC1_Ch5_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_MON4_Result_Get(void)
{
  return ADC1_Ch5_Result_Get();
}

/** \brief Read the converted value from the channel 6 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 6 (MON5) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON5 );
 *
 *     if (ADC1->FILT_OUT6.bit.VF6 == 1u)
 *     {
 *         adc_result = ADC1_Ch6_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_MON5_Result_Get(void)
{
  return ADC1_Ch6_Result_Get();
}

/** \brief Read the converted value from the channel 7 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 7 (P2.1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P21 );
 *
 *     if (ADC1->FILT_OUT7.bit.VF7 == 1u)
 *     {
 *         adc_result = ADC1_Ch7_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P21_Result_Get(void)
{
  return ADC1_Ch7_Result_Get();
}

/** \brief Read the converted value from the channel 8 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 8 (P2.2) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P22 );
 *
 *     if (ADC1->FILT_OUT8.bit.VF8 == 1u)
 *     {
 *         adc_result = ADC1_Ch8_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P22_Result_Get(void)
{
  return ADC1_Ch8_Result_Get();
}

/** \brief Read the converted value from the channel 9 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 9 (P2.3) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P23 );
 *
 *     if (ADC1->FILT_OUT9.bit.VF9 == 1u)
 *     {
 *         adc_result = ADC1_Ch9_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P23_Result_Get(void)
{
  return ADC1_Ch9_Result_Get();
}

/** \brief Read the converted value from the channel 10 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 10 (P2.6) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P26 );
 *
 *     if (ADC1->FILT_OUT10.bit.VF10 == 1u)
 *     {
 *         adc_result = ADC1_Ch10_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P26_Result_Get(void)
{
  return ADC1_Ch10_Result_Get();
}

/** \brief Read the converted value from the channel 11 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 11 (P2.7) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_P27 );
 *
 *     if (ADC1->FILT_OUT11.bit.VF11 == 1u)
 *     {
 *         adc_result = ADC1_Ch11_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P27_Result_Get(void)
{
  return ADC1_Ch11_Result_Get();
}

/** \brief Read the converted value from the channel 12 result register.
 *
 * \return 12-bit value, with left aligned 10-bit/8-bit result
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *
 *     if (ADC1->FILT_OUT12.bit.VF12 == 1u)
 *     {
 *         adc_result = ADC1_Ch12_Result_Get();
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE uint16 ADC1_P20_Result_Get(void)
{
  return ADC1_Ch12_Result_Get();
}


/* ADC1 Sequencer configuration*/
/** \brief Define the channels to be measured in Sequence 0, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 * 
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence0_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence0_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ0_1.reg, (uint8)ADC1_SQ0_1_SQ0_Pos, ADC1_SQ0_1_SQ0_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 1, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence1_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence1_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ0_1.reg, (uint8)ADC1_SQ0_1_SQ1_Pos, ADC1_SQ0_1_SQ1_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 2, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence2_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence2_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ2_3.reg, (uint8)ADC1_SQ2_3_SQ2_Pos, ADC1_SQ2_3_SQ2_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 3, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence3_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence3_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ2_3.reg, (uint8)ADC1_SQ2_3_SQ3_Pos, ADC1_SQ2_3_SQ3_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 4, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();


 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence4_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence4_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ4_5.reg, (uint8)ADC1_SQ4_5_SQ4_Pos, ADC1_SQ4_5_SQ4_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 5, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence5_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence5_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ4_5.reg, (uint8)ADC1_SQ4_5_SQ5_Pos, ADC1_SQ4_5_SQ5_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 6, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence6_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence6_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ6_7.reg, (uint8)ADC1_SQ6_7_SQ6_Pos, ADC1_SQ6_7_SQ6_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 7, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence7_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence7_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ6_7.reg, (uint8)ADC1_SQ6_7_SQ7_Pos, ADC1_SQ6_7_SQ7_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 8, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence8_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence8_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ8_9.reg, (uint8)ADC1_SQ8_9_SQ8_Pos, ADC1_SQ8_9_SQ8_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 9, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence9_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence9_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ8_9.reg, (uint8)ADC1_SQ8_9_SQ9_Pos, ADC1_SQ8_9_SQ9_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 10, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence10_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence10_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ10_11.reg, (uint8)ADC1_SQ10_11_SQ10_Pos, ADC1_SQ10_11_SQ10_Msk, mask_ch);
}

/** \brief Define the channels to be measured in Sequence 10, only valid in Sequencer Mode, see \link ADC1_Sequencer_Mode_Sel \endlink.
 *
 * \param mask_ch a bit mask of channels to be set for this sequence
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 3 (MON2) and channel 9 (P2.3).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_Sequence10_Set( ADC1_MASK_MON2 | ADC1_MASK_P23 );
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Sequence11_Set(uint32 mask_ch)
{
  Field_Mod32(&ADC1->SQ10_11.reg, (uint8)ADC1_SQ10_11_SQ11_Pos, ADC1_SQ10_11_SQ11_Msk, mask_ch);
}


/* ADC1 Postprocessing Control Register */
/** \brief Enable Upper threshold IIR filter Channel 0.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch0_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH0_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH0_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch1_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH1_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH1_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch2_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH2_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH2_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 3.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch3_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH3_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH3_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 4.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch4_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH4_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH4_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 5.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch5_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH5_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH5_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 6.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch6_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH6_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH6_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 7.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 7.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch7_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH7_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH7_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 8.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch8_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH8_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH8_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 9.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 9.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch9_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH9_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH9_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 10.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 10.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch10_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH10_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH10_EN_Msk, 1u);
}

/** \brief Enable Upper threshold IIR filter Channel 11.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Upper threshold IIR filter Channel 11.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch11_Filt_Up_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Filt_Up_En(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH11_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH11_EN_Msk, 1u);
}

/** \brief Disable Upper threshold IIR filter Channel 0.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch0_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH0_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH0_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch1_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH1_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH1_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch2_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH2_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH2_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 3.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch3_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH3_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH3_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 4.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch4_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH4_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH4_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 5.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch5_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH5_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH5_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 6.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch6_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH6_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH6_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 7.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 7.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch7_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH7_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH7_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 8.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch8_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH8_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH8_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 9.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 9.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch9_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH9_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH9_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 10.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 10.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch10_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH10_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH10_EN_Msk, 0u);
}

/** \brief Disable Upper threshold IIR filter Channel 11.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Upper threshold IIR filter Channel 11.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch11_Filt_Up_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Filt_Up_Dis(void)
{
  Field_Mod32(&ADC1->FILT_UP_CTRL.reg, (uint8)ADC1_FILT_UP_CTRL_FU_CH11_EN_Pos, ADC1_FILT_UP_CTRL_FU_CH11_EN_Msk, 0u);
}

/** \brief Enable Lower threshold IIR filter Channel 0.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch0_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH0_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH0_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch1_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH1_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH1_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch2_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH2_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH2_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 3.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch3_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH3_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH3_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 4.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch4_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH4_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH4_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 5.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch5_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH5_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH5_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 6.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch6_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH6_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH6_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 7.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 7.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch7_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH7_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH7_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 8.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch8_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH8_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH8_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 9.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 9.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch9_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH9_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH9_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 10.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 10.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch10_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH10_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH10_EN_Msk, 1u);
}

/** \brief Enable Lower threshold IIR filter Channel 11.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables Lower threshold IIR filter Channel 11.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch11_Filt_Lo_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Filt_Lo_En(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH11_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH11_EN_Msk, 1u);
}

/** \brief Disable Lower threshold IIR filter Channel 0.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch0_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH0_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH0_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 1.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch1_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH1_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH1_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 2.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch2_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH2_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH2_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 3.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch3_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH3_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH3_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 4.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch4_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH4_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH4_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 5.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 5.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch5_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH5_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH5_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 6.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch6_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH6_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH6_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 7.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 7.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch7_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH7_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH7_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 8.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch8_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH8_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH8_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 9.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 9.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch9_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH9_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH9_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 10.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 10.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch10_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH10_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH10_EN_Msk, 0u);
}

/** \brief Disable Lower threshold IIR filter Channel 11.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables Lower threshold IIR filter Channel 11.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_Ch11_Filt_Lo_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Filt_Lo_Dis(void)
{
  Field_Mod32(&ADC1->FILT_LO_CTRL.reg, (uint8)ADC1_FILT_LO_CTRL_FL_CH11_EN_Pos, ADC1_FILT_LO_CTRL_FL_CH11_EN_Msk, 0u);
}


/* ADC1 Differential Unit Status Clear*/
#if defined(TLE9845QX)
/** \brief Clear ADC1 Differential Unit 1 (DU1) lower Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 1 (DU1) lower Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Lo_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU1LO_SC_Pos, ADC1_STSCLR_1_DU1LO_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 1 (DU1) upper Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 1 (DU1) upper Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Up_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU1UP_SC_Pos, ADC1_STSCLR_1_DU1UP_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 2 (DU2) lower Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 2 lower Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Lo_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU2LO_SC_Pos, ADC1_STSCLR_1_DU2LO_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 2 (DU2) upper Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 2 upper Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Up_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU2UP_SC_Pos, ADC1_STSCLR_1_DU2UP_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 3 (DU3) lower Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 3 lower Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Lo_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU3LO_SC_Pos, ADC1_STSCLR_1_DU3LO_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 3 (DU3) upper Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 3 upper Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Up_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU3UP_SC_Pos, ADC1_STSCLR_1_DU3UP_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 4 (DU4) lower Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 4 lower Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Lo_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU4LO_SC_Pos, ADC1_STSCLR_1_DU4LO_SC_Msk, 1u);
}

/** \brief Clear ADC1 Differential Unit 4 (DU4) upper Channel flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example clears ADC1 Differential Unit 4 upper Channel flag.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Up_Sts_Clr();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Sts_Clr(void)
{
  Field_Wrt32(&ADC1->STSCLR_1.reg, (uint8)ADC1_STSCLR_1_DU4UP_SC_Pos, ADC1_STSCLR_1_DU4UP_SC_Msk, 1u);
}

#endif

/* ADC1 Differential Measurement Unit */
#if defined(TLE9845QX)
/** \brief Enable ADC1 Differential Unit 1 (DU1).
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Differential Unit 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_En(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU1_EN_Pos, ADC1_DUIN_SEL_DU1_EN_Msk, 1u);
}

/** \brief Enable ADC1 Differential Unit 2 (DU2).
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Differential Unit 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_En(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU2_EN_Pos, ADC1_DUIN_SEL_DU2_EN_Msk, 1u);
}

/** \brief Enable ADC1 Differential Unit 3 (DU3).
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Differential Unit 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_En(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU3_EN_Pos, ADC1_DUIN_SEL_DU3_EN_Msk, 1u);
}

/** \brief Enable ADC1 Differential Unit 4 (DU4).
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 Differential Unit 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_En(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU4_EN_Pos, ADC1_DUIN_SEL_DU4_EN_Msk, 1u);
}

/** \brief Disable ADC1 Differential Unit 1 (DU1).
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC1 Differential Unit 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Dis(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU1_EN_Pos, ADC1_DUIN_SEL_DU1_EN_Msk, 0u);
}

/** \brief Disable ADC1 Differential Unit 2 (DU2).
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC1 Differential Unit 2.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Dis(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU2_EN_Pos, ADC1_DUIN_SEL_DU2_EN_Msk, 0u);
}

/** \brief Disable ADC1 Differential Unit 3 (DU3).
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC1 Differential Unit 3.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Dis(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU3_EN_Pos, ADC1_DUIN_SEL_DU3_EN_Msk, 0u);
}

/** \brief Disable ADC1 Differential Unit 4 (DU4).
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC1 Differential Unit 4.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Dis(void)
{
  Field_Mod32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU4_EN_Pos, ADC1_DUIN_SEL_DU4_EN_Msk, 0u);
}

/** \brief Read the Differential Unit 1 result negative Status.
 *
 * \retval 0 if DU1 Result is positive.
 * \retval 1 if DU1 Result is negative.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Differential Unit 1 result negative Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *
 *     sts = ADC1_DU1_Result_Neg_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_DU1_Result_Neg_Sts(void)
{
  return u1_Field_Rd32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU1RES_NEG_Pos, ADC1_DUIN_SEL_DU1RES_NEG_Msk);
}

/** \brief Read the Differential Unit 2 result negative Status.
 *
 * \retval 0 if DU2 Result is positive.
 * \retval 1 if DU2 Result is negative.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Differential Unit 2 result negative Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *
 *     sts = ADC1_DU2_Result_Neg_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_DU2_Result_Neg_Sts(void)
{
  return u1_Field_Rd32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU2RES_NEG_Pos, ADC1_DUIN_SEL_DU2RES_NEG_Msk);
}

/** \brief Read the Differential Unit 3 result negative Status.
 *
 * \retval 0 if DU3 Result is positive.
 * \retval 1 if DU3 Result is negative.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Differential Unit 3 result negative Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *
 *     sts = ADC1_DU3_Result_Neg_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_DU3_Result_Neg_Sts(void)
{
  return u1_Field_Rd32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU3RES_NEG_Pos, ADC1_DUIN_SEL_DU3RES_NEG_Msk);
}

/** \brief Read the Differential Unit 4 result negative Status.
 *
 * \retval 0 if DU4 Result is positive.
 * \retval 1 if DU4 Result is negative.
 *
 * \brief <b>Example</b><br>
 * \brief This example reads the Differential Unit 4 result negative Status.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint8 sts;
 *
 *     sts = ADC1_DU4_Result_Neg_Sts();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE uint8 ADC1_DU4_Result_Neg_Sts(void)
{
  return u1_Field_Rd32(&ADC1->DUIN_SEL.reg, (uint8)ADC1_DUIN_SEL_DU4RES_NEG_Pos, ADC1_DUIN_SEL_DU4RES_NEG_Msk);
}

#endif

/* ADC1 Overvoltage Measurement Mode */
/** \brief Select the Measurement mode Channel 0.
 *
 * \param mmode Channel 0 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 0 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_0_Pos, ADC1_MMODE0_11_MMODE_0_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 1.
 *
 * \param mmode Channel 1 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 1 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_1_Pos, ADC1_MMODE0_11_MMODE_1_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 2.
 *
 * \param mmode Channel 2 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 2 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_2_Pos, ADC1_MMODE0_11_MMODE_2_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 3.
 *
 * \param mmode Channel 3 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 3 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_3_Pos, ADC1_MMODE0_11_MMODE_3_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 4.
 *
 * \param mmode Channel 4 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 4 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_4_Pos, ADC1_MMODE0_11_MMODE_4_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 5.
 *
 * \param mmode Channel 5 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 5 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_5_Pos, ADC1_MMODE0_11_MMODE_5_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 6.
 *
 * \param mmode Channel 6 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 6 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_6_Pos, ADC1_MMODE0_11_MMODE_6_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 7.
 *
 * \param mmode Channel 7 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 7 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_7_Pos, ADC1_MMODE0_11_MMODE_7_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 8.
 *
 * \param mmode Channel 8 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 8 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_8_Pos, ADC1_MMODE0_11_MMODE_8_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 9.
 *
 * \param mmode Channel 9 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 9 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_9_Pos, ADC1_MMODE0_11_MMODE_9_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 10.
 *
 * \param mmode Channel 10 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 10 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_10_Pos, ADC1_MMODE0_11_MMODE_10_Msk, mmode);
}

/** \brief Select the Measurement mode Channel 11.
 *
 * \param mmode Channel 11 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Channel 11 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_11_Pos, ADC1_MMODE0_11_MMODE_11_Msk, mmode);
}

#if defined(TLE9845QX)
/** \brief Select the Measurement mode Differential Channel 1.
 *
 * \param mmode Differential Channel 1 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Differential Channel 1 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_D1_Pos, ADC1_MMODE0_11_MMODE_D1_Msk, mmode);
}

/** \brief Select the Measurement mode Differential Channel 2.
 *
 * \param mmode Differential Channel 2 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Differential Channel 2 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_D2_Pos, ADC1_MMODE0_11_MMODE_D2_Msk, mmode);
}

/** \brief Select the Measurement mode Differential Channel 3.
 *
 * \param mmode Differential Channel 3 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Differential Channel 3 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_D3_Pos, ADC1_MMODE0_11_MMODE_D3_Msk, mmode);
}

/** \brief Select the Measurement mode Differential Channel 4.
 *
 * \param mmode Differential Channel 4 Measurement Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Measurement mode Differential Channel 4 to undervoltage limit measurement mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Mode_Meas_Sel(ADC1_MMODE_UV); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Mode_Meas_Sel(uint32 mmode)
{
  Field_Mod32(&ADC1->MMODE0_11.reg, (uint8)ADC1_MMODE0_11_MMODE_D4_Pos, ADC1_MMODE0_11_MMODE_D4_Msk, mmode);
}

#endif

/* ADC1 Postprocessing Lower Comparator Trigger Level */
/** \brief Select the Channel 0 lower trigger level.
 *
 * \param trig_val Channel 0 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_LOWER.reg, (uint8)ADC1_TH0_3_LOWER_CH0_LOW_Pos, ADC1_TH0_3_LOWER_CH0_LOW_Msk, trig_val);
}

/** \brief Select the Channel 1 lower trigger level.
 *
 * \param trig_val Channel 1 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_LOWER.reg, (uint8)ADC1_TH0_3_LOWER_CH1_LOW_Pos, ADC1_TH0_3_LOWER_CH1_LOW_Msk, trig_val);
}

/** \brief Select the Channel 2 lower trigger level.
 *
 * \param trig_val Channel 2 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_LOWER.reg, (uint8)ADC1_TH0_3_LOWER_CH2_LOW_Pos, ADC1_TH0_3_LOWER_CH2_LOW_Msk, trig_val);
}

/** \brief Select the Channel 3 lower trigger level.
 *
 * \param trig_val Channel 3 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_LOWER.reg, (uint8)ADC1_TH0_3_LOWER_CH3_LOW_Pos, ADC1_TH0_3_LOWER_CH3_LOW_Msk, trig_val);
}

/** \brief Select the Channel 4 lower trigger level.
 *
 * \param trig_val Channel 4 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_LOWER.reg, (uint8)ADC1_TH4_7_LOWER_CH4_LOW_Pos, ADC1_TH4_7_LOWER_CH4_LOW_Msk, trig_val);
}

/** \brief Select the Channel 5 lower trigger level.
 *
 * \param trig_val Channel 5 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_LOWER.reg, (uint8)ADC1_TH4_7_LOWER_CH5_LOW_Pos, ADC1_TH4_7_LOWER_CH5_LOW_Msk, trig_val);
}

/** \brief Select the Channel 6 lower trigger level.
 *
 * \param trig_val Channel 6 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_LOWER.reg, (uint8)ADC1_TH4_7_LOWER_CH6_LOW_Pos, ADC1_TH4_7_LOWER_CH6_LOW_Msk, trig_val);
}

/** \brief Select the Channel 7 lower trigger level.
 *
 * \param trig_val Channel 7 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_LOWER.reg, (uint8)ADC1_TH4_7_LOWER_CH7_LOW_Pos, ADC1_TH4_7_LOWER_CH7_LOW_Msk, trig_val);
}

/** \brief Select the Channel 8 lower trigger level.
 *
 * \param trig_val Channel 8 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_LOWER.reg, (uint8)ADC1_TH8_11_LOWER_CH8_LOW_Pos, ADC1_TH8_11_LOWER_CH8_LOW_Msk, trig_val);
}

/** \brief Select the Channel 9 lower trigger level.
 *
 * \param trig_val Channel 9 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_LOWER.reg, (uint8)ADC1_TH8_11_LOWER_CH9_LOW_Pos, ADC1_TH8_11_LOWER_CH9_LOW_Msk, trig_val);
}

/** \brief Select the Channel 10 lower trigger level.
 *
 * \param trig_val Channel 10 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_LOWER.reg, (uint8)ADC1_TH8_11_LOWER_CH10_LOW_Pos, ADC1_TH8_11_LOWER_CH10_LOW_Msk, trig_val);
}

/** \brief Select the Channel 11 lower trigger level.
 *
 * \param trig_val Channel 11 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_LOWER.reg, (uint8)ADC1_TH8_11_LOWER_CH11_LOW_Pos, ADC1_TH8_11_LOWER_CH11_LOW_Msk, trig_val);
}

#if defined(TLE9845QX)
/** \brief Select the Differential Channel 1 lower trigger level.
 *
 * \param trig_val Differential Channel 1 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_LOWER.reg, (uint8)ADC1_DCHTH1_4_LOWER_DCH1_LOW_Pos, ADC1_DCHTH1_4_LOWER_DCH1_LOW_Msk, trig_val);
}

/** \brief Select the Differential Channel 2 lower trigger level.
 *
 * \param trig_val Differential Channel 2 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_LOWER.reg, (uint8)ADC1_DCHTH1_4_LOWER_DCH2_LOW_Pos, ADC1_DCHTH1_4_LOWER_DCH2_LOW_Msk, trig_val);
}

/** \brief Select the Differential Channel 3 lower trigger level.
 *
 * \param trig_val Differential Channel 3 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_LOWER.reg, (uint8)ADC1_DCHTH1_4_LOWER_DCH3_LOW_Pos, ADC1_DCHTH1_4_LOWER_DCH3_LOW_Msk, trig_val);
}

/** \brief Select the Differential Channel 4 lower trigger level.
 *
 * \param trig_val Differential Channel 4 lower trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 lower trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Lo_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_LOWER.reg, (uint8)ADC1_DCHTH1_4_LOWER_DCH4_LOW_Pos, ADC1_DCHTH1_4_LOWER_DCH4_LOW_Msk, trig_val);
}

#endif

/* ADC1 Postprocessing Upper Comparator Trigger Level */
/** \brief Select the Channel 0 upper trigger level.
 *
 * \param trig_val Channel 0 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_UPPER.reg, (uint8)ADC1_TH0_3_UPPER_CH0_UP_Pos, ADC1_TH0_3_UPPER_CH0_UP_Msk, trig_val);
}

/** \brief Select the Channel 1 upper trigger level.
 *
 * \param trig_val Channel 1 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_UPPER.reg, (uint8)ADC1_TH0_3_UPPER_CH1_UP_Pos, ADC1_TH0_3_UPPER_CH1_UP_Msk, trig_val);
}

/** \brief Select the Channel 2 upper trigger level.
 *
 * \param trig_val Channel 2 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_UPPER.reg, (uint8)ADC1_TH0_3_UPPER_CH2_UP_Pos, ADC1_TH0_3_UPPER_CH2_UP_Msk, trig_val);
}

/** \brief Select the Channel 3 upper trigger level.
 *
 * \param trig_val Channel 3 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH0_3_UPPER.reg, (uint8)ADC1_TH0_3_UPPER_CH3_UP_Pos, ADC1_TH0_3_UPPER_CH3_UP_Msk, trig_val);
}

/** \brief Select the Channel 4 upper trigger level.
 *
 * \param trig_val Channel 4 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_UPPER.reg, (uint8)ADC1_TH4_7_UPPER_CH4_UP_Pos, ADC1_TH4_7_UPPER_CH4_UP_Msk, trig_val);
}

/** \brief Select the Channel 5 upper trigger level.
 *
 * \param trig_val Channel 5 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_UPPER.reg, (uint8)ADC1_TH4_7_UPPER_CH5_UP_Pos, ADC1_TH4_7_UPPER_CH5_UP_Msk, trig_val);
}

/** \brief Select the Channel 6 upper trigger level.
 *
 * \param trig_val Channel 6 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_UPPER.reg, (uint8)ADC1_TH4_7_UPPER_CH6_UP_Pos, ADC1_TH4_7_UPPER_CH6_UP_Msk, trig_val);
}

/** \brief Select the Channel 7 upper trigger level.
 *
 * \param trig_val Channel 7 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH4_7_UPPER.reg, (uint8)ADC1_TH4_7_UPPER_CH7_UP_Pos, ADC1_TH4_7_UPPER_CH7_UP_Msk, trig_val);
}

/** \brief Select the Channel 8 upper trigger level.
 *
 * \param trig_val Channel 8 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_UPPER.reg, (uint8)ADC1_TH8_11_UPPER_CH8_UP_Pos, ADC1_TH8_11_UPPER_CH8_UP_Msk, trig_val);
}

/** \brief Select the Channel 9 upper trigger level.
 *
 * \param trig_val Channel 9 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_UPPER.reg, (uint8)ADC1_TH8_11_UPPER_CH9_UP_Pos, ADC1_TH8_11_UPPER_CH9_UP_Msk, trig_val);
}

/** \brief Select the Channel 10 upper trigger level.
 *
 * \param trig_val Channel 10 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_UPPER.reg, (uint8)ADC1_TH8_11_UPPER_CH10_UP_Pos, ADC1_TH8_11_UPPER_CH10_UP_Msk, trig_val);
}

/** \brief Select the Channel 11 upper trigger level.
 *
 * \param trig_val Channel 11 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->TH8_11_UPPER.reg, (uint8)ADC1_TH8_11_UPPER_CH11_UP_Pos, ADC1_TH8_11_UPPER_CH11_UP_Msk, trig_val);
}

#if defined(TLE9845QX)
/** \brief Select the Differential Channel 1 upper trigger level.
 *
 * \param trig_val Differential Channel 1 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_UPPER.reg, (uint8)ADC1_DCHTH1_4_UPPER_DCH1_UP_Pos, ADC1_DCHTH1_4_UPPER_DCH1_UP_Msk, trig_val);
}

/** \brief Select the Differential Channel 2 upper trigger level.
 *
 * \param trig_val Differential Channel 2 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_UPPER.reg, (uint8)ADC1_DCHTH1_4_UPPER_DCH2_UP_Pos, ADC1_DCHTH1_4_UPPER_DCH2_UP_Msk, trig_val);
}

/** \brief Select the Differential Channel 3 upper trigger level.
 *
 * \param trig_val Differential Channel 3 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_UPPER.reg, (uint8)ADC1_DCHTH1_4_UPPER_DCH3_UP_Pos, ADC1_DCHTH1_4_UPPER_DCH3_UP_Msk, trig_val);
}

/** \brief Select the Differential Channel 4 upper trigger level.
 *
 * \param trig_val Differential Channel 4 upper trigger level
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 upper trigger level to Min threshold value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Up_Th_Set(0u); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Th_Set(uint32 trig_val)
{
  Field_Mod32(&ADC1->DCHTH1_4_UPPER.reg, (uint8)ADC1_DCHTH1_4_UPPER_DCH4_UP_Pos, ADC1_DCHTH1_4_UPPER_DCH4_UP_Msk, trig_val);
}

/* Differential Unit Trigger Selection */
/** \brief Select the Differential Unit 1 Trigger Enable.
 *
 * \param trig_Ch Differential Unit 1 Trigger Enable Options, \link du_triggen_sel \endlink
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Unit 1 Trigger Enable to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Trigger_Sel(DU_TRIGGEN_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Trigger_Sel(du_triggen_sel trig_Ch)
{
  Field_Mod32(&SCU->MODPISEL4.reg, (uint8)SCU_MODPISEL4_DU1TRIGGEN_Pos, SCU_MODPISEL4_DU1TRIGGEN_Msk, trig_Ch);
}

/** \brief Select the Differential Unit 2 Trigger Enable.
 *
 * \param trig_Ch Differential Unit 2 Trigger Enable Options, \link du_triggen_sel \endlink
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Unit 2 Trigger Enable to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Trigger_Sel(DU_TRIGGEN_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Trigger_Sel(du_triggen_sel trig_Ch)
{
  Field_Mod32(&SCU->MODPISEL4.reg, (uint8)SCU_MODPISEL4_DU2TRIGGEN_Pos, SCU_MODPISEL4_DU2TRIGGEN_Msk, trig_Ch);
}

/** \brief Select the Differential Unit 3 Trigger Enable.
 *
 * \param trig_Ch Differential Unit 3 Trigger Enable Options, \link du_triggen_sel \endlink
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Unit 3 Trigger Enable to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Trigger_Sel(DU_TRIGGEN_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Trigger_Sel(du_triggen_sel trig_Ch)
{
  Field_Mod32(&SCU->MODPISEL4.reg, (uint8)SCU_MODPISEL4_DU3TRIGGEN_Pos, SCU_MODPISEL4_DU3TRIGGEN_Msk, trig_Ch);
}

/** \brief Select the Differential Unit 4 Trigger Enable.
 *
 * \param trig_Ch Differential Unit 4 Trigger Enable Options, \link du_triggen_sel \endlink
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Unit 4 Trigger Enable to CC61.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Trigger_Sel(DU_TRIGGEN_CC61); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Trigger_Sel(du_triggen_sel trig_Ch)
{
  Field_Mod32(&SCU->MODPISEL4.reg, (uint8)SCU_MODPISEL4_DU4TRIGGEN_Pos, SCU_MODPISEL4_DU4TRIGGEN_Msk, trig_Ch);
}

#endif

/* ADC1 Lower Hysteresis Trigger Level  */
/** \brief Set the Channel 0 lower hysteresis Mode.
 *
 * \param hyst Channel 0 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_HYST_LO_CH0_Pos, ADC1_CNT0_3_LOWER_HYST_LO_CH0_Msk, hyst);
}

/** \brief Set the Channel 1 lower hysteresis Mode.
 *
 * \param hyst Channel 1 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_HYST_LO_CH1_Pos, ADC1_CNT0_3_LOWER_HYST_LO_CH1_Msk, hyst);
}

/** \brief Set the Channel 2 lower hysteresis Mode.
 *
 * \param hyst Channel 2 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_HYST_LO_CH2_Pos, ADC1_CNT0_3_LOWER_HYST_LO_CH2_Msk, hyst);
}

/** \brief Set the Channel 3 lower hysteresis Mode.
 *
 * \param hyst Channel 3 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_HYST_LO_CH3_Pos, ADC1_CNT0_3_LOWER_HYST_LO_CH3_Msk, hyst);
}

/** \brief Set the Channel 4 lower hysteresis Mode.
 *
 * \param hyst Channel 4 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_HYST_LO_CH4_Pos, ADC1_CNT4_7_LOWER_HYST_LO_CH4_Msk, hyst);
}

/** \brief Set the Channel 5 lower hysteresis Mode.
 *
 * \param hyst Channel 5 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_HYST_LO_CH5_Pos, ADC1_CNT4_7_LOWER_HYST_LO_CH5_Msk, hyst);
}

/** \brief Set the Channel 6 lower hysteresis Mode.
 *
 * \param hyst Channel 6 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_HYST_LO_CH6_Pos, ADC1_CNT4_7_LOWER_HYST_LO_CH6_Msk, hyst);
}

/** \brief Set the Channel 7 lower hysteresis Mode.
 *
 * \param hyst Channel 7 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_HYST_LO_CH7_Pos, ADC1_CNT4_7_LOWER_HYST_LO_CH7_Msk, hyst);
}

/** \brief Set the Channel 8 lower hysteresis Mode.
 *
 * \param hyst Channel 8 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_HYST_LO_CH8_Pos, ADC1_CNT8_11_LOWER_HYST_LO_CH8_Msk, hyst);
}

/** \brief Set the Channel 9 lower hysteresis Mode.
 *
 * \param hyst Channel 9 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_HYST_LO_CH9_Pos, ADC1_CNT8_11_LOWER_HYST_LO_CH9_Msk, hyst);
}

/** \brief Set the Channel 10 lower hysteresis Mode.
 *
 * \param hyst Channel 10 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_HYST_LO_CH10_Pos, ADC1_CNT8_11_LOWER_HYST_LO_CH10_Msk, hyst);
}

/** \brief Set the Channel 11 lower hysteresis Mode.
 *
 * \param hyst Channel 11 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_HYST_LO_CH11_Pos, ADC1_CNT8_11_LOWER_HYST_LO_CH11_Msk, hyst);
}

#if defined(TLE9845QX)
/** \brief Set the Differential Channel 1 lower hysteresis Mode.
 *
 * \param hyst Differential Channel 1 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH1_Pos, ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH1_Msk, hyst);
}

/** \brief Set the Differential Channel 2 lower hysteresis Mode.
 *
 * \param hyst Differential Channel 2 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH2_Pos, ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH2_Msk, hyst);
}

/** \brief Set the Differential Channel 3 lower hysteresis Mode.
 *
 * \param hyst Differential Channel 3 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH3_Pos, ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH3_Msk, hyst);
}

/** \brief Set the Differential Channel 4 lower hysteresis Mode.
 *
 * \param hyst Differential Channel 4 lower hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 lower hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Lo_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH4_Pos, ADC1_DCHCNT1_4_LOWER_HYST_LO_DCH4_Msk, hyst);
}

#endif

/* ADC1 Upper Hysteresis Trigger Level  */
/** \brief Set the Channel 0 upper hysteresis Mode.
 *
 * \param hyst Channel 0 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_HYST_UP_CH0_Pos, ADC1_CNT0_3_UPPER_HYST_UP_CH0_Msk, hyst);
}

/** \brief Set the Channel 1 upper hysteresis Mode.
 *
 * \param hyst Channel 1 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_HYST_UP_CH1_Pos, ADC1_CNT0_3_UPPER_HYST_UP_CH1_Msk, hyst);
}

/** \brief Set the Channel 2 upper hysteresis Mode.
 *
 * \param hyst Channel 2 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_HYST_UP_CH2_Pos, ADC1_CNT0_3_UPPER_HYST_UP_CH2_Msk, hyst);
}

/** \brief Set the Channel 3 upper hysteresis Mode.
 *
 * \param hyst Channel 3 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_HYST_UP_CH3_Pos, ADC1_CNT0_3_UPPER_HYST_UP_CH3_Msk, hyst);
}

/** \brief Set the Channel 4 upper hysteresis Mode.
 *
 * \param hyst Channel 4 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_HYST_UP_CH4_Pos, ADC1_CNT4_7_UPPER_HYST_UP_CH4_Msk, hyst);
}

/** \brief Set the Channel 5 upper hysteresis Mode.
 *
 * \param hyst Channel 5 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_HYST_UP_CH5_Pos, ADC1_CNT4_7_UPPER_HYST_UP_CH5_Msk, hyst);
}

/** \brief Set the Channel 6 upper hysteresis Mode.
 *
 * \param hyst Channel 6 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_HYST_UP_CH6_Pos, ADC1_CNT4_7_UPPER_HYST_UP_CH6_Msk, hyst);
}

/** \brief Set the Channel 7 upper hysteresis Mode.
 *
 * \param hyst Channel 7 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_HYST_UP_CH7_Pos, ADC1_CNT4_7_UPPER_HYST_UP_CH7_Msk, hyst);
}

/** \brief Set the Channel 8 upper hysteresis Mode.
 *
 * \param hyst Channel 8 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_HYST_UP_CH8_Pos, ADC1_CNT8_11_UPPER_HYST_UP_CH8_Msk, hyst);
}

/** \brief Set the Channel 9 upper hysteresis Mode.
 *
 * \param hyst Channel 9 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_HYST_UP_CH9_Pos, ADC1_CNT8_11_UPPER_HYST_UP_CH9_Msk, hyst);
}

/** \brief Set the Channel 10 upper hysteresis Mode.
 *
 * \param hyst Channel 10 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_HYST_UP_CH10_Pos, ADC1_CNT8_11_UPPER_HYST_UP_CH10_Msk, hyst);
}

/** \brief Set the Channel 11 upper hysteresis Mode.
 *
 * \param hyst Channel 11 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_HYST_UP_CH11_Pos, ADC1_CNT8_11_UPPER_HYST_UP_CH11_Msk, hyst);
}

#if defined(TLE9845QX)
/** \brief Set the Differential Channel 1 upper hysteresis Mode.
 *
 * \param hyst Differential Channel 1 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH1_Pos, ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH1_Msk, hyst);
}

/** \brief Set the Differential Channel 2 upper hysteresis Mode.
 *
 * \param hyst Differential Channel 2 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH2_Pos, ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH2_Msk, hyst);
}

/** \brief Set the Differential Channel 3 upper hysteresis Mode.
 *
 * \param hyst Differential Channel 3 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH3_Pos, ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH3_Msk, hyst);
}

/** \brief Set the Differential Channel 4 upper hysteresis Mode.
 *
 * \param hyst Differential Channel 4 upper hysteresis Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 upper hysteresis Mode to HYST8.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Up_hys_Set(ADC1_HYST_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_hys_Set(uint32 hyst)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH4_Pos, ADC1_DCHCNT1_4_UPPER_HYST_UP_DCH4_Msk, hyst);
}

#endif

/** \brief Set the Channel 0 Lower timer trigger threshold.
 *
 * \param cnt Channel 0 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_CNT_LO_CH0_Pos, ADC1_CNT0_3_LOWER_CNT_LO_CH0_Msk, cnt);
}

/** \brief Set the Channel 1 Lower timer trigger threshold.
 *
 * \param cnt Channel 1 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_CNT_LO_CH1_Pos, ADC1_CNT0_3_LOWER_CNT_LO_CH1_Msk, cnt);
}

/** \brief Set the Channel 2 Lower timer trigger threshold.
 *
 * \param cnt Channel 2 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_CNT_LO_CH2_Pos, ADC1_CNT0_3_LOWER_CNT_LO_CH2_Msk, cnt);
}

/** \brief Set the Channel 3 Lower timer trigger threshold.
 *
 * \param cnt Channel 3 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_LOWER.reg, (uint8)ADC1_CNT0_3_LOWER_CNT_LO_CH3_Pos, ADC1_CNT0_3_LOWER_CNT_LO_CH3_Msk, cnt);
}

/** \brief Set the Channel 4 Lower timer trigger threshold.
 *
 * \param cnt Channel 4 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_CNT_LO_CH4_Pos, ADC1_CNT4_7_LOWER_CNT_LO_CH4_Msk, cnt);
}

/** \brief Set the Channel 5 Lower timer trigger threshold.
 *
 * \param cnt Channel 5 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_CNT_LO_CH5_Pos, ADC1_CNT4_7_LOWER_CNT_LO_CH5_Msk, cnt);
}

/** \brief Set the Channel 6 Lower timer trigger threshold.
 *
 * \param cnt Channel 6 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_CNT_LO_CH6_Pos, ADC1_CNT4_7_LOWER_CNT_LO_CH6_Msk, cnt);
}

/** \brief Set the Channel 7 Lower timer trigger threshold.
 *
 * \param cnt Channel 7 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_LOWER.reg, (uint8)ADC1_CNT4_7_LOWER_CNT_LO_CH7_Pos, ADC1_CNT4_7_LOWER_CNT_LO_CH7_Msk, cnt);
}

/** \brief Set the Channel 8 Lower timer trigger threshold.
 *
 * \param cnt Channel 8 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_CNT_LO_CH8_Pos, ADC1_CNT8_11_LOWER_CNT_LO_CH8_Msk, cnt);
}

/** \brief Set the Channel 9 Lower timer trigger threshold.
 *
 * \param cnt Channel 9 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_CNT_LO_CH9_Pos, ADC1_CNT8_11_LOWER_CNT_LO_CH9_Msk, cnt);
}

/** \brief Set the Channel 10 Lower timer trigger threshold.
 *
 * \param cnt Channel 10 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_CNT_LO_CH10_Pos, ADC1_CNT8_11_LOWER_CNT_LO_CH10_Msk, cnt);
}

/** \brief Set the Channel 11 Lower timer trigger threshold.
 *
 * \param cnt Channel 11 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_LOWER.reg, (uint8)ADC1_CNT8_11_LOWER_CNT_LO_CH11_Pos, ADC1_CNT8_11_LOWER_CNT_LO_CH11_Msk, cnt);
}

#if defined(TLE9845QX)
/** \brief Set the Differential Channel 1 Lower timer trigger threshold.
 *
 * \param cnt Differential Channel 1 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH1_Pos, ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH1_Msk, cnt);
}

/** \brief Set the Differential Channel 2 Lower timer trigger threshold.
 *
 * \param cnt Differential Channel 2 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH2_Pos, ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH2_Msk, cnt);
}

/** \brief Set the Differential Channel 3 Lower timer trigger threshold.
 *
 * \param cnt Differential Channel 3 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH3_Pos, ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH3_Msk, cnt);
}

/** \brief Set the Differential Channel 4 Lower timer trigger threshold.
 *
 * \param cnt Differential Channel 4 Lower timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 Lower timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Lo_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_LOWER.reg, (uint8)ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH4_Pos, ADC1_DCHCNT1_4_LOWER_CNT_LO_DCH4_Msk, cnt);
}

#endif

/* ADC1 Upper Counter Trigger Level  */
/** \brief Set the Channel 0 Upper timer trigger threshold.
 *
 * \param cnt Channel 0 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 0 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch0_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch0_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_CNT_UP_CH0_Pos, ADC1_CNT0_3_UPPER_CNT_UP_CH0_Msk, cnt);
}

/** \brief Set the Channel 1 Upper timer trigger threshold.
 *
 * \param cnt Channel 1 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 1 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch1_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch1_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_CNT_UP_CH1_Pos, ADC1_CNT0_3_UPPER_CNT_UP_CH1_Msk, cnt);
}

/** \brief Set the Channel 2 Upper timer trigger threshold.
 *
 * \param cnt Channel 2 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 2 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch2_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch2_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_CNT_UP_CH2_Pos, ADC1_CNT0_3_UPPER_CNT_UP_CH2_Msk, cnt);
}

/** \brief Set the Channel 3 Upper timer trigger threshold.
 *
 * \param cnt Channel 3 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 3 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch3_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch3_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT0_3_UPPER.reg, (uint8)ADC1_CNT0_3_UPPER_CNT_UP_CH3_Pos, ADC1_CNT0_3_UPPER_CNT_UP_CH3_Msk, cnt);
}

/** \brief Set the Channel 4 Upper timer trigger threshold.
 *
 * \param cnt Channel 4 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 4 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch4_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch4_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_CNT_UP_CH4_Pos, ADC1_CNT4_7_UPPER_CNT_UP_CH4_Msk, cnt);
}

/** \brief Set the Channel 5 Upper timer trigger threshold.
 *
 * \param cnt Channel 5 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 5 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch5_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch5_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_CNT_UP_CH5_Pos, ADC1_CNT4_7_UPPER_CNT_UP_CH5_Msk, cnt);
}

/** \brief Set the Channel 6 Upper timer trigger threshold.
 *
 * \param cnt Channel 6 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 6 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch6_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch6_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_CNT_UP_CH6_Pos, ADC1_CNT4_7_UPPER_CNT_UP_CH6_Msk, cnt);
}

/** \brief Set the Channel 7 Upper timer trigger threshold.
 *
 * \param cnt Channel 7 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 7 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch7_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch7_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT4_7_UPPER.reg, (uint8)ADC1_CNT4_7_UPPER_CNT_UP_CH7_Pos, ADC1_CNT4_7_UPPER_CNT_UP_CH7_Msk, cnt);
}

/** \brief Set the Channel 8 Upper timer trigger threshold.
 *
 * \param cnt Channel 8 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 8 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch8_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch8_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_CNT_UP_CH8_Pos, ADC1_CNT8_11_UPPER_CNT_UP_CH8_Msk, cnt);
}

/** \brief Set the Channel 9 Upper timer trigger threshold.
 *
 * \param cnt Channel 9 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 9 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch9_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch9_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_CNT_UP_CH9_Pos, ADC1_CNT8_11_UPPER_CNT_UP_CH9_Msk, cnt);
}

/** \brief Set the Channel 10 Upper timer trigger threshold.
 *
 * \param cnt Channel 10 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 10 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch10_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch10_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_CNT_UP_CH10_Pos, ADC1_CNT8_11_UPPER_CNT_UP_CH10_Msk, cnt);
}

/** \brief Set the Channel 11 Upper timer trigger threshold.
 *
 * \param cnt Channel 11 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Channel 11 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Ch11_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_Ch11_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->CNT8_11_UPPER.reg, (uint8)ADC1_CNT8_11_UPPER_CNT_UP_CH11_Pos, ADC1_CNT8_11_UPPER_CNT_UP_CH11_Msk, cnt);
}

#if defined(TLE9845QX)
/** \brief Set the Differential Channel 1 Upper timer trigger threshold.
 *
 * \param cnt Differential Channel 1 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 1 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU1_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH1_Pos, ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH1_Msk, cnt);
}

/** \brief Set the Differential Channel 2 Upper timer trigger threshold.
 *
 * \param cnt Differential Channel 2 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 2 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU2_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH2_Pos, ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH2_Msk, cnt);
}

/** \brief Set the Differential Channel 3 Upper timer trigger threshold.
 *
 * \param cnt Differential Channel 3 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 3 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU3_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH3_Pos, ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH3_Msk, cnt);
}

/** \brief Set the Differential Channel 4 Upper timer trigger threshold.
 *
 * \param cnt Differential Channel 4 Upper timer trigger threshold
 *
 * \brief <b>Example</b><br>
 * \brief This example sets the Differential Channel 4 Upper timer trigger threshold to 8 measurements.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_DU4_Up_Cnt_Set(ADC1_CNT_8); 
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Cnt_Set(uint32 cnt)
{
  Field_Mod32(&ADC1->DCHCNT1_4_UPPER.reg, (uint8)ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH4_Pos, ADC1_DCHCNT1_4_UPPER_CNT_UP_DCH4_Msk, cnt);
}

#endif

/* ADC1 Interrupt Status Clear */
/** \brief Clear VBAT Overvoltage Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VBAT Overvoltage Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VBAT_OV_IE == 1u)
 *   {	 
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VBAT_OV_IS == 1u)
 *     {
 *       ADC1_CH0_UP_CALLBACK();
 *       ADC1_VBAT_OV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VBAT_OV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VBAT_OV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VBAT_OV_ISC_Msk, 1u);
}

/** \brief Clear VBAT Undervoltage Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the VBAT Undervoltage Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   if (SCUPM->SYS_SUPPLY_IRQ_CTRL.bit.VBAT_UV_IE == 1u)
 *   {	 
 *     if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VBAT_UV_IS == 1u)
 *     {
 *       ADC1_CH0_LO_CALLBACK();
 *       ADC1_VBAT_UV_Int_Clr();
 *     }
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VBAT_UV_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_SUPPLY_IRQ_CLR.reg, (uint8)SCUPM_SYS_SUPPLY_IRQ_CLR_VBAT_UV_ISC_Pos, SCUPM_SYS_SUPPLY_IRQ_CLR_VBAT_UV_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 0 (VbatSense) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 0 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VBatSense_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VBATSEN_IS == 1u)
 *   {
 *     ADC1_CH0_CALLBACK();
 *     ADC1_VbatSense_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VbatSense_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_VBATSEN_ISC_Pos, ADC1_IRQCLR_1_VBATSEN_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 1 (VS) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 1 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VS_IS == 1u)
 *   {
 *     ADC1_CH1_CALLBACK();
 *     ADC1_VS_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_VS_ISC_Pos, ADC1_IRQCLR_1_VS_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 2 (MON1) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 2 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON1_IS == 1u)
 *   {
 *     ADC1_CH2_CALLBACK();
 *     ADC1_MON1_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_MON1_ISC_Pos, ADC1_IRQCLR_1_MON1_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 3 (MON2) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 3 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON2_IS == 1u)
 *   {
 *     ADC1_CH3_CALLBACK();
 *     ADC1_MON2_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_MON2_ISC_Pos, ADC1_IRQCLR_1_MON2_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 4 (MON3) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 4 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON3_IS == 1u)
 *   {
 *     ADC1_CH4_CALLBACK();
 *     ADC1_MON3_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_MON3_ISC_Pos, ADC1_IRQCLR_1_MON3_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 5 (MON4) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 5 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON4_IS == 1u)
 *   {
 *     ADC1_CH5_CALLBACK();
 *     ADC1_MON4_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_MON4_ISC_Pos, ADC1_IRQCLR_1_MON4_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 6 (MON5) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 6 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON5_IS == 1u)
 *   {
 *     ADC1_CH6_CALLBACK();
 *     ADC1_MON5_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_MON5_ISC_Pos, ADC1_IRQCLR_1_MON5_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 7 (P2.1) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 7 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_1_IS == 1u)
 *   {
 *     ADC1_CH7_CALLBACK();
 *     ADC1_P21_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_1_ISC_Pos, ADC1_IRQCLR_1_P2_1_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 8 (P2.2) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 8 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_2_IS == 1u)
 *   {
 *     ADC1_CH8_CALLBACK();
 *     ADC1_P22_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_2_ISC_Pos, ADC1_IRQCLR_1_P2_2_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 9 (P2.3) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 9 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_3_IS == 1u)
 *   {
 *     ADC1_CH9_CALLBACK();
 *     ADC1_P23_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_3_ISC_Pos, ADC1_IRQCLR_1_P2_3_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 10 (P2.6) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 10 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_6_IS == 1u)
 *   {
 *     ADC1_CH10_CALLBACK();
 *     ADC1_P26_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_6_ISC_Pos, ADC1_IRQCLR_1_P2_6_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 11 (P2.7) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 11 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_7_IS == 1u)
 *   {
 *     ADC1_CH11_CALLBACK();
 *     ADC1_P27_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_7_ISC_Pos, ADC1_IRQCLR_1_P2_7_ISC_Msk, 1u);
}

/** \brief Clear ADC1 Channel 12 (P2.0) Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 12 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P20_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_0_IS == 1u)
 *   {
 *     ADC1_CH12_CALLBACK();
 *     ADC1_P20_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P20_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_P2_0_ISC_Pos, ADC1_IRQCLR_1_P2_0_ISC_Msk, 1u);
}

/** \brief Clear Exceptional Interrupt Measurement (EIM) flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_EIM_En();
 *   ADC1_EIM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.EIM_IS == 1u)
 *   {
 *     ADC1_EIM_CALLBACK();
 *     ADC1_EIM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_EIM_ISC_Pos, ADC1_IRQCLR_1_EIM_ISC_Msk, 1u);
}

/** \brief Clear Exceptional Sequence Measurement (ESM) flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Sequence Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_ESM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.ESM_IS == 1u)
 *   {
 *     ADC1_ESM_CALLBACK();
 *     ADC1_ESM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_ESM_ISC_Pos, ADC1_IRQCLR_1_ESM_ISC_Msk, 1u);
}

/** \brief Clear ADC1 VS Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_LO_IS == 1u)
 *   {
 *     ADC1_CH1_LO_CALLBACK();
 *     ADC1_VS_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_VS_LO_ISC_Pos, ADC1_IRQCLR_2_VS_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON1 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_LO_IS == 1u)
 *   {
 *     ADC1_CH2_LO_CALLBACK();
 *     ADC1_MON1_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON1_LO_ISC_Pos, ADC1_IRQCLR_2_MON1_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON2 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_LO_IS == 1u)
 *   {
 *     ADC1_CH3_LO_CALLBACK();
 *     ADC1_MON2_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON2_LO_ISC_Pos, ADC1_IRQCLR_2_MON2_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON3 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_LO_IS == 1u)
 *   {
 *     ADC1_CH4_LO_CALLBACK();
 *     ADC1_MON3_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON3_LO_ISC_Pos, ADC1_IRQCLR_2_MON3_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON4 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_LO_IS == 1u)
 *   {
 *     ADC1_CH5_LO_CALLBACK();
 *     ADC1_MON4_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON4_LO_ISC_Pos, ADC1_IRQCLR_2_MON4_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON5 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_LO_IS == 1u)
 *   {
 *     ADC1_CH6_LO_CALLBACK();
 *     ADC1_MON5_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON5_LO_ISC_Pos, ADC1_IRQCLR_2_MON5_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.1 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_LO_IS == 1u)
 *   {
 *     ADC1_CH7_LO_CALLBACK();
 *     ADC1_P21_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_1_LO_ISC_Pos, ADC1_IRQCLR_2_P2_1_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.2 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_LO_IS == 1u)
 *   {
 *     ADC1_CH8_LO_CALLBACK();
 *     ADC1_P22_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_2_LO_ISC_Pos, ADC1_IRQCLR_2_P2_2_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.3 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_LO_IS == 1u)
 *   {
 *     ADC1_CH9_LO_CALLBACK();
 *     ADC1_P23_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_3_LO_ISC_Pos, ADC1_IRQCLR_2_P2_3_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.6 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_LO_IS == 1u)
 *   {
 *     ADC1_CH10_LO_CALLBACK();
 *     ADC1_P26_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_6_LO_ISC_Pos, ADC1_IRQCLR_2_P2_6_LO_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.7 Lower Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_LO_IS == 1u)
 *   {
 *     ADC1_CH11_LO_CALLBACK();
 *     ADC1_P27_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_7_LO_ISC_Pos, ADC1_IRQCLR_2_P2_7_LO_ISC_Msk, 1u);
}


/** \brief Clear ADC1 VS Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_UP_IS == 1u)
 *   {
 *     ADC1_CH1_UP_CALLBACK();
 *     ADC1_VS_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_VS_UP_ISC_Pos, ADC1_IRQCLR_2_VS_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON1 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_UP_IS == 1u)
 *   {
 *     ADC1_CH2_UP_CALLBACK();
 *     ADC1_MON1_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON1_UP_ISC_Pos, ADC1_IRQCLR_2_MON1_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON2 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_UP_IS == 1u)
 *   {
 *     ADC1_CH3_UP_CALLBACK();
 *     ADC1_MON2_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON2_UP_ISC_Pos, ADC1_IRQCLR_2_MON2_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON3 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_UP_IS == 1u)
 *   {
 *     ADC1_CH4_UP_CALLBACK();
 *     ADC1_MON3_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON3_UP_ISC_Pos, ADC1_IRQCLR_2_MON3_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON4 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_UP_IS == 1u)
 *   {
 *     ADC1_CH5_UP_CALLBACK();
 *     ADC1_MON4_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON4_UP_ISC_Pos, ADC1_IRQCLR_2_MON4_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 MON5 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_UP_IS == 1u)
 *   {
 *     ADC1_CH6_UP_CALLBACK();
 *     ADC1_MON5_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_MON5_UP_ISC_Pos, ADC1_IRQCLR_2_MON5_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.1 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_UP_IS == 1u)
 *   {
 *     ADC1_CH7_UP_CALLBACK();
 *     ADC1_P21_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_1_UP_ISC_Pos, ADC1_IRQCLR_2_P2_1_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.2 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_UP_IS == 1u)
 *   {
 *     ADC1_CH8_UP_CALLBACK();
 *     ADC1_P22_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_2_UP_ISC_Pos, ADC1_IRQCLR_2_P2_2_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.3 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_UP_IS == 1u)
 *   {
 *     ADC1_CH9_UP_CALLBACK();
 *     ADC1_P23_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_3_UP_ISC_Pos, ADC1_IRQCLR_2_P2_3_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.6 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_UP_IS == 1u)
 *   {
 *     ADC1_CH10_UP_CALLBACK();
 *     ADC1_P26_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_6_UP_ISC_Pos, ADC1_IRQCLR_2_P2_6_UP_ISC_Msk, 1u);
}

/** \brief Clear ADC1 P2.7 Upper Threshold Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_UP_IS == 1u)
 *   {
 *     ADC1_CH11_UP_CALLBACK();
 *     ADC1_P27_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_2.reg, (uint8)ADC1_IRQCLR_2_P2_7_UP_ISC_Pos, ADC1_IRQCLR_2_P2_7_UP_ISC_Msk, 1u);
}


#if defined(TLE9845QX)
/** \brief Clear Differential Unit 1 lower Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1LO_IS == 1u)
 *   {
 *     ADC1_DCH1_LO_CALLBACK();
 *     ADC1_DU1_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU1LO_ISC_Pos, ADC1_IRQCLR_1_DU1LO_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 1 upper Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1UP_IS == 1u)
 *   {
 *     ADC1_DCH1_UP_CALLBACK();
 *     ADC1_DU1_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU1UP_ISC_Pos, ADC1_IRQCLR_1_DU1UP_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 2 lower Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2LO_IS == 1u)
 *   {
 *     ADC1_DCH2_LO_CALLBACK();
 *     ADC1_DU2_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU2LO_ISC_Pos, ADC1_IRQCLR_1_DU2LO_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 2 upper Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2UP_IS == 1u)
 *   {
 *     ADC1_DCH2_UP_CALLBACK();
 *     ADC1_DU2_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU2UP_ISC_Pos, ADC1_IRQCLR_1_DU2UP_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 3 lower Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3LO_IS == 1u)
 *   {
 *     ADC1_DCH3_LO_CALLBACK();
 *     ADC1_DU3_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU3LO_ISC_Pos, ADC1_IRQCLR_1_DU3LO_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 3 upper Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3UP_IS == 1u)
 *   {
 *     ADC1_DCH3_UP_CALLBACK();
 *     ADC1_DU3_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU3UP_ISC_Pos, ADC1_IRQCLR_1_DU3UP_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 4 lower Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4LO_IS == 1u)
 *   {
 *     ADC1_DCH4_LO_CALLBACK();
 *     ADC1_DU4_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU4LO_ISC_Pos, ADC1_IRQCLR_1_DU4LO_ISC_Msk, 1u);
}

/** \brief Clear Differential Unit 4 upper Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4UP_IS == 1u)
 *   {
 *     ADC1_DCH4_UP_CALLBACK();
 *     ADC1_DU4_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Int_Clr(void)
{
  Field_Wrt32(&ADC1->IRQCLR_1.reg, (uint8)ADC1_IRQCLR_1_DU4UP_ISC_Pos, ADC1_IRQCLR_1_DU4UP_ISC_Msk, 1u);
}

#endif

/* ADC1 Interrupt Enable (ADC1_IRQEN_1) */
/** \brief Enable ADC1 Channel 0 (VbatSense) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 0 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VBatSense_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VBATSEN_IS == 1u)
 *   {
 *     ADC1_CH0_CALLBACK();
 *     ADC1_VbatSense_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VBatSense_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_VBATSEN_IEN_Pos, ADC1_IRQEN_1_VBATSEN_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 1 (VS) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 1 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VS_IS == 1u)
 *   {
 *     ADC1_CH1_CALLBACK();
 *     ADC1_VS_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_VS_IEN_Pos, ADC1_IRQEN_1_VS_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 2 (MON1) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 2 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON1_IS == 1u)
 *   {
 *     ADC1_CH2_CALLBACK();
 *     ADC1_MON1_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON1_IEN_Pos, ADC1_IRQEN_1_MON1_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 3 (MON2) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 3 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON2_IS == 1u)
 *   {
 *     ADC1_CH3_CALLBACK();
 *     ADC1_MON2_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON2_IEN_Pos, ADC1_IRQEN_1_MON2_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 4 (MON3) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 4 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON3_IS == 1u)
 *   {
 *     ADC1_CH4_CALLBACK();
 *     ADC1_MON3_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON3_IEN_Pos, ADC1_IRQEN_1_MON3_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 5 (MON4) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 5 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON4_IS == 1u)
 *   {
 *     ADC1_CH5_CALLBACK();
 *     ADC1_MON4_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON4_IEN_Pos, ADC1_IRQEN_1_MON4_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 6 (MON5) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 6 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON5_IS == 1u)
 *   {
 *     ADC1_CH6_CALLBACK();
 *     ADC1_MON5_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON5_IEN_Pos, ADC1_IRQEN_1_MON5_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 7 (P2.1) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 7 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_1_IS == 1u)
 *   {
 *     ADC1_CH7_CALLBACK();
 *     ADC1_P21_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_1_IEN_Pos, ADC1_IRQEN_1_P2_1_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 8 (P2.2) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 8 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_2_IS == 1u)
 *   {
 *     ADC1_CH8_CALLBACK();
 *     ADC1_P22_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_2_IEN_Pos, ADC1_IRQEN_1_P2_2_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 9 (P2.3) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 9 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_3_IS == 1u)
 *   {
 *     ADC1_CH9_CALLBACK();
 *     ADC1_P23_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_3_IEN_Pos, ADC1_IRQEN_1_P2_3_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 10 (P2.6) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 10 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_6_IS == 1u)
 *   {
 *     ADC1_CH10_CALLBACK();
 *     ADC1_P26_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_6_IEN_Pos, ADC1_IRQEN_1_P2_6_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 11 (P2.7) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 11 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_7_IS == 1u)
 *   {
 *     ADC1_CH11_CALLBACK();
 *     ADC1_P27_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_7_IEN_Pos, ADC1_IRQEN_1_P2_7_IEN_Msk, 1u);
}

/** \brief Enable ADC1 Channel 12 (P2.0) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 12 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P20_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_0_IS == 1u)
 *   {
 *     ADC1_CH12_CALLBACK();
 *     ADC1_P20_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P20_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_0_IEN_Pos, ADC1_IRQEN_1_P2_0_IEN_Msk, 1u);
}

/** \brief Enable Exceptional Interrupt Measurement (EIM).
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_EIM_En();
 *   ADC1_EIM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.EIM_IS == 1u)
 *   {
 *     ADC1_EIM_CALLBACK();
 *     ADC1_EIM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_EIM_IEN_Pos, ADC1_IRQEN_1_EIM_IEN_Msk, 1u);
}

/** \brief Enable Exceptional Sequence Measurement (ESM).
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Sequence Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_ESM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.ESM_IS == 1u)
 *   {
 *     ADC1_ESM_CALLBACK();
 *     ADC1_ESM_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_ESM_IEN_Pos, ADC1_IRQEN_1_ESM_IEN_Msk, 1u);
}

#if defined(TLE9845QX)
/** \brief Enable Differential Unit 1 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1LO_IS == 1u)
 *   {
 *     ADC1_DCH1_LO_CALLBACK();
 *     ADC1_DU1_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU1LO_IEN_Pos, ADC1_IRQEN_1_DU1LO_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 1 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1UP_IS == 1u)
 *   {
 *     ADC1_DCH1_UP_CALLBACK();
 *     ADC1_DU1_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU1UP_IEN_Pos, ADC1_IRQEN_1_DU1UP_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 2 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2LO_IS == 1u)
 *   {
 *     ADC1_DCH2_LO_CALLBACK();
 *     ADC1_DU2_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU2LO_IEN_Pos, ADC1_IRQEN_1_DU2LO_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 2 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2UP_IS == 1u)
 *   {
 *     ADC1_DCH2_UP_CALLBACK();
 *     ADC1_DU2_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU2UP_IEN_Pos, ADC1_IRQEN_1_DU2UP_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 3 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3LO_IS == 1u)
 *   {
 *     ADC1_DCH3_LO_CALLBACK();
 *     ADC1_DU3_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU3LO_IEN_Pos, ADC1_IRQEN_1_DU3LO_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 3 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3UP_IS == 1u)
 *   {
 *     ADC1_DCH3_UP_CALLBACK();
 *     ADC1_DU3_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU3UP_IEN_Pos, ADC1_IRQEN_1_DU3UP_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 4 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4LO_IS == 1u)
 *   {
 *     ADC1_DCH4_LO_CALLBACK();
 *     ADC1_DU4_Lo_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU4LO_IEN_Pos, ADC1_IRQEN_1_DU4LO_IEN_Msk, 1u);
}

/** \brief Enable Differential Unit 4 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4UP_IS == 1u)
 *   {
 *     ADC1_DCH4_UP_CALLBACK();
 *     ADC1_DU4_Up_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU4UP_IEN_Pos, ADC1_IRQEN_1_DU4UP_IEN_Msk, 1u);
}

#endif

/* ADC1 Interrupt Disable (ADC1_IRQEN_1) */
/** \brief Disable ADC1 Channel 0 (VbatSense) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 0 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VBatSense_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VBATSEN_IS == 1u)
 *   {
 *     ADC1_CH0_CALLBACK();
 *     ADC1_VbatSense_Int_Clr();
 *   } 
 *   ADC1_VBatSense_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VbatSense_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_VBATSEN_IEN_Pos, ADC1_IRQEN_1_VBATSEN_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 1 (VS) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 1 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Int_En();	 
 *   if (ADC1->IRQS_1.bit.VS_IS == 1u)
 *   {
 *     ADC1_CH1_CALLBACK();
 *     ADC1_VS_Int_Clr();
 *   } 
 *   ADC1_VS_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_VS_IEN_Pos, ADC1_IRQEN_1_VS_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 2 (MON1) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 2 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON1_IS == 1u)
 *   {
 *     ADC1_CH2_CALLBACK();
 *     ADC1_MON1_Int_Clr();
 *   } 
 *   ADC1_MON1_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON1_IEN_Pos, ADC1_IRQEN_1_MON1_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 3 (MON2) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 3 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON2_IS == 1u)
 *   {
 *     ADC1_CH3_CALLBACK();
 *     ADC1_MON2_Int_Clr();
 *   } 
 *   ADC1_MON2_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON2_IEN_Pos, ADC1_IRQEN_1_MON2_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 4 (MON3) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 4 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON3_IS == 1u)
 *   {
 *     ADC1_CH4_CALLBACK();
 *     ADC1_MON3_Int_Clr();
 *   } 
 *   ADC1_MON3_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON3_IEN_Pos, ADC1_IRQEN_1_MON3_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 5 (MON4) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 5 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON4_IS == 1u)
 *   {
 *     ADC1_CH5_CALLBACK();
 *     ADC1_MON4_Int_Clr();
 *   } 
 *   ADC1_MON4_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON4_IEN_Pos, ADC1_IRQEN_1_MON4_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 6 (MON5) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 6 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Int_En();	 
 *   if (ADC1->IRQS_1.bit.MON5_IS == 1u)
 *   {
 *     ADC1_CH6_CALLBACK();
 *     ADC1_MON5_Int_Clr();
 *   } 
 *   ADC1_MON5_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_MON5_IEN_Pos, ADC1_IRQEN_1_MON5_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 7 (P2.1) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 7 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_1_IS == 1u)
 *   {
 *     ADC1_CH7_CALLBACK();
 *     ADC1_P21_Int_Clr();
 *   }
 *   ADC1_P21_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_1_IEN_Pos, ADC1_IRQEN_1_P2_1_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 8 (P2.2) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 8 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_2_IS == 1u)
 *   {
 *     ADC1_CH8_CALLBACK();
 *     ADC1_P22_Int_Clr();
 *   } 
 *   ADC1_P22_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_2_IEN_Pos, ADC1_IRQEN_1_P2_2_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 9 (P2.3) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 9 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_3_IS == 1u)
 *   {
 *     ADC1_CH9_CALLBACK();
 *     ADC1_P23_Int_Clr();
 *   } 
 *   ADC1_P23_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_3_IEN_Pos, ADC1_IRQEN_1_P2_3_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 10 (P2.6) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 10 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_6_IS == 1u)
 *   {
 *     ADC1_CH10_CALLBACK();
 *     ADC1_P26_Int_Clr();
 *   } 
 *   ADC1_P26_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_6_IEN_Pos, ADC1_IRQEN_1_P2_6_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 11 (P2.7) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 11 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_7_IS == 1u)
 *   {
 *     ADC1_CH11_CALLBACK();
 *     ADC1_P27_Int_Clr();
 *   } 
 *   ADC1_P27_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_7_IEN_Pos, ADC1_IRQEN_1_P2_7_IEN_Msk, 0u);
}

/** \brief Disable ADC1 Channel 12 (P2.0) Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 Channel 12 Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P20_Int_En();	 
 *   if (ADC1->IRQS_1.bit.P2_0_IS == 1u)
 *   {
 *     ADC1_CH12_CALLBACK();
 *     ADC1_P20_Int_Clr();
 *   } 
 *   ADC1_P20_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P20_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_P2_0_IEN_Pos, ADC1_IRQEN_1_P2_0_IEN_Msk, 0u);
}

/** \brief Disable Exceptional Interrupt Measurement (EIM).
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Interrupt Measurement (EIM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_EIM_En();
 *   ADC1_EIM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.EIM_IS == 1u)
 *   {
 *     ADC1_EIM_CALLBACK();
 *     ADC1_EIM_Int_Clr();
 *   }
 *   ADC1_EIM_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_EIM_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_EIM_IEN_Pos, ADC1_IRQEN_1_EIM_IEN_Msk, 0u);
}

/** \brief Disable Exceptional Sequence Measurement (ESM).
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Exceptional Sequence Measurement (ESM).
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_ESM_Int_En(); 
 *   if (ADC1->IRQS_1.bit.ESM_IS == 1u)
 *   {
 *     ADC1_ESM_CALLBACK();
 *     ADC1_ESM_Int_Clr();
 *   }
 *   ADC1_ESM_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_ESM_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_ESM_IEN_Pos, ADC1_IRQEN_1_ESM_IEN_Msk, 0u);
}

#if defined(TLE9845QX)         
/** \brief Disable Differential Unit 1 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1LO_IS == 1u)
 *   {
 *     ADC1_DCH1_LO_CALLBACK();
 *     ADC1_DU1_Lo_Int_Clr();
 *   }
 *   ADC1_DU1_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU1LO_IEN_Pos, ADC1_IRQEN_1_DU1LO_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 1 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 1 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU1_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU1UP_IS == 1u)
 *   {
 *     ADC1_DCH1_UP_CALLBACK();
 *     ADC1_DU1_Up_Int_Clr();
 *   }
 *   ADC1_DU1_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU1_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU1UP_IEN_Pos, ADC1_IRQEN_1_DU1UP_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 2 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2LO_IS == 1u)
 *   {
 *     ADC1_DCH2_LO_CALLBACK();
 *     ADC1_DU2_Lo_Int_Clr();
 *   }
 *   ADC1_DU2_Lo_Int_Dis();  
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU2LO_IEN_Pos, ADC1_IRQEN_1_DU2LO_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 2 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 2 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU2_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU2UP_IS == 1u)
 *   {
 *     ADC1_DCH2_UP_CALLBACK();
 *     ADC1_DU2_Up_Int_Clr();
 *   }
 *   ADC1_DU2_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU2_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU2UP_IEN_Pos, ADC1_IRQEN_1_DU2UP_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 3 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3LO_IS == 1u)
 *   {
 *     ADC1_DCH3_LO_CALLBACK();
 *     ADC1_DU3_Lo_Int_Clr();
 *   }
 *   ADC1_DU3_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU3LO_IEN_Pos, ADC1_IRQEN_1_DU3LO_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 3 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 3 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU3_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU3UP_IS == 1u)
 *   {
 *     ADC1_DCH3_UP_CALLBACK();
 *     ADC1_DU3_Up_Int_Clr();
 *   }
 *   ADC1_DU3_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU3_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU3UP_IEN_Pos, ADC1_IRQEN_1_DU3UP_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 4 lower Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 lower Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Lo_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4LO_IS == 1u)
 *   {
 *     ADC1_DCH4_LO_CALLBACK();
 *     ADC1_DU4_Lo_Int_Clr();
 *   }
 *   ADC1_DU4_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU4LO_IEN_Pos, ADC1_IRQEN_1_DU4LO_IEN_Msk, 0u);
}

/** \brief Disable Differential Unit 4 upper Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the Differential Unit 4 upper Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_DU4_Up_Int_En(); 
 *   if (ADC1->IRQS_1.bit.DU4UP_IS == 1u)
 *   {
 *     ADC1_DCH4_UP_CALLBACK();
 *     ADC1_DU4_Up_Int_Clr();
 *   }
 *   ADC1_DU4_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_DU4_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_1.reg, (uint8)ADC1_IRQEN_1_DU4UP_IEN_Pos, ADC1_IRQEN_1_DU4UP_IEN_Msk, 0u);
}

#endif

/* ADC1 Threshold Interrupt Enable (ADC1_IRQEN_2) */
/** \brief Enable ADC1 VS Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_LO_IS == 1u)
 *   {
 *     ADC1_CH1_LO_CALLBACK();
 *     ADC1_VS_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_VS_LO_IEN_Pos, ADC1_IRQEN_2_VS_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON1 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_LO_IS == 1u)
 *   {
 *     ADC1_CH2_LO_CALLBACK();
 *     ADC1_MON1_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON1_LO_IEN_Pos, ADC1_IRQEN_2_MON1_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON2 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_LO_IS == 1u)
 *   {
 *     ADC1_CH3_LO_CALLBACK();
 *     ADC1_MON2_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON2_LO_IEN_Pos, ADC1_IRQEN_2_MON2_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON3 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_LO_IS == 1u)
 *   {
 *     ADC1_CH4_LO_CALLBACK();
 *     ADC1_MON3_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON3_LO_IEN_Pos, ADC1_IRQEN_2_MON3_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON4 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_LO_IS == 1u)
 *   {
 *     ADC1_CH5_LO_CALLBACK();
 *     ADC1_MON4_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON4_LO_IEN_Pos, ADC1_IRQEN_2_MON4_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON5 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_LO_IS == 1u)
 *   {
 *     ADC1_CH6_LO_CALLBACK();
 *     ADC1_MON5_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON5_LO_IEN_Pos, ADC1_IRQEN_2_MON5_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.1 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_LO_IS == 1u)
 *   {
 *     ADC1_CH7_LO_CALLBACK();
 *     ADC1_P21_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_1_LO_IEN_Pos, ADC1_IRQEN_2_P2_1_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.2 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_LO_IS == 1u)
 *   {
 *     ADC1_CH8_LO_CALLBACK();
 *     ADC1_P22_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_2_LO_IEN_Pos, ADC1_IRQEN_2_P2_2_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.3 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_LO_IS == 1u)
 *   {
 *     ADC1_CH9_LO_CALLBACK();
 *     ADC1_P23_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_3_LO_IEN_Pos, ADC1_IRQEN_2_P2_3_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.6 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_LO_IS == 1u)
 *   {
 *     ADC1_CH10_LO_CALLBACK();
 *     ADC1_P26_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_6_LO_IEN_Pos, ADC1_IRQEN_2_P2_6_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.7 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_LO_IS == 1u)
 *   {
 *     ADC1_CH11_LO_CALLBACK();
 *     ADC1_P27_Lo_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Lo_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_7_LO_IEN_Pos, ADC1_IRQEN_2_P2_7_LO_IEN_Msk, 1u);
}

/** \brief Enable ADC1 VS Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_UP_IS == 1u)
 *   {
 *     ADC1_CH1_UP_CALLBACK();
 *     ADC1_VS_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_VS_UP_IEN_Pos, ADC1_IRQEN_2_VS_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON1 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_UP_IS == 1u)
 *   {
 *     ADC1_CH2_UP_CALLBACK();
 *     ADC1_MON1_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON1_UP_IEN_Pos, ADC1_IRQEN_2_MON1_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON2 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_UP_IS == 1u)
 *   {
 *     ADC1_CH3_UP_CALLBACK();
 *     ADC1_MON2_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON2_UP_IEN_Pos, ADC1_IRQEN_2_MON2_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON3 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_UP_IS == 1u)
 *   {
 *     ADC1_CH4_UP_CALLBACK();
 *     ADC1_MON3_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON3_UP_IEN_Pos, ADC1_IRQEN_2_MON3_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON4 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_UP_IS == 1u)
 *   {
 *     ADC1_CH5_UP_CALLBACK();
 *     ADC1_MON4_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON4_UP_IEN_Pos, ADC1_IRQEN_2_MON4_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 MON5 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_UP_IS == 1u)
 *   {
 *     ADC1_CH6_UP_CALLBACK();
 *     ADC1_MON5_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON5_UP_IEN_Pos, ADC1_IRQEN_2_MON5_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.1 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_UP_IS == 1u)
 *   {
 *     ADC1_CH7_UP_CALLBACK();
 *     ADC1_P21_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_1_UP_IEN_Pos, ADC1_IRQEN_2_P2_1_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.2 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_UP_IS == 1u)
 *   {
 *     ADC1_CH8_UP_CALLBACK();
 *     ADC1_P22_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_2_UP_IEN_Pos, ADC1_IRQEN_2_P2_2_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.3 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_UP_IS == 1u)
 *   {
 *     ADC1_CH9_UP_CALLBACK();
 *     ADC1_P23_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_3_UP_IEN_Pos, ADC1_IRQEN_2_P2_3_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.6 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_UP_IS == 1u)
 *   {
 *     ADC1_CH10_UP_CALLBACK();
 *     ADC1_P26_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_6_UP_IEN_Pos, ADC1_IRQEN_2_P2_6_UP_IEN_Msk, 1u);
}

/** \brief Enable ADC1 P2.7 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_UP_IS == 1u)
 *   {
 *     ADC1_CH11_UP_CALLBACK();
 *     ADC1_P27_Up_Int_Clr();
 *   } 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Up_Int_En(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_7_UP_IEN_Pos, ADC1_IRQEN_2_P2_7_UP_IEN_Msk, 1u);
}


/* ADC1 Threshold Interrupt Disable (ADC1_IRQEN_2) */
/** \brief Disable ADC1 VS Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_LO_IS == 1u)
 *   {
 *     ADC1_CH1_LO_CALLBACK();
 *     ADC1_VS_Lo_Int_Clr();
 *   } 
 *   ADC1_VS_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_VS_LO_IEN_Pos, ADC1_IRQEN_2_VS_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON1 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_LO_IS == 1u)
 *   {
 *     ADC1_CH2_LO_CALLBACK();
 *     ADC1_MON1_Lo_Int_Clr();
 *   } 
 *   ADC1_MON1_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON1_LO_IEN_Pos, ADC1_IRQEN_2_MON1_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON2 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_LO_IS == 1u)
 *   {
 *     ADC1_CH3_LO_CALLBACK();
 *     ADC1_MON2_Lo_Int_Clr();
 *   } 
 *   ADC1_MON2_Lo_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON2_LO_IEN_Pos, ADC1_IRQEN_2_MON2_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON3 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_LO_IS == 1u)
 *   {
 *     ADC1_CH4_LO_CALLBACK();
 *     ADC1_MON3_Lo_Int_Clr();
 *   } 
 *   ADC1_MON3_Lo_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON3_LO_IEN_Pos, ADC1_IRQEN_2_MON3_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON4 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_LO_IS == 1u)
 *   {
 *     ADC1_CH5_LO_CALLBACK();
 *     ADC1_MON4_Lo_Int_Clr();
 *   } 
 *   ADC1_MON4_Lo_Int_Dis();	  
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON4_LO_IEN_Pos, ADC1_IRQEN_2_MON4_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON5 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_LO_IS == 1u)
 *   {
 *     ADC1_CH6_LO_CALLBACK();
 *     ADC1_MON5_Lo_Int_Clr();
 *   } 
 *   ADC1_MON5_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON5_LO_IEN_Pos, ADC1_IRQEN_2_MON5_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.1 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_LO_IS == 1u)
 *   {
 *     ADC1_CH7_LO_CALLBACK();
 *     ADC1_P21_Lo_Int_Clr();
 *   } 
 *   ADC1_P21_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_1_LO_IEN_Pos, ADC1_IRQEN_2_P2_1_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.2 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_LO_IS == 1u)
 *   {
 *     ADC1_CH8_LO_CALLBACK();
 *     ADC1_P22_Lo_Int_Clr();
 *   } 
 *   ADC1_P22_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_2_LO_IEN_Pos, ADC1_IRQEN_2_P2_2_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.3 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_LO_IS == 1u)
 *   {
 *     ADC1_CH9_LO_CALLBACK();
 *     ADC1_P23_Lo_Int_Clr();
 *   } 
 *   ADC1_P23_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_3_LO_IEN_Pos, ADC1_IRQEN_2_P2_3_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.6 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_LO_IS == 1u)
 *   {
 *     ADC1_CH10_LO_CALLBACK();
 *     ADC1_P26_Lo_Int_Clr();
 *   } 
 *   ADC1_P26_Lo_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_6_LO_IEN_Pos, ADC1_IRQEN_2_P2_6_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.7 Lower Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Lower Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Lo_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_LO_IS == 1u)
 *   {
 *     ADC1_CH11_LO_CALLBACK();
 *     ADC1_P27_Lo_Int_Clr();
 *   } 
 *   ADC1_P27_Lo_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Lo_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_7_LO_IEN_Pos, ADC1_IRQEN_2_P2_7_LO_IEN_Msk, 0u);
}

/** \brief Disable ADC1 VS Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 VS Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_VS_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.VS_UP_IS == 1u)
 *   {
 *     ADC1_CH1_UP_CALLBACK();
 *     ADC1_VS_Up_Int_Clr();
 *   } 
 *   ADC1_VS_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_VS_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_VS_UP_IEN_Pos, ADC1_IRQEN_2_VS_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON1 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON1_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON1_UP_IS == 1u)
 *   {
 *     ADC1_CH2_UP_CALLBACK();
 *     ADC1_MON1_Up_Int_Clr();
 *   }
 *   ADC1_MON1_Up_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON1_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON1_UP_IEN_Pos, ADC1_IRQEN_2_MON1_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON2 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON2_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON2_UP_IS == 1u)
 *   {
 *     ADC1_CH3_UP_CALLBACK();
 *     ADC1_MON2_Up_Int_Clr();
 *   } 
 *   ADC1_MON2_Up_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON2_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON2_UP_IEN_Pos, ADC1_IRQEN_2_MON2_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON3 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON3_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON3_UP_IS == 1u)
 *   {
 *     ADC1_CH4_UP_CALLBACK();
 *     ADC1_MON3_Up_Int_Clr();
 *   } 
 *   ADC1_MON3_Up_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON3_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON3_UP_IEN_Pos, ADC1_IRQEN_2_MON3_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON4 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON4 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON4_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON4_UP_IS == 1u)
 *   {
 *     ADC1_CH5_UP_CALLBACK();
 *     ADC1_MON4_Up_Int_Clr();
 *   } 
 *   ADC1_MON4_Up_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON4_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON4_UP_IEN_Pos, ADC1_IRQEN_2_MON4_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 MON5 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 MON5 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_MON5_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.MON5_UP_IS == 1u)
 *   {
 *     ADC1_CH6_UP_CALLBACK();
 *     ADC1_MON5_Up_Int_Clr();
 *   } 
 *   ADC1_MON5_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_MON5_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_MON5_UP_IEN_Pos, ADC1_IRQEN_2_MON5_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.1 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.1 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P21_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_1_UP_IS == 1u)
 *   {
 *     ADC1_CH7_UP_CALLBACK();
 *     ADC1_P21_Up_Int_Clr();
 *   } 
 *   ADC1_P21_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P21_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_1_UP_IEN_Pos, ADC1_IRQEN_2_P2_1_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.2 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.2 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P22_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_2_UP_IS == 1u)
 *   {
 *     ADC1_CH8_UP_CALLBACK();
 *     ADC1_P22_Up_Int_Clr();
 *   } 
 *   ADC1_P22_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P22_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_2_UP_IEN_Pos, ADC1_IRQEN_2_P2_2_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.3 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.3 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P23_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_3_UP_IS == 1u)
 *   {
 *     ADC1_CH9_UP_CALLBACK();
 *     ADC1_P23_Up_Int_Clr();
 *   } 
 *   ADC1_P23_Up_Int_Dis();	 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P23_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_3_UP_IEN_Pos, ADC1_IRQEN_2_P2_3_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.6 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.6 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P26_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_6_UP_IS == 1u)
 *   {
 *     ADC1_CH10_UP_CALLBACK();
 *     ADC1_P26_Up_Int_Clr();
 *   } 
 *   ADC1_P26_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P26_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_6_UP_IEN_Pos, ADC1_IRQEN_2_P2_6_UP_IEN_Msk, 0u);
}

/** \brief Disable ADC1 P2.7 Upper Threshold Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC1 P2.7 Upper Threshold Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   ADC1_P27_Up_Int_En();	 
 *   if (ADC1->IRQS_2.bit.P2_7_UP_IS == 1u)
 *   {
 *     ADC1_CH11_UP_CALLBACK();
 *     ADC1_P27_Up_Int_Clr();
 *   } 
 *   ADC1_P27_Up_Int_Dis(); 
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */
INLINE void ADC1_P27_Up_Int_Dis(void)
{
  Field_Mod32(&ADC1->IRQEN_2.reg, (uint8)ADC1_IRQEN_2_P2_7_UP_IEN_Pos, ADC1_IRQEN_2_P2_7_UP_IEN_Msk, 0u);
}

/** \brief Set ADC1 EIM channel
 *
 * \param channel EIM mode channel
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Sequencer Mode, and programs channel 2 (MON1) and reads the result.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     uint16 adc_result;
 *
 *     ADC1_Power_On();
 *     ADC1_Sequencer_Mode_Sel();
 *     ADC1_SetEIMChannel( ADC1_CH2 );
 *
 *     while (ADC1->FILT_OUTEIM.bit.VF_EIM == 0u)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */
INLINE void ADC1_SetEIMChannel(uint8 channel)
{
  Field_Mod32(&ADC1->CHx_EIM.reg, (uint8)ADC1_CHx_EIM_EIM_CHx_Pos, ADC1_CHx_EIM_EIM_CHx_Msk, channel);
}                               /* End of ADC1_SetEIMChannel */

/** \brief Set ADC1 SW_MODE channel
 *
 * \param channel ADC1 SW mode channel
 *
 * \brief <b>Example</b><br>
 * \brief This example starts an ADC1 conversion on channel 12 - P2.0.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_Software_Mode_Sel(); 
 *     ADC1_SetSwMode_Channel(ADC1_P20);
 *     ADC1_SOS_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */ 
INLINE void ADC1_SetSwMode_Channel(uint8 channel)
{
  ADC1->CTRL_STS.bit.SW_CH_SEL = channel;
}

/** \brief Set ADC1 Sequencer Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC and sets Sequencer Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_SetSeqMode();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup ADC1_api
 */ 
INLINE void ADC1_SetSeqMode(void)
{
  ADC1->CTRL3.bit.SW_MODE = 0;
  while (ADC1_SQ_Stop_Sts() == 1u)
  {
  }
}

/** \brief Set ADC1 Software Mode
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC and sets Software Mode.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_SetSwMode();
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */ 
INLINE void ADC1_SetSwMode(void)
{
  ADC1->CTRL3.bit.SW_MODE = 1;
  while (ADC1_SQ_Stop_Sts() == 0u)
  {
  }
}

/** \brief Start ADC1 SW_MODE conversion
 *
 * \param channel  ADC channel number to be converted 
 *
 * \brief This example enables ADC and starts Software Mode conversion for channel 6.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On(); 
 *     ADC1_SetSwMode(); 
 *     ADC1_SetSocSwMode(ADC1_MON5);
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */ 
INLINE void ADC1_SetSocSwMode(uint8 channel)
{
  ADC1->CTRL_STS.bit.SW_CH_SEL = channel;
  ADC1->CTRL_STS.bit.SOS = 1;
}

/** \brief Get ADC1 Software Mode End Of Conversion Status
 *
 * \retval true conversion done
 * \retval false conversion ongoing
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC1 and switches to Software Mode, starts a measurement and waits for it to be finished.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     ADC1_Power_On();
 *     ADC1_Software_Mode_Sel();
 *     ADC1_Sequencer_Mode_Sel( ADC1_MON1 );
 *     ADC1_SOS_Set();
 *
 *     while ( ADC1_GetEocSwMode() != true ) {}
 * }
 * ~~~~~~~~~~~~~~~ 
 * \ingroup ADC1_api
 */ 
INLINE bool ADC1_GetEocSwMode(void)
{
  bool res = false;
  
  if (ADC1_EOC_Sts() == (uint8)1)
  {
    res = true;
  }
  return (res);
}

#endif /* ADC1_H */
